#include "Cell.h"

// Static Data Initialization

// Permittivity and permeability of free space in MKS
// Taken from http://physics.nist.gov/cuu/Constants/index.html
const double Cell::EPS0 = 8.854187817E-12;
const double Cell::MU0 = 12.566370614E-7;

// Simulation Parameters
double Cell::delX=0.0;
double Cell::delY=0.0;
double Cell::delZ=0.0;
double Cell::delT=0.0;


// Cell constructor
Cell::Cell(){

	Ex=Ey=Ez=Hx=Hy=Hz=0.0;	
	cellBackX=0;
	cellBackY=0;
	cellBackZ=0;
	cellFrontX=0;
	cellFrontY=0;
	cellFrontZ=0;	
};

// Each cell has 6 neighbours in 3D space. checkNeighbours() tests whether they exist or not.
void Cell::checkNeighbours(int i,int j,int k){
	
	if (cellBackX==0 || cellBackY==0 || cellBackZ==0 || cellFrontX==0 || cellFrontY==0 || cellFrontZ==0){
		printf("Neighbour Problem at Cell (%d,%d,%d): ",i,j,k);
		if (cellBackX == 0)
			printf("No backX ");
		if (cellBackY == 0)
			printf("No backY ");
		if (cellBackZ == 0)
			printf("No backZ ");
		if (cellFrontX == 0)
			printf("No frontX ");
		if (cellFrontY == 0)
			printf("No frontY ");
		if (cellFrontZ == 0)
			printf("No frontZ ");
		printf(". Exiting...\n");
		exit(EXIT_FAILURE);
	}	
};