#ifndef cellGUARD
#define cellGUARD

// Include for printf() function
#include <stdio.h>

// Include for exit() function
#include <stdlib.h>

// The abstract base class out of which every other space object is derived
// All derived classes should implement the updateFields() method and define the necessary constants

class Cell{
public:
	
	double Ex, Ey, Ez, Hx, Hy, Hz;				// Field parameters in the Yee cell
	static double delX, delY, delZ, delT;		// Global parameters of the simulation domain
	const static double EPS0, MU0;				// Permittivity and permeability of free space
	Cell *cellBackX, *cellBackY, *cellBackZ ;	// neigbouring cells one step back in space
	Cell *cellFrontX, *cellFrontY, *cellFrontZ; // neigbouring cells one step front in space

	// Constructor method
	// Make sure you instantiate objects after setting the static variables delX, delY, delZ and delT
	// Otherwise, simulation constants for each cell may not be set correctly
	Cell();
	//Destructor method
	virtual ~Cell() {};
	
	// Debugging methods to check whether all neighbours are set correctly
	void checkNeighbours(int i,int j,int k);
	// This method is for checking brick type cells
	virtual void checkNeighbours(int i) {printf("%d - This method should be called from brick cells.\n",i);};
	
	// In order to implement the total field / scattered field (TF/SF) formalism, equivalent current sources
	// should be added on the surface of the TF/SF boundary. The functions below are used to add the current
	// sources
	virtual void addJx(double Jx){printf("addJx(%f) not defined in this cell.\n",Jx);exit(EXIT_FAILURE);};
	virtual void addJy(double Jy){printf("addJy(%f) not defined in this cell.\n",Jy);exit(EXIT_FAILURE);};
	virtual void addJz(double Jz){printf("addJz(%f) not defined in this cell.\n",Jz);exit(EXIT_FAILURE);};
	virtual void addMx(double Mx){printf("addMx(%f) not defined in this cell.\n",Mx);exit(EXIT_FAILURE);}
	virtual void addMy(double My){printf("addMy(%f) not defined in this cell.\n",My);exit(EXIT_FAILURE);};
	virtual void addMz(double Mz){printf("addMz(%f) not defined in this cell.\n",Mz);exit(EXIT_FAILURE);}
	// Virtual methods to be implemented by child classes
	// The Cell object is made abstract by setting updateFields()=0
		
	virtual void updateEFields()=0;
	virtual void updateHFields()=0;

};

#endif