#include "DielectricECell.h"

// Constructor for DielectricECell Class
DielectricECell::DielectricECell(double epsr){
	EC = delT / (EPS0*epsr);
	HC = delT / MU0;
};


void DielectricECell::updateEFields(){
	Ex = Ex + EC * ( (Hz-cellBackY->Hz)/delY - (Hy-cellBackZ->Hy)/delZ );
	Ey = Ey + EC * ( (Hx-cellBackZ->Hx)/delZ - (Hz-cellBackX->Hz)/delX );
	Ez = Ez + EC * ( (Hy-cellBackX->Hy)/delX - (Hx-cellBackY->Hx)/delY );
};

void DielectricECell::updateHFields(){
	Hx = Hx + HC * ( (cellFrontZ->Ey-Ey)/delZ - (cellFrontY->Ez-Ez)/delY );
	Hy = Hy + HC * ( (cellFrontX->Ez-Ez)/delX - (cellFrontZ->Ex-Ex)/delZ );
	Hz = Hz + HC * ( (cellFrontY->Ex-Ex)/delY - (cellFrontX->Ey-Ey)/delX );
};

void DielectricECell::addJx(double Jx){
	Ex = Ex - EC * Jx;
};

void DielectricECell::addJy(double Jy){
	Ey = Ey - EC * Jy;
};

void DielectricECell::addJz(double Jz){
	Ez = Ez - EC * Jz;
};

void DielectricECell::addMx(double Mx){
	Hx = Hx - HC * Mx;
};

void DielectricECell::addMy(double My){
	Hy = Hy - HC * My;
};

void DielectricECell::addMz(double Mz){
	Hz = Hz - HC * Mz;
};
