% noLayers is the number of PML layers to be added, it should have 3
% elements [lx ly lz] denoting the number of layers in each direction.
% objectMapIn is the input structure of cell arrays
% objectColotIn is the 3D matrix that holds the material info
% in the big structure that holds the simulation params

function [objectMap,objectColor,material] = addPML(noLayers,objectMapIn,objectColorIn,in)
% Set simulation params
NX = size(objectMapIn,1);
NY = size(objectMapIn,2);
NZ = size(objectMapIn,3);
delX = in.dx*in.unitLength;
delY = in.dy*in.unitLength;
delZ = in.dz*in.unitLength;

% Load physical constants
physicalConstants;

% Add the PML boundaries around objectMapIn and set the coefficients of PML medium
% Number of PML layers in x,y,z dimensions
lx=noLayers(1);
ly=noLayers(2);
lz=noLayers(3);


% Power m used in polynomial grading
m=4;
% Create the objectMap, objectColor and copy objectMapIn,objectColorIn into it
objectMap = cell(NX+2*lx,NY+2*ly,NZ+2*lz);
objectColor = zeros(size(objectMap));
objectMap(lx+1:NX+lx,ly+1:NY+ly,lz+1:NZ+lz) = objectMapIn;
objectColor(lx+1:NX+lx,ly+1:NY+ly,lz+1:NZ+lz) = objectColorIn;

% set the PML layer color
material = in.material;
PMLColor = length(in.material)+1;
material{PMLColor}.type = 'PMLCell';
material{PMLColor}.name = 'SimplePML';

% Create the polynomially graded PML regions

% Eta is taken to be that in free space, if performance of PML is low, try
% changing the eta value.

eta = sqrt(PC.mu0 / PC.eps0);
Sxmax=0.8*(m+1)/(eta*delX);
Symax=0.8*(m+1)/(eta*delY);
Szmax=0.8*(m+1)/(eta*delZ);

% getParent gives the parent cell from which PML parameters should be
% derived. Polynomial grading is specified as explaiend in Taflove 2nd Ed.
% pp 306-307.

% Due to the for loops, this part of code is really slow.. Do something to
% make it faster...
disp('Adding PMLs... Due to unoptimized code, it''ll take a while...');
for ii=1:NX+2*lx
    for jj=1:NY+2*ly
        for kk=1:NZ+2*lz
            try
                [x,y,z]=getParent(ii,jj,kk,NX,NY,NZ,noLayers);
            catch
                if length(findstr(lasterr,'Not in PML layer!'))==1
                    continue;
                else
                    error(lasterr);
                end
            end
            objectColor(ii,jj,kk) = PMLColor;
            objectMap{ii,jj,kk}.type = 'PMLCell';
            if lx~=0
                objectMap{ii,jj,kk}.Sx = (abs(x-ii)/lx)^m*Sxmax;
            else
                objectMap{ii,jj,kk}.Sx = 0;
            end
            if ly~=0
                objectMap{ii,jj,kk}.Sy = (abs(y-jj)/ly)^m*Symax;
            else
                objectMap{ii,jj,kk}.Sy = 0;
            end
            if lz~=0
                objectMap{ii,jj,kk}.Sz = (abs(z-kk)/lz)^m*Szmax;
            else
                objectMap{ii,jj,kk}.Sz = 0;
            end
            objectMap{ii,jj,kk}.Kx = 1.0;
            objectMap{ii,jj,kk}.Ky = 1.0;
            objectMap{ii,jj,kk}.Kz = 1.0;
            % Here epsr is matched for regular cells, for Lorentz cells, it
            % is taken to be one. If unacceptable behaviour occurs,
            % implement the PML for dispersive media and terminate with it.
            if strcmp(objectMap{x,y,z}.type,'DielectricECell')
                objectMap{ii,jj,kk}.epsr = objectMap{x,y,z}.epsr;
            elseif strcmp(objectMap{x,y,z}.type,'LorentzCell')
                objectMap{ii,jj,kk}.epsr = 1.0;
            end
        end
    end
end