% This function puts the objects into objectMapIn
% As new shape types are added, this function needs to be modified
% Implemented shapes are as follows:

% shape='Block'

function createObjects(input,objectName)
global objectMapIn
global objectColorIn

object = input.(objectName);
material = input.material;
origin = input.centerPoint;
dx = input.dx;
dy = input.dy;
dz = input.dz;

s = size(objectMapIn);
NX = s(1);
NY = s(2);
NZ = s(3);
if mod(NX,2)~=0 || mod(NY,2)~=0 || mod(NZ,2)~=0
    error('objectMapIn is not of expected size');
end

for ii=1:length(object)
    matIndex = object{ii}.materialIndex;
    mat = material{matIndex};    
    if strcmp(object{ii}.shape,'Block')==1
        p1 = object{ii}.start;
        p2 = object{ii}.end;
        % Calculate the distance to the origin in terms of delta's
        p1 = (p1 - origin) ./ [dx dy dz];
        p2 = (p2 - origin) ./ [dx dy dz];
        
        % If an object intersects a cell, make the cell a member of the
        % object by the floor function.
        p1 = floor(p1);
        p2 = floor(p2);
        
        % Sort p1 and p2 so that for arbitrary opposing corners the
        % algorithm works.
        pstart = min(p1,p2);
        pend = max(p1,p2);
        
        % The origin has the indices [ NX/2 +1,NY/2 +1,NZ/2 +1] in the
        % objectMapIn matrix. Thus convert from coordinates wrt origin to
        % coordinates wrt objectMapIn
        pstart = pstart + [ (NX/2+1), (NY/2+1) , (NZ/2+1)];
        pend = pend + [ (NX/2+1), (NY/2+1) , (NZ/2+1)];
        
        % Clip the limits that are out of bounds        
        % Make sure there are no terms less than 1
        pstart = max(pstart,[1 1 1]);
        pend = max(pend,[1 1 1]);
        
        % Make sure there are no terms greater than matrix limits
        pstart = min(pstart,[NX NY NZ]);
        pend = min(pend,[NX NY NZ]);    
        
        % Update the objectMapIn
        objectMapIn(pstart(1):pend(1),pstart(2):pend(2),pstart(3):pend(3))={mat};
        objectColorIn(pstart(1):pend(1),pstart(2):pend(2),pstart(3):pend(3))=matIndex;
    else
        error(sprintf('Shape %s not known',objects{ii}.shape));
    end
end
