% This function calculates the indices of the specified output locations

function indices=createOutputIndices(output,input,objectColor)

dx = input.dx;
dy = input.dy;
dz = input.dz;
origin = input.centerPoint;

NX = size(objectColor,1);
NY = size(objectColor,2);
NZ = size(objectColor,3);

indices=[];

for ii=1:length(output)
    if output{ii}.type == 'Block'
        p1 = output{ii}.start;
        p2 = output{ii}.end;
        
        % Calculate the distance to the origin in terms of delta's
        p1 = (p1 - origin) ./ [dx dy dz];
        p2 = (p2 - origin) ./ [dx dy dz];
        
        % If the outout box intersects a cell, put the cell into the box
        p1 = floor(p1);
        p2 = floor(p2);
        
        % Sort p1 and p2 so that for arbitrary opposing corners the
        % algorithm works.
        pstart = min(p1,p2);
        pend = max(p1,p2);
        
        % The origin has the indices [ NX/2 +1,NY/2 +1,NZ/2 +1] in the
        % objectMapIn matrix. Thus convert from coordinates wrt origin to
        % coordinates wrt objectMapIn
        pstart = pstart + [ (NX/2+1), (NY/2+1) , (NZ/2+1)];
        pend = pend + [ (NX/2+1), (NY/2+1) , (NZ/2+1)];
        
        % Clip the limits that are out of bounds        
        % Make sure there are no terms less than 1
        pstart = max(pstart,[1 1 1]);
        pend = max(pend,[1 1 1]);
        
        % Make sure there are no terms greater than matrix limits
        pstart = min(pstart,[NX NY NZ]);
        pend = min(pend,[NX NY NZ]);    
        
        [ilist,jlist,klist]=ndgrid(pstart(1):pend(1),pstart(2):pend(2),pstart(3):pend(3));
        
        ilist = reshape(ilist,[1 prod(size(ilist))]);
        jlist = reshape(jlist,[1 prod(size(jlist))]);
        klist = reshape(klist,[1 prod(size(klist))]);
        
        indices = [indices [ilist;jlist;klist]];
        
    elseif output{ii}.type == 'Point'
        % Algorithm very similar to above.
        p1 = output{ii}.point;
        p1 = (p1 - origin) ./ [dx dy dz];
        p1 = floor(p1);
        p1 = p1 + [ (NX/2+1), (NY/2+1) , (NZ/2+1)];
        p1 = max(p1,[1 1 1]);
        p1 = min(p1,[NX NY NZ]);
        indices = [indices p1'];
    end
    
end


