function source=getSource(lambda,delT,delF,numCycles)
% lambda = wavelength in meters
% delT = time discretization in seconds
% delF = Width of Gaussian in terms of center frequency
% numCycles = number of center frequency cycles

physicalConstants
f0=PC.c / lambda; % center frequency in Hz

% Number of time steps for simulation
timeSteps = ceil(numCycles* lambda/PC.c / delT); % total time steps

% Gaussian Width in frequency in terms of the center frequency
delf=f0*delF;

% Shifted Gaussian Source

time = (0:timeSteps-1)*delT;
% 1/(pi*delf) is the time for 1/e magnitude, Gaussian starts at the time
% when it is exp(-5^2)~=1E-11
t0=5*1/(pi*delf);  
source = exp(-(time - t0).^2 * pi^2 * delf^2).*sin(2*pi*f0*(time-t0));

figure
plot(time*f0,source)
xlabel('Time Step');ylabel('Magnitude');

sourceFreq = fft(source);

figure
fs=1 / delT; %sampling Frequency
f=linspace(0,fs,length(time)); % the max frequency is fs/2 in fourier domain, and thus 2*pi corresponds to fs
subplot(2,1,1)
plot(f/f0,abs(sourceFreq)/max(abs(sourceFreq)))
xlim([0.0 2.0])
xlabel('f/f_0');ylabel('Magnitude');
subplot(2,1,2)
semilogy(f/f0,abs(sourceFreq)/max(abs(sourceFreq)))
xlim([0.0 10.0])
xlabel('f/f_0');ylabel('Magnitude');
