% This file parses the text file that defines the simulation parameters
% Input is the structure that defines the simulation domain and parameters
function input = parseInput(filename)

rawInput = {''}; % rawInput is the input file, stripped of empty lines and comment lines
index=1; % index used for rawInput
fid = fopen(filename); %fid is the file identifier
% Locations of definitions in rawInput file
sourceStart = 1; % Source start row
sourceEnd = 1;     % Source end row
layerStart =1 ; %Layer start row
layerEnd =1 ; %Layer end row
wireStart = 1; % Wire start row
wireEnd = 1; % Wire end row
antennaStart = 1; % Antenna start row
antennaEnd = 1; % Antenna end row
materialStart =1 ; %Material start row
materialEnd =1 ; %Material end row
outputLayerStart =1;% Outputs for the layer simulation
outputLayerEnd =1;
outputWireStart =1;% Outputs for the wired layer simulation
outputWireEnd =1;
outputAntennaStart =1;% Outputs for the antenna layer simulation
outputAntennaEnd =1;
objects = {}; % Cell array of strings that define different objects
materials ={}; % Cell array of strings that define different materials


while 1
    tline = fgetl(fid);
    % Stop when end of file is reached
    if ~ischar(tline)
        break
    end
    % Ignore lines made out of blank characters
    if length(tline)==sum(isspace(tline))
        continue;
    end
    % Ignore lines that start with '//'
    token = strtok(tline);
    if (token(1)=='/' && token(2)=='/')
        continue;
    end
    rawInput{index}=tline;
    index = index +1;
    disp(tline)
end
fclose(fid);

% Get basic simulation parameters unitLength, dx, dy, dz, sizeX, sizeY,
% sizeZ, centerPoint

input.unitLength = parseParam('unitLength',rawInput);
input.dx = parseParam('dx',rawInput);
input.dy = parseParam('dy',rawInput);
input.dz = parseParam('dz',rawInput);
input.sizeX = parseParam('sizeX',rawInput);
input.sizeY = parseParam('sizeY',rawInput);
input.sizeZ = parseParam('sizeZ',rawInput);
input.centerPoint = parseParam('centerPoint',rawInput);
input.isTE = parseParam('isTE',rawInput);
input.lambda = parseParam('centerLambda',rawInput);
input.delF = parseParam('freqWidth',rawInput);
input.numCycles = parseParam('numCycles',rawInput);



% Get the line numbers of the objects and materials
materialStart=strmatch('materialStart',rawInput)+1;
materialEnd=strmatch('materialEnd',rawInput)-1;
layerStart=strmatch('layerStart',rawInput)+1;
layerEnd=strmatch('layerEnd',rawInput)-1;
wireStart=strmatch('wireStart',rawInput)+1;
wireEnd=strmatch('wireEnd',rawInput)-1;
antennaStart = strmatch('antennaStart',rawInput)+1;
antennaEnd = strmatch('antennaEnd',rawInput)-1;
outputLayerStart = strmatch('outputLayerStart',rawInput)+1;
outputLayerEnd = strmatch('outputLayerEnd',rawInput)-1;
outputWireStart = strmatch('outputWireStart',rawInput)+1;
outputWireEnd = strmatch('outputWireEnd',rawInput)-1;
outputAntennaStart = strmatch('outputAntennaStart',rawInput)+1;
outputAntennaEnd = strmatch('outputAntennaEnd',rawInput)-1;

% Copy the objects and material strings

if materialStart <= materialEnd
    material = rawInput(materialStart:materialEnd);
else
    material = {};
end


if layerStart<=layerEnd
    layer = rawInput(layerStart:layerEnd);
else
    layer = {};
end

if wireStart<=wireEnd
    wire = rawInput(wireStart:wireEnd);
else
    wire = {};
end

if antennaStart <= antennaEnd
    antenna = rawInput(antennaStart:antennaEnd);
else
    antenna = {};
end

if outputLayerStart <= outputLayerEnd
    outputLayer = rawInput(outputLayerStart:outputLayerEnd);
else
    outputLayer = {};
end

if outputWireStart <= outputWireEnd
    outputWire = rawInput(outputWireStart:outputWireEnd);
else
    outputWire = {};
end

if outputAntennaStart <= outputAntennaEnd
    outputAntenna = rawInput(outputAntennaStart:outputAntennaEnd);
else
    outputAntenna = {};
end

input.layer = {};
input.wire = {};
input.antenna = {};
input.material = {};
input.outputLayer = {};
input.outputWire = {};
input.outputAntenna = {};

% Parse the material strings into valid material structures
for ii=1:length(material)
    input.material{ii}=parseMaterial(material{ii});    
end

% Parse the layer strings into valid object structures
for ii=1:length(layer)
    input.layer{ii}=parseObject(layer{ii},input.material);
end

% Parse the wire strings into valid object structures
for ii=1:length(wire)
    input.wire{ii}=parseObject(wire{ii},input.material);
end

% Parse the antenna strings into valid object structures
for ii=1:length(antenna)
    input.antenna{ii}=parseObject(antenna{ii},input.material);
end

% Parse the output specifications
for ii=1:length(outputLayer)
    input.outputLayer{ii}=parseOutputParams(outputLayer{ii});
end

for ii=1:length(outputWire)
    input.outputWire{ii}=parseOutputParams(outputWire{ii});
end

for ii=1:length(outputAntenna)
    input.outputAntenna{ii}=parseOutputParams(outputAntenna{ii});
end

