% This function parses the material parameters for a given material type
% As new material types are added this function needs to be modified
% The implemented types and their structures are as follows

% type='Dielectric'
% name
% epsr -> relative permittivity

% type='Lorentz'
% name
% numPoles -> number of poles
% plasmon -> plasmon frequencies
% oscillator -> oscillator frequencies
% damping -> damping constants

function out=parseMaterial(matString)
% out is the structure corresponding to the type with given parameters

[token,rem] = strtok(matString);
if strcmp(token,'Dielectric')==1
    out.type = 'DielectricECell';
    [token,rem]=strtok(rem);
    out.name = token;
    out.epsr = str2double(strtok(rem));
elseif strcmp(token,'Lorentz')==1
    out.type = 'LorentzCell';
    [token,rem]=strtok(rem);
    out.name = token;
    params = str2num(rem);
    if mod(length(params),3)~=0
        error('Incorrect Lorentz parameters!');
    end
    out.numPoles = length(params)/3;
    out.plasmon = params(1:3:length(params));
    out.oscillator = params(2:3:length(params));
    out.damping = params(3:3:length(params));
else
    error('Unknown material type!');
end
    