clear all
% Load the variables for visualization purposes
load postSim.mat
physicalConstants

WIREFILENAME='./SimFiles/layer.out';
ANTENNAFILENAME = './SimFiles/antenna.out';

numWireOutputs   =size(objectMap2DWire_outputIndices,2);
numAntennaOutputs=size(objectMap3D_outputIndices,2);

% First read in the normalization data point
fid=fopen(WIREFILENAME);
[wireOutput,count]=fread(fid,inf,'double');
fclose(fid);
wireOutput=reshape(wireOutput,6,numWireOutputs,[]);

% Calculate the fourier transform of the point
wireFFT = zeros(size(wireOutput));
%  fft(X,[],dim) applies the FFT operation across the dimension dim
wireFFT = fft(wireOutput,[],3);

% Calculate the indices of the starting and stopping points for the
% frequency distribution. The variables below determine the limits.

lambdaStart = 500; % in nm.
lambdaEnd = 2000;  % in nm.

time = (0:size(wireFFT,3)-1)*delT;
fs=1 / delT; %sampling Frequency
f=linspace(0,fs,length(time)); % the max frequency is fs/2 in fourier domain, and thus 2*pi corresponds to fs
en=f*PC.h/PC.q; % Energy in terms of eV
lambdaList = PC.c ./ f * 1e9; % in terms of nanometers
[dellambda,lambdaStartIndex] = min(abs(lambdaList-lambdaStart));
[dellambda,lambdaEndIndex] = min(abs(lambdaList-lambdaEnd));
clear dellambda
% Sort the indices to prevent a starting index larger than the ending index
startIndex = min(lambdaStartIndex,lambdaEndIndex);
endIndex = max(lambdaStartIndex,lambdaEndIndex);
clear lambdaStartIndex lambdaEndIndex
% Modify the plotting variables
lambdaList = lambdaList(startIndex:endIndex);
f = f(startIndex:endIndex);
en = en(startIndex:endIndex);

% Get rid of all the unnecessary frequency components and calculate the
% normalization parameters
wireFFT = squeeze(abs(wireFFT(:,1,startIndex:endIndex)).^2);
% The first three rows are Ex,Ey,Ez
normE = sum(wireFFT(1:3,:));
% The last three are Hx,Hy,Hz
normH = sum(wireFFT(4:6,:));

EfieldEnhancement = zeros(numAntennaOutputs,length(lambdaList));
for ii=1:numAntennaOutputs
    x=getEnhancementE(ANTENNAFILENAME,numAntennaOutputs,ii,startIndex,endIndex,normE,size(source,2));
    EfieldEnhancement(ii,:)=x;
    disp(sprintf('Point %d of %d at simulation point [%d %d %d]',ii,numAntennaOutputs,objectMap3D_outputIndices(1,ii),objectMap3D_outputIndices(2,ii),objectMap3D_outputIndices(3,ii)));
end
save 'EfieldEnhancement' EfieldEnhancement

% Find the space coordinates by first detecting which of the indices change
% The following line finds the index of the nonzero element
index=find(objectMap3D_outputIndices(:,1)-objectMap3D_outputIndices(:,2));
spaceCoord = objectMap3D_outputIndices(index,:);

figure
h=pcolor(lambdaList,spaceCoord,10*log10(EfieldEnhancement));
set(h,'EdgeColor','none');
xlabel('Wavelength in nm');
switch index
    case 1
        ylabel('Space coordinate (x)');
    case 2
        ylabel('Space coordinate (y)');
    case 3
        ylabel('Space coordinate (z)');
    otherwise
        ylabel('Space coordinate screwed!');
end
title(sprintf('|E|^2 Ratio in dB'));
colorbar('East');
axis([lambdaStart lambdaEnd spaceCoord(1) spaceCoord(numAntennaOutputs)])

figure
h=surf(lambdaList,spaceCoord,10*log10(EfieldEnhancement));
set(h,'EdgeColor','none');
xlabel('Wavelength in nm');

switch index
    case 1
        ylabel('Space coordinate (x)');
    case 2
        ylabel('Space coordinate (y)');
    case 3
        ylabel('Space coordinate (z)');
    otherwise
        ylabel('Space coordinate screwed!');
end

title(sprintf('|E|^2 Ratio in dB'));
colorbar('East');
axis([lambdaStart lambdaEnd spaceCoord(1) spaceCoord(numAntennaOutputs) -30 10*log10(max(max(EfieldEnhancement)))])
return
% The lines below are for individual layer visualization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for ii=1:numAntennaOutputs
    figure(123)
    spaceValue = spaceCoord(ii);
    [delSpaceValue,spaceCoordIndex] = min(abs(spaceCoord-spaceValue));
    h=plot(lambdaList,EfieldEnhancement(spaceCoordIndex,:));
    xlabel('Wavelength in nm');
    ylabel('Enhancement');
    title(sprintf('|E|^2 Ratio at Point %d',spaceValue));
    pause;
end