clear all

load('./SimFiles/postSim.mat');
physicalConstants

WIREFILENAME='./SimFiles/layer.out';
ANTENNAFILENAME = './SimFiles/antenna.out';
IMAGEFILENAME = './SimFiles/Plots/Dipole3DView.png'

numOutputs=size(objectMap3D_outputIndices,2);

fid=fopen(WIREFILENAME);
[wireOutput,count]=fread(fid,inf,'double');
fclose(fid);
wireOutput=reshape(wireOutput,6,numOutputs,[]);

fid=fopen(ANTENNAFILENAME);
[antennaOutput,count]=fread(fid,inf,'double');
fclose(fid);
antennaOutput=reshape(antennaOutput,6,numOutputs,[]);

% Frequency domain conversion for wire
Exwire = squeeze(wireOutput(1,1,:));
Eywire = squeeze(wireOutput(2,1,:));
Ezwire = squeeze(wireOutput(3,1,:));
Hxwire = squeeze(wireOutput(4,1,:));
Hywire = squeeze(wireOutput(5,1,:));
Hzwire = squeeze(wireOutput(6,1,:));
ExwireFreq = abs(fft(Exwire)).';
EywireFreq = abs(fft(Eywire)).';
EzwireFreq = abs(fft(Ezwire)).';
HxwireFreq = abs(fft(Hxwire)).';
HywireFreq = abs(fft(Hywire)).';
HzwireFreq = abs(fft(Hzwire)).';

% Frequency domain conversion for antenna
Exantenna = squeeze(antennaOutput(1,1,:));
Eyantenna = squeeze(antennaOutput(2,1,:));
Ezantenna = squeeze(antennaOutput(3,1,:));
Hxantenna = squeeze(antennaOutput(4,1,:));
Hyantenna = squeeze(antennaOutput(5,1,:));
Hzantenna = squeeze(antennaOutput(6,1,:));
ExantennaFreq = abs(fft(Exantenna)).';
EyantennaFreq = abs(fft(Eyantenna)).';
EzantennaFreq = abs(fft(Ezantenna)).';
HxantennaFreq = abs(fft(Hxantenna)).';
HyantennaFreq = abs(fft(Hyantenna)).';
HzantennaFreq = abs(fft(Hzantenna)).';

% Power ratio calculation
ratioEField = (ExantennaFreq.^2+EyantennaFreq.^2+EzantennaFreq.^2)./(ExwireFreq.^2 + EywireFreq.^2 + EzwireFreq.^2);
ratioHField = (HxantennaFreq.^2+HyantennaFreq.^2+HzantennaFreq.^2)./(HxwireFreq.^2 + HywireFreq.^2 + HzwireFreq.^2);

time = (0:length(Exantenna)-1)*delT;
lambda = in.lambda*in.unitLength;
f0 = PC.c / lambda;


fs=1 / delT; %sampling Frequency
f=linspace(0,fs,length(time)); % the max frequency is fs/2 in fourier domain, and thus 2*pi corresponds to fs
en=f*PC.h/PC.q;
lambdaList = PC.c ./ f * 1e9; % in terms of nanometers
enStart = PC.c*PC.h/(PC.q*4e-6);
enEnd   = PC.c*PC.h/(PC.q*0.5e-6);


figure
plot(lambdaList,ratioEField,'Linewidth',2)
xlim([500 3000])
xlabel('Wavelength in nm');
ylabel('Power Enhancement');
title(sprintf('E Power Ratio'));
hLegend = axes('Layer','top','Position',[0.7 0.8 0.30 0.20],'XTickLabel','','YTickLabel','','ZTickLabel','','Color',[0 0 0]);
img = imread(IMAGEFILENAME);
image(img);
axis image
set(hLegend, 'XTick',[],'YTick',[])


figure
plot(lambdaList,ratioHField,'Linewidth',2)
xlim([500 3000])
xlabel('Wavelength in nm');
ylabel('Power Enhancement');
title(sprintf('H Power Ratio'));
