% This file reads in simulation results for a volume of data, calculates
% the total electric field energy in that volume and plots it with respect
% to wavelength.

% Because the file is a modified verion of the readLineDipole.m file, there
% are some variables that are not actually necessary but are there for
% convenience.

clear all
% Load the variables for visualization purposes
load './simFiles/postSim.mat'
physicalConstants

WIREFILENAME='./SimFiles/layer.out';
ANTENNAFILENAME = './SimFiles/antenna.out';

numWireOutputs   =size(objectMap2DWire_outputIndices,2);
numAntennaOutputs=size(objectMap3D_outputIndices,2);

% First read in the normalization data point
fid=fopen(WIREFILENAME);
[wireOutput,count]=fread(fid,inf,'double');
fclose(fid);
wireOutput=reshape(wireOutput,6,numWireOutputs,[]);

% Calculate the fourier transform of the point
wireFFT = zeros(size(wireOutput));
%  fft(X,[],dim) applies the FFT operation across the dimension dim
wireFFT = fft(wireOutput,[],3);

% Calculate the indices of the starting and stopping points for the
% frequency distribution. The variables below determine the limits.

lambdaStart = 500; % in nm.
lambdaEnd = 2000;  % in nm.

time = (0:size(wireFFT,3)-1)*delT;
fs=1 / delT; %sampling Frequency
f=linspace(0,fs,length(time)); % the max frequency is fs/2 in fourier domain, and thus 2*pi corresponds to fs
en=f*PC.h/PC.q; % Energy in terms of eV
lambdaList = PC.c ./ f * 1e9; % in terms of nanometers
[dellambda,lambdaStartIndex] = min(abs(lambdaList-lambdaStart));
[dellambda,lambdaEndIndex] = min(abs(lambdaList-lambdaEnd));
clear dellambda
% Sort the indices to prevent a starting index larger than the ending index
startIndex = min(lambdaStartIndex,lambdaEndIndex);
endIndex = max(lambdaStartIndex,lambdaEndIndex);
clear lambdaStartIndex lambdaEndIndex
% Modify the plotting variables
lambdaList = lambdaList(startIndex:endIndex);
f = f(startIndex:endIndex);
en = en(startIndex:endIndex);

% Get rid of all the unnecessary frequency components and calculate the
% normalization parameters
wireFFT = squeeze(abs(wireFFT(:,1,startIndex:endIndex)).^2);
% The first three rows are Ex,Ey,Ez
normE = sum(wireFFT(1:3,:));
% The last three are Hx,Hy,Hz
normH = sum(wireFFT(4:6,:));
normE=1; %delete later on
EfieldEnhancement = zeros(1,length(lambdaList));
for ii=1:numAntennaOutputs
    x=getEnhancementE(ANTENNAFILENAME,numAntennaOutputs,ii,startIndex,endIndex,normE,size(source,2));
    EfieldEnhancement=EfieldEnhancement + x./numAntennaOutputs;
    disp(sprintf('Point %d of %d at simulation point [%d %d %d]',ii,numAntennaOutputs,objectMap3D_outputIndices(1,ii),objectMap3D_outputIndices(2,ii),objectMap3D_outputIndices(3,ii)));
end
save './simFiles/EfieldVolumeEnhancement' EfieldEnhancement

figure
h=plot(lambdaList,EfieldEnhancement);
xlabel('Wavelength in nm');
ylabel('Enhancement');
title(sprintf('Volumetric |E|^2 Enhancement Ratio'));
