function visualizeWorld(material, objectColor,varargin)

% See if there are any output indices
if length(varargin)==0
    indices = [];
elseif length(varargin)==1
    indices = varargin{1};
else
    error('Too many inputs');
end

% If more than three materials are used, define which color should be
% used by the material. The first color is the color of the first defined
% material, the second that of the second material and so on.
map(1,:) = [1 0 0]; % Red
map(2,:) = [0 1 0]; % Green
map(3,:) = [0 0 1]; % Blue
map(4,:) = [1 0.62 0.40]; % Copper

% The last color is the PML
map(5,:) = [0.3 0.3 0.3]; % Gray for PML

% Get the size of the screen and position the figures accordingly
scrsz = get(0,'ScreenSize');
figmain=figure('Position',[scrsz(3)/10 scrsz(4)/10 scrsz(3)*0.85 scrsz(4)*0.85],'name','OOFDTD - Matlab Visualization','NumberTitle','off','Colormap',map);
set(figmain,'KeyPressFcn',@keyPressed);
figure(figmain)

% Create the name and color info on a black background corner
hLegend = axes('Layer','top','Position',[0 0.8 0.15 0.15],'XTickLabel','','YTickLabel','','ZTickLabel','','Color',[0 0 0]);
for jj=1:length(material)
    text(0.0,1-(jj-1)*1/length(material),material{jj}.name,'BackgroundColor',map(jj,:),'Color',[1 1 1]);
end
xlim([-0.1,0.6]);
ylim([0,1.2]);
view(2);

% v is just the shor named version of object color
v=objectColor;

% xslice, yslice and zslice are the slice numbers to be visulized.
xslice=1;yslice=1;zslice=1;

% drawLines turns drawing of lines on and off
drawLines=1;

% Create the axis
h3D = axes('Layer','bottom','Position',[0.025 0.5025 0.45 0.45]);colormap(map);
hxslice = axes('Layer','bottom','Position',[0.575 0.575 0.35 0.35]);colormap(map);
hyslice = axes('Layer','bottom','Position',[0.575 0.075 0.35 0.35]);colormap(map);
hzslice = axes('Layer','bottom','Position',[0.075 0.075 0.35 0.35]);colormap(map);

% Set the application data
setappdata(figmain,'v',objectColor);
setappdata(figmain,'h3D',h3D);
setappdata(figmain,'hxslice',hxslice);
setappdata(figmain,'hyslice',hyslice);
setappdata(figmain,'hzslice',hzslice);
setappdata(figmain,'map',map);
setappdata(figmain,'xslice',xslice);
setappdata(figmain,'yslice',yslice);
setappdata(figmain,'zslice',zslice);
setappdata(figmain,'drawLines',drawLines);

if length(indices)~=0
    setappdata(figmain,'indices',indices);
end

% Call the necessary functions
update3DView(figmain);
updateSliceView(figmain,'x',1);
updateSliceView(figmain,'y',1);
updateSliceView(figmain,'z',1);

% Adjust the initial view point of h3D
axes(h3D);
view(3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function update3DView(figmain)
% Plot the 3D slice
% Note that I tried the slice function first but it did not work properly. So,
% I'm doing the slices manually now.
v=getappdata(figmain,'v');
xslice=getappdata(figmain,'xslice');
yslice=getappdata(figmain,'yslice');
zslice=getappdata(figmain,'zslice');
h3D = getappdata(figmain,'h3D');

s=size(v);
x=1:s(1);
y=1:s(2);
z=1:s(3);

axes(h3D);
t=view;
cla;
m=[];


% First draw the fixed slices
% x=xmax slice
%[Y,Z]=ndgrid(y,z);
%m=[m,surface(s(1)*ones(size(Y)),Y,Z,squeeze(v(s(1),:,:)))];
% y=ymax slice
%[X,Z]=ndgrid(x,z);
%m=[m,surface(X,s(2)*ones(size(X)),Z,squeeze(v(:,s(2),:)))];
% z=1 slice
%[X,Y]=ndgrid(x,y);
%m=[m,surface(X,Y,ones(size(X)),squeeze(v(:,:,1)))];    

% Now draw the moving slices
% moving x-slice
[Y,Z]=ndgrid(y,z);
m=surface(xslice*ones(size(Y)),Y,Z,squeeze(v(xslice,:,:)));

% moving y-slice
[X,Z]=ndgrid(x,z);
m=[m,surface(X,yslice*ones(size(X)),Z,squeeze(v(:,yslice,:)))];

% moving z-slice
[X,Y]=ndgrid(x,y);
m=[m,surface(X,Y,zslice*ones(size(X)),squeeze(v(:,:,zslice)))];    
set(m,'EdgeColor','none','CdataMapping', 'direct','FaceAlpha',0.8,'EdgeAlpha',1);
axis equal
title(sprintf('X=%d Y=%d Z=%d slices',xslice,yslice,zslice));
xlabel('X');ylabel('Y');zlabel('Z');
view(t);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function updateSliceView(figmain,sliceType, slicePlane)

v=getappdata(figmain,'v');
hxslice = getappdata(figmain,'hxslice');
hyslice = getappdata(figmain,'hyslice');
hzslice = getappdata(figmain,'hzslice');
drawLines = getappdata(figmain,'drawLines');
map = getappdata(figmain,'map');
indices = getappdata(figmain,'indices');

% Plot the 2D slice. Note that function "image" gives a 1-to-1 correspondance with
% the underlying matrix whereas slice uses the matrix to color vertex
% points which is problematic. 
figure(figmain)
if sliceType=='x'
    axes(hxslice);
    cla
    slicePlane=min(size(v,1),slicePlane);
    slicePlane=max(1,slicePlane);
    setappdata(figmain,'xslice',slicePlane);
    data=squeeze(v(slicePlane,:,:))';
    title(sprintf('X=%d slice',slicePlane));
    xlabel('Y');ylabel('Z');
    if ~isempty(indices)
        % Find if there are any output points in this plane
        indices = indices(:,find(indices(1,:)==slicePlane));
        % Get the y and z coordinates only
        if ~isempty(indices)
            indices = indices([2 3],:);
        end
    end        
elseif sliceType=='y'
    axes(hyslice);
    cla
    slicePlane=min(size(v,2),slicePlane);
    slicePlane=max(1,slicePlane);
    setappdata(figmain,'yslice',slicePlane);
    data=squeeze(v(:,slicePlane,:))';
    title(sprintf('Y=%d slice',slicePlane));
    xlabel('X');ylabel('Z');
    if ~isempty(indices)
        % Find if there are any output points in this plane
        indices = indices(:,find(indices(2,:)==slicePlane));
        % Get the x and z coordinates only
        if ~isempty(indices)
            indices = indices([1 3],:);
        end
    end
elseif sliceType=='z'
    axes(hzslice);
    cla
    slicePlane=min(size(v,3),slicePlane);
    slicePlane=max(1,slicePlane);
    setappdata(figmain,'zslice',slicePlane);
    data=squeeze(v(:,:,slicePlane))';
    title(sprintf('Z=%d slice',slicePlane));
    xlabel('X');ylabel('Y');
    if ~isempty(indices)
        % Find if there are any output points in this plane
        indices = indices(:,find(indices(3,:)==slicePlane));
        % Get the x and y coordinates only
        if ~isempty(indices) 
            indices = indices([1 2],:);
        end
    end
end

sd=size(data);
image('CData',data,'CDataMapping','direct','AlphaData',0.4)
view(2);
axis xy
axis equal
axis tight
if ~isempty(indices)
    hold on
    plot(indices(1,:),indices(2,:),'x','markersize',10,'markerfacecolor','yellow','markeredgecolor','black');
    hold off
end

% Since the center of each cell has 0.5 shift, adjust ticks accordingly
% To understand, play with "image" by plotting a simple matrix. You'll
% see that the center of each cell with non-integer locations gives the
% center.
delTick=5;
xticks = 0:delTick:sd(2);
yticks = 0:delTick:sd(1);
set(gca,'XGrid','off','YGrid','off','XTick',xticks,'YTick',yticks,'XTickLabel',0:delTick:sd(2),'YTickLabel',0:delTick:sd(1))
% Draw lines to show grids for each cell
% Since the center of each cell has integer indices, adjust ticks accordingly
% To understand, play with "image" by plotting a simple matrix. You'll
% see that the center of each cell with non-integer locations gives the
% center.
if drawLines ==1
    delTick=1;
    xticks = 0.5:delTick:sd(2)+0.5;
    yticks = 0.5:delTick:sd(1)+0.5;
    line([xticks;xticks],[min(yticks)*ones(size(xticks));max(yticks)*ones(size(xticks))],'LineStyle','-','Color',[0 0 0]);
    line([min(xticks)*ones(size(yticks));max(xticks)*ones(size(yticks))],[yticks;yticks],'LineStyle','-','Color',[0 0 0]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function keeps track of which keys are pressed and then updates the
% view.
function keyPressed(obj,eventdata)
c=get(obj,'CurrentCharacter');
drawLines = getappdata(obj,'drawLines');
sprintf('%d key pressed',c)
sprintf('Current type=%s, slice=%s.',getappdata(obj,'type'),getappdata(obj,'slice'))

if c=='x'
    setappdata(obj,'type','x');
    return;
elseif c=='y'
    setappdata(obj,'type','y');
    return;
elseif c=='z'
    setappdata(obj,'type','z');
    return;
elseif c=='l'
    if drawLines == 0
        setappdata(obj,'drawLines',1);
    else
        setappdata(obj,'drawLines',0);
    end
    updateSliceView(obj,'x', getappdata(obj,'xslice'));
    updateSliceView(obj,'y', getappdata(obj,'yslice'));
    updateSliceView(obj,'z', getappdata(obj,'zslice'));
    return;
end

if c==13 %enter key
    type = getappdata(obj,'type');
    if type=='x'
        slice=str2num(getappdata(obj,'slice'));
        sprintf('Updating x=%d slice',slice)
        updateSliceView(obj,'x', slice);
        update3DView(obj);
    elseif type=='y'
        slice=str2num(getappdata(obj,'slice'));
        sprintf('Updating y=%d slice',slice)
        updateSliceView(obj,'y', slice);
        update3DView(obj);
    elseif type=='z'
        slice=str2num(getappdata(obj,'slice'));
        sprintf('Updating z=%d slice',slice)
        updateSliceView(obj,'z', slice);
        update3DView(obj);
    end
    setappdata(obj,'type','');
    setappdata(obj,'slice','');
    return;
end
if (c<=57 && c>=48) %0-9
    dum = getappdata(obj,'slice');
    setappdata(obj,'slice',[dum c]);
    return;
end
setappdata(obj,'type','');
setappdata(obj,'slice','');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

