#include "PMLCell.h"

// Constructor for PMLCell Class
PMLCell::PMLCell(double Sx, double Sy, double Sz, double Kx, double Ky, double Kz, double epsr){
	Dx=Dy=Dz=Bx=By=Bz=0.0;
	Dxold=Dyold=Dzold=Bxold=Byold=Bzold=0.0;
	Spx = Sx*delT;
	Spy = Sy*delT;
	Spz = Sz*delT;
	Kpx = 2*EPS0*Kx;
	Kpy = 2*EPS0*Ky;
	Kpz = 2*EPS0*Kz;
	epsDt = 2*EPS0*delT;
	(*this).eps = EPS0*epsr;
	
};


void PMLCell::updateEFields(){
	Dxold = Dx;
	Dyold = Dy;
	Dzold = Dz;

	Dx = (Kpy - Spy)/(Kpy + Spy)*Dx + epsDt / (Kpy + Spy) * ( (Hz-cellBackY->Hz)/delY - (Hy-cellBackZ->Hy)/delZ );
	Dy = (Kpz - Spz)/(Kpz + Spz)*Dy + epsDt / (Kpz + Spz) * ( (Hx-cellBackZ->Hx)/delZ - (Hz-cellBackX->Hz)/delX );
	Dz = (Kpx - Spx)/(Kpx + Spx)*Dz + epsDt / (Kpx + Spx) * ( (Hy-cellBackX->Hy)/delX - (Hx-cellBackY->Hx)/delY );

	Ex = (Kpz - Spz)/(Kpz + Spz)*Ex + 1/(eps*(Kpz+Spz))*( (Kpx+Spx)*Dx - (Kpx-Spx)*Dxold );
	Ey = (Kpx - Spx)/(Kpx + Spx)*Ey + 1/(eps*(Kpx+Spx))*( (Kpy+Spy)*Dy - (Kpy-Spy)*Dyold );
	Ez = (Kpy - Spy)/(Kpy + Spy)*Ez + 1/(eps*(Kpy+Spy))*( (Kpz+Spz)*Dz - (Kpz-Spz)*Dzold );
};

void PMLCell::updateHFields(){
	Bxold = Bx;
	Byold = By;
	Bzold = Bz;

	Bx = (Kpy - Spy)/(Kpy + Spy)*Bx + epsDt / (Kpy + Spy) * ( (cellFrontZ->Ey-Ey)/delZ - (cellFrontY->Ez-Ez)/delY );
	By = (Kpz - Spz)/(Kpz + Spz)*By + epsDt / (Kpz + Spz) * ( (cellFrontX->Ez-Ez)/delX - (cellFrontZ->Ex-Ex)/delZ );
	Bz = (Kpx - Spx)/(Kpx + Spx)*Bz + epsDt / (Kpx + Spx) * ( (cellFrontY->Ex-Ex)/delY - (cellFrontX->Ey-Ey)/delX );

	Hx = (Kpz - Spz)/(Kpz + Spz)*Hx + 1/(MU0*(Kpz+Spz))*( (Kpx+Spx)*Bx - (Kpx-Spx)*Bxold );
	Hy = (Kpx - Spx)/(Kpx + Spx)*Hy + 1/(MU0*(Kpx+Spx))*( (Kpy+Spy)*By - (Kpy-Spy)*Byold );
	Hz = (Kpy - Spy)/(Kpy + Spy)*Hz + 1/(MU0*(Kpy+Spy))*( (Kpz+Spz)*Bz - (Kpz-Spz)*Bzold );
};
