#ifndef PMLCellGUARD
#define PMLCellGUARD

#include "Cell.h"

// This object implements the Uniaxial Perfectly Matched Layer (UPML) as described in pp 308-313 at
/*
				  ISBN: 1580530761
                Author: Taflove, Allen.
                 Title: Computational electrodynamics : the finite-difference
                        time-domain method.
               Edition: 2nd ed. / Allen Taflove, Susan C. Hagness.
               Imprint: Boston : Artech House, c2000.

*/

// The object assumes that the surrounding space has its permittivity equal to MU0.

class PMLCell: public Cell{
public:
	
	// Constructor method
	// S stands for sigma, K stands for kappa. x,y,z denote the directions
	
	PMLCell(double Sx, double Sy, double Sz, double Kx, double Ky, double Kz, double epsr);

	// Field update equations for the class
	void updateEFields();
	void updateHFields();

private:
	double Dx, Dy, Dz, Bx, By, Bz ;			// Extra field elements used in update equations
	double Dxold, Dyold, Dzold, Bxold, Byold, Bzold ; // Extra field elements used in update equations
	double Spx, Spy, Spz, Kpx, Kpy, Kpz;	// "Primed" sigma and kappa values for update equations
	double epsDt;							// 2*EPS0*delT
	double eps;								// Permittivity (not relative!)

};

#endif