#include "TEMBrick.h"

// Constructor for TEMBrick Class
TEMBrick::TEMBrick(const char* type){

	if( strcmp(type,"TE") == 0 ){
		isTE = true;
	}else if( strcmp(type,"TM") == 0 ){
		isTE = false;
	}else{
		printf("TEMBrick does not understand type %s. Exiting...\n",type);
		exit(EXIT_FAILURE);
	}
	// Even though fields were initialized to zero, there's no harm in initializing them again.
	Ex=Ey=Ez=Hx=Hy=Hz=0.0;
};

// updateFields() to be called at the end of each time cycle
// depending on whether the TE or TM termination is made, different values of neighbouring cell is set to zero.

void TEMBrick::updateEFields(){
	updateBothFields();
}

void TEMBrick::updateHFields(){
	updateBothFields();
}

void TEMBrick::updateBothFields(){
	if (isTE) {
		if (cellFrontY != 0) {
			cellFrontY->Ez = 0.0;
			cellFrontY->Ex = 0.0;
			cellFrontY->Hy = 0.0;
		}
	}else{
		if (cellBackY != 0) {
			cellBackY->Hx = 0.0;
			cellBackY->Hz = 0.0;
			cellBackY->Ey = 0.0;
		}	

	}


};

// TEMBrick should only have one neighbour set and it should be in the plus or minus y direction
void TEMBrick::checkNeighbours(int i){
	int numNeighbours = 0;

	if (cellBackY != 0)
		numNeighbours++;
	if (cellFrontY != 0)
		numNeighbours++;
	if (numNeighbours != 1){
		if(isTE)
			printf("Neighbour Problem at TEBrick (%d). Exiting...\n",i);
		else
			printf("Neighbour Problem at TMBrick (%d). Exiting...\n",i);
		exit(EXIT_FAILURE);
	}

};