#include "World2D.h"

// Constructor

World2D::World2D(const char *file, const char *varName,const char *outputFile):World3D(file, varName,outputFile){	
	if (NY!=1){
		printf("Variable %s in file %s has NY=%d. It is not 2D. Exiting...\n",varName,file,NY);
		exit(EXIT_FAILURE);
	}	
	MATFile *pmat = matOpen(file, "r");
	double dum=0;
	World3D::getScalar(pmat,"TE",&dum);
	if (dum > 0)
		isTE = true;
	else if (dum <  0)
		isTE = false;
	else{
		printf("Something wrong with TE variable. Exiting...\n");
		exit(EXIT_FAILURE);
	}
	
}


void World2D::addBoundaryBricks(){

	printf("Adding boundary bricks for World2D.\n");

	const int imin=0,imax=NX-1,jmin=0,jmax=NY-1,kmin=0,kmax=NZ-1;
	numBricks=2*(NX*NY+NX*NZ+NY*NZ);
	brickMap = new Cell*[numBricks];
	int ib = 0;


	// Top face z=max(z)
	for(int i=0;i<NX;i++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(i,j,kmax)->cellFrontZ=new PECBrick();
			brickMap[ib]=ObjectMap3D(i,j,kmax)->cellFrontZ;
			brickMap[ib]->cellBackZ=ObjectMap3D(i,j,kmax);
			ib++;
		}
	}

	// Bottom face z=min(z)
	for(int i=0;i<NX;i++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(i,j,kmin)->cellBackZ=new PECBrick();
			brickMap[ib]=ObjectMap3D(i,j,kmin)->cellBackZ;
			brickMap[ib]->cellFrontZ=ObjectMap3D(i,j,kmin);
			ib++;
		}
	}

	// Left face x=min(x)
	for(int k=0;k<NZ;k++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(imin,j,k)->cellBackX=new PECBrick();
			brickMap[ib]=ObjectMap3D(imin,j,k)->cellBackX;
			brickMap[ib]->cellFrontX=ObjectMap3D(imin,j,k);
			ib++;
		}
	}

	// Right face x=max(x)
	for(int k=0;k<NZ;k++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(imax,j,k)->cellFrontX=new PECBrick();
			brickMap[ib]=ObjectMap3D(imax,j,k)->cellFrontX;
			brickMap[ib]->cellBackX=ObjectMap3D(imax,j,k);
			ib++;
		}
	}

	// For 2D simulations, front and back y-face bricks should be TEMBricks
	// Front face y=min(y)


	for(int k=0;k<NZ;k++){
		for(int i=0;i<NX;i++){
			if(isTE)
				ObjectMap3D(i,jmin,k)->cellBackY=new TEMBrick("TE");
			else
				ObjectMap3D(i,jmin,k)->cellBackY=new TEMBrick("TM");
			brickMap[ib]=ObjectMap3D(i,jmin,k)->cellBackY;
			brickMap[ib]->cellFrontY=ObjectMap3D(i,jmin,k);
			ib++;
		}
	}

	// Back face y=max(y)
	for(int k=0;k<NZ;k++){
		for(int i=0;i<NX;i++){
			if(isTE)
				ObjectMap3D(i,jmax,k)->cellFrontY=new TEMBrick("TE");
			else
				ObjectMap3D(i,jmax,k)->cellFrontY=new TEMBrick("TM");
			brickMap[ib]=ObjectMap3D(i,jmax,k)->cellFrontY;
			brickMap[ib]->cellBackY=ObjectMap3D(i,jmax,k);
			ib++;
		}
	}

}


// Very similar to addBoundaryBricks() but makes sure x=min(x) side sees
// x=max(x) side as its neighbour.
void World2D::addPeriodicBoundaryBricks(){

	printf("Adding periodic boundary bricks for World2D.\n");

	const int imin=0,imax=NX-1,jmin=0,jmax=NY-1,kmin=0,kmax=NZ-1;
	numBricks=2*(NX*NY+NX*NZ);
	brickMap = new Cell*[numBricks];
	int ib = 0;


	// Top face z=max(z)
	for(int i=0;i<NX;i++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(i,j,kmax)->cellFrontZ=new PECBrick();
			brickMap[ib]=ObjectMap3D(i,j,kmax)->cellFrontZ;
			brickMap[ib]->cellBackZ=ObjectMap3D(i,j,kmax);
			ib++;
		}
	}

	// Bottom face z=min(z)
	for(int i=0;i<NX;i++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(i,j,kmin)->cellBackZ=new PECBrick();
			brickMap[ib]=ObjectMap3D(i,j,kmin)->cellBackZ;
			brickMap[ib]->cellFrontZ=ObjectMap3D(i,j,kmin);
			ib++;
		}
	}

	// Make the simulation periodic in x-direction.
	// Could have done in one for loop, but for the sake of clarity both loops are
	// preserved.

	// Left face x=min(x)
	for(int k=0;k<NZ;k++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(imin,j,k)->cellBackX=ObjectMap3D(imax,j,k);
		}
	}

	// Right face x=max(x)
	for(int k=0;k<NZ;k++){
		for(int j=0;j<NY;j++){
			ObjectMap3D(imax,j,k)->cellFrontX=ObjectMap3D(imin,j,k);			
		}
	}

	// For 2D simulations, front and back y-face bricks should be TEMBricks
	// Front face y=min(y)


	for(int k=0;k<NZ;k++){
		for(int i=0;i<NX;i++){
			if(isTE)
				ObjectMap3D(i,jmin,k)->cellBackY=new TEMBrick("TE");
			else
				ObjectMap3D(i,jmin,k)->cellBackY=new TEMBrick("TM");
			brickMap[ib]=ObjectMap3D(i,jmin,k)->cellBackY;
			brickMap[ib]->cellFrontY=ObjectMap3D(i,jmin,k);
			ib++;
		}
	}

	// Back face y=max(y)
	for(int k=0;k<NZ;k++){
		for(int i=0;i<NX;i++){
			if(isTE)
				ObjectMap3D(i,jmax,k)->cellFrontY=new TEMBrick("TE");
			else
				ObjectMap3D(i,jmax,k)->cellFrontY=new TEMBrick("TM");
			brickMap[ib]=ObjectMap3D(i,jmax,k)->cellFrontY;
			brickMap[ib]->cellBackY=ObjectMap3D(i,jmax,k);
			ib++;
		}
	}

}


void World2D::setSourceWorld(World2D *source,int xmin,int xmax,int zmin,int zmax,int x0, int z0){
	sourceTF = source;
	xminTFSF = xmin;
	xmaxTFSF = xmax;
	zminTFSF = zmin;
	zmaxTFSF = zmax;
	x0TFSF = x0;
	z0TFSF = z0;
}

void World2D::TFSFsourceE(){
	if(sourceTF == 0){
		printf("sourceTF called with no pointer to source. Exiting...");
		exit(EXIT_FAILURE);
	}

	// A very bad programming choice has led to a bug that needs the following fix
	// Note that the macro ObjectMap3D defined in World3D.h has
	// ObjectMap3D(I,J,K) objectMap3D[(K)*NX*NY+(J)*NX + (I)]
	// note that when one does sourceTF->ObjectMap3D(i,0,zminTFSF)
	// the NX in the macro is *this.NX, not *sourceTF.NX . That's a major bug.
	// But here's the fix. In the future, never call ObjectMap outside of the object

	int sNX = sourceTF->NX;
	int sNY = sourceTF->NY;
	
#define ObjectMap3DSource(I,J,K) objectMap3D[(K)*sNX*sNY+(J)*sNX + (I)]

	if(isTE){
		// z=min(z) boundary
		for(int i=xminTFSF+1;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zminTFSF+z0TFSF)->addJy(+(sourceTF->ObjectMap3DSource(i,0,zminTFSF)->Hx)/delZ);

		// z=max(z) boundary
		for(int i=xminTFSF+1;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zmaxTFSF+z0TFSF)->addJy(-(sourceTF->ObjectMap3DSource(i,0,zmaxTFSF-1)->Hx)/delZ);

		// x=min(x) boundary
		for(int k=zminTFSF+1;k<zmaxTFSF;k++)
			ObjectMap3D(xminTFSF+x0TFSF,0,k+z0TFSF)->addJy(-(sourceTF->ObjectMap3DSource(xminTFSF,0,k)->Hz)/delX);

		// x=max(x) boundary
		for(int k=zminTFSF+1;k<zmaxTFSF;k++)
			ObjectMap3D(xmaxTFSF+x0TFSF,0,k+z0TFSF)->addJy(+(sourceTF->ObjectMap3DSource(xmaxTFSF-1,0,k)->Hz)/delX);



	}else{
		//TM implementation

		// z=min(z) boundary
		for(int i=xminTFSF;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zminTFSF+z0TFSF)->addJx(-(sourceTF->ObjectMap3DSource(i,0,zminTFSF)->Hy)/delZ);

		// z=max(z) boundary
		for(int i=xminTFSF;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zmaxTFSF+z0TFSF)->addJx(+(sourceTF->ObjectMap3DSource(i,0,zmaxTFSF-1)->Hy)/delZ);

		// x=min(x) boundary
		for(int k=zminTFSF;k<zmaxTFSF;k++)
			ObjectMap3D(xminTFSF+x0TFSF,0,k+z0TFSF)->addJz(+(sourceTF->ObjectMap3DSource(xminTFSF,0,k)->Hy)/delX);

		// x=max(x) boundary
		for(int k=zminTFSF;k<zmaxTFSF;k++)
			ObjectMap3D(xmaxTFSF+x0TFSF,0,k+z0TFSF)->addJz(-(sourceTF->ObjectMap3DSource(xmaxTFSF-1,0,k)->Hy)/delX);


	}

#undef ObjectMap3DSource
}

void World2D::TFSFsourceH(){
	if(sourceTF == 0){
		printf("sourceTF called with no pointer to source. Exiting...");
		exit(EXIT_FAILURE);
	}

	// A very bad programming choice has led to a bug that needs the following fix
	// Note that the macro ObjectMap3D defined in World3D.h has
	// ObjectMap3D(I,J,K) objectMap3D[(K)*NX*NY+(J)*NX + (I)]
	// note that when one does sourceTF->ObjectMap3D(i,0,zminTFSF)
	// the NX in the macro is *this.NX, not *sourceTF.NX . That's a major bug.
	// But here's the fix. In the future, never call ObjectMap outside of the object

	int sNX = sourceTF->NX;
	int sNY = sourceTF->NY;

#define ObjectMap3DSource(I,J,K) objectMap3D[(K)*sNX*sNY+(J)*sNX + (I)]

	if(isTE){
		// z=min(z) boundary
		for(int i=xminTFSF+1;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zminTFSF+z0TFSF)->addMx(+(sourceTF->ObjectMap3DSource(i,0,zminTFSF)->Ey)/delZ);

		// z=max(z) boundary
		for(int i=xminTFSF+1;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zmaxTFSF-1+z0TFSF)->addMx(-(sourceTF->ObjectMap3DSource(i,0,zmaxTFSF)->Ey)/delZ);

		// x=min(x) boundary
		for(int k=zminTFSF+1;k<zmaxTFSF;k++)
			ObjectMap3D(xminTFSF+x0TFSF,0,k+z0TFSF)->addMz(-(sourceTF->ObjectMap3DSource(xminTFSF,0,k)->Ey)/delX);

		// x=max(x) boundary
		for(int k=zminTFSF+1;k<zmaxTFSF;k++)
			ObjectMap3D(xmaxTFSF-1+x0TFSF,0,k+z0TFSF)->addMz(+(sourceTF->ObjectMap3DSource(xmaxTFSF,0,k)->Ey)/delX);



	}else{
		//TM implementation
		// z=min(z) boundary
		for(int i=xminTFSF;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zminTFSF+z0TFSF)->addMy(-(sourceTF->ObjectMap3DSource(i,0,zminTFSF)->Ex)/delZ);

		// z=max(z) boundary
		for(int i=xminTFSF;i<xmaxTFSF;i++)
			ObjectMap3D(i+x0TFSF,0,zmaxTFSF-1+z0TFSF)->addMy(+(sourceTF->ObjectMap3DSource(i,0,zmaxTFSF)->Ex)/delZ);

		// x=min(x) boundary
		for(int k=zminTFSF;k<zmaxTFSF;k++)
			ObjectMap3D(xminTFSF+x0TFSF,0,k+z0TFSF)->addMy(+(sourceTF->ObjectMap3DSource(xminTFSF,0,k)->Ez)/delX);

		// x=max(x) boundary
		for(int k=zminTFSF;k<zmaxTFSF;k++)
			ObjectMap3D(xmaxTFSF-1+x0TFSF,0,k+z0TFSF)->addMy(-(sourceTF->ObjectMap3DSource(xmaxTFSF,0,k)->Ez)/delX);
	}

// End of bug-fixing macro definition... What an ugly solution... But what the heck...
#undef ObjectMap3DSource

}