(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   1246586,      23381]*)
(*NotebookOutlinePosition[   1247570,      23412]*)
(*  CellTagsIndexPosition[   1247526,      23408]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Drude-Lorentz Fit To Experimental Permittivity Data", \
"Title"]], "Subtitle"],

Cell["\<\
S. Ekin Kocabas
kocabas@stanford.edu\
\>", "Author",
  TextAlignment->Right],

Cell[TextData[{
  "This code attempts to find a good Drude-Lorentz fit to the experimental \
(n,k) pairs of a given metal. The code uses the ",
  Cell[BoxData[
      \(TraditionalForm\`\[ExponentialE]\^\(+\[ImaginaryJ]\[Omega]t\)\)]],
  " convention and the unknowns to be solved for are defined as have been \
done in [1]. The relative permittivity is the Drude-Lorentz function:"
}], "Text"],

Cell[BoxData[
    \(TraditionalForm\`\(\[Epsilon]\_r\)(\[Omega])\  = \ \(1 + \[Sum]\+\(m = \
1\)\%P\((\[Omega]\_pm)\)\^2\/\(\[Omega]\_m\^2 - \[Omega]\^2 + \[ImaginaryI]\ \
\[Omega]\ \[CapitalGamma]\_m\)\ with\ \ \[Omega]\_1 = 0\)\)], "DisplayFormula"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega]\_pm\)]],
  " is the plasmon frequency of the ",
  Cell[BoxData[
      \(TraditionalForm\`m\^th\)]],
  " pole, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega]\_m\)]],
  " is the oscillator frequency and ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalGamma]\_m\)]],
  " is the damping constant. Note that, in the above expansion setting any \
one of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega]\_m\)]],
  "'s equal to zero makes it a Drude pole. That's why, the first pole has its \
oscillator frequency, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega]\_1\)]],
  ", set to zero to make sure we have a mixture of Drude and Lorentz type of \
poles."
}], "Text",
  FormatType->TextForm],

Cell[TextData[{
  Cell[TextData[{
    "[",
    
    CounterBox["Reference"],
    "]\[ThickSpace]\[MediumSpace]"
  }],
    CellSize->{24, Inherited},
    TextAlignment->Right],
  "Optical properties of metallic films for vertical-cavity optoelectronic \
devices. Rakic, AD ; Djurisic, AB ; Elazar, JM ; Majewski, ML. Applied \
Optics; 1 Aug. 1998; vol.37, no.22, p.5271-83."
}], "Reference"],

Cell["\<\
Read the experimental data values and create the lists that holds the real \
and imaginary parts of the permittivity.\
\>", "Text",
  CellFrame->{{0, 0}, {0, 2}}],

Cell[BoxData[
    \(\(\(\ \)\(x = x; Remove["\<Global`*\>"];\[IndentingNewLine]
    \(Off[General::spell];\)\[IndentingNewLine]
    \(Off[General::spell1];\)\[IndentingNewLine]
    \(SetDirectory["\<c:\\Documents and Settings\\Ekin\\My \
Documents\\research\\NumericalModels\\FDTD\\OOFDTD\\MathematicaFiles\>"];\)\n \
(*\ Physical\ Constants\ from\ \(\(http\)\(:\)\) // \(\(physics . nist . gov/
              cuu\)/Constants\)/index . html\ *) \[IndentingNewLine]
    \(hbar = 1.05457168*10^\((\(-34\))\);\)\n
    \(me = 9.1093826\ *10^\((\(-31\))\);\)\n
    \(c = 299792458;\)\n
    \(q = 1.60217653\ *10^\((\(-19\))\);\)\[IndentingNewLine]
    \(mu0 = 4  \[Pi]*10^\((\(-7\))\);\)\n
    \(eps0 = 8.854187817*10^\((\(-12\))\);\)\[IndentingNewLine] (*\ 
      Read\ the\ data\ into\ the\ memory\ *) \[IndentingNewLine]
    \(data = Import["\<goldPalik.txt\>", "\<TSV\>"];\)\n
    \(data = Drop[data, 2];\)\n
    \(realEpsilon = Array[0, {Length[data], 2}];\)\n
    \(imagEpsilon = Array[0, {Length[data], 2}];\)\[IndentingNewLine] (*\ 
      Assuming\ \[ExponentialE]\^\(\(\ \)\(\[ImaginaryJ]\ \[Omega]\ t\)\)\ \
time\ convention, \ 
      one\ has\ \[Epsilon]\  = \ \(\((n - \[ImaginaryI]\ k)\)\^2 = \(n\^2 - 
              k\^2 - \[ImaginaryI]*2*n*
                k\  = \ \[Epsilon]\_r + \[ImaginaryI]\ \[Epsilon]\_i\)\)\ \
*) \n
    \(For[i = 1, 
        i \[LessEqual] \ 
          Length[data], \(i++\), \[IndentingNewLine]realEpsilon[\([i, 1]\)] = 
          hbar*2*\[Pi]*c\ /\((data[\([i, 1]\)]*10\^\(-6\)*q)\); \  (*\ 
          Energy\ in\ eV\ *) \[IndentingNewLine]realEpsilon[\([i, 2]\)] = 
          Re[\((data[\([i, 2]\)]\  - \ \[ImaginaryI]*data[\([i, 3]\)])\)\^2]; \
 (*\ Real\ part\ of\ epsilon\ *) \[IndentingNewLine]imagEpsilon[\([i, 1]\)] = 
          hbar*2*\[Pi]*c\ /\((data[\([i, 1]\)]*10\^\(-6\)*q)\); \  (*\ 
          Energy\ in\ eV\ *) \[IndentingNewLine]imagEpsilon[\([i, 2]\)] = 
          Im[\((data[\([i, 2]\)]\  - \ \[ImaginaryI]*data[\([i, 3]\)])\)\^2]; \
\  (*\ Imaginary\ part\ of\ epsilon\ *) \[IndentingNewLine]];\)\)\)\)], \
"InputOnly"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*\ 
      Set\ frequency\ range\ over\ which\ the\ fit\ is\ required\ *) \)\(\
\[IndentingNewLine]\)\( (*\ Also, \ 
      set\ the\ number\ of\ Lorentz\ poles\ to\ be\ fitted\ *) \)\(\
\[IndentingNewLine]\)\(\(lambdaStart\  = \ 1.2;\)\  (*\ 
      In\ \[Mu]m\ *) \[IndentingNewLine]
    lambdaEnd\  = \ 2.0; \  (*\ 
      In\ \[Mu]m\ *) \[IndentingNewLine]numPoles\  = \ 3;\[IndentingNewLine]
    enStart = 
      hbar*2*\[Pi]*c\ /\((lambdaStart*10\^\(-6\)*q)\)\[IndentingNewLine]
    enEnd = 
      hbar*2*\[Pi]*c\ /\((lambdaEnd*10\^\(-6\)*q)\)\[IndentingNewLine] (*\ 
      Find\ the\ indices\ of\ experimental\ data\ which\ fall\ in\ the\ \
required\ range\ *) \[IndentingNewLine]
    \(indexStart\  = \ 
        First[Ordering[
            Abs[realEpsilon[\([All, 1]\)] - enStart]]];\)\[IndentingNewLine]
    \(indexEnd\  = \ 
        First[Ordering[
            Abs[realEpsilon[\([All, 1]\)] - enEnd]]];\)\)\)\)], "Input"],

Cell[BoxData[
    \(1.0332015858200152`\)], "Output"],

Cell[BoxData[
    \(0.6199209514920092`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The first pole is always of  Drude type. If numPoles>1, the other Lorentz \
poles are added accordingly. Some string manipulation is done to convert \
variable number of variables into legitimate ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expressions. Also note that all frequency names have \[HBar] to imply \
that they're in terms of eV's."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(\(fitParameters\  = \ 
      ToExpression[
        Flatten[Append[{"\<\[HBar]\[Omega]p1\>", "\<\[CapitalGamma]1\>"}, 
            Table[{"\<\[HBar]\[Omega]\>" <> 
                  ToString[m], "\<\[HBar]\[Omega]p\>" <> 
                  ToString[m], "\<\[CapitalGamma]\>" <> ToString[m]}, {m, 2, 
                numPoles}]]]]\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(constraints\  = \ 
      Block[{dum = True}, \[IndentingNewLine]For[i = 1, 
          i <= \ Length[fitParameters], \(i++\), \[IndentingNewLine]\(dum = \ 
              And[dum, 
                0.0\  <= \ 
                  fitParameters[\([i]\)]];\)\[IndentingNewLine]]; \
\[IndentingNewLine]LogicalExpand[
          dum]\[IndentingNewLine]]\), "\[IndentingNewLine]", 
    \(epsilon\  = \ 
      1\  + \ fitParameters[\([1]\)]\^2\/\(\[ImaginaryI]\ \((\[HBar]\[Omega])\
\) fitParameters[\([2]\)] - \((\[HBar]\[Omega])\)\^2\)\  + \ 
        Sum[fitParameters[\([3*\((m - 1)\) + 1]\)]\^2/\((\[ImaginaryI]\ \((\
\[HBar]\[Omega])\) fitParameters[\([3*\((m - 1)\) + 2]\)] + 
                fitParameters[\([3*\((m - 1)\)]\)]\^2 - \((\[HBar]\[Omega])\)\
\^2)\), {m, 2, numPoles}]\)}], "Input"],

Cell[BoxData[
    \({\[HBar]\[Omega]p1, \[CapitalGamma]1, \[HBar]\[Omega]2, \
\[HBar]\[Omega]p2, \[CapitalGamma]2, \[HBar]\[Omega]3, \[HBar]\[Omega]p3, \
\[CapitalGamma]3}\)], "Output"],

Cell[BoxData[
    \(0.` \[LessEqual] \[CapitalGamma]1 && 0.` \[LessEqual] \[CapitalGamma]2 && 
      0.` \[LessEqual] \[CapitalGamma]3 && 0.` \[LessEqual] \[HBar]\[Omega]2 && 
      0.` \[LessEqual] \[HBar]\[Omega]3 && 
      0.` \[LessEqual] \[HBar]\[Omega]p1 && 
      0.` \[LessEqual] \[HBar]\[Omega]p2 && 
      0.` \[LessEqual] \[HBar]\[Omega]p3\)], "Output"],

Cell[BoxData[
    \(1 + \[HBar]\[Omega]p1\^2\/\(\[ImaginaryI]\ \[CapitalGamma]1\ \[HBar]\
\[Omega] - \[HBar]\[Omega]\^2\) + \[HBar]\[Omega]p2\^2\/\(\[ImaginaryI]\ \
\[CapitalGamma]2\ \[HBar]\[Omega] - \[HBar]\[Omega]\^2 + \
\[HBar]\[Omega]2\^2\) + \[HBar]\[Omega]p3\^2\/\(\[ImaginaryI]\ \
\[CapitalGamma]3\ \[HBar]\[Omega] - \[HBar]\[Omega]\^2 + \
\[HBar]\[Omega]3\^2\)\)], "Output"]
}, Open  ]],

Cell["\<\
Define the fit function as the one which is closest to the experimental data \
in the range of wavelengths.\
\>", "Text"],

Cell[BoxData[
    \(\(fitFunction\  = \ \[IndentingNewLine]Block[{en = 0, 
            err = 0}, \[IndentingNewLine]For[i = indexStart, 
            i \[LessEqual] indexEnd, \(i++\), \[IndentingNewLine]en = 
              realEpsilon[\([i, 1]\)]; \[IndentingNewLine]err = 
              err + \((Re[epsilon /. \[HBar]\[Omega] -> en] - \
realEpsilon[\([i, 2]\)])\)\^2/\((realEpsilon[\([i, 2]\)])\)\^2 + \
\((Im[epsilon /. \[HBar]\[Omega] -> en] - imagEpsilon[\([i, \
2]\)])\)\^2/\((imagEpsilon[\([i, 2]\)])\)\^2;\[IndentingNewLine]]; \
\[IndentingNewLine]err\[IndentingNewLine]];\)\)], "Input"],

Cell["\<\
Here, different minimization techniques are tried. At the end, the one that \
produced the best results is assigned to the solnFinal variable for plotting \
purposes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(soln1 = 
      NMinimize[{fitFunction, constraints}, fitParameters, 
        MaxIterations \[Rule] 1000, 
        Method \[Rule] {"\<NelderMead\>", \ "\<ShrinkRatio\>" \[Rule]  .95, "\
\<ContractRatio\>" \[Rule]  .95, "\<ReflectRatio\>" \[Rule] 1.5}]\)], "Input"],

Cell[BoxData[
    \({0.05093483670285249`, {\[CapitalGamma]1 \[Rule] 
          0.`, \[CapitalGamma]2 \[Rule] 
          0.2890102765721252`, \[CapitalGamma]3 \[Rule] 
          2.331464640953873`, \[HBar]\[Omega]2 \[Rule] 
          0.000013468166711396699`, \[HBar]\[Omega]3 \[Rule] 
          0.5413895588222749`, \[HBar]\[Omega]p1 \[Rule] 
          6.674774852419044`, \[HBar]\[Omega]p2 \[Rule] 
          4.516370183741735`, \[HBar]\[Omega]p3 \[Rule] 
          1.2059635667831137`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(soln2 = 
      NMinimize[{fitFunction, constraints}, fitParameters, 
        MaxIterations \[Rule] 1000, 
        Method \[Rule] {"\<DifferentialEvolution\>", "\<ScalingFactor\>" \
\[Rule] 1}]\)], "Input"],

Cell[BoxData[
    \({0.08979913161787144`, {\[CapitalGamma]1 \[Rule] 
          0.05706524282560901`, \[CapitalGamma]2 \[Rule] 
          114.90186880286842`, \[CapitalGamma]3 \[Rule] 
          1.563095203364922`, \[HBar]\[Omega]2 \[Rule] 
          23.636412839420228`, \[HBar]\[Omega]3 \[Rule] 
          0.5683748261115856`, \[HBar]\[Omega]p1 \[Rule] 
          8.022580640918369`, \[HBar]\[Omega]p2 \[Rule] 
          0.038893475709143546`, \[HBar]\[Omega]p3 \[Rule] 
          2.1460085314827664`}}\)], "Output"]
}, Open  ]],

Cell["\<\
Choose the solution to be plotted below. And output the parameters of Lorentz \
media for use in the FDTD simulation.\
\>", "Text"],

Cell[BoxData[{
    \(\(solnFinal = soln1;\)\), "\[IndentingNewLine]", 
    \(\(plasmonList\  = \((\ 
            Table[ToExpression[
                  Table["\<\[HBar]\[Omega]p\>" <> ToString[m], {m, 1, 
                      numPoles}]]] /. \ solnFinal[\([2]\)])\)*
          q/hbar;\)\), "\[IndentingNewLine]", 
    \(oscillatorList = 
      Table[ToExpression[
          Table["\<\[HBar]\[Omega]\>" <> ToString[m], {m, 2, numPoles}]]]; 
    oscillatorList = \((Prepend[oscillatorList, 0.0] /. \ 
            solnFinal[\([2]\)])\)*q/hbar;\), "\[IndentingNewLine]", 
    \(\(dampingList\  = \((Table[
                ToExpression[
                  Table["\<\[CapitalGamma]\>" <> ToString[m], {m, 1, 
                      numPoles}]]] /. \ solnFinal[\([2]\)])\)*
          q/hbar;\)\), "\[IndentingNewLine]", 
    \(\(StylePrint["\<--FDTD Parameters For Copying into OOFDTD--\>", "\<Text\
\>"];\)\), "\[IndentingNewLine]", 
    \(\(StylePrint[
        PaddedForm[
          TableForm[
            Flatten[Table[{plasmonList[\([i]\)], oscillatorList[\([i]\)], 
                  dampingList[\([i]\)]}, {i, 1, numPoles}]], 
            TableDirections -> Row], {10, 9}, ExponentFunction -> Identity, 
          NumberFormat \[Rule] \((SequenceForm[#1, "\<e\>", #3] &)\)], \
"\<Text\>"];\)\)}], "Input"],

Cell["--FDTD Parameters For Copying into OOFDTD--", "Text",
  GeneratedCell->True,
  CellAutoOverwrite->True],

Cell["\<\
 1.014076882e16  0.000000000e0  0.000000000e0  6.861574653e15  2.046174861e10 \
 4.390839341e14  1.832181311e15  8.225155873e14  3.542118567e15\
\>", "Text",
  GeneratedCell->True,
  CellAutoOverwrite->True],

Cell["\<\
Below are the plotting options. It looks kind of messy, but that seems to be \
an inevitable consequence of Mathematica syntax for plot options.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(<< Graphics`Graphics`\), "\n", 
    \(\($TextStyle = {FontFamily \[Rule] "\<Times\>", 
          FontSize \[Rule] 12};\)\), "\[IndentingNewLine]", 
    \(\($FormatType\  = TraditionalForm;\)\), "\[IndentingNewLine]", 
    \(\(plotOps = {Frame -> True, ImageSize \[Rule] \ 72\ 6, 
          FrameLabel \[Rule] {"\<Energy (eV)\>", \*"\"\<\!\(\[Epsilon]\_r\)\>\
\"", "\<Real and Imaginary Parts Log-Log Plot\>", "\<\>"}, 
          Axes \[Rule] False, 
          PlotRange \[Rule] {{enEnd, enStart}, {0.1, 1100}}, 
          DisplayFunction \[Rule] Identity};\)\), "\[IndentingNewLine]", 
    \(\(realDataPlot = 
        LogLogListPlot[Abs[realEpsilon], Evaluate[plotOps], 
          PlotStyle \[Rule] {PointSize[0.013], 
              RGBColor[0, 0, 0]}];\)\), "\n", 
    \(\(imagDataPlot = 
        LogLogListPlot[Abs[imagEpsilon], Evaluate[plotOps], 
          PlotStyle \[Rule] {PointSize[0.013], 
              RGBColor[1, 0, 0]}];\)\), "\[IndentingNewLine]", 
    \(\(realPlot = 
        LogLogPlot[\(-Re[epsilon /. solnFinal[\([2]\)]]\), {\[HBar]\[Omega], 
            enEnd, enStart}, Evaluate[plotOps], 
          PlotStyle \[Rule] {Thickness[0.007], 
              RGBColor[0, 0, 0]}];\)\), "\n", 
    \(\(imagPlot = 
        LogLogPlot[\(-Im[epsilon /. solnFinal[\([2]\)]]\), {\[HBar]\[Omega], 
            enEnd, enStart}, Evaluate[plotOps], 
          PlotStyle \[Rule] {Thickness[0.007], 
              RGBColor[1, 0, 0]}];\)\), "\n", 
    \(\(Show[realPlot, imagPlot, realDataPlot, imagDataPlot, 
        DisplayFunction \[Rule] $DisplayFunction];\)\), \
"\[IndentingNewLine]", 
    \(\(plotOps = {Frame -> True, ImageSize \[Rule] \ 72\ 6, 
          FrameLabel \[Rule] {"\<Energy (eV)\>", \
\*"\"\<Re(\!\(\[Epsilon]\_r\))\>\"", "\<Real and Imaginary Parts of \
\[Epsilon] - Linear Plot\>", "\<\>"}, Axes \[Rule] False, 
          DisplayFunction \[Rule] Identity, 
          PlotRange \[Rule] {{enEnd, 
                enStart}, {Min[
                  Take[realEpsilon[\([All, 2]\)], {indexStart, indexEnd}]], 
                Max[Take[
                    realEpsilon[\([All, 2]\)], {indexStart, 
                      indexEnd}]]}}};\)\), "\[IndentingNewLine]", 
    \(\(realDataPlot = 
        ListPlot[realEpsilon, Evaluate[plotOps], 
          PlotStyle \[Rule] {PointSize[0.013], 
              RGBColor[0, 0, 0]}];\)\), "\n", 
    \(\(realPlot = 
        Plot[Re[epsilon /. solnFinal[\([2]\)]], {\[HBar]\[Omega], enEnd, 
            enStart}, Evaluate[plotOps], 
          PlotStyle \[Rule] {Thickness[0.005], 
              RGBColor[0, 0, 0]}];\)\), "\[IndentingNewLine]", 
    \(\(p1 = Show[realPlot, realDataPlot];\)\), "\[IndentingNewLine]", 
    \(\(plotOps = {Frame -> True, ImageSize \[Rule] \ 72\ 6, 
          FrameLabel \[Rule] {"\<Energy (eV)\>", \
\*"\"\<Im(\!\(\[Epsilon]\_r\))\>\""}, Axes \[Rule] False, 
          DisplayFunction \[Rule] Identity, 
          PlotRange \[Rule] {{enEnd, 
                enStart}, {Min[
                  Take[imagEpsilon[\([All, 2]\)], {indexStart, indexEnd}]], 
                Max[Take[
                    imagEpsilon[\([All, 2]\)], {indexStart, 
                      indexEnd}]]}}};\)\), "\[IndentingNewLine]", 
    \(\(imagDataPlot = 
        ListPlot[imagEpsilon, Evaluate[plotOps], 
          PlotStyle \[Rule] {PointSize[0.02], 
              RGBColor[1, 0, 0]}];\)\), "\[IndentingNewLine]", 
    \(\(imagPlot = 
        Plot[Im[epsilon /. solnFinal[\([2]\)]], {\[HBar]\[Omega], enEnd, 
            enStart}, Evaluate[plotOps], 
          PlotStyle \[Rule] {Thickness[0.007], 
              RGBColor[1, 0, 0]}];\)\), "\[IndentingNewLine]", 
    \(\(p2 = Show[imagPlot, imagDataPlot];\)\), "\[IndentingNewLine]", 
    \(\(Export["\<out.png\>", 
        Show[Graphics[{Rectangle[{0, 0}, {1.1, 1.1}, p1], 
              Rectangle[{0.45, 0.15}, {0.95, 0.65}, p2]}], 
          DisplayFunction \[Rule] $DisplayFunction, 
          ImageSize \[Rule] \ 72\ 6]];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 432 266.991 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.93606 4.50758 0.152926 0.152926 [
[.93606 -0.0125 -5 -15.6875 ]
[.93606 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -31.25 -31.375 ]
[.5 -0.0125 31.25 -15.6875 ]
[-0.0125 .15293 -10 -7.84375 ]
[-0.0125 .15293 0 7.84375 ]
[-0.0125 .30585 -16 -7.84375 ]
[-0.0125 .30585 0 7.84375 ]
[-0.0125 .45878 -22 -7.84375 ]
[-0.0125 .45878 0 7.84375 ]
[-0.0125 .6117 -28 -7.84375 ]
[-0.0125 .6117 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -43.6875 -6.25 ]
[-0.01875 .30902 -28 6.25 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -97.5625 0 ]
[.5 .63053 97.5625 15.6875 ]
[ 1 0 .125 0 ]
[ -0.0005 0 0 0 ]
[ 1.0005 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.93606 0 m
.93606 .00625 L
s
gsave
.93606 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
69.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
s
gsave
.5 -0.0125 -92.25 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Energy) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
100.000 15.813 moveto
(H) show
103.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
117.688 15.813 moveto
(L) show
121.500 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .15293 m
.00625 .15293 L
s
gsave
-0.0125 .15293 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
69.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .30585 m
.00625 .30585 L
s
gsave
-0.0125 .30585 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
75.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .45878 m
.00625 .45878 L
s
gsave
-0.0125 .45878 -83 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
81.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .6117 m
.00625 .6117 L
s
gsave
-0.0125 .6117 -89 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1000) show
87.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.001 w
0 .04604 m
.00375 .04604 L
s
0 .07296 m
.00375 .07296 L
s
0 .09207 m
.00375 .09207 L
s
0 .10689 m
.00375 .10689 L
s
0 .119 m
.00375 .119 L
s
0 .12924 m
.00375 .12924 L
s
0 .13811 m
.00375 .13811 L
s
0 .14593 m
.00375 .14593 L
s
0 .19896 m
.00375 .19896 L
s
0 .22589 m
.00375 .22589 L
s
0 .245 m
.00375 .245 L
s
0 .25982 m
.00375 .25982 L
s
0 .27193 m
.00375 .27193 L
s
0 .28216 m
.00375 .28216 L
s
0 .29103 m
.00375 .29103 L
s
0 .29885 m
.00375 .29885 L
s
0 .35189 m
.00375 .35189 L
s
0 .37882 m
.00375 .37882 L
s
0 .39792 m
.00375 .39792 L
s
0 .41274 m
.00375 .41274 L
s
0 .42485 m
.00375 .42485 L
s
0 .43509 m
.00375 .43509 L
s
0 .44396 m
.00375 .44396 L
s
0 .45178 m
.00375 .45178 L
s
0 .50481 m
.00375 .50481 L
s
0 .53174 m
.00375 .53174 L
s
0 .55085 m
.00375 .55085 L
s
0 .56567 m
.00375 .56567 L
s
0 .57778 m
.00375 .57778 L
s
0 .58802 m
.00375 .58802 L
s
0 .59688 m
.00375 .59688 L
s
0 .60471 m
.00375 .60471 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -103.094 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
67.25 11.8438 translate
90 rotate
-67.25 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(e) show
67.688 17.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 8.500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(r) show
71.500 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.93606 .61178 m
.93606 .61803 L
s
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -158.563 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Real) show
87.813 15.813 moveto
(and) show
108.063 15.813 moveto
(Imaginary) show
159.375 15.813 moveto
(Parts) show
186.250 15.813 moveto
(Log) show
205.188 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
212.875 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Log) show
234.813 15.813 moveto
(Plot) show
254.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 0 m
1 0 L
s
.99375 .15293 m
1 .15293 L
s
.99375 .30585 m
1 .30585 L
s
.99375 .45878 m
1 .45878 L
s
.99375 .6117 m
1 .6117 L
s
.001 w
.99625 .04604 m
1 .04604 L
s
.99625 .07296 m
1 .07296 L
s
.99625 .09207 m
1 .09207 L
s
.99625 .10689 m
1 .10689 L
s
.99625 .119 m
1 .119 L
s
.99625 .12924 m
1 .12924 L
s
.99625 .13811 m
1 .13811 L
s
.99625 .14593 m
1 .14593 L
s
.99625 .19896 m
1 .19896 L
s
.99625 .22589 m
1 .22589 L
s
.99625 .245 m
1 .245 L
s
.99625 .25982 m
1 .25982 L
s
.99625 .27193 m
1 .27193 L
s
.99625 .28216 m
1 .28216 L
s
.99625 .29103 m
1 .29103 L
s
.99625 .29885 m
1 .29885 L
s
.99625 .35189 m
1 .35189 L
s
.99625 .37882 m
1 .37882 L
s
.99625 .39792 m
1 .39792 L
s
.99625 .41274 m
1 .41274 L
s
.99625 .42485 m
1 .42485 L
s
.99625 .43509 m
1 .43509 L
s
.99625 .44396 m
1 .44396 L
s
.99625 .45178 m
1 .45178 L
s
.99625 .50481 m
1 .50481 L
s
.99625 .53174 m
1 .53174 L
s
.99625 .55085 m
1 .55085 L
s
.99625 .56567 m
1 .56567 L
s
.99625 .57778 m
1 .57778 L
s
.99625 .58802 m
1 .58802 L
s
.99625 .59688 m
1 .59688 L
s
.99625 .60471 m
1 .60471 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 0 r
.007 w
0 .48941 m
.05224 .48602 L
.10767 .48241 L
.15833 .47911 L
.20585 .47601 L
.25519 .47278 L
.30154 .46974 L
.34963 .46658 L
.39487 .4636 L
.43743 .4608 L
.48171 .45788 L
.52342 .45512 L
.56271 .45252 L
.60369 .4498 L
.64234 .44723 L
.68261 .44456 L
.72064 .44202 L
.75652 .43963 L
.79401 .43713 L
.82943 .43476 L
.86638 .43229 L
.90133 .42995 L
.93439 .42773 L
.96894 .42541 L
1 .42332 L
s
1 0 0 r
0 .35616 m
.05224 .3516 L
.10767 .34673 L
.15833 .34227 L
.20585 .33806 L
.25519 .33368 L
.30154 .32955 L
.34963 .32525 L
.39487 .3212 L
.43743 .31738 L
.48171 .31339 L
.52342 .30962 L
.56271 .30607 L
.60369 .30236 L
.64234 .29885 L
.68261 .29519 L
.72064 .29173 L
.75652 .28847 L
.79401 .28505 L
.82943 .28181 L
.86638 .27843 L
.90133 .27524 L
.93439 .27221 L
.96894 .26904 L
1 .2662 L
s
0 0 0 r
.013 w
.93581 .44122 Mdot
.89673 .43047 Mdot
.85691 .43307 Mdot
.8149 .43577 Mdot
.72073 .43842 Mdot
.72924 .44117 Mdot
.68569 .44401 Mdot
.64037 .44679 Mdot
.59475 .44979 Mdot
.54757 .45287 Mdot
.49898 .45601 Mdot
.4491 .45988 Mdot
.39926 .46247 Mdot
.34715 .46628 Mdot
.29298 .46998 Mdot
.23805 .47357 Mdot
.1814 .47707 Mdot
.12217 .48161 Mdot
.06266 .4849 Mdot
0 .48917 Mdot
1 0 0 r
.93581 .2775 Mdot
.89673 .2751 Mdot
.85691 .27875 Mdot
.8149 .28269 Mdot
.72073 .28666 Mdot
.72924 .29073 Mdot
.68569 .29487 Mdot
.64037 .29902 Mdot
.59475 .30342 Mdot
.54757 .30774 Mdot
.49898 .31199 Mdot
.4491 .31671 Mdot
.39926 .32091 Mdot
.34715 .3257 Mdot
.29298 .33052 Mdot
.23805 .33534 Mdot
.1814 .34027 Mdot
.12217 .34582 Mdot
.06266 .35085 Mdot
0 .35644 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{432, 266.938},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3clo>a
0??cl`00jP3clo<3000000030??cl`00000000000<00lo?c003Y0??cl`050000003clo<0lo?c0??c
l`0000000P3clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c00/0lo?c00<000000??c
l`3clo<0Y03clo<00>T0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c
00H0lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`2S0??cl`00d`3clo<600000005
0??cl`00000000000000003clo<00P0000000`3clo<00000000000020??cl`<000000P3clo<30000
00@0lo?c00<000000??cl`3clo<01@3clo<010000000lo?c0??cl`3clo<2000000D0lo?c00<00000
0??cl`3clo<0103clo<00`000000lo?c0??cl`2R0??cl`00e03clo<00`000000lo?c0??cl`020??c
l`030000003clo<000000080lo?c00<000000??cl`0000000P3clo<010000000lo?c0??cl`000002
0??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`0000001P3clo<010000000lo?c0??cl`00
00020??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??c
l`2R0??cl`00e03clo<00`000000lo?c0??cl`040??cl`060000003clo<0lo?c0000003clo<00000
1@3clo<010000000lo?c0??cl`3clo<2000000@0lo?c00<000000??cl`0000001P3clo<010000000
lo?c0??cl`0000060??cl`030000003clo<0000000D0lo?c00<000000??cl`3clo<0XP3clo<00=@0
lo?c00@000000??cl`3clo<000000`3clo<01@000000lo?c0??cl`000000lo?c00@000000P3clo<0
10000000lo?c0??cl`0000020??cl`040000003clo<0lo?c00000080lo?c00<000000??cl`3clo<0
103clo<00`000000lo?c0??cl`04000000<0lo?c00<000000??cl`0000001@3clo<00`000000lo?c
0??cl`2R0??cl`00e03clo<4000000<0lo?c0P000000103clo<000000??cl`0000020??cl`030000
003clo<0lo?c0080000000D0lo?c0000003clo<0lo?c000000020??cl`050000003clo<0lo?c0??c
l`0000001@3clo<010000000lo?c0??cl`0000020??cl`070000003clo<0lo?c0??cl`000000lo?c
000000050??cl`030000003clo<0lo?c0:80lo?c003D0??cl`040000003clo<0lo?c00000080lo?c
0P0000000`3clo<000000??cl`020??cl`8000000P3clo<2000000030??cl`000000lo?c00L00000
00@0lo?c0000000000000000103clo<010000000lo?c0??cl`3clo<2000000<0lo?c00D000000??c
l`3clo<0lo?c000000040??cl`030000003clo<0lo?c0:80lo?c003D0??cl`030000003clo<0lo?c
0280lo?c00<000000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00<00000
0??cl`3clo<0XP3clo<00=@0lo?c00D000000??cl`3clo<0lo?c0000000Q0??cl`030000003clo<0
lo?c00D0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c0:<0lo?c003C
0??cl`H000008P3clo<00`000000lo?c0??cl`030??cl`<0000000H0lo?c00000000000000000??c
l`00002V0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00
o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>30??cl`<00000:`3clo<00?l0
lo?cQ03clo<00`000000lo?c0??cl`0Z0??cl`00o`3clo>40??cl`030000003clo<0lo?c02X0lo?c
003o0??clh@0lo?c00<000000??cl`3clo<0:P3clo<00?l0lo?cQ03clo<00`000000lo?c0??cl`0Z
0??cl`00o`3clo>40??cl`030000003clo<0lo?c02X0lo?c003o0??clh@0lo?c00<000000??cl`3c
lo<0:P3clo<00?l0lo?cP`3clo<2000002`0lo?c003o0??clh@0lo?c00<000000??cl`3clo<0:P3c
lo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3c
lo<00?l0lo?c/@3clo<004H0lo?co`00001F000001D0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?c?03clo<00`000000lo?c0??cl`0C0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000
003clo<0lo?c0?l0lo?c?03clo<00`000000lo?c0??cl`0C0??cl`030000003clo<0lo?c01<0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<0
0`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c
00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003c
lo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C
0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0
lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000
003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??c
l`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B
0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`00
0000lo?c0??cl`0C0??cl`00AP3clo<200000?l0lo?cDP3clo<2000001D0lo?c00160??cl`030000
003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??c
l`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B
0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`00
0000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3c
lo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<0
0`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c0P00003o0??c
le80lo?c0P00000E0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3c
lo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<0
0`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<200000?l0lo?cDP3clo<2000001D0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<0
0`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<200000?l0lo?cDP3clo<2000001D0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<0
0`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`800000o`3clo=B0??cl`80
00005@3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<2
00000?l0lo?cDP3clo<2000001D0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`00
0000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3c
lo<04`3clo<004H0lo?c0P00003o0??cle80lo?c0P00000E0??cl`00AP3clo<00`000000lo?c0??c
l`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<003/0lo?c0`0000080??cl`030000003clo<0
lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00?03clo<00`000000lo?c0??cl`070??c
l`800000o`3clo=B0??cl`8000005@3clo<003`0lo?c00<000000??cl`3clo<01`3clo<00`000000
lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<003`0lo?c00<000000??cl`3clo<0
1`3clo<300000?l0lo?cD03clo<3000001D0lo?c000l0??cl`030000003clo<0lo?c00L0lo?c00<0
00000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c000l0??cl`030000003clo<0
lo?c00L0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c000l0??c
l`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c
01<0lo?c000k0??cl`8000002@3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3c
lo<04`3clo<003`0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`3o0??cle80lo?c
00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003c
lo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C
0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0
lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000
003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??c
l`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B
0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`00
0000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3c
lo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c
00160??cl`800000o`3clo=B0??cl`8000005@3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B
0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`00
0000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3c
lo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<0
0`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<200000?l0lo?cDP3clo<2000001D0lo?c
00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<0
0`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c
00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003c
lo<0lo?c01<0lo?c00160??cl`800000o`3clo=B0??cl`8000005@3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c
00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003c
lo<0lo?c01<0lo?c00160??cl`800000o`3clo=B0??cl`8000005@3clo<004H0lo?c00<000000??c
l`3clo<0o`3clo=B0??cl`030?l0003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0
lo?cB03clo<;0?l001D0lo?c00160??cl`030000003clo<0lo?c0?l0lo?c?03clo<F0?l000030000
003clo<0lo?c01<0lo?c00160??cl`800000o`3clo<_0??cl`80o`0000<0lo?c0?l0003o00005@3o
00090??cl`8000005@3clo<004H0lo?c00<000000??cl`3clo<0o`3clo<Q0??cl`80o`000P3clo<J
0?l001<0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo<B0??c
l`80o`00103clo<I0?l000X0lo?c103o000C0??cl`030000003clo<0lo?c01<0lo?c00160??cl`80
0000lP3clo<20?l001P0lo?c6@3o00090??cl`80o`00303clo<20?l001<0lo?c0P00000E0??cl`00
AP3clo<00`000000lo?c0??cl`3`0??cl`@0o`002P3clo<I0?l000T0lo?c0P3o000_0??cl`030000
003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?00lo?c6`3o00060??cl`80o`00?P3c
lo<00`000000lo?c0??cl`0C0??cl`00AP3clo<200000>H0lo?c0P3o00050??cla80o`00DP3clo<2
000001D0lo?c00160??cl`030000003clo<0lo?c0>@0lo?c3P3o00000`3clo<0o`000?l000020?l0
05X0lo?c00<000000??cl`3clo<04`3clo<003D0lo?c0`0000030??cl`<00000203clo<00`000000
lo?c0??cl`3F0??claH0o`00203clo<20?l005/0lo?c00<000000??cl`3clo<04`3clo<001`0lo?c
0`00000G0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`80
0000b03clo<20?l000030??cl`3o0000o`0001/0o`00J@3clo<2000001D0lo?c000K0??cl`800000
0P3clo<00`000000lo?c0??cl`0D0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000
lo?c0??cl`050??cl`030000003clo<0lo?c0;`0lo?c7@3o001h0??cl`030000003clo<0lo?c01<0
lo?c000J0??cl`030000003clo<000000080lo?c00<000000??cl`3clo<0503clo<01@000000lo?c
0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<01@3clo<300000;H0lo?c503o000<0??cl`80
o`00M`3clo<3000001D0lo?c000J0??cl`030000003clo<000000080lo?c00<000000??cl`3clo<0
503clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<01@3clo<00`000000
lo?c0??cl`2V0??cl`80o`001`3clo<=0?l000/0lo?c0P3o00280??cl`030000003clo<0lo?c01<0
lo?c000J0??cl`030000003clo<0lo?c01T0lo?c00D000000??cl`3clo<0lo?c000000030??cl`03
0000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0X@3clo<H0?l009P0lo?c00<000000??cl`3c
lo<04`3clo<001`0lo?c00<000000??cl`3clo<00P00000E0??cl`050000003clo<0lo?c0??cl`00
00000`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0980lo?c7@3o00070??cl`80
o`00V@3clo<00`000000lo?c0??cl`0C0??cl`00703clo<3000001H0lo?c0P0000030??cl`050000
003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`240??cl`80o`001@3clo<F0?l000D0
lo?c0P3o002Y0??cl`030000003clo<0lo?c01<0lo?c000M0??cl`030000003clo<0lo?c01H0lo?c
00<000000??cl`3clo<00P3clo<3000000P0lo?c00<000000??cl`3clo<0P`3clo<?0?l00080lo?c
103o002i0??cl`030000003clo<0lo?c01<0lo?c000L0??cl`030000003clo<0lo?c02L0lo?c00<0
00000??cl`3clo<0LP3clo<I0?l000X0lo?c0P3o002j0??cl`030000003clo<0lo?c01<0lo?c0016
0??cl`030000003clo<0lo?c0600lo?c0P3o00000`3clo<0o`000?l0000N0?l000030??cl`3o0000
o`000</0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0G03clo<I0?l0
0=`0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0C@3clo<20?l000D0
lo?c3`3o000?0??cl`80o`00g@3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c
0??cl`130??claT0o`00103clo<20?l00>l0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<0
00000??cl`3clo<0>P3clo<J0?l00?d0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<00000
0??cl`3clo<0<`3clo<@0?l000X0lo?c0P3o003o0??cl`<0lo?c00<000000??cl`3clo<04`3clo<0
04H0lo?c00<000000??cl`3clo<08@3clo<L0?l00?l0lo?c5@3clo<00`000000lo?c0??cl`0C0??c
l`00AP3clo<00`000000lo?c0??cl`0A0??cl`80o`001@3clo<K0?l000L0lo?c0P3o003o0??claH0
lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<03`3clo<B0?l000@0lo?c
103o003o0??clbT0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c6`3o000>0??cl`80o`00o`3c
lo<Y0??cl`8000005@3clo<004H0lo?c4P3o00020??cl`80o`00o`3clo<o0??cl`030000003clo<0
lo?c01<0lo?c00160??cl`80o`00o`3clo=C0??cl`030000003clo<0lo?c01<0lo?c00160??cl`03
0?l0003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c
0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3c
lo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0
lo?c00160??cl`800000o`3clo=B0??cl`8000005@3clo<004H0lo?c00<000000??cl`3clo<0o`3c
lo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0
lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3c
lo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c0P00003o
0??cle80lo?c0P00000E0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0
lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3c
lo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c0P00003o
0??cle80lo?c0P00000E0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0
lo?c00160??cl`030000003clo<0lo?c0?l0lo?cB03clo<;000001D0lo?c00160??cl`800000o`3c
lo<_0??cl`800000303clo<G000001D0lo?c00160??cl`030000003clo<0lo?c0?l0lo?c8@3clo<2
000000X0lo?c6`00000:0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c
0?l0lo?c4P3clo<2000000@0lo?c9000000F0??cl`030000003clo<0lo?c01<0lo?c00160??cl`80
0000lP3clo<2000001P0lo?c9@00000P0??cl`8000005@3clo<004H0lo?c00<000000??cl`3clo<0
l03clo<6000000P0lo?c6@0000090??cl`8000006@3clo<2000001@0lo?c00<000000??cl`3clo<0
4`3clo<004H0lo?c00<000000??cl`3clo<0i@3clo<2000000T0lo?c6`0000060??cl`8000009`3c
lo<4000001<0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c0P00003G0??cl`800000303clo<J
000003`0lo?c1000000B0??cl`8000005@3clo<004H0lo?c00<000000??cl`3clo<0a`3clo<20000
00030??cl`000000000002H000000P3clo<2000004D0lo?c0P00000D0??cl`030000003clo<0lo?c
01<0lo?c000_0??cl`<000000`3clo<3000000<0lo?c0`0000080??cl`030000003clo<0lo?c0;H0
lo?c0P0000040??clb/00000JP3clo<00`000000lo?c0??cl`0C0??cl`00<03clo<01@000000lo?c
0??cl`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`80
0000Y`3clo<2000000L0lo?c6`00000<0??cl`800000N03clo<2000001D0lo?c000`0??cl`050000
003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00D0
lo?c00<000000??cl`3clo<0U@3clo<2000000X0lo?c6`00000;0??cl`800000R03clo<00`000000
lo?c0??cl`0C0??cl`00<03clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`00
00000`3clo<00`000000lo?c0??cl`050??cl`<00000Q03clo<2000000`0lo?c7@0000070??cl`80
0000U`3clo<3000001D0lo?c000`0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000
lo?c000000030??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0P`3clo<N000000D0
lo?c0P00002Y0??cl`030000003clo<0lo?c01<0lo?c000`0??cl`050000003clo<0lo?c0??cl`00
00000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3c
lo<0LP3clo<P000000<0lo?c0P00002j0??cl`030000003clo<0lo?c01<0lo?c000`0??cl`050000
003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00D0
lo?c00<000000??cl`3clo<0H03clo<2000000030??cl`000000000001h0000000<0lo?c00000000
0000b`3clo<00`000000lo?c0??cl`0C0??cl`00;`3clo<2000000<0lo?c00L000000??cl`3clo<0
lo?c0000003clo<0000000<0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`1=0??c
l`8000001@3clo<Q00000=`0lo?c00<000000??cl`3clo<04`3clo<00300lo?c00<000000??cl`3c
lo<00P3clo<3000000<0lo?c0`0000080??cl`030000003clo<0lo?c03X0lo?c0P0000070??clb00
00003`3clo<200000=d0lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0
<`3clo<Q000000`0lo?c0P00003_0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003c
lo<0lo?c0240lo?c8P00000:0??cl`800000o`3clo<30??cl`030000003clo<0lo?c01<0lo?c0016
0??cl`030000003clo<0lo?c0140lo?c0P0000050??cla/000001`3clo<200000?l0lo?c5P3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`0?0??cla800000103clo<40000
0?l0lo?c:@3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<K000000h0lo?c0P00003o0??clbX0
lo?c00<000000??cl`3clo<04`3clo<004H0lo?c4P0000020??cl`800000o`3clo<o0??cl`030000
003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??c
l`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<0
04H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`03
0000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<200000?l0lo?c
DP3clo<2000001D0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??c
l`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<0
04H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`03
0000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c
0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3c
lo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c0P00003o0??cle80lo?c0P00000E0??cl`00AP3clo<00`000000lo?c
0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3c
lo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0
lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3c
lo<200000?l0lo?cDP3clo<2000001D0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0
lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3c
lo<200000?l0lo?cDP3clo<2000001D0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??c
l`3clo<04`3clo<004H0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0
lo?c00160??cl`800000o`3clo=B0??cl`8000005@3clo<004H0lo?c00<000000??cl`3clo<0o`3c
lo=B0??cl`030000003clo<0lo?c01<0lo?c00160??cl`030000003clo<0lo?c0?l0lo?cDP3clo<0
0`000000lo?c0??cl`0C0??cl`00AP3clo<200000?l0lo?cDP3clo<2000001D0lo?c00160??cl`03
0000003clo<0lo?c0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00AP3clo<00`000000lo?c
0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<004H0lo?c0P00003o0??cle80lo?c0P00
000E0??cl`00AP3clo<00`000000lo?c0??cl`3o0??cle80lo?c00<000000??cl`3clo<04`3clo<0
02T0lo?c0`0000030??cl`<000000`3clo<3000000<0lo?c0`0000080??cl`030000003clo<0lo?c
0?l0lo?cDP3clo<00`000000lo?c0??cl`0C0??cl`00:P3clo<01@000000lo?c0??cl`3clo<00000
00<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c
00D0lo?c0P00003o0??cle80lo?c0P00000E0??cl`00:P3clo<01@000000lo?c0??cl`3clo<00000
00<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c
00D0lo?c00<000000??cl`3clo<0o`3clo=B0??cl`030000003clo<0lo?c01<0lo?c000Z0??cl`05
0000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<00000
00<0lo?c00<000000??cl`3clo<01@3clo<300000?l0lo?c?03clo<00`000000lo?c0??cl`0A0??c
l`<000005@3clo<002X0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<00000
00<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c
0?l0lo?c?03clo<00`000000lo?c0??cl`0C0??cl`030000003clo<0lo?c01<0lo?c000Z0??cl`05
0000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<00000
00<0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`3o0??clc`0lo?c00<000000??c
l`3clo<04`3clo<00`000000lo?c0??cl`0C0??cl`00:P3clo<01@000000lo?c0??cl`3clo<00000
00<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c
00D0lo?co`00001F000001D0lo?c000Y0??cl`8000000`3clo<01`000000lo?c0??cl`3clo<00000
0??cl`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c0?l0lo?cL03clo<0
02X0lo?c00<000000??cl`3clo<00P3clo<3000000<0lo?c0`0000030??cl`<00000o`3clo=c0??c
l`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00d03clo<3000001<0lo?c0P00000b
0??cl`<000005`3clo<3000007T0lo?c003?0??cl`050000003clo<0lo?c0??cl`000000503clo<0
0`000000lo?c0??cl`0^0??cl`050000003clo<0lo?c0??cl`0000005@3clo<01@000000lo?c0??c
l`3clo<0000007P0lo?c003?0??cl`050000003clo<0lo?c0??cl`0000005@3clo<00`000000lo?c
0??cl`0]0??cl`050000003clo<0lo?c0??cl`0000005@3clo<01@000000lo?c0??cl`3clo<00000
07P0lo?c002@0??cl`<000000`3clo<2000000030??cl`000000000000<0lo?c1`0000050??cl`L0
000000@0lo?c000000000000lo?c1@0000030??cl`<0000000D0lo?c00000000000000000??cl`02
000000040??cl`00000000000??cl`@0000000D0lo?c00000000000000000??cl`06000000040??c
l`00000000000??cl`L00000103clo<00`000000lo?c0??cl`050??cl`<000001@3clo<700000080
lo?c0P000000103clo<000000000000000040??cl`H000000P3clo<3000000<0lo?c0`0000090??c
l`H000000P3clo<3000000<0lo?c0`0000050??cl`<00000103clo<3000000040??cl`0000000000
000000<0lo?c0P00001Q0??cl`00T@3clo<01@000000lo?c0??cl`3clo<000000080lo?c00H00000
0??cl`3clo<000000??cl`0000020??cl`040000003clo<0lo?c000000D0lo?c00@000000??cl`3c
lo<000000P3clo<01P000000lo?c0??cl`000000lo?c000000<0lo?c00<000000??cl`3clo<00`3c
lo<01@000000lo?c0??cl`3clo<000000080lo?c00H000000??cl`3clo<000000??cl`0000020??c
l`030000003clo<0000000D0lo?c00@000000??cl`3clo<000000P3clo<00`000000lo?c00000002
0??cl`040000003clo<0lo?c000000@0lo?c00<000000??cl`0000001`3clo<00`000000lo?c0??c
l`030??cl`040000003clo<0lo?c00000080lo?c00D000000??cl`3clo<0lo?c000000020??cl`04
0000003clo<0lo?c000000@0lo?c00<000000??cl`3clo<00P3clo<2000000<0lo?c00<000000??c
l`0000003@3clo<00`000000lo?c0??cl`020??cl`8000000`3clo<00`000000lo?c000000090??c
l`030000003clo<0lo?c00@0lo?c00<000000??cl`0000000`3clo<010000000lo?c0??cl`00001R
0??cl`00T@3clo<010000000lo?c0??cl`0000030??cl`030000003clo<0lo?c0080lo?c00@00000
0??cl`3clo<000000P3clo<00`000000lo?c0??cl`030??cl`040000003clo<0lo?c00000080lo?c
00H000000??cl`3clo<000000??cl`0000030??cl`030000003clo<0lo?c00<0lo?c00D000000??c
l`3clo<0lo?c000000020??cl`060000003clo<0lo?c0000003clo<000000P3clo<00`000000lo?c
0??cl`02000000<0lo?c00@000000??cl`3clo<000000P3clo<00`000000lo?c000000020??cl`04
0000003clo<0lo?c000000@0lo?c00<000000??cl`0000001`3clo<00`000000lo?c0??cl`030??c
l`040000003clo<0lo?c00000080lo?c00D000000??cl`3clo<0lo?c000000040??cl`800000103c
lo<00`000000lo?c0??cl`030??cl`050000003clo<0lo?c0??cl`0000000P3clo<2000000/0lo?c
00<000000??cl`3clo<00`3clo<01@000000lo?c0??cl`3clo<000000080lo?c0P0000070??cl`03
0000003clo<0lo?c00@0lo?c00<000000??cl`0000000`3clo<010000000lo?c0??cl`00001R0??c
l`00T@3clo<010000000lo?c0??cl`0000030??cl`@000000P3clo<300000080lo?c00<000000??c
l`3clo<0103clo<300000080lo?c00H000000??cl`3clo<000000??cl`0000030??cl`030000003c
lo<0lo?c00<0lo?c00D000000??cl`3clo<0lo?c000000020??cl`040000003clo<0lo?c00000080
lo?c0`0000001@3clo<000000??cl`3clo<000000080lo?c00@000000??cl`3clo<000000P3clo<0
0`000000lo?c0??cl`0300000080lo?c00D000000??cl`3clo<0lo?c000000020??cl`030000003c
lo<0lo?c00D0lo?c00<000000??cl`3clo<0103clo<300000080lo?c00D000000??cl`3clo<0lo?c
000000020??cl`8000001P3clo<00`000000lo?c0??cl`030??cl`070000003clo<0lo?c0??cl`00
0000lo?c000000020??cl`040000003clo<0lo?c0??cl`P000001@3clo<01`000000lo?c0??cl`3c
lo<000000??cl`0000000P3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c00@0lo?c
00<000000??cl`0000000`3clo<010000000lo?c0??cl`00001R0??cl`00T@3clo<4000000<0lo?c
00H000000??cl`3clo<000000??cl`0000020??cl`040000003clo<0lo?c000000D0lo?c00@00000
0??cl`3clo<000000P3clo<2000000040??cl`000000lo?c00000080lo?c0P0000050??cl`040000
003clo<0lo?c0??cl`80000000L0lo?c000000000000lo?c0000003clo<000000080lo?c00<00000
0??cl`0000000P3clo<010000000lo?c0??cl`0000020??cl`80000000@0lo?c0000003clo<00000
0P3clo<00`000000lo?c0??cl`0200000080lo?c00D000000??cl`3clo<0lo?c000000060??cl`@0
00000P3clo<010000000lo?c0??cl`0000020??cl`8000000P3clo<010000000lo?c0??cl`000002
0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<00`3clo<01`000000lo?c0??cl`3c
lo<000000??cl`0000000P3clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c00<0lo?c
00L000000??cl`3clo<0lo?c0000003clo<000000080lo?c00<000000??cl`3clo<0103clo<40000
00<0lo?c00<000000??cl`0000000`3clo<010000000lo?c0??cl`00001R0??cl`00T@3clo<01@00
0000lo?c0??cl`3clo<0000000<0lo?c0P0000030??cl`8000000`3clo<00`000000lo?c0??cl`04
0??cl`8000000P3clo<2000000030??cl`000000lo?c0080lo?c0P0000000`3clo<000000??cl`04
0??cl`030000003clo<0lo?c0080000000D0lo?c0000003clo<0lo?c000000030??cl`8000000`3c
lo<6000000050??cl`00000000000??cl`0000000`3clo<200000080lo?c0P0000000`3clo<00000
00000002000000040??cl`0000000000000000D0lo?c00D000000??cl`3clo<0lo?c000000020??c
l`8000000P3clo<2000000030??cl`0000000000008000000P3clo<3000000@0lo?c00<000000??c
l`3clo<0103clo<3000000<0lo?c100000090??cl`030000003clo<0lo?c00@0lo?c0`0000030??c
l`@000001@3clo<01@000000lo?c0??cl`3clo<000000080lo?c00<000000??cl`3clo<00`000002
0??cl`<00000H@3clo<00940lo?c00D000000??cl`3clo<0lo?c0000000=0??cl`030000003clo<0
lo?c0180lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`0a0??cl`050000003clo<0
lo?c0??cl`0000002`3clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c01L0lo?c00<0
00000??cl`3clo<04`3clo<01@000000lo?c0??cl`3clo<000000080lo?c00<000000??cl`3clo<0
1P3clo<00`000000lo?c0??cl`1P0??cl`00T@3clo<01@000000lo?c0??cl`3clo<0000000d0lo?c
00<000000??cl`3clo<04P3clo<00`000000lo?c0??cl`030??cl`030000003clo<0lo?c0340lo?c
00D000000??cl`3clo<0lo?c0000000;0??cl`030000003clo<0lo?c00P0lo?c00<000000??cl`3c
lo<05`3clo<00`000000lo?c0??cl`0C0??cl`050000003clo<0lo?c0??cl`0000000P3clo<00`00
0000lo?c0??cl`060??cl`030000003clo<0lo?c0600lo?c002@0??cl`D000003@3clo<2000001<0
lo?c0P0000040??cl`<000005@3clo<00`000000lo?c0??cl`0I0??cl`D000005P3clo<3000001L0
lo?c0`00000C0??cl`D000000P3clo<2000006/0lo?c003o0??clk40lo?c003o0??clk40lo?c0000
\
\>"],
  ImageRangeCache->{{{0, 431}, {265.938, 0}} -> {-0.253458, -1.68452, \
0.000651772, 0.0192114}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 432 266.991 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.865801 0.0147151 0.535094 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0147151 0.97619 0.603319 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.5 2.41966 1.05318 0.00666411 [
[.19376 -0.0125 -9.5 -15.6875 ]
[.19376 -0.0125 9.5 0 ]
[.43573 -0.0125 -9.5 -15.6875 ]
[.43573 -0.0125 9.5 0 ]
[.6777 -0.0125 -9.5 -15.6875 ]
[.6777 -0.0125 9.5 0 ]
[.91966 -0.0125 -5 -15.6875 ]
[.91966 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -31.25 -31.375 ]
[.5 -0.0125 31.25 -15.6875 ]
[-0.0125 .1202 -30.3125 -7.84375 ]
[-0.0125 .1202 0 7.84375 ]
[-0.0125 .25349 -30.3125 -7.84375 ]
[-0.0125 .25349 0 7.84375 ]
[-0.0125 .38677 -30.3125 -7.84375 ]
[-0.0125 .38677 0 7.84375 ]
[-0.0125 .52005 -24.3125 -7.84375 ]
[-0.0125 .52005 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -46 -16.6875 ]
[-0.01875 .30902 -30.3125 16.6875 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -106.438 0 ]
[.5 .63053 106.438 15.6875 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.19376 0 m
.19376 .00625 L
s
gsave
.19376 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.7) show
1.000 setlinewidth
grestore
.43573 0 m
.43573 .00625 L
s
gsave
.43573 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.6777 0 m
.6777 .00625 L
s
gsave
.6777 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.9) show
1.000 setlinewidth
grestore
.91966 0 m
.91966 .00625 L
s
gsave
.91966 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.24216 0 m
.24216 .00375 L
s
.29055 0 m
.29055 .00375 L
s
.33894 0 m
.33894 .00375 L
s
.38734 0 m
.38734 .00375 L
s
.48412 0 m
.48412 .00375 L
s
.53252 0 m
.53252 .00375 L
s
.58091 0 m
.58091 .00375 L
s
.6293 0 m
.6293 .00375 L
s
.72609 0 m
.72609 .00375 L
s
.77448 0 m
.77448 .00375 L
s
.82288 0 m
.82288 .00375 L
s
.87127 0 m
.87127 .00375 L
s
.14537 0 m
.14537 .00375 L
s
.09698 0 m
.09698 .00375 L
s
.04858 0 m
.04858 .00375 L
s
.00019 0 m
.00019 .00375 L
s
.96806 0 m
.96806 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -92.25 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Energy) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
100.000 15.813 moveto
(H) show
103.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
117.688 15.813 moveto
(L) show
121.500 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .1202 m
.00625 .1202 L
s
gsave
-0.0125 .1202 -91.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(140) show
1.000 setlinewidth
grestore
0 .25349 m
.00625 .25349 L
s
gsave
-0.0125 .25349 -91.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(120) show
1.000 setlinewidth
grestore
0 .38677 m
.00625 .38677 L
s
gsave
-0.0125 .38677 -91.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
0 .52005 m
.00625 .52005 L
s
gsave
-0.0125 .52005 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(80) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .15352 m
.00375 .15352 L
s
0 .18684 m
.00375 .18684 L
s
0 .22017 m
.00375 .22017 L
s
0 .28681 m
.00375 .28681 L
s
0 .32013 m
.00375 .32013 L
s
0 .35345 m
.00375 .35345 L
s
0 .42009 m
.00375 .42009 L
s
0 .45341 m
.00375 .45341 L
s
0 .48673 m
.00375 .48673 L
s
0 .08688 m
.00375 .08688 L
s
0 .05356 m
.00375 .05356 L
s
0 .02024 m
.00375 .02024 L
s
0 .55337 m
.00375 .55337 L
s
0 .58669 m
.00375 .58669 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -115.844 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
77.6875 11.8438 translate
90 rotate
-77.6875 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Re) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
76.250 15.813 moveto
(H) show
80.063 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(e) show
84.750 17.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 8.500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(r) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
88.563 15.813 moveto
(L) show
92.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.19376 .61178 m
.19376 .61803 L
s
.43573 .61178 m
.43573 .61803 L
s
.6777 .61178 m
.6777 .61803 L
s
.91966 .61178 m
.91966 .61803 L
s
.125 Mabswid
.24216 .61428 m
.24216 .61803 L
s
.29055 .61428 m
.29055 .61803 L
s
.33894 .61428 m
.33894 .61803 L
s
.38734 .61428 m
.38734 .61803 L
s
.48412 .61428 m
.48412 .61803 L
s
.53252 .61428 m
.53252 .61803 L
s
.58091 .61428 m
.58091 .61803 L
s
.6293 .61428 m
.6293 .61803 L
s
.72609 .61428 m
.72609 .61803 L
s
.77448 .61428 m
.77448 .61803 L
s
.82288 .61428 m
.82288 .61803 L
s
.87127 .61428 m
.87127 .61803 L
s
.14537 .61428 m
.14537 .61803 L
s
.09698 .61428 m
.09698 .61803 L
s
.04858 .61428 m
.04858 .61803 L
s
.00019 .61428 m
.00019 .61803 L
s
.96806 .61428 m
.96806 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -167.438 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Real) show
87.813 15.813 moveto
(and) show
108.063 15.813 moveto
(Imaginary) show
159.375 15.813 moveto
(Parts) show
186.250 15.813 moveto
(of) show
200.375 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(e) show
208.063 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
218.750 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Linear) show
252.563 15.813 moveto
(Plot) show
271.875 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .1202 m
1 .1202 L
s
.99375 .25349 m
1 .25349 L
s
.99375 .38677 m
1 .38677 L
s
.99375 .52005 m
1 .52005 L
s
.125 Mabswid
.99625 .15352 m
1 .15352 L
s
.99625 .18684 m
1 .18684 L
s
.99625 .22017 m
1 .22017 L
s
.99625 .28681 m
1 .28681 L
s
.99625 .32013 m
1 .32013 L
s
.99625 .35345 m
1 .35345 L
s
.99625 .42009 m
1 .42009 L
s
.99625 .45341 m
1 .45341 L
s
.99625 .48673 m
1 .48673 L
s
.99625 .08688 m
1 .08688 L
s
.99625 .05356 m
1 .05356 L
s
.99625 .02024 m
1 .02024 L
s
.99625 .55337 m
1 .55337 L
s
.99625 .58669 m
1 .58669 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 0 r
.005 w
.00287 0 m
.04057 .04889 L
.08481 .10195 L
.12636 .1481 L
.16632 .18942 L
.20885 .23041 L
.2498 .2672 L
.29331 .3037 L
.33524 .33655 L
.37557 .36618 L
.41848 .39576 L
.4598 .4225 L
.49953 .44672 L
.54183 .47104 L
.58254 .4931 L
.62583 .51523 L
.66752 .53535 L
.70762 .55367 L
.7503 .57214 L
.79139 .58899 L
.83505 .60595 L
s
.83505 .60595 m
.86784 .61803 L
s
.013 w
.91935 .54155 Mdot
.87154 .61803 Mdot
.82378 .60069 Mdot
.77445 .58189 Mdot
.66763 .56271 Mdot
.67707 .542 Mdot
.62917 .51966 Mdot
.58044 .49686 Mdot
.53252 .47111 Mdot
.48413 .44349 Mdot
.43548 .41393 Mdot
.38679 .37564 Mdot
.33936 .34866 Mdot
.29104 .3071 Mdot
.24216 .26436 Mdot
.19396 .22046 Mdot
.14564 .1754 Mdot
.09659 .1134 Mdot
.04879 .06563 Mdot
0 0 Mdot
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.41342 0.0949792 0.84632 0.362526 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.5 2.41966 0.875202 0.0408591 [
[.19376 -0.0125 -9.5 -15.6875 ]
[.19376 -0.0125 9.5 0 ]
[.43573 -0.0125 -9.5 -15.6875 ]
[.43573 -0.0125 9.5 0 ]
[.6777 -0.0125 -9.5 -15.6875 ]
[.6777 -0.0125 9.5 0 ]
[.91966 -0.0125 -5 -15.6875 ]
[.91966 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -31.25 -31.375 ]
[.5 -0.0125 31.25 -15.6875 ]
[-0.0125 .05802 -24.3125 -7.84375 ]
[-0.0125 .05802 0 7.84375 ]
[-0.0125 .13974 -24.3125 -7.84375 ]
[-0.0125 .13974 0 7.84375 ]
[-0.0125 .22146 -24.3125 -7.84375 ]
[-0.0125 .22146 0 7.84375 ]
[-0.0125 .30317 -24.3125 -7.84375 ]
[-0.0125 .30317 0 7.84375 ]
[-0.0125 .38489 -24.3125 -7.84375 ]
[-0.0125 .38489 0 7.84375 ]
[-0.0125 .46661 -24.3125 -7.84375 ]
[-0.0125 .46661 0 7.84375 ]
[-0.0125 .54833 -18.3125 -7.84375 ]
[-0.0125 .54833 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40 -16.5938 ]
[-0.01875 .30902 -24.3125 16.5938 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.19376 0 m
.19376 .00625 L
s
gsave
.19376 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.7) show
1.000 setlinewidth
grestore
.43573 0 m
.43573 .00625 L
s
gsave
.43573 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.6777 0 m
.6777 .00625 L
s
gsave
.6777 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.9) show
1.000 setlinewidth
grestore
.91966 0 m
.91966 .00625 L
s
gsave
.91966 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.24216 0 m
.24216 .00375 L
s
.29055 0 m
.29055 .00375 L
s
.33894 0 m
.33894 .00375 L
s
.38734 0 m
.38734 .00375 L
s
.48412 0 m
.48412 .00375 L
s
.53252 0 m
.53252 .00375 L
s
.58091 0 m
.58091 .00375 L
s
.6293 0 m
.6293 .00375 L
s
.72609 0 m
.72609 .00375 L
s
.77448 0 m
.77448 .00375 L
s
.82288 0 m
.82288 .00375 L
s
.87127 0 m
.87127 .00375 L
s
.14537 0 m
.14537 .00375 L
s
.09698 0 m
.09698 .00375 L
s
.04858 0 m
.04858 .00375 L
s
.00019 0 m
.00019 .00375 L
s
.96806 0 m
.96806 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -92.25 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Energy) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
100.000 15.813 moveto
(H) show
103.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
117.688 15.813 moveto
(L) show
121.500 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .05802 m
.00625 .05802 L
s
gsave
-0.0125 .05802 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .13974 m
.00625 .13974 L
s
gsave
-0.0125 .13974 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(18) show
1.000 setlinewidth
grestore
0 .22146 m
.00625 .22146 L
s
gsave
-0.0125 .22146 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(16) show
1.000 setlinewidth
grestore
0 .30317 m
.00625 .30317 L
s
gsave
-0.0125 .30317 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(14) show
1.000 setlinewidth
grestore
0 .38489 m
.00625 .38489 L
s
gsave
-0.0125 .38489 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(12) show
1.000 setlinewidth
grestore
0 .46661 m
.00625 .46661 L
s
gsave
-0.0125 .46661 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
0 .54833 m
.00625 .54833 L
s
gsave
-0.0125 .54833 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .07845 m
.00375 .07845 L
s
0 .09888 m
.00375 .09888 L
s
0 .11931 m
.00375 .11931 L
s
0 .16017 m
.00375 .16017 L
s
0 .1806 m
.00375 .1806 L
s
0 .20103 m
.00375 .20103 L
s
0 .24189 m
.00375 .24189 L
s
0 .26232 m
.00375 .26232 L
s
0 .28274 m
.00375 .28274 L
s
0 .3236 m
.00375 .3236 L
s
0 .34403 m
.00375 .34403 L
s
0 .36446 m
.00375 .36446 L
s
0 .40532 m
.00375 .40532 L
s
0 .42575 m
.00375 .42575 L
s
0 .44618 m
.00375 .44618 L
s
0 .48704 m
.00375 .48704 L
s
0 .50747 m
.00375 .50747 L
s
0 .5279 m
.00375 .5279 L
s
0 .03759 m
.00375 .03759 L
s
0 .01716 m
.00375 .01716 L
s
0 .56876 m
.00375 .56876 L
s
0 .58919 m
.00375 .58919 L
s
0 .60962 m
.00375 .60962 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -109.75 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
77.5938 11.8438 translate
90 rotate
-77.5938 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Im) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
76.063 15.813 moveto
(H) show
79.875 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(e) show
84.563 17.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 8.500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(r) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
88.375 15.813 moveto
(L) show
92.188 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.19376 .61178 m
.19376 .61803 L
s
.43573 .61178 m
.43573 .61803 L
s
.6777 .61178 m
.6777 .61803 L
s
.91966 .61178 m
.91966 .61803 L
s
.125 Mabswid
.24216 .61428 m
.24216 .61803 L
s
.29055 .61428 m
.29055 .61803 L
s
.33894 .61428 m
.33894 .61803 L
s
.38734 .61428 m
.38734 .61803 L
s
.48412 .61428 m
.48412 .61803 L
s
.53252 .61428 m
.53252 .61803 L
s
.58091 .61428 m
.58091 .61803 L
s
.6293 .61428 m
.6293 .61803 L
s
.72609 .61428 m
.72609 .61803 L
s
.77448 .61428 m
.77448 .61803 L
s
.82288 .61428 m
.82288 .61803 L
s
.87127 .61428 m
.87127 .61803 L
s
.14537 .61428 m
.14537 .61803 L
s
.09698 .61428 m
.09698 .61803 L
s
.04858 .61428 m
.04858 .61803 L
s
.00019 .61428 m
.00019 .61803 L
s
.96806 .61428 m
.96806 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .05802 m
1 .05802 L
s
.99375 .13974 m
1 .13974 L
s
.99375 .22146 m
1 .22146 L
s
.99375 .30317 m
1 .30317 L
s
.99375 .38489 m
1 .38489 L
s
.99375 .46661 m
1 .46661 L
s
.99375 .54833 m
1 .54833 L
s
.125 Mabswid
.99625 .07845 m
1 .07845 L
s
.99625 .09888 m
1 .09888 L
s
.99625 .11931 m
1 .11931 L
s
.99625 .16017 m
1 .16017 L
s
.99625 .1806 m
1 .1806 L
s
.99625 .20103 m
1 .20103 L
s
.99625 .24189 m
1 .24189 L
s
.99625 .26232 m
1 .26232 L
s
.99625 .28274 m
1 .28274 L
s
.99625 .3236 m
1 .3236 L
s
.99625 .34403 m
1 .34403 L
s
.99625 .36446 m
1 .36446 L
s
.99625 .40532 m
1 .40532 L
s
.99625 .42575 m
1 .42575 L
s
.99625 .44618 m
1 .44618 L
s
.99625 .48704 m
1 .48704 L
s
.99625 .50747 m
1 .50747 L
s
.99625 .5279 m
1 .5279 L
s
.99625 .03759 m
1 .03759 L
s
.99625 .01716 m
1 .01716 L
s
.99625 .56876 m
1 .56876 L
s
.99625 .58919 m
1 .58919 L
s
.99625 .60962 m
1 .60962 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.007 w
0 .00372 m
.04057 .06158 L
.08481 .11904 L
.12636 .16821 L
.16632 .21158 L
.20885 .25395 L
.2498 .2914 L
.29331 .32798 L
.33524 .36039 L
.37557 .38919 L
.41848 .41751 L
.4598 .44273 L
.49953 .46525 L
.54183 .48754 L
.58254 .50747 L
.62583 .52719 L
.66752 .54485 L
.70762 .56072 L
.7503 .5765 L
.79139 .5907 L
.83505 .6048 L
.87712 .61751 L
s
.87712 .61751 m
.87895 .61803 L
s
.02 w
.91935 .6086 Mdot
.87154 .61803 Mdot
.82378 .60351 Mdot
.77445 .5869 Mdot
.66763 .56915 Mdot
.67707 .54983 Mdot
.62917 .52887 Mdot
.58044 .50658 Mdot
.53252 .48129 Mdot
.48413 .45481 Mdot
.43548 .42708 Mdot
.38679 .39402 Mdot
.33936 .36261 Mdot
.29104 .32429 Mdot
.24216 .28287 Mdot
.19396 .23819 Mdot
.14564 .18918 Mdot
.09659 .12934 Mdot
.04879 .07065 Mdot
0 0 Mdot
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{432, 266.938},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3clo>a
0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a
0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00j`3clo<3000000030??cl`00000000000;l0lo?c
003Z0??cl`050000003clo<0lo?c0??cl`0000000P3clo<00`000000lo?c0??cl`080??cl`030000
003clo<0lo?c00/0lo?c00<000000??cl`3clo<0X`3clo<00>X0lo?c00D000000??cl`3clo<0lo?c
000000030??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c
0??cl`2R0??cl`00e03clo<6000000050??cl`00000000000000003clo<00P0000000`3clo<00000
000000020??cl`<000000P3clo<3000000@0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c
0??cl`02000000D0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`2Q0??cl`00e@3c
lo<00`000000lo?c0??cl`020??cl`030000003clo<000000080lo?c00<000000??cl`0000000P3c
lo<010000000lo?c0??cl`0000020??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`000000
1P3clo<00`000000lo?c000000020??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0
1@3clo<00`000000lo?c0??cl`2Q0??cl`00e@3clo<00`000000lo?c0??cl`040??cl`060000003c
lo<0lo?c0000003clo<000001@3clo<010000000lo?c0??cl`3clo<2000000@0lo?c00<000000??c
l`0000001P3clo<00`000000lo?c000000060??cl`030000003clo<0000000H0lo?c00<000000??c
l`3clo<0X@3clo<00=D0lo?c00@000000??cl`3clo<000000`3clo<01@000000lo?c0??cl`000000
lo?c00@000000P3clo<010000000lo?c0??cl`0000020??cl`040000003clo<0lo?c00000080lo?c
00<000000??cl`3clo<0103clo<00`000000lo?c00000003000000<0lo?c00<000000??cl`000000
1P3clo<00`000000lo?c0??cl`2Q0??cl`00e@3clo<4000000<0lo?c0P000000103clo<000000??c
l`0000020??cl`030000003clo<0lo?c0080000000D0lo?c0000003clo<0lo?c000000020??cl`05
0000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c000000020??cl`070000003clo<0lo?c
0??cl`000000lo?c000000060??cl`030000003clo<0lo?c0:40lo?c003E0??cl`040000003clo<0
lo?c00000080lo?c0P0000000`3clo<000000??cl`020??cl`8000000P3clo<2000000030??cl`00
0000lo?c00L0000000@0lo?c0000000000000000103clo<00`000000lo?c0??cl`02000000<0lo?c
00D000000??cl`3clo<0lo?c000000050??cl`030000003clo<0lo?c0:40lo?c003E0??cl`030000
003clo<0lo?c0280lo?c00<000000??cl`3clo<01@3clo<01@000000lo?c0??cl`3clo<0000000D0
lo?c00<000000??cl`3clo<0X@3clo<00=D0lo?c00D000000??cl`3clo<0lo?c0000000Q0??cl`03
0000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c000000040??cl`030000003clo<0lo?c
0:80lo?c003D0??cl`H000008P3clo<00`000000lo?c0??cl`020??cl`<0000000@0lo?c00000000
000000000P3clo<00`000000lo?c0??cl`2S0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3c
lo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00o`3clo>a0??cl`00R03clo<3000000<0lo?c
00D000000??cl`3clo<0lo?c000000130??cl`<000000`3clo<00`000000lo?c0??cl`0300000440
lo?c0`0000030??cl`040000003clo<00000000004P0lo?c0`00000h0??cl`00Q`3clo<01@000000
lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`3clo<0@03clo<01@000000lo?c0??cl`3clo<0
000000@0lo?c00D000000??cl`3clo<0lo?c0000000o0??cl`050000003clo<0lo?c0??cl`000000
1P3clo<00`000000lo?c0??cl`160??cl`030000003clo<0lo?c03L0lo?c00270??cl`050000003c
lo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`0o0??cl`050000003clo<0lo?c0??cl`00
0000103clo<01@000000lo?c0??cl`3clo<0000003l0lo?c00D000000??cl`3clo<0lo?c00000007
0??cl`030000003clo<0lo?c04D0lo?c00<000000??cl`3clo<0=`3clo<008L0lo?c00D000000??c
l`3clo<0lo?c000000070??cl`030000003clo<0lo?c03l0lo?c00D000000??cl`3clo<0lo?c0000
00050??cl`030000003clo<000000400lo?c00D000000??cl`3clo<0lo?c000000050??cl`@00000
AP3clo<00`000000lo?c0??cl`0g0??cl`00Q`3clo<01@000000lo?c0??cl`3clo<0000000L0lo?c
00<000000??cl`3clo<0?`3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`3c
lo<0?`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c00000016
0??cl`030000003clo<0lo?c03L0lo?c00270??cl`050000003clo<0lo?c0??cl`0000001`3clo<0
0`000000lo?c0??cl`0o0??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0000
00100??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<0000004H0
lo?c00<000000??cl`3clo<0=`3clo<008L0lo?c00D000000??cl`3clo<0lo?c000000080??cl`03
0000003clo<0lo?c03h0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c
0??cl`000000?`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c
000000160??cl`030000003clo<0lo?c03L0lo?c00270??cl`050000003clo<0lo?c0??cl`000000
103clo<01@000000lo?c0??cl`3clo<000000400lo?c00D000000??cl`3clo<0lo?c000000040??c
l`050000003clo<0lo?c0??cl`000000?`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D0
00000??cl`3clo<0lo?c000000150??cl`800000>@3clo<008P0lo?c0`0000060??cl`@00000@@3c
lo<3000000H0lo?c0`0000110??cl`<000001P3clo<3000004L0lo?c00<000000??cl`3clo<0=`3c
lo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3c
lo<00?l0lo?c/@3clo<00540lo?co`000012000001h0lo?c001A0??cl`<00000303clo<00`000000
lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<00`000000
lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<00`000000
lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<00`000000
lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<010000000
lo?c0??cl`3clo<3000000030??cl`000000000000H0lo?c00<000000??cl`3clo<0303clo<00`00
0000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<00`00
0000lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<01`3clo<00`00
0000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0000000<0??cl`030000003clo<0lo?c00d0
lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0
lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0
lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0
lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0<0??cl`040000003clo<0lo?c0000
00<0lo?c00@000000??cl`3clo<000001@3clo<00`000000lo?c0??cl`020??cl`030000003clo<0
lo?c00L0lo?c00D000000??cl`3clo<0lo?c0000000;0??cl`030000003clo<0lo?c00`0lo?c00<0
00000??cl`3clo<03@3clo<00`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0lo?c00<0
00000??cl`3clo<01`3clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<010000000lo?c0??cl`00
00380??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`060??cl`030000
003clo<0lo?c00d0lo?c00<000000??cl`3clo<0D03clo<00`000000lo?c0??cl`0L0??cl`00D@3c
lo<01@000000lo?c0??cl`3clo<000000;40lo?c1P0000001@3clo<00000000000000000lo?c0080
000000<0lo?c0000000000000P3clo<300000080lo?c0`0000040??cl`030000003clo<0lo?c00D0
lo?c00@000000??cl`3clo<0lo?c0P0000050??cl`030000003clo<0lo?c00@0lo?c00<000000??c
l`3clo<0C`3clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`020??cl`03
0000003clo<0lo?c0:l0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c000000020??cl`03
0000003clo<000000080lo?c00@000000??cl`3clo<000000P3clo<00`000000lo?c0??cl`040??c
l`030000003clo<0000000H0lo?c00@000000??cl`3clo<000000P3clo<00`000000lo?c0??cl`02
0??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0C`3clo<00`000000lo?c0??cl`0L
0??cl`00D@3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c0:l0lo?c00<000000??c
l`3clo<0103clo<01P000000lo?c0??cl`000000lo?c000000D0lo?c00@000000??cl`3clo<0lo?c
0P0000040??cl`030000003clo<0000000H0lo?c00@000000??cl`3clo<000001P3clo<00`000000
lo?c000000050??cl`030000003clo<0lo?c04l0lo?c00<000000??cl`3clo<0703clo<00540lo?c
0P0000040??cl`030000003clo<0lo?c0:h0lo?c00@000000??cl`3clo<000000`3clo<01@000000
lo?c0??cl`000000lo?c00@000000P3clo<010000000lo?c0??cl`0000020??cl`040000003clo<0
lo?c00000080lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`04000000<0lo?c00<0
00000??cl`0000001@3clo<00`000000lo?c0??cl`1>0??cl`8000007P3clo<00540lo?c00<00000
0??cl`3clo<0103clo<00`000000lo?c0??cl`2]0??cl`@000000`3clo<2000000040??cl`000000
lo?c00000080lo?c00<000000??cl`3clo<00P0000001@3clo<000000??cl`3clo<000000080lo?c
00D000000??cl`3clo<0lo?c000000050??cl`040000003clo<0lo?c00000080lo?c00L000000??c
l`3clo<0lo?c0000003clo<0000000D0lo?c00<000000??cl`3clo<0C`3clo<00`000000lo?c0??c
l`0L0??cl`00D@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0:`0lo?c00@00000
0??cl`3clo<000000P3clo<2000000030??cl`000000lo?c0080lo?c0P0000020??cl`80000000<0
lo?c0000003clo<01`000000103clo<000000000000000040??cl`040000003clo<0lo?c0??cl`80
00000`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00<000000??cl`3clo<0C`3clo<00`00
0000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0:`0
lo?c00<000000??cl`3clo<08P3clo<00`000000lo?c0??cl`060??cl`050000003clo<0lo?c0??c
l`000000103clo<00`000000lo?c0??cl`1?0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`03
0000003clo<0lo?c00H0lo?c00<000000??cl`3clo<0Z`3clo<01@000000lo?c0??cl`3clo<00000
0240lo?c00<000000??cl`3clo<01@3clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<00000
0??cl`3clo<0D03clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`070??c
l`030000003clo<0lo?c0:T0lo?c1P00000R0??cl`030000003clo<0lo?c00<0lo?c0`0000001P3c
lo<00000000000000000lo?c000005<0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<00000
0??cl`3clo<0203clo<00`000000lo?c0??cl`3o0??clc<0lo?c00<000000??cl`3clo<0703clo<0
0540lo?c00<000000??cl`3clo<0203clo<00`000000lo?c0??cl`3o0??clc<0lo?c00<000000??c
l`3clo<0703clo<00540lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`3o0??clc80
lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c
0??cl`3o0??clc40lo?c00<000000??cl`3clo<0703clo<00540lo?c0P00000<0??cl`030000003c
lo<0lo?c0?l0lo?c;`3clo<2000001h0lo?c001A0??cl`030000003clo<0lo?c00`0lo?c00<00000
0??cl`3clo<0o`3clo<_0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c
00`0lo?c00<000000??cl`3clo<0o`3clo<_0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`03
0000003clo<0lo?c00`0lo?c0P00002C0??cl`<000000`3clo<01@000000lo?c0??cl`3clo<00000
01D0lo?c0`0000030??cl`030000003clo<0lo?c00<00000503clo<3000000<0lo?c00@000000??c
l`00000000006`3clo<3000003@0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??c
l`3clo<02`3clo<400000940lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000003clo<0
lo?c0180lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`000000
4P3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`3clo<06@3clo<00`000000
lo?c0??cl`0c0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c00/0lo?c
1@00002@0??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`0A0??cl`05
0000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<000000180lo?c00D00000
0??cl`3clo<0lo?c000000070??cl`030000003clo<0lo?c01P0lo?c00<000000??cl`3clo<0<`3c
lo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`0<0??cl`8000000P3clo<0
0`000000lo?c0??cl`2=0??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??c
l`0A0??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0000000C0??cl`050000
003clo<0lo?c0??cl`0000001@3clo<4000001T0lo?c00<000000??cl`3clo<0<`3clo<00`000000
lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`0A0??cl`030000003clo<0lo?c08`0lo?c
00D000000??cl`3clo<0lo?c000000070??cl`030000003clo<0lo?c0140lo?c00D000000??cl`3c
lo<0lo?c000000060??cl`030000003clo<0lo?c0180lo?c00D000000??cl`3clo<0lo?c00000004
0??cl`050000003clo<0lo?c0??cl`0000006@3clo<00`000000lo?c0??cl`0c0??cl`030000003c
lo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c0140lo?c00<000000??cl`3clo<0S03clo<0
1@000000lo?c0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<04@3clo<01@000000lo?c0??c
l`3clo<0000000D0lo?c00<000000??cl`0000004`3clo<01@000000lo?c0??cl`3clo<0000000@0
lo?c00D000000??cl`3clo<0lo?c0000000I0??cl`030000003clo<0lo?c03<0lo?c00<000000??c
l`3clo<0703clo<00540lo?c00<000000??cl`3clo<04P3clo<00`000000lo?c0??cl`2;0??cl`05
0000003clo<0lo?c0??cl`000000203clo<00`000000lo?c0??cl`0@0??cl`050000003clo<0lo?c
0??cl`000000103clo<01@000000lo?c0??cl`3clo<000000180lo?c00D000000??cl`3clo<0lo?c
000000040??cl`050000003clo<0lo?c0??cl`0000006@3clo<00`000000lo?c0??cl`0c0??cl`03
0000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c01<0lo?c00<000000??cl`3clo<0
RP3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c0000000B0??c
l`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<000000180lo?c00D0
00000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`000000603clo<2000003D0
lo?c00<000000??cl`3clo<0703clo<00540lo?c0P00000E0??cl`030000003clo<0lo?c08X0lo?c
0`0000060??cl`@000004`3clo<3000000H0lo?c0`00000D0??cl`<000001P3clo<3000001X0lo?c
00<000000??cl`3clo<0<P3clo<2000001h0lo?c001A0??cl`030000003clo<0lo?c01D0lo?c00<0
00000??cl`3clo<0o`3clo<V0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0
lo?c01H0lo?c00<000000??cl`3clo<0o`3clo<U0??cl`030000003clo<0lo?c01`0lo?c001A0??c
l`030000003clo<0lo?c01H0lo?c00<000000??cl`3clo<0o`3clo<U0??cl`030000003clo<0lo?c
01`0lo?c001A0??cl`030000003clo<0lo?c01L0lo?c00<000000??cl`3clo<0M03clo>5000002X0
lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0603clo<00`000000lo?c
0??cl`1c0??cl`030?l0003clo<0lo?c00<0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c
0??cl`040??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c
0??cl`030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c
0??cl`030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c
0??cl`040??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c
0??cl`030??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c
0??cl`030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<00`3clo<01@000000lo?c
0??cl`3clo<0000002X0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0
6@3clo<00`000000lo?c0??cl`1b0??cl`030000003o0000lo?c0840lo?c00<000000??cl`3clo<0
:03clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`0J0??cl`<00000L@3c
lo<00`000000o`000??cl`210??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703c
lo<003X0lo?c0`0000060??cl`030000003clo<0lo?c00<00000203clo<00`000000lo?c0??cl`0J
0??cl`@00000L03clo<00`000000lo?c0?l000210??cl`030000003clo<0lo?c02P0lo?c00<00000
0??cl`3clo<0703clo<003/0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c000000030??c
l`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<06P3clo<400000600lo?c1@0000020??c
l`<000001P3clo<00`000000lo?c0?l000210??cl`030000003clo<0lo?c02P0lo?c00<000000??c
l`3clo<0703clo<003/0lo?c00@000000??cl`3clo<0lo?c1`0000030??cl`030000003clo<0lo?c
00D0lo?c00<000000??cl`3clo<06`3clo<200000680lo?c00H000000??cl`3clo<000000??cl`00
00030??cl`030000003clo<0lo?c00<0lo?c00@000000??cl`3clo<0o`00P03clo<00`000000lo?c
0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000a0??cl`L000000`3clo<01@000000lo?c0??c
l`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`<00000
7@3clo<00`000000lo?c0??cl`1P0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000
lo?c0??cl`030??cl`050000003clo<0lo?c0??cl`3o0000O`3clo<00`000000lo?c0??cl`0V0??c
l`<000007P3clo<003/0lo?c00<000000??cl`3clo<00P3clo<01P000000lo?c0??cl`000000lo?c
000000<0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0N0??cl`030000003clo<0
lo?c05H0lo?c1`0000030??cl`040000003clo<0lo?c000000<0lo?c00<000000??cl`3clo<00`3c
lo<01@000000lo?c0??cl`3clo<0o`0007l0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c
0??cl`0L0??cl`00>`3clo<00`000000lo?c0??cl`030??cl`050000003clo<000000??cl`000000
0`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c01l0lo?c00<000000??cl`3clo<0
H03clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0
0P3clo<00`3o0000lo?c0??cl`1l0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0
703clo<003/0lo?c00<000000??cl`3clo<00`3clo<01@000000lo?c0000003clo<0000000<0lo?c
00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0P0??cl`030000003clo<0lo?c05l0lo?c
00<000000??cl`0000000`3clo<00`000000lo?c0??cl`030??cl`030000003clo<0lo?c0080lo?c
0P3o001m0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<003X0lo?c0P00
00060??cl`80000000<0lo?c0000003clo<00P3clo<00`000000lo?c0??cl`050??cl`030000003c
lo<0lo?c0240lo?c00<000000??cl`3clo<0GP3clo<00`000000lo?c000000030??cl`030000003c
lo<0lo?c00<0lo?c00<000000??cl`3clo<00P3clo<00`3o0000lo?c0?l0001l0??cl`030000003c
lo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<003/0lo?c00<000000??cl`3clo<01@3clo<0
0`000000lo?c0??cl`03000000P0lo?c00<000000??cl`3clo<08P3clo<00`000000lo?c0??cl`1I
0??cl`070000003clo<0lo?c0??cl`000000lo?c000000030??cl`030000003clo<0lo?c00<0lo?c
00<000000??cl`3clo<01@3clo<00`3o0000lo?c0??cl`1i0??cl`030000003clo<0lo?c02P0lo?c
00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<08P3clo<00`000000lo?c0??c
l`1J0??cl`<000000`3clo<3000000H0lo?c00<000000??cl`3clo<01@3clo<00`3o0000lo?c0??c
l`1i0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<00000
0??cl`3clo<08`3clo<00`000000lo?c0??cl`1X0??cl`030000003clo<0lo?c00H0lo?c00<0o`00
0??cl`3clo<0N03clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c001A0??c
l`030000003clo<0lo?c02@0lo?c00<000000??cl`3clo<0I`3clo<00`000000lo?c0??cl`070??c
l`030?l0003clo<0lo?c07L0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??c
l`00D@3clo<00`000000lo?c0??cl`0U0??cl`030000003clo<0lo?c05L0lo?c0`0000030??cl`<0
00001P3clo<00`000000lo?c0??cl`080??cl`030?l0003clo<0lo?c07H0lo?c00<000000??cl`3c
lo<0:03clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`0V0??cl`030000
003clo<0lo?c05L0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00<0
lo?c00<000000??cl`3clo<02@3clo<00`3o0000lo?c0??cl`1e0??cl`030000003clo<0lo?c02P0
lo?c00<000000??cl`3clo<0703clo<00540lo?c0P00000X0??cl`030000003clo<0lo?c05H0lo?c
00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3c
lo<02@3clo<00`3o0000lo?c0??cl`1e0??cl`030000003clo<0lo?c02L0lo?c0P00000N0??cl`00
D@3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c04`0lo?c1`0000020??cl`030000
003clo<0lo?c0080lo?c00<000000??cl`0000001P3clo<00`000000lo?c0??cl`0:0??cl`030?l0
003clo<0lo?c07@0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`00D@3c
lo<00`000000lo?c0??cl`0Y0??cl`030000003clo<0lo?c05@0lo?c00<000000??cl`3clo<00`3c
lo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c00/0lo?c00<0o`000??cl`3clo<0L`3c
lo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0
lo?c02X0lo?c00<000000??cl`3clo<0D`3clo<00`000000lo?c0??cl`020??cl`030000003clo<0
000000H0lo?c00<000000??cl`3clo<0303clo<00`3o0000lo?c0??cl`1b0??cl`030000003clo<0
lo?c02P0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0:`3clo<00`00
0000lo?c0??cl`1B0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`03
0??cl`030000003clo<0lo?c00`0lo?c00<0o`000??cl`3clo<0LP3clo<00`000000lo?c0??cl`0X
0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c02/0lo?c0P00001B0??c
l`8000000`3clo<01@000000lo?c0??cl`3clo<0000000D0lo?c00<000000??cl`3clo<03@3clo<0
0`3o0000lo?c0??cl`1a0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<0
0540lo?c00<000000??cl`3clo<0:P3clo<400000580lo?c00<000000??cl`3clo<00P3clo<30000
00H0lo?c00<000000??cl`3clo<03P3clo<00`3o0000lo?c0??cl`1`0??cl`030000003clo<0lo?c
02P0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0:P3clo<5000005l0
lo?c00<000000??cl`3clo<03`3clo<00`3o0000lo?c0??cl`1_0??cl`030000003clo<0lo?c02P0
lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0:`3clo<200000080lo?c
00<000000??cl`3clo<0G03clo<00`000000lo?c0??cl`0?0??cl`80o`00L03clo<00`000000lo?c
0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c0300lo?c00<0
00000??cl`3clo<0C03clo<3000000<0lo?c0`0000060??cl`030000003clo<0lo?c0140lo?c00<0
o`000??cl`3clo<0K@3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c001A
0??cl`800000<P3clo<00`000000lo?c0??cl`0b0??cl`030000003clo<0lo?c00D0lo?c00<00000
0??cl`3clo<03`3clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<00`3c
lo<00`000000lo?c0??cl`0B0??cl`030?l0003clo<0lo?c06`0lo?c00<000000??cl`3clo<09`3c
lo<2000001h0lo?c001A0??cl`030000003clo<0lo?c0380lo?c00<000000??cl`3clo<0<@3clo<9
00000140lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00<0lo?c00<0
00000??cl`3clo<04`3clo<00`3o0000lo?c0??cl`1[0??cl`030000003clo<0lo?c02P0lo?c00<0
00000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0<`3clo<00`000000lo?c0??cl`0`
0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<01P3clo<700000080lo?c00D00000
0??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0503c
lo<00`3o0000lo?c0??cl`1Z0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703c
lo<00540lo?c00<000000??cl`3clo<0=03clo<00`000000lo?c0??cl`190??cl`040000003clo<0
lo?c0??cl`8000000P3clo<00`000000lo?c0??cl`030??cl`030000003clo<0lo?c01D0lo?c00<0
o`000??cl`3clo<0J@3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c001A
0??cl`030000003clo<0lo?c03D0lo?c00<000000??cl`3clo<0<@3clo<00`000000lo?c0??cl`02
0??cl`030000003clo<0lo?c00l0lo?c00H000000??cl`3clo<0lo?c0000003clo<2000000H0lo?c
00<000000??cl`3clo<05P3clo<00`3o0000lo?c0??cl`1X0??cl`030000003clo<0lo?c02P0lo?c
00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0=P3clo<00`000000lo?c0??c
l`0`0??cl`H000004@3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c00H0lo?c00<0
00000??cl`3clo<05P3clo<20?l006T0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??c
l`0L0??cl`00D@3clo<00`000000lo?c0??cl`0g0??cl`030000003clo<0lo?c02l0lo?c00<00000
0??cl`3clo<00P3clo<00`000000lo?c0??cl`0>0??cl`8000001@3clo<00`000000lo?c0??cl`05
0??cl`030000003clo<0lo?c01P0lo?c00<0o`000??cl`3clo<0IP3clo<00`000000lo?c0??cl`0X
0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c03P0lo?c00<000000??c
l`3clo<0;P3clo<00`000000lo?c0??cl`0D0??cl`030000003clo<0lo?c00@0lo?c0P0000050??c
l`030000003clo<0lo?c01T0lo?c00<0o`000??cl`3clo<0I@3clo<00`000000lo?c0??cl`0X0??c
l`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c03T0lo?c0P00000_0??cl`D0
00007`3clo<00`000000lo?c0??cl`0J0??cl`030?l0003clo<0lo?c06@0lo?c00<000000??cl`3c
lo<0:03clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`0k0??cl`030000
003clo<0lo?c02/0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0M0??cl`030000
003clo<0lo?c01/0lo?c0P3o001T0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0
703clo<00540lo?c00<000000??cl`3clo<0>P3clo<3000002`0lo?c00<000000??cl`3clo<04`3c
lo<3000000H0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`0L0??cl`80o`00H`3c
lo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`800000>P3clo<5
000002`0lo?c1@00000A0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<00`3clo<0
0`000000lo?c0??cl`0L0??cl`030?l0003clo<0o`000680lo?c00<000000??cl`3clo<09`3clo<2
000001h0lo?c001A0??cl`030000003clo<0lo?c03T0lo?c100000000`3clo<000000??cl`0^0??c
l`030000003clo<0lo?c00l0lo?c00@000000??cl`3clo<0lo?c1P0000040??cl`030000003clo<0
lo?c01l0lo?c00<0o`000??cl`3clo<0G`3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0
lo?c01`0lo?c001A0??cl`030000003clo<0lo?c03X0lo?c0P0000030??cl`030000003clo<0lo?c
03D0lo?c1`0000020??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`03
0??cl`030000003clo<0lo?c0200lo?c00<0o`000??cl`3clo<0GP3clo<00`000000lo?c0??cl`0X
0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c0400lo?c00<000000??c
l`3clo<09@3clo<700000140lo?c00<000000??cl`3clo<00P3clo<010000000lo?c0??cl`000005
0??cl`030000003clo<0lo?c0240lo?c0P3o001N0??cl`030000003clo<0lo?c02P0lo?c00<00000
0??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0@@3clo<00`000000lo?c0??cl`0S0??c
l`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<03P3clo<00`000000lo?c0??cl`030??c
l`030000003clo<0000000D0lo?c00<000000??cl`3clo<08P3clo<20?l005d0lo?c00<000000??c
l`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`120??cl`03
0000003clo<0lo?c0240lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c00<0lo?c00<000000??cl`0000001@3clo<00`000000lo?c0??cl`0R0??cl`03
0?l0003clo<0o`0005`0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`00
D@3clo<00`000000lo?c0??cl`130??cl`030000003clo<0lo?c02D0lo?c0`00000A0??cl`800000
1P3clo<2000000D0lo?c00<000000??cl`3clo<09@3clo<00`3o0000lo?c0??cl`1I0??cl`030000
003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0A03c
lo<00`000000lo?c0??cl`0S0??cl`8000000P3clo<00`000000lo?c0??cl`0?0??cl`030000003c
lo<0lo?c00D0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`0V0??cl`80o`00F@3c
lo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000j0??cl`<000000P3clo<5
00000080lo?c0`0000080??cl`030000003clo<0lo?c04D0lo?c00<000000??cl`3clo<08@3clo<0
0`000000lo?c000000020??cl`030000003clo<0lo?c01d0lo?c00<000000??cl`3clo<0:03clo<0
0`3o0000lo?c0??cl`1F0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<0
03/0lo?c00<000000??cl`3clo<00P3clo<01P000000lo?c0??cl`000000lo?c000000<0lo?c00<0
00000??cl`3clo<01@3clo<00`000000lo?c0??cl`160??cl`8000008@3clo<00`000000lo?c0000
00020??cl`030000003clo<0lo?c01d0lo?c00<000000??cl`3clo<0:@3clo<00`3o0000lo?c0??c
l`1E0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<003/0lo?c00<00000
0??cl`3clo<00`3clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<01@3c
lo<00`000000lo?c0??cl`180??cl`030000003clo<0lo?c01h0lo?c00@000000??cl`3clo<00000
0P3clo<2000000h0lo?c0`0000020??cl`D000001@3clo<00`000000lo?c0??cl`0Y0??cl`<0o`00
E@3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000a0??cl`L000000`3c
lo<00`000000lo?c0??cl`040??cl`040000003clo<0lo?c000000<0lo?c00<000000??cl`3clo<0
1@3clo<3000004T0lo?c00<000000??cl`3clo<0803clo<300000140lo?c00<000000??cl`3clo<0
0P3clo<010000000lo?c0??cl`0000050??cl`030000003clo<0lo?c02`0lo?c00<0o`000??cl`3c
lo<0DP3clo<00`000000lo?c0??cl`0V0??cl`<000007P3clo<003/0lo?c00<000000??cl`3clo<0
1@3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0
BP3clo<00`000000lo?c0??cl`0P0??cl`030000003clo<0lo?c0100lo?c00<000000??cl`3clo<0
0`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c02d0lo?c0P3o001B0??cl`030000
003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<003/0lo?c00<000000??cl`3clo<01@3c
lo<00`000000lo?c000000030??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0BP3c
lo<200000200lo?c00<000000??cl`3clo<0203clo<700000080lo?c00<000000??cl`3clo<0103c
lo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c02l0lo?c00<0o`000??cl`3clo<0C`3c
lo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000k0??cl`030000003clo<0
lo?c00D0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0
lo?c04T0lo?c1000000I0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<03@3clo<0
0`000000lo?c0??cl`050??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0;`3clo<2
0?l00500lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`007`3clo<00`00
0000lo?c0??cl`050??cl`030000003clo<0lo?c0100lo?c0P0000030??cl`070000003clo<0lo?c
0??cl`000000lo?c000000030??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0B@3c
lo<5000001T0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0>0??cl`030000003c
lo<0lo?c00D0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`0_0??cl`040?l0003c
lo<0o`000?l004h0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`007`3c
lo<9000001<0lo?c00<000000??cl`3clo<00P3clo<3000000<0lo?c0`0000080??cl`030000003c
lo<0lo?c04X0lo?c0P0000020??cl`030000003clo<0lo?c01L0lo?c1`00000A0??cl`030000003c
lo<0lo?c00D0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`0c0??cl`030?l0003c
lo<0lo?c04/0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`007`3clo<0
1@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<09`3clo<00`000000lo?c0??c
l`1?0??cl`030000003clo<0lo?c02d0lo?c0P0000030??cl`050000003clo<0lo?c0??cl`000000
1@3clo<00`000000lo?c0??cl`0d0??cl`030?l0003clo<0lo?c04X0lo?c00<000000??cl`3clo<0
:03clo<00`000000lo?c0??cl`0L0??cl`007`3clo<01@000000lo?c0??cl`3clo<0000002d0lo?c
00<000000??cl`3clo<0D03clo<2000002h0lo?c00<000000??cl`3clo<00P3clo<3000000H0lo?c
00<000000??cl`3clo<0=@3clo<20?l004X0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c
0??cl`0L0??cl`007`3clo<010000000lo?c0??cl`3clo<3000002/0lo?c00<000000??cl`3clo<0
DP3clo<00`000000lo?c0??cl`0i0??cl`030000003clo<0lo?c03H0lo?c0`3o00180??cl`030000
003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<00200lo?c0`0000030??cl`030000003c
lo<0lo?c02P0lo?c00<000000??cl`3clo<0D`3clo<00`000000lo?c0??cl`0Y0??cl`<000000`3c
lo<3000000H0lo?c00<000000??cl`3clo<0>@3clo<20?l004H0lo?c00<000000??cl`3clo<0:03c
lo<00`000000lo?c0??cl`0L0??cl`009`3clo<00`000000lo?c0??cl`0W0??cl`800000E@3clo<0
0`000000lo?c0??cl`0Y0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??c
l`030??cl`030000003clo<0lo?c03/0lo?c0P3o00140??cl`030000003clo<0lo?c02L0lo?c0P00
000N0??cl`009`3clo<00`000000lo?c0??cl`0W0??cl`030000003clo<0lo?c05D0lo?c00<00000
0??cl`3clo<0:03clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<00`3c
lo<00`000000lo?c0??cl`0l0??cl`<0o`00@P3clo<00`000000lo?c0??cl`0X0??cl`030000003c
lo<0lo?c01`0lo?c000S0??cl`@00000:P3clo<00`000000lo?c0??cl`1F0??cl`030000003clo<0
lo?c01h0lo?c1`0000020??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??c
l`030??cl`030000003clo<0lo?c03`0lo?c00@0o`000??cl`3clo<0o`00@@3clo<00`000000lo?c
0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000R0??cl`030000003clo<000000080lo?c00<0
00000??cl`3clo<09`3clo<00`000000lo?c0??cl`1G0??cl`8000009`3clo<01@000000lo?c0??c
l`3clo<0000000<0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`100??cl`80o`00
?`3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000R0??cl`030000003c
lo<000000080lo?c00<000000??cl`3clo<09`3clo<00`000000lo?c0??cl`1I0??cl`030000003c
lo<0lo?c02@0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00<0lo?c
00<000000??cl`3clo<0@P3clo<20?l003d0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c
0??cl`0L0??cl`008`3clo<2000000030??cl`000000lo?c02T0lo?c00<000000??cl`3clo<0F@3c
lo<2000002D0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00<0lo?c
00<000000??cl`3clo<0@`3clo<30?l003/0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c
0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`1H0??cl`@000008`3clo<2000000<0lo?c00D0
00000??cl`3clo<0lo?c000000050??cl`030000003clo<0lo?c04H0lo?c0P3o000i0??cl`030000
003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0F03c
lo<5000002<0lo?c00<000000??cl`3clo<00P3clo<3000000H0lo?c00<000000??cl`3clo<0B03c
lo<20?l003L0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`008@3clo<7
000002T0lo?c00<000000??cl`3clo<0F@3clo<200000080lo?c0P00000_0??cl`030000003clo<0
lo?c04T0lo?c0`3o000e0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<0
0200lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0V0??cl`030000003clo<0lo?c
05l0lo?c00<000000??cl`3clo<0;03clo<00`000000lo?c0??cl`190??cl`030?l0003clo<0lo?c
00<0o`00<P3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000O0??cl`03
0000003clo<0lo?c00L0lo?c00<000000??cl`3clo<09@3clo<00`000000lo?c0??cl`1P0??cl`03
0000003clo<0lo?c0280lo?c0`0000060??cl`030000003clo<0lo?c04l0lo?c0P3o000`0??cl`03
0000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<002@0lo?c0`00000Z0??cl`800000
HP3clo<200000240lo?c00D000000??cl`3clo<0lo?c000000050??cl`030000003clo<0lo?c04l0
lo?c00@0o`000??cl`3o0000o`00;P3clo<00`000000lo?c0??cl`0W0??cl`8000007P3clo<002<0
lo?c0P0000020??cl`030000003clo<0lo?c02L0lo?c00<000000??cl`3clo<0H`3clo<00`000000
lo?c0??cl`0N0??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0??cl`1?0??c
l`040?l0003clo<0lo?c0??cl`80o`00;03clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0
lo?c01`0lo?c000R0??cl`030000003clo<000000080lo?c00<000000??cl`3clo<09`3clo<00`00
0000lo?c0??cl`1T0??cl`030000003clo<0lo?c01H0lo?c1`000000103clo<000000??cl`000006
0??cl`030000003clo<0lo?c05D0lo?c0P3o000Z0??cl`030000003clo<0lo?c02P0lo?c00<00000
0??cl`3clo<0703clo<00280lo?c00<000000??cl`0000000P3clo<00`000000lo?c0??cl`0W0??c
l`030000003clo<0lo?c06D0lo?c0P00000O0??cl`030000003clo<0lo?c00D0lo?c00<000000??c
l`3clo<0EP3clo<40?l002L0lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??c
l`008P3clo<010000000lo?c0??cl`0000020??cl`8000009`3clo<00`000000lo?c0??cl`1W0??c
l`030000003clo<0lo?c01/0lo?c00<000000??cl`0000001P3clo<00`000000lo?c0??cl`1J0??c
l`<0o`00903clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000U0??cl`<0
0000:@3clo<00`000000lo?c0??cl`1X0??cl`030000003clo<0lo?c01T0lo?c00D000000??cl`3c
lo<0lo?c000000050??cl`030000003clo<0lo?c05@0lo?c00<0o`000??cl`3clo<01P3clo<20?l0
0280lo?c00<000000??cl`3clo<0:03clo<00`000000lo?c0??cl`0L0??cl`009P3clo<00`000000
lo?c0??cl`0X0??cl`030000003clo<0lo?c06T0lo?c00<000000??cl`3clo<0603clo<01@000000
lo?c0??cl`3clo<0000000D0lo?c00<000000??cl`3clo<0E03clo<00`3o0000lo?c0??cl`080??c
l`<0o`007`3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01`0lo?c000U0??cl`03
0000003clo<0lo?c02T0lo?c00<000000??cl`3clo<0JP3clo<2000001T0lo?c0`0000060??cl`03
0000003clo<0lo?c0680lo?c0`3o000L0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3c
lo<0703clo<001l0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`0U0??cl`030000
003clo<0lo?c06T0lo?c0P0000000`3clo<000000??cl`0P0??cl`030000003clo<0lo?c06<0lo?c
00<0o`000??cl`3o00000P3o000I0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0
703clo<00200lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0V0??cl`030000003c
lo<0lo?c06P0lo?c100000000`3clo<000000??cl`0O0??cl`030000003clo<0lo?c06P0lo?c0`3o
000F0??cl`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0703clo<00240lo?c1`00000Y
0??cl`030000003clo<0lo?c06P0lo?c100000020??cl`8000007P3clo<00`000000lo?c0??cl`1Y
0??cl`030?l0003clo<0o`0000<0o`001`3clo<00`3o0000lo?c0??cl`080??cl`030000003clo<0
lo?c02P0lo?c00<000000??cl`3clo<0703clo<00540lo?c0P00001Z0??cl`8000001@3clo<00`00
0000lo?c0??cl`0K0??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<00`3clo<00`00
0000lo?c0??cl`040??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0103clo<00`00
0000lo?c0??cl`030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<00`3clo<00`00
0000lo?c0??cl`030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<00`3clo<00`00
0000lo?c0??cl`040??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0103clo<00`00
0000lo?c0??cl`030??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0103clo<00`00
0000lo?c0??cl`020??cl`<0o`00103clo<00`3o000000000??cl`040??cl`050000003clo<0lo?c
0??cl`000000:@3clo<2000001h0lo?c001A0??cl`030000003clo<0lo?c0740lo?c00<000000??c
l`3clo<06P3clo=c00000080o`004000000Z0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`03
0000003clo<0lo?c0780lo?c0P0000390??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000
003clo<0lo?c07@0lo?c00<000000??cl`3clo<0aP3clo<00`000000lo?c0??cl`0L0??cl`00D@3c
lo<00`000000lo?c0??cl`1e0??cl`030000003clo<0lo?c0<D0lo?c00<000000??cl`3clo<0703c
lo<00540lo?c00<000000??cl`3clo<0MP3clo<400000<<0lo?c00<000000??cl`3clo<0703clo<0
0540lo?c00<000000??cl`3clo<0M`3clo<400000<80lo?c00<000000??cl`3clo<0703clo<003X0
lo?c0`0000030??cl`<000000`3clo<3000000P0lo?c00<000000??cl`3clo<0M`3clo<400000<80
lo?c00<000000??cl`3clo<0703clo<003/0lo?c00D000000??cl`3clo<0lo?c000000030??cl`03
0000003clo<0000000<0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`1h0??cl`80
000000<0lo?c0000003clo<0`03clo<00`000000lo?c0??cl`0L0??cl`00>`3clo<01@000000lo?c
0??cl`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`03
0000003clo<0lo?c07`0lo?c0P00002o0??cl`030000003clo<0lo?c01`0lo?c000a0??cl`L00000
0`3clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000
lo?c0??cl`050??cl`<00000OP3clo<200000;/0lo?c0`00000N0??cl`00>`3clo<01@000000lo?c
0??cl`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`03
0000003clo<0lo?c0800lo?c00<000000??cl`3clo<0^P3clo<00`000000lo?c0??cl`0L0??cl`00
>`3clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000
lo?c0??cl`050??cl`030000003clo<0lo?c0840lo?c0P00002j0??cl`030000003clo<0lo?c01`0
lo?c000k0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c000000030??cl`03
0000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0P`3clo<00`000000lo?c0??cl`2g0??cl`03
0000003clo<0lo?c01`0lo?c000j0??cl`8000000`3clo<01`000000lo?c0??cl`3clo<000000??c
l`0000000`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c08@0lo?c0P00002g0??c
l`030000003clo<0lo?c01`0lo?c000k0??cl`030000003clo<0lo?c0080lo?c0`0000030??cl`<0
0000203clo<00`000000lo?c0??cl`260??cl`030000003clo<0lo?c0;@0lo?c00<000000??cl`3c
lo<0703clo<00540lo?c00<000000??cl`3clo<0Q`3clo<200000;@0lo?c00<000000??cl`3clo<0
703clo<00540lo?c00<000000??cl`3clo<0R@3clo<00`000000lo?c0??cl`2a0??cl`030000003c
lo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c08P0lo?c0`00002b0??cl`030000003clo<0
lo?c01`0lo?c001A0??cl`030000003clo<0lo?c08L0lo?c1P00002`0??cl`030000003clo<0lo?c
01`0lo?c001A0??cl`030000003clo<0lo?c08L0lo?c100000020??cl`030000003clo<0lo?c0:d0
lo?c00<000000??cl`3clo<0703clo<00540lo?c0P0000290??cl`800000103clo<200000:`0lo?c
0P00000N0??cl`00D@3clo<00`000000lo?c0??cl`2@0??cl`030000003clo<0lo?c0:X0lo?c00<0
00000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0T@3clo<200000:X0lo?c00<00000
0??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0T`3clo<200000:P0lo?c00<000000??c
l`3clo<0703clo<00540lo?c00<000000??cl`3clo<0U@3clo<00`000000lo?c0??cl`2U0??cl`03
0000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c09H0lo?c0P00002U0??cl`030000
003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c09L0lo?c0P00002T0??cl`030000003c
lo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c09H0lo?c1@00002R0??cl`030000003clo<0
lo?c01`0lo?c001A0??cl`030000003clo<0lo?c09H0lo?c100000000`3clo<000000000002P0??c
l`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c09L0lo?c0P0000040??cl`03
0000003clo<0lo?c09d0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0
WP3clo<2000009d0lo?c00<000000??cl`3clo<0703clo<00540lo?c0P00002Q0??cl`800000VP3c
lo<2000001h0lo?c001A0??cl`030000003clo<0lo?c0:80lo?c0P00002I0??cl`030000003clo<0
lo?c01`0lo?c001A0??cl`030000003clo<0lo?c0:@0lo?c0P00002G0??cl`030000003clo<0lo?c
01`0lo?c001A0??cl`030000003clo<0lo?c0:H0lo?c0P00002E0??cl`030000003clo<0lo?c01`0
lo?c001A0??cl`030000003clo<0lo?c0:L0lo?c0`00002C0??cl`030000003clo<0lo?c01`0lo?c
001A0??cl`030000003clo<0lo?c0:H0lo?c1@00002B0??cl`030000003clo<0lo?c01`0lo?c001A
0??cl`030000003clo<0lo?c0:H0lo?c100000000`3clo<000000000002@0??cl`030000003clo<0
lo?c01`0lo?c001A0??cl`030000003clo<0lo?c0:L0lo?c0P0000040??cl`800000SP3clo<00`00
0000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`2_0??cl`800000S03clo<00`000000
lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`2a0??cl`800000RP3clo<00`000000lo?c
0??cl`0L0??cl`00D@3clo<200000;@0lo?c0P0000270??cl`8000007P3clo<00540lo?c00<00000
0??cl`3clo<0]@3clo<2000008H0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??c
l`3clo<0]P3clo<2000008D0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3c
lo<0]@3clo<5000008<0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0
]@3clo<4000000030??cl`00000000000840lo?c00<000000??cl`3clo<0703clo<00540lo?c00<0
00000??cl`3clo<0]P3clo<2000000@0lo?c0P00001o0??cl`030000003clo<0lo?c01`0lo?c001A
0??cl`030000003clo<0lo?c0;h0lo?c00<000000??cl`3clo<0O03clo<00`000000lo?c0??cl`0L
0??cl`00D@3clo<00`000000lo?c0??cl`2o0??cl`800000O03clo<00`000000lo?c0??cl`0L0??c
l`00@03clo<3000000<0lo?c0`0000080??cl`030000003clo<0lo?c0<40lo?c0P00001j0??cl`03
0000003clo<0lo?c01`0lo?c000o0??cl`070000003clo<0lo?c0??cl`000000lo?c000000030??c
l`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0``3clo<2000007P0lo?c00<000000??c
l`3clo<0703clo<003l0lo?c00L000000??cl`3clo<0lo?c0000003clo<0000000<0lo?c00<00000
0??cl`3clo<01@3clo<00`000000lo?c0??cl`350??cl`<00000M@3clo<00`000000lo?c0??cl`0L
0??cl`00=`3clo<700000080lo?c00<000000??cl`0000000P3clo<01@000000lo?c0??cl`3clo<0
000000L0lo?c0`0000350??cl`@00000LP3clo<3000001h0lo?c00110??cl`050000003clo<0lo?c
0??cl`0000000`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0<D0lo?c1P00001b
0??cl`030000003clo<0lo?c01`0lo?c00100??cl`030000003clo<000000080lo?c00D000000??c
l`3clo<0lo?c000000070??cl`030000003clo<0lo?c0<H0lo?c0P0000030??cl`<00000K`3clo<0
0`000000lo?c0??cl`0L0??cl`00?`3clo<01`000000lo?c0??cl`3clo<000000??cl`0000000`3c
lo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0<h0lo?c0P00001]0??cl`030000003c
lo<0lo?c01`0lo?c000o0??cl`070000003clo<0lo?c0??cl`000000lo?c000000030??cl`030000
003clo<0lo?c00D0lo?c00<000000??cl`3clo<0d03clo<2000006/0lo?c00<000000??cl`3clo<0
703clo<00400lo?c0`0000030??cl`<00000203clo<00`000000lo?c0??cl`3B0??cl`800000J@3c
lo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c0??cl`3D0??cl`<00000C03clo<2
000001P0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3clo<0e03clo<40000
04X0lo?c1000000G0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c0=@0
lo?c1P0000180??cl`@000005`3clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c
0??cl`3E0??cl`8000000`3clo<2000004L0lo?c0P00000H0??cl`030000003clo<0lo?c01`0lo?c
001A0??cl`030000003clo<0lo?c0=`0lo?c0P00001O0??cl`030000003clo<0lo?c01`0lo?c001A
0??cl`800000g`3clo<3000005/0lo?c0P00000N0??cl`00D@3clo<00`000000lo?c0??cl`3B0??c
l`8000003@3clo<2000005X0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<000000??cl`3c
lo<0d@3clo<4000000h0lo?c0P00001H0??cl`030000003clo<0lo?c01`0lo?c001A0??cl`030000
003clo<0lo?c0=40lo?c1000000@0??cl`<00000E@3clo<00`000000lo?c0??cl`0L0??cl`00D@3c
lo<00`000000lo?c0??cl`3B0??cl`800000503clo<2000005<0lo?c00<000000??cl`3clo<0703c
lo<00540lo?c00<000000??cl`3clo<0jP3clo<200000540lo?c00<000000??cl`3clo<0703clo<0
0540lo?c00<000000??cl`3clo<0k03clo<3000004h0lo?c00<000000??cl`3clo<0703clo<00540
lo?c00<000000??cl`3clo<0k`3clo<2000000<0lo?c0P0000170??cl`030000003clo<0lo?c01`0
lo?c001A0??cl`030000003clo<0lo?c0?40lo?c1P0000160??cl`030000003clo<0lo?c01`0lo?c
001A0??cl`030000003clo<0lo?c0?<0lo?c100000160??cl`030000003clo<0lo?c01`0lo?c001A
0??cl`800000m@3clo<2000000030??cl`000000000004<0lo?c0P00000N0??cl`00D@3clo<00`00
0000lo?c0??cl`3i0??cl`<00000@@3clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000
lo?c0??cl`3l0??cl`800000?`3clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<00`000000lo?c
0??cl`3n0??cl`800000103clo<2000003L0lo?c00<000000??cl`3clo<0703clo<00540lo?c00<0
00000??cl`3clo<0o`3clo<10??cl`L00000=P3clo<00`000000lo?c0??cl`0L0??cl`00D@3clo<0
0`000000lo?c0??cl`3o0??cl`@0lo?c1000000f0??cl`030000003clo<0lo?c01`0lo?c001A0??c
l`030000003clo<0lo?c0?l0lo?c1@3clo<3000003H0lo?c00<000000??cl`3clo<0703clo<00540
lo?c00<000000??cl`3clo<0o`3clo<80??cl`<00000<`3clo<00`000000lo?c0??cl`0L0??cl`00
D@3clo<00`000000lo?c0??cl`0k0??cl`030000003clo<0lo?c04X0lo?c00<000000??cl`3clo<0
B`3clo<00`000000lo?c0??cl`0a0??cl`@000005@3clo<00`000000lo?c0??cl`0G0??cl`030000
003clo<0lo?c01`0lo?c001A0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3c
lo<00`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303c
lo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3c
lo<00`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303c
lo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3c
lo<00`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303c
lo<00`000000lo?c0??cl`070??cl`@0000000<0lo?c0000000000003P3clo<00`000000lo?c0??c
l`0=0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0703clo<00540lo?co`000012
000001h0lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??clk40lo?c003o
0??clk40lo?c00390??cl`<000004`3clo<200000<l0lo?c00380??cl`050000003clo<0lo?c0??c
l`000000503clo<00`000000lo?c0??cl`3<0??cl`00b03clo<01@000000lo?c0??cl`3clo<00000
01D0lo?c00<000000??cl`3clo<0b`3clo<008P0lo?c0`0000030??cl`80000000<0lo?c00000000
00000`3clo<7000000D0lo?c1`000000103clo<000000000003clo<5000000@0lo?c0`0000001@3c
lo<00000000000000000lo?c0080000000@0lo?c000000000000lo?c100000001@3clo<000000000
00000000lo?c00H0000000@0lo?c000000000000lo?c1`0000040??cl`030000003clo<0lo?c00@0
lo?c0`0000050??cl`L000000P3clo<2000000040??cl`0000000000000000D0lo?c0`0000020??c
l`<000001P3clo<3000000h0lo?c1P0000000`3clo<0000000000004000000040??cl`0000000000
0??cl`8000000`3clo<7000000D0lo?c0`0000040??cl`<0000000@0lo?c00000000000000000`3c
lo<2000005L0lo?c00290??cl`050000003clo<0lo?c0??cl`0000000P3clo<01P000000lo?c0??c
l`000000lo?c00000080lo?c00@000000??cl`3clo<000001@3clo<010000000lo?c0??cl`000002
0??cl`060000003clo<0lo?c0000003clo<000000`3clo<00`000000lo?c0??cl`040??cl`050000
003clo<0lo?c0??cl`0000000P3clo<01P000000lo?c0??cl`000000lo?c00000080lo?c00<00000
0??cl`0000001@3clo<010000000lo?c0??cl`0000020??cl`030000003clo<000000080lo?c00@0
00000??cl`3clo<00000103clo<00`000000lo?c000000060??cl`030000003clo<0lo?c00<0lo?c
00@000000??cl`3clo<000000P3clo<01@000000lo?c0??cl`3clo<000000080lo?c00@000000??c
l`3clo<000000`3clo<01@000000lo?c0??cl`3clo<000000080lo?c00<000000??cl`3clo<0103c
lo<00`000000lo?c0??cl`0@0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`0000000P3c
lo<01P000000lo?c0??cl`000000lo?c00000080lo?c00<000000??cl`0000000P3clo<010000000
lo?c0??cl`0000070??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`0000000`3clo<01000
0000lo?c0??cl`00001H0??cl`00R@3clo<010000000lo?c0??cl`0000030??cl`030000003clo<0
lo?c0080lo?c00@000000??cl`3clo<000000P3clo<00`000000lo?c0??cl`030??cl`040000003c
lo<0lo?c00000080lo?c00H000000??cl`3clo<000000??cl`0000030??cl`030000003clo<0lo?c
00@0lo?c00D000000??cl`3clo<0lo?c000000020??cl`060000003clo<0lo?c0000003clo<00000
0P3clo<00`000000lo?c0??cl`02000000<0lo?c00@000000??cl`3clo<000000P3clo<00`000000
lo?c000000020??cl`040000003clo<0lo?c000000@0lo?c00<000000??cl`0000001P3clo<00`00
0000lo?c0??cl`030??cl`040000003clo<0lo?c00000080lo?c00D000000??cl`3clo<0lo?c0000
00040??cl`8000000`3clo<01@000000lo?c0??cl`3clo<000000080lo?c00<000000??cl`3clo<0
103clo<00`000000lo?c0??cl`0@0??cl`030000003clo<0lo?c00@0lo?c00@000000??cl`3clo<0
00000P3clo<00`000000lo?c000000040??cl`040000003clo<0lo?c00000080lo?c00<000000??c
l`3clo<01@3clo<00`000000lo?c0??cl`040??cl`030000003clo<0000000<0lo?c00@000000??c
l`3clo<00000F03clo<008T0lo?c00@000000??cl`3clo<000000`3clo<400000080lo?c0`000002
0??cl`030000003clo<0lo?c00@0lo?c0`0000020??cl`060000003clo<0lo?c0000003clo<00000
0`3clo<00`000000lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`0000000P3clo<010000000
lo?c0??cl`0000020??cl`<0000000D0lo?c0000003clo<0lo?c000000020??cl`040000003clo<0
lo?c00000080lo?c00<000000??cl`3clo<00`0000020??cl`050000003clo<0lo?c0??cl`000000
0P3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c00@0lo?c0`0000020??cl`050000
003clo<0lo?c0??cl`0000000P3clo<2000000D0lo?c00D000000??cl`3clo<0lo?c000000020??c
l`030000003clo<0lo?c00@0lo?c100000050??cl`L000000`3clo<00`000000lo?c0??cl`040??c
l`040000003clo<0lo?c00000080lo?c00<000000??cl`0000000`0000020??cl`<000000P3clo<0
0`000000lo?c0??cl`050??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`0000000`3clo<0
10000000lo?c0??cl`00001H0??cl`00R@3clo<4000000<0lo?c00H000000??cl`3clo<000000??c
l`0000020??cl`040000003clo<0lo?c000000D0lo?c00@000000??cl`3clo<000000P3clo<20000
00040??cl`000000lo?c00000080lo?c0P0000060??cl`040000003clo<0lo?c0??cl`80000000L0
lo?c000000000000lo?c0000003clo<000000080lo?c00<000000??cl`0000000P3clo<010000000
lo?c0??cl`0000020??cl`80000000@0lo?c0000003clo<000000P3clo<00`000000lo?c0??cl`02
00000080lo?c00D000000??cl`3clo<0lo?c000000050??cl`@000000P3clo<010000000lo?c0??c
l`0000020??cl`8000000P3clo<010000000lo?c0??cl`0000020??cl`050000003clo<0lo?c0??c
l`0000000`3clo<010000000lo?c0??cl`0000070??cl`030000003clo<0lo?c00l0lo?c00<00000
0??cl`3clo<0103clo<00`000000lo?c0??cl`02000000040??cl`000000lo?c00000080lo?c00<0
00000??cl`0000000P3clo<00`000000lo?c0??cl`02000000H0lo?c100000030??cl`030000003c
lo<0000000<0lo?c00@000000??cl`3clo<00000F03clo<008T0lo?c00D000000??cl`3clo<0lo?c
000000030??cl`8000000`3clo<2000000<0lo?c00<000000??cl`3clo<0103clo<200000080lo?c
0P0000000`3clo<000000??cl`020??cl`80000000<0lo?c0000003clo<01@3clo<00`000000lo?c
0??cl`02000000050??cl`000000lo?c0??cl`0000000`3clo<2000000<0lo?c1P0000001@3clo<0
00000000003clo<0000000<0lo?c0P0000020??cl`80000000<0lo?c0000000000000P000000103c
lo<000000000000000040??cl`050000003clo<0lo?c0??cl`0000000P3clo<200000080lo?c0P00
00000`3clo<000000000000200000080lo?c0`0000040??cl`<000000P3clo<3000000L0lo?c0`00
000>0??cl`030000003clo<0lo?c00<0lo?c0P0000001@3clo<000000000003clo<0000000<0lo?c
0P0000030??cl`8000000P3clo<2000000030??cl`000000lo?c00@0lo?c00D000000??cl`3clo<0
lo?c000000020??cl`030000003clo<0lo?c00<000000P3clo<3000005L0lo?c00290??cl`050000
003clo<0lo?c0??cl`0000003@3clo<00`000000lo?c0??cl`0B0??cl`030000003clo<0lo?c00@0
lo?c00<000000??cl`3clo<0<03clo<01@000000lo?c0??cl`3clo<0000000/0lo?c00<000000??c
l`3clo<03P3clo<00`000000lo?c0??cl`0G0??cl`030000003clo<0lo?c01l0lo?c00D000000??c
l`3clo<0lo?c000000020??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<0EP3clo<0
08T0lo?c00D000000??cl`3clo<0lo?c0000000=0??cl`030000003clo<0lo?c0180lo?c00<00000
0??cl`3clo<0103clo<00`000000lo?c0??cl`0`0??cl`050000003clo<0lo?c0??cl`0000002`3c
lo<00`000000lo?c0??cl`0>0??cl`040000003clo<0lo?c000001H0lo?c00<000000??cl`3clo<0
7`3clo<01@000000lo?c0??cl`3clo<000000080lo?c00<000000??cl`3clo<01P3clo<00`000000
lo?c0??cl`1F0??cl`00R03clo<5000000d0lo?c0P00000C0??cl`8000001@3clo<3000001D0lo?c
00<000000??cl`3clo<0603clo<5000001h0lo?c0`00000E0??cl`<000001@3clo<00`000000lo?c
0??cl`0G0??cl`D000000P3clo<200000640lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??c
lk40lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??c
lk40lo?c003o0??clk40lo?c0000\
\>"],
  ImageRangeCache->{{{0, 431}, {265.938, 0}} -> {-0.028447, -0.0275058, \
0.00268421, 0.00434314}, {{10.5625, 420.375}, {259.563, 6.3125}} -> \
{0.515406, -177.58, 0.0012889, 0.467984}, {{178.188, 364.5}, {225.063, \
109.938}} -> {-0.0836361, -35.039, 0.00311151, 0.184262}}]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
  "In order to test FDTD codes, transmission through a three layer structure \
is calculated. Frequency response of the transmission can be found \
analytically as well as numerically and a comparison can be made. In the \
following, normally illuminated three layer slab problem will be solved. Note \
that the formulation is genereal and can be applied to an arbitrary number of \
layers with an arbitrary angle of incidence and TE or TM polarization.\nThe \
formulation below assumes ",
  Cell[BoxData[
      \(TraditionalForm\`\[ExponentialE]\^\(+\[ImaginaryJ]\[Omega]t\)\)]],
  " convention. Note that the definition of reflection coefficient, \
\[CapitalGamma], is ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalGamma] = \(E\_-\)/\(E\_+\)\)]],
  " for TE and ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalGamma] = \(-\ \(H\_-\)\)/\(H\_+\)\)]],
  " for TM polarization. "
}], "Text"],

Cell[BoxData[
    \(\(TransmissionCoef[epsListSymbolic_, dList_, \[Theta]in_, pol_, 
          en_] := \[IndentingNewLine]Block[{\[Omega], epsList, kx, kzList, 
            ZList, \[Gamma]List, \[CapitalGamma]List, i, 
            fieldList}, \[IndentingNewLine]\[Omega] = 
            en\ *q/hbar; \[IndentingNewLine]epsList = 
            epsListSymbolic\  /. \ \[HBar]\[Omega] -> 
                en; \[IndentingNewLine]kx\  = 
            Sin[\[Theta]in]*\ \[Omega] \@\( mu0*eps0*epsList[\([1]\)]\); 
          If[Im[kx > 0], \(kx = \(-kx\);\)]; \  (*\ 
            In\ \[ExponentialE]\^\(+\[ImaginaryJ]\[Omega]t\)\ convention, \ 
            Im[kx]\ should\ be\ negative*) \[IndentingNewLine]kzList = 
            Table[\@\(\[Omega]\^2*mu0*eps0*epsList[\([i]\)] - kx\^2\), {i, 1, 
                Length[epsList]}]; \[IndentingNewLine]For[i = 1, 
            i <= \ Length[kzList], \[IndentingNewLine]If[
              Im[kzList[\([i]\)]] > 0, 
              kzList[\([i]\)] = \(-kzList[\([i]\)]\)]; \[IndentingNewLine]\(i\
++\);\[IndentingNewLine]]; \[IndentingNewLine]If[pol\  == "\<TE\>", 
            ZList = \[Omega]*mu0\ /\ kzList, 
            ZList = kzList\ /\ \((\[Omega]*eps0*epsList)\)]; \  (*\ 
            Calculation\ of\ impedances\ *) \[IndentingNewLine]\[Gamma]List\  \
= \ Table[\(ZList[\([i + 1]\)] - ZList[\([i]\)]\)\/\(ZList[\([i + 1]\)] + \
ZList[\([i]\)]\), {i, 1, 
                Length[ZList] - 
                  1}]; \[IndentingNewLine]\[CapitalGamma]List\  = \ {0}; \
\[IndentingNewLine]For[i = Length[\[Gamma]List], 
            i >= 1, \[IndentingNewLine]\[CapitalGamma]List\  = \ 
              Prepend[\[CapitalGamma]List, \((\[Gamma]List[\([i]\)] + \ 
                      Exp[\(-\[ImaginaryJ]\)*2*kzList[\([i + 1]\)]*
                            dList[\([i + 
                                  1]\)]]*\[CapitalGamma]List[\([1]\)])\)/\((1 \
+ \ \[Gamma]List[\([i]\)]*
                        Exp[\(-\[ImaginaryJ]\)*2*kzList[\([i + 1]\)]*
                            dList[\([i + 
                                  1]\)]]*\[CapitalGamma]List[\([1]\)])\)]; \
\[IndentingNewLine]\(i--\);\[IndentingNewLine]]; \[IndentingNewLine]fieldList \
= {1}; \[IndentingNewLine]For[i = 1, 
            i < Length[\[CapitalGamma]List], \[IndentingNewLine]If[
              pol == "\<TE\>", \[IndentingNewLine]fieldList\  = \ 
                Append[fieldList, 
                  Last[fieldList]*\ \((1 + \
\[CapitalGamma]List[\([i]\)])\)/\((Exp[\(+\[ImaginaryJ]\)*kzList[\([i + 1]\)]*
                              dList[\([i + 
                                    1]\)]] + \[CapitalGamma]List[\([i + 1]\)]*
                            Exp[\(-\[ImaginaryJ]\)*kzList[\([i + 1]\)]*
                                dList[\([i + 
                                      1]\)]])\)], \
\[IndentingNewLine]fieldList\  = \ 
                Append[fieldList, 
                  Last[fieldList]*\ \((1 - \
\[CapitalGamma]List[\([i]\)])\)/\((Exp[\(+\[ImaginaryJ]\)*kzList[\([i + 1]\)]*
                              dList[\([i + 
                                    1]\)]] - \[CapitalGamma]List[\([i + 1]\)]*
                            Exp[\(-\[ImaginaryJ]\)*kzList[\([i + 1]\)]*
                                dList[\([i + 
                                      1]\)]])\)], \[IndentingNewLine]]; \
\[IndentingNewLine]\(i++\);\[IndentingNewLine]]; \[IndentingNewLine] \
(*\(Print[\[CapitalGamma]List];\)*) \[IndentingNewLine]Last[
            fieldList]\[IndentingNewLine]];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Test Cases", "Section"],

Cell[CellGroupData[{

Cell["Case 1: PHYSICAL REVIEW B 71, 085416 (2005)", "Subsection"],

Cell[TextData[{
  "In this example, the Lorentz model is defined a bit differently. Note that \
the below definition is for ",
  Cell[BoxData[
      \(TraditionalForm\`\[ExponentialE]\^\(-\[ImaginaryJ]\[Omega]t\)\)]],
  " convention, thus needs to be conjugated to be used for the above \
formulation. This has been the bug source, which took a long time (2 days) to \
debug. ",
  StyleBox["Thus, the main lesson is to determine the convention that's being \
used.", "TB"]
}], "Text"],

Cell[CellGroupData[{

Cell[GraphicsData["Metafile", "\<\
CF5dJ6E]HGAYHf4PEfU^I6mgLb15CDHPAVmbKF5d0@00159<0@0006`000000000000008/2001/0000
00000000000=@`009`/00215CDH00040C58400<000010000000000000000000000@00003000>0@00
b`0000000000000000000;0N103h60<0D@000<aA10000000000008/2001/00000000000000000000
000008`2001]0000D00002P0001h0000E5440000000P0<`0S08006d0000X0000R@8006d000010200
000005AA100000000000000000000000oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
Q8:4024L8@3>clh0oooo0?ooo`3oool0in?W07]mN`2]ZZd0oooo0?Ogm`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>ock`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3FemH0c/_>0?ooo`3oool0oooo0?ooo`3oool0oooo0<K3aP0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H7240Jfe[0:FVY@3oool0oooo0?ooo`3oool0oooo0>O[i`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3Ngmh0
Jfec0>ock`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP0Q8240]KJe0?ooo`3oool0oooo0?ooo`3oool0W9ZL0?Ogm`1[KF/0
in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
Q7f4024L8@3>clh0oooo0?ooo`3oool0in?W07]mN`2]ZZd0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kSgP1cKG<0moOg0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ024P8@2LVY`0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo07]mN`0Q82T0g^?N0?ooo`3oool0moOg0:FZ[@24PX@08A`Q0<K3aP3oool0
oooo0?ooo`2m][d0YJZ]0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH0FUUJ0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0@T5208B2Q03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0kno_05YIFP3Nfmh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP0Q8240]KJe0?ooo`3oool0oooo0?ooo`3oool0W9ZL0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP1JFEX0in_W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNW00H7240
8B0Q0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH08A`Q0;Ff]@3oool0oooo0?ooo`3oool0
oooo0;Ff]@0Q7240[JZ]0?ooo`3oool0oooo0:fZ[@2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3WjnL0@T520=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNW012@D80
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH0@T520>o[k`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0@T5208AmQ03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>O[i`12ADX0
e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3WjnL08B0Y024L8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0495BP1kOG/0
oooo0?ooo`3oool0oooo0?ooo`3oool0W9jL024L8@2m][d0oooo0?ooo`3oool0Q8:40:FZ[@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Lfec09bJW03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`36`lH08B0Q0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNY@1[KF/0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNY@12@D80
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`1[KF/0W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q8240Ngek0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2<SX`0@T520?ooo`3oool0oooo0?ooo`3oool0oooo0>ock`12@D80FUUJ0?ooo`3oool0
oooo0>O[i`0Q8240a/_60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe01PL8@3_jnl0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`3>blh0e]OF0?ooo`3oool0oooo0?Ogm`36blH0
c/o>0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0kn__01PL8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3WjnL08B0Q0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2m`kd08B0Q0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3Ngmh0
c/o>0=KGeP3oool0oooo0?ooo`3_lnl0c/o>0<k;cP3_jnl0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3WhnL0c/_>0=kKgP3oool0oooo0?ooo`3gmoL0c/_>0<k?cP3WjnL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2m][d08A`Q0>o_k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
in?W0<k;cP3Nfmh0oooo0?ooo`3oool0moOg0<k;cP3>clh0in_W0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Ngek024P:@3gmoL0
oooo0?ooo`3oool0oooo0?ooo`3oool0a/?6024L8@3>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0
Ngek024P8@3FemH0oooo0?ooo`3oool0S8j<0495BP3gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
moOg0491@P24PX@0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg08B2Q00Q72408B0Q024L8@1JFF<0
kn__0>OSi`12@D808B0Y024L8@0Q8240S8j<0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0S8j<024P8@0Q72408B0Y024L8@0Q8240Jfe[0?ooo`3oool0oooo0=KGeP0Q82T0
8A`Q024P8@0Q72408B0Y024L8@3_knl0oooo0?ooo`3oool0oooo0?ooo`1cKG<0Ngek0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`12ADX0moOg0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08B2Q00Q7240Q8:40=kSgP3oool0
oooo0?ooo`3>blh08B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q07]mN`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0kno_024L8@2m][d0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`1kOG/08B0Q024L8@0Q82T0FUUJ0>o_k`3Nhmh0@TE:024L8@0Q82408A`Q09B>U03gmoL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ocm`24OH@061`Q024P8@0Q7240FUUJ0>O[i`3WhnL0@T52024P8@0H7240
8B0Q08b>S03gmoL0oooo0?ooo`3oool0oooo0?ooo`3gmoL0@T5208AmQ03oool0oooo0?ooo`3oool0
oooo0?ooo`3gloL0Q7f401PL8@0Q82408A`Q05YIFP3WjnL0in?W0491@P0Q824061`Q024P8@2<SX`0
moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo07]mN`0Q8240Ngek0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0>o[k`12@D80a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0>O[i`0Q8240W9ZL0?ooo`3oool0
oooo0?ooo`3oool0oooo0:fZ[@0H7240]KJe0?ooo`3oool0oooo0=KGeP0Q7240[JZ]0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0;Ff]@0Q8240YJJU0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S00Q7240
@T520=kOgP3oool0_L>m0491@P1[KF/0Q7f40?Ocm`3_jnl0Ngek024P8@1JFEX0moOg0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0mo?g024P8@0H72408B0Q024L8@0Q824061`Q0<k?cP3oool0
oooo0?ooo`3oool0Jfe[024P8@0H72408B0Q024L8@0Q8240YJJU0?ooo`3oool0oooo0?ooo`3oool0
]KJe0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2]ZZd08A`Q0;Ff]@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2]ZZd0
S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0
8B0Q0:FVY@2e][D0]KJe0?ooo`3oool0oooo0?ooo`3oool061`Q024P8@0Q7240e]OF0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KF/0Ngek0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2<SX`08B0Q0491@P3WhnL0oooo0<K3aP12@D80Jfe[07]mN`3gmoL0
in_W08AmQ00Q7240FUUJ0>ock`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2DSY@08A`Q0495BP3Nhmh0oooo0;g3_@12ADX0
Jfe[08B2Q03gmoL0kno_07]mN`0Q8240FUUJ0?Ogm`3oool0oooo0?ooo`3oool0]KJe024P8@2UZZD0
oooo0?ooo`3oool0oooo0?ooo`3oool0U8jD024L8@12ADX0g^?N0?ooo`2m`kd0@TE:06]]J`24PX@0
moOg0>o_k`1kOG/08B0Q05YIFP3gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ogm`0Q7240in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`1JFEX08A`Q0:fZ[@2e][D0]KJe0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ0491@P3_knl0oooo0?ooo`3oool0oooo0?ooo`3oool0
FUUJ05YIH`3oool0oooo0?ooo`3oool0oooo0?ooo`2UZZD08B0Y0;Ff]@3oool0oooo0?ooo`3oool0
8B0Q0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1cKG<08A`Q0<k?cP3oool0oooo0?ooo`3oool0
oooo0?Ogm`0Q72408B0Q08b>S03oool0oooo0?ooo`2<SX`08B0Y0<K3aP3oool0oooo0?Ogm`12@D80
8B0Q07]mN`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP1JFEX0W9jU09bJW02LWY`0
Ngek05YIH`3oool0oooo0?ooo`3oool0oooo0<K3aP12@D80W9jU09bJW02LWY`0S8j<05YIH`3oool0
oooo0?ooo`3oool0oooo0=kKgP0Q7240g]_N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
c/_>024P8@1[KF/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0YJZU09B>U03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0[JZ]024L8@24PX@0oooo0:fZ[@3gmoL0oooo0?ooo`3oool0oooo024P:@0Q7240
8B0Q0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
]KJe0495BP3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL08B0Q024L8@2DSY@0oooo0?ooo`3oool0
U8jD024L8@36`lH0oooo0?ooo`3gmoL0@TE:024L8@24PX@0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL061`Q024P8@2<SX`0
oooo0?ooo`3oool0S8j<024P8@2m`kd0oooo0?ooo`3gmoL0@T52024P8@1kOG/0oooo0?ooo`3oool0
oooo06]]L`0H7240c/o>0?ooo`3oool0oooo0?ooo`3oool0moOg01PL8@0Q8240S8j<0?ooo`3oool0
oooo08b>S00Q8240_L>m0?ooo`3oool0moOg0491@P0Q8240Ngek0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_lnl08B0Q0=kOgP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJJU024P8@1kOG/0oooo0:FVY@3gmoL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo01PL8@0Q8240Ngek0?ooo`3oool0
oooo0?ooo`3oool0oooo0:fZ[@0Q7240kn__0?ooo`3oool0oooo0?ooo`3oool0Q7f4024L8@3FemH0
oooo0?ooo`3oool0oooo06]]J`0Q8240e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL08A`Q024P8@3Ngmh0
oooo0?ooo`3oool0oooo0?ooo`3FemH08B0Q01PL8@2e][D0oooo0?ooo`3oool0]KJe024L8@2]ZZd0
oooo0?ooo`3oool0]KJe01PL8@0Q8240c/_>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`0
c/o>0?ooo`3oool0in_W0491@P2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0@T520:FVY@3oool0
oooo0?ooo`1JFEX0in?W0?ooo`3oool0oooo0?ooo`3WjnL08B0Q07]mN`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>OSi`0H72408B0Q0?Ocm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP0Q8240@T520?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`0Q72408B0Q01PL8@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP0H7240e]OF0?ooo`3oool0oooo0?ooo`3oool0e]OF01PL8@0Q8240
]KJe0?ooo`3oool0oooo0;Ff]@0Q8240YJJU0?ooo`3oool0oooo0;Ff]@0Q82408A`Q0<k?cP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
e]OF024P:@0Q7240]KJe0?ooo`3oool0oooo0;Ff]@0Q7240[JZ]0?ooo`3oool0oooo0;ff_@0Q7240
8B0Q0<k;cP3oool0oooo0?Ogm`0Q72408B0Y0=kSgP3oool0oooo0?ooo`3oool0oooo0=KGeP0Q82T0
8A`Q0;Ff]@3oool0oooo0?ooo`2e][D08A`Q0:fZ[@3oool0oooo0?ooo`2m][d08A`Q024P8@3>blh0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024L8@3WhnL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kKgP0Q7240
@TE:0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q82T0
@T52024P8@2UZZD0oooo0?ooo`3oool0oooo0?ooo`3FemH08B0Q0;g3_@3oool0oooo0?ooo`3oool0
in?W024L8@1cKG<0oooo0?ooo`3oool0oooo0?ooo`2DSY@08A`Q0:fZ[@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
e]OF024P8@0Q7240kno_0?ooo`3oool0oooo0?ooo`3oool0g]_N024L8@0Q82T0YJZ]0?ooo`3oool0
oooo0;Ff]@0Q8240@T520?ooo`3oool0oooo0>O[i`0Q82T08A`Q09B>U03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0@TE:0?Ogm`3oool0moOg05YIH`0Q7240kno_0?ooo`3oool0oooo0?ooo`3oool0
oooo09bJW00Q8240_L>m0?ooo`3oool0]KJe0;Ff]@3oool0oooo0?ooo`2e][D08B0Y024L8@0Q8240
8A`Q07=]L`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL08B0Y024L8@3Nfmh0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2UZZd0U8jD0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ024P8@3>blh0
oooo0?ooo`3oool0oooo0?ooo`3oool08B0Q024L8@0Q82T0e]OF0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL08B0Y07]mN`3oool0oooo0?ooo`3oool0
oooo0=KGeP0Q82T08A`Q0:fZ[@3oool0oooo0?ooo`2e][D08A`Q0495BP3oool0oooo0?ooo`3_knl0
8A`Q024P8@2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP0Q72408B0Q0:FVY@3oool0oooo0?ooo`2e][D08B0Q0491@P3oool0
oooo0?ooo`3WjnL08B0Q01PL8@2<SX`0oooo0?ooo`3FemH08B0Q024L8@3_jnl0oooo0?ooo`3oool0
oooo0?ooo`3FemH08A`Q024P8@2UYZD0oooo0?ooo`3oool0]KJe024P8@12@D80oooo0?ooo`3oool0
in_W024P8@0H7240S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ocm`0Q8240g]oN0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0FUUJ01PL8@3>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3gmoL08A`Q08AmQ02LVY`08B0Q0=KGeP3oool0oooo0?ooo`3oool0e]OF01PL8@2LWY`0
oooo0?ooo`3oool0[JZ]0491@P1JFEX0in_W0?ooo`3oool0oooo0?ooo`3oool0W9ZL024P8@1[KF/0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0<K3aP0H72408B0Q0?ooo`3oool0oooo0?ooo`3oool0oooo0>ock`0Q8240
8A`Q08AmQ03oool0oooo0?ooo`2LWY`061`Q024P8@3>blh0oooo0?ooo`3oool0@T52024P8@1JFEX0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S00Q7240Jfec0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3FemH061`Q024P8@3Ngmh0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024L8@0Q8240
_L>m0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
[JZ]08b>S03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo09bNW00H7240W9jL0?ooo`3oool0oooo0?ooo`3oool0oooo01PL8@0Q82408A`Q0=KGeP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Q024L8@0Q8240
61`Q06]]J`3oool0oooo0?ooo`3gmoL08A`Q024P8@1kOG/0oooo0?ooo`3oool0W9ZL024P8@0Q7240
c/o>0?ooo`3oool0oooo0491@P0H7240FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL08B0Q024L8@24PX@0oooo0?ooo`3oool0
W9jU024L8@0Q8240c/_>0?ooo`3oool0oooo0491@P0Q82T0FUUJ0?ooo`3oool0a/?6024L8@0Q8240
oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024P8@0Q7240Q8:40?ooo`3oool0oooo09bNY@0Q7240
8B0Q0<k;cP3oool0oooo0?ooo`12@D808B0Y05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3gmoL08A`Q0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bJW00Q82T0W9ZL0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0kn__024P8@1kOG/0moOg0491@P1JFEX0moOg0?ooo`3oool0
]KJe024L8@0Q82T08A`Q024P8@0Q72408B0Y05YIFP2]ZZd0moOg0?ooo`3oool0oooo0?ooo`3oool0
oooo0;Ff]@0Q7240FUUS0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Y024L8@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0Ngek024P8@12@D80oooo0?ooo`3oool0YJZU024P:@0Q7240Q8:40?ooo`3oool0
oooo08B2Q00Q72408B0Y0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kKgP0Q7240
8B0Q0:FZY@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024P:@0Q7240Q8:40?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`0Q82408A`Q0;ff_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0:FZY@2DSY@0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3>blh08B0Y05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`0Q82T0
8A`Q024P8@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08B2Q00Q7240@TE:0?ooo`3oool0
oooo0:fZ[@0Q72408B0Q07]mN`3oool0oooo0?ooo`1kOG/08B0Y024L8@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`0Q8240
@T520?ooo`3oool0oooo0:FVY@0Q824061`Q08AmQ03oool0oooo0?ooo`24OH@08A`Q024P8@3oool0
oooo0;Ff]@0Q824061`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/08B0Q0491@P3oool0
oooo0?ooo`2UYZD08B0Q01PL8@24OH@0oooo0?ooo`3oool0Q7f4024L8@0Q8240oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0ko?_024P8@3Ngmh0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>clh08A`Q05YIFP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP0H7240Q7f40?ooo`2e][D0
61`Q09bNW03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Q024L8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe024L8@0Q8240
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP0H72408B0Q0=KGeP3oool0oooo0=KGeP0Q7240
8B0Q05YIFP3oool0oooo0?ooo`1kOG/08B0Q05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`1kOG/08B0Q01PL8@3>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KF/0
8B0Q01PL8@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool061`Q024P8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2]ZZd0S8j<0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0491@P0Q8240e]OF0?ooo`3oool0
oooo0?ooo`3oool08A`Q024P8@0H7240e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36blH0
8B0Q024L8@3FemH0oooo0?ooo`3FemH08B0Q01PL8@1JFEX0oooo0?ooo`3oool0Q7f4024L8@1JFEX0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3>clh08A`Q024P8@3FemH0oooo0?ooo`3Nfmh08A`Q024P:@1JFEX0oooo0?ooo`3oool0
Ngek024P8@1JFEX0oooo0?ooo`2e][D08A`Q024P:@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
c/o>024L8@0Q8240e]OF0?ooo`3oool0g]_N024L8@0Q82T0FUUJ0?ooo`3oool0oooo07]mN`0Q8240
FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`0Q7240
in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0495BP0Q7240g]_N0?ooo`3oool0oooo0?ooo`3oool0g]_N0?ooo`3oool0oooo0?ooo`2LVY`0
8B0Y09bJW03oool0oooo05YIH`0Q7240in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0_KJm024L8@0Q8240oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0;Ff]@0Q82408A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Q8:4024L8@24PX@0
oooo0?ooo`3oool0c/o>09bJW03Nfmh0oooo0?ooo`3oool0Lfec024L8@2DSY@0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:FZY@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0Jfe[0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_jnl08B0Q024L8@0Q82T0g^?N0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0U8jD024L8@0Q82T0W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024P:@0Q7240]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3gmoL08A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@3>blh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2DSY@0
8A`Q0;ff_@3oool0oooo0?ooo`3oool0oooo024P8@0Q72408B0Y0=KGeP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo07]mN`0Q8240Ngek0?ooo`3oool0oooo0<k;cP2LWZD0e]OF0?ooo`3oool0
oooo06]]J`0Q8240S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08AmQ00H7240Q7f40?ooo`3oool0oooo0<k?cP2LVY`0
e]OF0?ooo`3oool0oooo06]]L`0H7240S8j<0?ooo`3oool0]KJe024P8@0Q7240oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`24OH@061`Q08AmQ03oool0oooo0?ooo`3>clh0W9ZL0=KGeP3oool0
oooo0?ooo`1[KG<061`Q08b>S03oool0oooo0?ooo`3oool0oooo0?ooo`2UYZD08B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q06]]J`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2<SX`08B0Q0;Ff]@3oool0oooo0?ooo`3oool0kn__0491@P3_jnl0
oooo0?ooo`3_lnl0@T52024L8@2e][D0oooo0?ooo`2m`kd08B0Q06]]J`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q8240
61`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3>clh061`Q024P8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ocm`1JFEX061`Q0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0kn__024L8@0Q8240
g]oN0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0c/o>01PL8@0Q82408A`Q0?Ogm`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bJW00Q82408A`Q05YIFP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q7240
8B0Q0<K;aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0:fZ[@2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0]KJe024P8@1[KF/0oooo0?ooo`3oool0oooo0?ooo`0H72408B0Q024L8@3FemH0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0FUUJ024P8@2m`kd0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>O[i`0Q824061`Q0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0FUUS024L8@36`lH0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o_k`0Q72408B0Y0=kSgP3oool0oooo0<k?cP0Q7240
8B0Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg05YIH`0Q7240a/?60?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_knl08A`Q024P:@3Nhmh0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0_KJm024L8@1cKG<0oooo0?ooo`3oool0
oooo0?ooo`24PX@0FUUJ0;Ff]@2m`kd0Lfec024L8@0Q8240in_W0?ooo`3oool0oooo0491@P0Q82T0
e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2]ZZd08A`Q05YIH`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0g^?N024P:@0Q7240kno_0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0moOg09B>U00Q7240a/?60?ooo`3oool0oooo0?ooo`3oool0
oooo08b>S00Q8240_L>m0?ooo`3oool0Lfec024L8@0Q82T08A`Q024P8@0Q72408B0Y07]mN`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q82T0
8A`Q024P8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2m][d08A`Q024P8@0Q7240
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3_jnl08B0Q024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2UZZD0U8jD0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D08A`Q024P:@0Q72408B0Q0>O[i`3oool0oooo0?ooo`3oool0
8B0Y024L8@0Q8240e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`2<SX`0
8B0Q0;g3_@3oool0oooo0?ooo`3oool0oooo0?ooo`2DSY@08A`Q0<K3aP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ocm`2<SY@061`Q0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`08B0Q0;g3_@3oool0
oooo0?ooo`3Ngmh08B0Q01PL8@3_jnl0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gloL0
S8jD01PL8@36`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0S8j<024P8@2m`kd0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>ock`0Q8240g]oN0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Q024L8@0Q8240
61`Q0>o[k`3oool0oooo0?ooo`3oool0in_W0491@P0H72408B0Q024L8@0Q8240S8j<0?ooo`3oool0
oooo0?ooo`2e][D08A`Q08b>U03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0W9ZL024P8@1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q7240
8B0Q0=kOgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF0;Ff]@3oool0
oooo0?ooo`3oool0oooo0?Ocm`2e][D0g]oN0?ooo`3oool0oooo0?ooo`24OH@08A`Q024P8@0H7240
8B0Q024L8@12@D80ko?_0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D08A`Q024P8@0H7240oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
]KJe024P8@0H72408B0Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0in?W01PL8@0Q8240oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0[JZ]08b>S03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o[k`2UYZD0[JZ]05YIFP36`lH0
oooo0?ooo`3oool0oooo024L8@0Q824061`Q0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP2e][D0oooo0?ooo`3oool0oooo0?ooo`3gmoL0]KJe0>OSi`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kKgP2e][D0oooo0?ooo`3oool0oooo0?ooo`3gmoL0
_KJm0=kSgP3oool0oooo0?ooo`3oool0oooo024L8@0Q82T0g^?N0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3Nfmh0]KJe0?ooo`3oool0oooo0?ooo`3oool0moOg0;ff_@3Nhmh0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0
8A`Q0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0>o[k`2]ZZd0YJZU05YIH`36`lH0oooo0?ooo`3oool0oooo0?ooo`3WjnL0U8jD0491@P12ADX0
W9ZL0?Ogm`3oool0oooo0?ooo`3oool0oooo08B2Q01kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08B2Q00Q7240]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0U8jD024L8@36`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08B2Q03FemH0oooo0?ooo`3oool0]KJe0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0c/_>024P8@0Q72408B0Y0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0:fZ[@0Q72408B0Y05YIFP3oool0oooo0?ooo`2UZZd08B0Q024L8@0Q82T0
8A`Q024P8@0Q7240@TE:0?ooo`3oool0oooo0?ooo`3oool0oooo0;g3_@0Q82T0S8j<0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:FZ[@2DSY@0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q82408A`Q024P:@3FemH0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`08A`Q0<K3aP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0mo?g024P8@3Ngmh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0
8B0Q0=kOgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K;aP0Q8240W9ZL0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0Q7f40=kOgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o[k`0H72408B0Q024L8@3WhnL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`08B0Q024L8@2<SY@0oooo0?ooo`3oool0
oooo0:FVY@0Q82408A`Q024P8@0H72408B0Q024L8@3>clh0oooo0?ooo`3oool0oooo0?ooo`2LWY`0
8A`Q0<k?cP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2]ZZd0S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool061`Q024P8@0Q7240
e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0g]oN0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
c/_>024P8@2LVY`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`0Q7240in?W0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3gmoL08B0Q06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ05YIFP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`24PX@0in_W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
Q8:4024L8@0Q8240_L>m0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUS024L8@0Q8240
e]OF0?ooo`3oool0oooo0?ooo`3oool0YJZ]0;Ff]@3oool0oooo0?ooo`3WhnL0YJZU0?ooo`3oool0
oooo0?ooo`3oool0FUUJ05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0YJZU09B>U03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo024P:@0Q72408B0Q0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg05YIH`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0in_W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K3aP0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
Jfe[0:FVY@3oool0oooo0?ooo`3oool0@T5201PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3Ngmh0Jfe[0?Ocm`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0
Jfe[0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`1JFEX0FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_lnl08B0Q0=kOgP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0a/?601PL8@36`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0=KGeP0H7240e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
g]oN05YIFP3gloL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=kOgP0Q824061`Q08AmQ03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
e]OF024L8@0Q8240Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2UYZD0]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF024P8@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@2<SX`0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`12@D80
@T520>O[i`3oool0oooo0?ooo`0Q72408B0Q01PL8@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0<k?cP1JFEX0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0YJJU0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0YJJU0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`0Q82T0
g^?N0?ooo`3oool0moOg0:FZ[@24PX@08A`Q0<K3aP3oool0oooo0?ooo`3>clh08A`Q07=]L`3oool0
oooo0?ooo`3oool0]KJe0495BP3gmoL0oooo0?ooo`3oool0oooo0?ooo`3FemH0FUUS0>o[k`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>o_k`1JFEX0g]_N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0g]_N024L8@3Nfmh0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
moOg024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=kSgP12ADX0@T520>o_k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`1cKG<0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0W9jU06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2UZZd0U8jD0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0_L>m024P:@0Q7240c/o>0?ooo`3oool0
oooo0?ooo`3oool0oooo06]]J`0Q8240FUUJ0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08b>S03>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07=]L`2LVY`0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH08B0Y0491@P3_knl0oooo0?ooo`3oool08B0Q024L8@0Q82T0e]OF0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ogm`1JFEX0FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0a/?609bJW03_knl0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0S8j<0:fZ[@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@2<SX`0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH08A`Q0;Ff]@3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q7240[JZ]0?ooo`3oool0
oooo08AmQ012@D80oooo0?ooo`3oool0oooo0?ooo`3FemH0]KJe0?ooo`3oool0oooo0?ooo`3WjnL0
@T520=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF0491@P3_jnl0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>O[i`3_jnl0
oooo0?ooo`3oool0moOg0;g3_@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LWY`0
Jfe[0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH08A`Q0491@P3WjnL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K;aP12@D80
in_W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`24OH@0]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF01PL8@3WhnL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>blh0
@T5201PL8@2e][D0mo?g0?ooo`3oool0in?W07]mN`0Q8240S8j<0?Ogm`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ0=KGeP3oool0oooo0?ooo`3oool0oooo0;Ff]@1kOG/0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo01PL8@0Q8240
8A`Q0=KGeP3oool0oooo0?ooo`3oool0g]oN0;Ff]@12@D808B0Q05YIFP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3gmoL0Ngek0<k?cP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9ZL08AmQ03oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0Q7f40:FVY@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0495BP1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0
W9jL024L8@2m][d0oooo0?ooo`2e][D08B0Y0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0Lfec09bJW03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LWY`0
Jfe[0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
moOg07]mN`0Q82408A`Q09B>U03oool0in?W0491@P0Q82T08A`Q08B2Q03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo08B2Q02e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`1JFF<0e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
moOg08B2Q036`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`0Q82T0
W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ogm`36`lH0Ngek05YIFP1JFEX0FUUS06]]J`2LWY`0g^?N0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP1JFEX0oooo0?ooo`3oool0
oooo0<K3aP1kOG/0moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>blh0
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@1JFF<0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`12@D80moOg0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
c/_>024P:@3_jnl0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o[k`0Q8240c/_>0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`0@T520?ooo`3oool0
oooo0?ooo`3oool0oooo0>ock`12@D80FUUJ0?ooo`3oool0kn__024L8@2e][D0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe024L8@3_jnl0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0>OSi`36blH0e]OF0?ooo`3oool0oooo0?Ogm`3>blh0c/o>0>O[i`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0in_W024P8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo09bJW03>clh0oooo0=KGeP12@D80S8jD07]mN`3>clh0oooo0>o[k`2<SX`0
a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gloL0Q7f40;g3_@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>ock`24OH@0c/_>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0W9jL0>ock`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe0?Ogm`3oool0
oooo0?ooo`3WhnL08A`Q08b>U03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
@T520;Ff]@3oool0oooo0?Ogm`2LVY`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0S8j<0<k;cP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0ko?_0491@P2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0S8j<0491@P3gloL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0<K3aP24PX@0@T52024P:@0Q7240FUUJ06]]J`3_knl0oooo0?ooo`3oool0
a/?6024L8@3>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0Ngek024P8@3FemH0oooo0?ooo`1JFEX0
Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg0491@P24PX@0oooo0?ooo`3oool0
oooo0?ooo`3oool0moOg08B2Q00Q72408B0Y024L8@1JFEX0in_W0>OSi`12@D808B0Q024L8@0Q82T0
S8j<0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/0@TE:0?Ogm`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH0
Q8:40491@P0Q82T08A`Q05YIFP1[KF/0kno_0?ooo`24PX@0oooo0?ooo`3oool0g]_N024L8@2]ZZd0
oooo0?ooo`3oool0moOg08b>S03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LWZD0
moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2m][d0e]OF0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`1[KF/0in?W0?ooo`3_knl0Jfe[024P8@2m`kd0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo06]]J`1JFF<0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q7240
8B0Y024L8@3_knl0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@0Q7240g]_N0?ooo`3oool0oooo0?ooo`3oool0
_KJm024L8@0Q82408A`Q024P:@1kOG/0]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0=kKgP0Q7240[JZ]0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`0Q824061`Q0491@P1kOG/0W9jL09bJW024OH@0
c/_>0?ooo`3oool0oooo0>O[i`0Q8240W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@0H7240
]KJe0?ooo`3oool0W9ZL0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q8240
YJJU0?ooo`3oool0oooo0?ooo`3oool0oooo08b>U00H7240@T520=kOgP3oool0_L>m0491@P1[KF/0
Q7f40>ock`3_jnl0Ngek024P8@1JFEX0moOg0?ooo`3oool0oooo0?ooo`3oool0[JZ]01PL8@2e][D0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`1kOG/08B0Q01PL8@12@D80Ngek09bNW02LVY`0Q7f40<k;cP3oool0Ngek0?ooo`3oool0
oooo0?ooo`12@D80@T520?Ogm`3oool0oooo0?Ocm`2<SY@0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0in?W07]mN`0Q82408A`Q06]]L`2m`kd0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH0a/_60?ooo`3oool0oooo0?ooo`1JFEX0@T520?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`36blH08B0Q024L8@36`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`12@D808B0Q0?ooo`3oool0
oooo0?ooo`3oool0oooo06]]J`0Q8240S8j<0<k?cP3oool0oooo0:FVY@1JFEX0g]oN0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool08B0Q0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q82408A`Q08B2Q03gmoL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ05YIH`3oool0oooo0?ooo`3oool0
oooo0?ooo`2UZZD08B0Y0;Ff]@3oool0oooo0<k?cP0Q7240g]_N0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`1cKG<08A`Q0<k?cP3oool0oooo0?ooo`3oool0oooo0?Ogm`0Q72408B0Y08b>S03oool0
oooo0?ooo`2<SX`08B0Q0;g3_@3oool0oooo0?Ogm`12@D808B0Y07]mN`3oool0oooo0?ooo`3oool0
oooo0<k;cP0Q82T0Jfe[0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Q024L8@24PX@0moOg0?ooo`3oool0oooo0?ooo`3oool0
oooo07=]L`36`lH0oooo0>O[i`2]ZZd0YJZ]0495BP1kOG/0moOg0>o[k`2DSY@0e]OF0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08B2Q036`lH0oooo0?ooo`24PX@08A`Q09bNW03oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH0
8B0Q06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
]KJe0495BP3WjnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`0Q82T0e]OF0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`24OH@0
61`Q024P8@3Ngmh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@0Q7240
kn__0?ooo`3oool0oooo0?ooo`3oool0Q7f4024L8@3FemH0oooo0?ooo`3oool0@T520;Ff]@3oool0
oooo0?ooo`3oool0oooo0?ooo`3gmoL061`Q024P8@3Ngmh0oooo0?ooo`3oool0oooo0?ooo`3FemH0
8B0Q024L8@2e][D0oooo0?ooo`3oool0]KJe01PL8@2]ZZd0oooo0?ooo`3oool0]KJe024L8@0Q8240
a/_60?ooo`3oool0oooo0?ooo`3WhnL08A`Q024P8@3_lnl0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Q7f401PL8@0Q8240g]oN0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3Ngmh0@T5201PL8@0Q8240Ngek0?ooo`3Ngmh0@T52024L8@0Q8240
]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gloL0W9jL01PL8@0Q8240
@T520;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0[JZ]01PL8@2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`12@D80S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`0
8A`Q0:fZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0FUUJ024P:@0Q7240in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH08B0Q0;g3_@3oool0oooo0?ooo`3oool0in?W024L8@1cKG<0oooo0?ooo`3oool0
oooo07]mN`24PX@0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF024P:@0Q7240kno_0?ooo`3oool0
oooo0?ooo`3oool0g]_N024L8@0Q8240YJZU0?ooo`3oool0oooo0;Ff]@0Q82T0@T520?ooo`3oool0
oooo0>o[k`0Q82408A`Q09B>U03oool0oooo0?ooo`3oool0kn__024P8@0Q7240g]_N0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP0Q82T0
8A`Q0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`36`lH0c/_>0?ooo`3oool0
oooo0?ooo`3Nfmh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`0Q82T0W9ZL0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0@TE:0491@P3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0W9ZL024P8@1[KF/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S00Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q0;g3_@3oool0oooo0?ooo`3oool0e]OF01PL8@2LWY`0oooo0?ooo`3oool0[JZ]0491@P1JFEX0
in_W0?ooo`3oool0oooo0?ooo`2LWY`08A`Q0<k?cP3oool0oooo0?ooo`3oool0oooo0<K3aP0Q7240
8B0Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ocm`0Q824061`Q08AmQ03oool0oooo0?ooo`2LWY`0
8A`Q024P8@36blH0oooo0?ooo`3oool0@T52024P8@1JFEX0oooo0?ooo`3oool0oooo0?ooo`0H7240
8B0Q0;g3_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2<SX`08A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@2m`kd0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX08A`Q0;Ff]@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0491@P0Q8240
g]oN0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0H7240FUUJ0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH08B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@2DSY@0oooo0?ooo`3oool0]KJe024L8@0Q82T08A`Q024P8@0Q7240
8B0Y05YIFP2]ZZd0moOg0?ooo`3oool0oooo0?ooo`12ADX08A`Q024P8@0Q72408B0Y0<k;cP3oool0
oooo0?ooo`2e][D08B0Q024L8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Ngek024P:@12@D80
oooo0?ooo`3oool0YJZ]024P8@0Q7240Q8:40?ooo`3oool0oooo08B2Q00Q72408B0Q0?ooo`3oool0
oooo0?ooo`3oool08B0Y024L8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0a/?6024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
U8jD0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
8A`Q024P8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0<k;cP0Q82408A`Q0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Y024L8@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0491@P0Q82408A`Q0<k?cP3FemH0e]OF0=KGeP3FemH0g]oN0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe01PL8@0Q8240oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0<k?cP0Q72408B0Q0=KGeP3oool0oooo0=KGeP0H72408B0Q05YIFP3oool0oooo0?ooo`1kOG/0
8B0Q05YIFP3oool0oooo0?ooo`3oool0oooo024L8@0Q8240]KJe0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2UYZD08B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`12@D808B0Q024L8@3>clh0
e]OF0=KGeP3FemH0e]OF0=kOgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo024P8@0H7240]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ocm`1[KG<061`Q0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
]KJe024L8@0Q8240oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3>clh08A`Q024P8@1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q82T08A`Q0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0Q8:4024L8@24PX@0oooo0?ooo`3oool0c/o>09bJW03Nfmh0
oooo0?ooo`3oool0Lfec024L8@2DSY@0oooo0?ooo`3oool0oooo0?ooo`0Q82408A`Q0;ff_@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
c/o>024L8@0Q8240Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240c/o>0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJZ]024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@1[KF/0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q72408B0Y0;Ff]@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:FZ[@12ADX08A`Q024P:@2UZZd0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0;Ff]@0Q82408A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0H72408B0Q07]mN`3WhnL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>clh0
8A`Q024P8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>ock`1JFEX08A`Q0<K3aP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0kn__01PL8@0Q8240g]oN0?ooo`3oool0oooo0?ooo`3oool0
61`Q024P8@3>blh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D061`Q024P8@1kOG/0in?W0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool08A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ024L8@2]ZZd0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0@T52024L8@0Q824061`Q024P8@1kOG/0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2]ZZd061`Q05YIFP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
c/o>05YIFP0Q82408A`Q07=]L`2<SX`0W9jL09bJW03gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0g^?N024P8@0Q7240kno_0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
moOg09B>U00Q7240a/?60?ooo`3oool0oooo0?ooo`3oool0oooo08b>S00Q82T0a/?60?ooo`3oool0
oooo0?ooo`3oool0in_W024P:@0Q7240in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP1JFEX08B0Q024L8@1cKG<0
S8j<09bNW02LVY`0moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo07]mN`0Q8240W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH0
Ngek024P:@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9ZL024P:@1kOG/0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0a/_609bNW01[KF/0@T5201PL8@0Q8240e]OF0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0H72408B0Q0=kOgP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF0;Ff]@3oool0oooo0?ooo`3oool0oooo0>ock`2e][D0
g]oN0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`0Q72408B0Q0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0YJJU024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@1[KF/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0<K;aP2LWY`0Jfe[0491@P0H72408B0Q0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D061`Q08AmQ03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D0Jfec0>ock`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08AmQ00Q7240]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0U8jD024L8@36`lH0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH08B0Q08b>S03oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0g^?N024P:@1JFEX0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`1cKG<0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`1JFEX08B0Q0=kSgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0<k;cP0Q8240W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
W9jL01PL8@3>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KF/0
8B0Q0>ock`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`0
8A`Q0<k?cP3oool0oooo0?ooo`3oool0oooo0?ooo`3gloL08B0Q06]]J`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ05YIH`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo05YIFP1JFF<0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
kn__024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@12@D80[JZ]0>O[i`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
]KJe024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y0>o[k`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0a/?6024L8@36`lH0oooo0?ooo`3oool0oooo0?ooo`2]ZZd0_L>m0?ooo`3oool0
oooo0?ooo`3oool0FUUJ024P8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0a/?6024L8@36`lH0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP0Q7240e]OF0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP0Q8240e]OF0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0[JZ]024L8@2e][D0oooo0?ooo`3oool0]KJe08b>S00Q8240
W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0Q7f401PL8@3>clh0oooo0?ooo`3oool0in?W07]mN`2]ZZd0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KF/0Jfe[0?ooo`3oool0oooo0?ooo`3oool0
ko?_05YIFP2LVY`0oooo0?ooo`3_jnl0Ngek024P8@0H7240c/o>0?ooo`3oool0oooo0?ooo`3oool0
oooo06]]J`1[KF/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
W9jL06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1cKG<0W9ZL0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`12ADX0Ngek0?ooo`3oool0
oooo0?ooo`3oool0c/_>0495BP1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP0Q82T0]KJe0?ooo`3oool0
oooo0?ooo`3oool0W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0kn__0495BP3>blh0
oooo0?ooo`3oool0oooo0?ooo`3FemH08B0Q024L8@0Q82T08A`Q024P8@0Q7240Q8:40?ooo`3oool0
oooo0?ooo`3oool0oooo0<k;cP12ADX0kn__0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`24OH@0]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
Ngek0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`2m`kd08B0Q08b>S03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
@T5208AmQ03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH0FUUJ0?Ocm`3oool0oooo0?ooo`3oool0oooo0=kOgP24OH@08A`Q024P8@12@D80
]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0>ock`1JFEX0e]OF0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg08B2Q02m`kd0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`36`lH0Ngek0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0;ff_@0Q7240g]_N0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`0Q8240
g^?N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo09bNW012@D80oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k;cP24PX@0moOg0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`24PX@0c/_>0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
W9jL0?Ocm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3gmoL0W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kOgP3>clh0e]OF0?ooo`3oool0oooo0>ock`3>clh0
c/_>0>o[k`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL08B0Q0:FVY@3oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D061`Q0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2m`kd08B0Q0=KGeP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF0;Ff]@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg07]mN`0Q82408A`Q024P:@1JFEX0
kno_0=kSgP12ADX08A`Q024P8@0Q7240U8jD0?Ogm`3oool0oooo0?ooo`3oool0oooo05YIFP1JFF<0
oooo0?ooo`3oool0oooo0?ooo`3oool0c/_>024P:@2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo06]]J`0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@3Nhmh0oooo0?ooo`3oool0oooo0<K3aP1kOG/0@TE:024L8@0Q8240FUUJ07=]L`3_jnl0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH08A`Q0495BP3_jnl0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_knl08A`Q024P:@0Q72408B0Q024L8@0Q82T0e]OF0?ooo`3oool0
oooo06]]J`0Q82408A`Q024P:@0Q72408B0Q08b>S03oool0oooo0?ooo`3oool0kno_024L8@2e][D0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S00Q8240
@T520>OSi`3oool0a/?60491@P1[KF/0Ngek0?Ogm`3WjnL0Q7f4024L8@1JFEX0ko?_0?ooo`3oool0
oooo0?ooo`2LWY`08A`Q0>o[k`3oool0oooo0?ooo`3oool0oooo0<k?cP0Q7240S8jD0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`36`lH061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@1[KF/0oooo0?ooo`3oool0oooo08AmQ00H72408B0Q0491@P24OH@0
W9ZL09bNW01kOG/0c/o>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0_L>m024P8@12@D80
kn__0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJJU024P8@0Q72408B0Q01PL8@0Q8240
Jfe[0?ooo`3oool0oooo0?ooo`3>clh061`Q024P8@0Q72408B0Q01PL8@0Q8240mo?g0?ooo`3oool0
oooo0?ooo`1[KG<0Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`0Q82408A`Q09B>U03oool0oooo0?ooo`2DSY@08A`Q0<K3aP3oool0oooo0?Ogm`12ADX0
8A`Q08B2Q03oool0oooo0?ooo`3oool0c/_>024P8@2m`kd0oooo0?ooo`3oool0oooo0?ooo`2e][D0
8B0Q0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024P:@1[KF/0Q8:407]mN`24PX@0
Ngek08B2Q01kOG/0Q8:407]mN`24PX@08A`Q024P:@0Q7240a/?60?ooo`3oool0oooo0;Ff]@0Q7240
8B0Y07]mN`3gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIH`2<SX`0
W9jL09bJW02LWZD0@T520<K3aP3oool0oooo0?ooo`3oool0oooo05YIH`1kOG/0W9jL09bJW02LWZD0
FUUJ0<k?cP3oool0oooo0?ooo`3oool0]KJe0495BP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3FemH061`Q024P8@2e][D0oooo0?ooo`3oool0]KJe024P8@2UYZD0
oooo0?ooo`3oool0]KJe024P8@0Q7240c/o>0?ooo`3oool0oooo0?ooo`12@D80W9jL0?ooo`3oool0
oooo0?ooo`3oool0FUUJ0491@P3gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LVY`0
]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF024P8@0Q7240@T520?ooo`3oool0
oooo0?ooo`1kOG/08B0Q024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0>OSi`1JFEX0oooo0?ooo`3oool0YJJU0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
@T520>O[i`3oool0oooo0<k?cP2<SX`0oooo0?ooo`3oool0oooo0=KGeP0Q7240e]OF0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF024P:@0Q7240[JZ]0?ooo`3oool0
oooo0;Ff]@0Q7240@TE:0?ooo`3oool0oooo0>o_k`0Q72408B0Q08b>S03oool0oooo0?ooo`3oool0
@TE:05YIFP3oool0oooo0?ooo`2e][D0Lfec024L8@3>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0in?W05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09B>U00Q7240
8B0Q09bJW03oool0oooo0?ooo`3oool0FUUS024L8@0Q8240g^?N0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D0]KJe0?ooo`3oool0a/?6024P8@2LVY`0oooo0?ooo`3oool0
oooo0?ooo`3oool0kno_024L8@1JFF<0moOg0?ooo`3gmoL0@TE:0?ooo`3oool0oooo0?ooo`3_jnl0
8B0Q07]mN`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`0Q7240
8B0Q07]mN`3oool0oooo0?ooo`2LVY`08B0Q024L8@3>clh0oooo0?ooo`3oool0@T5201PL8@1JFEX0
oooo0?ooo`3Ngmh08B0Q024L8@0Q824061`Q024P8@0Q72408B0Q0:FVY@3WhnL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KF/0e]OF0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`0Q72408B0Q01PL8@3WhnL0oooo0?ooo`3oool0oooo08b>S00Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0g]oN024P8@0H7240
e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KF/08A`Q08b>U03oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D08B0Q01PL8@0Q82408A`Q06]]L`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0Q8:4024L8@12ADX0oooo0?ooo`3oool0[JZ]024L8@0Q8240Ngek0?ooo`3oool0
oooo07]mN`0Q82T08A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0a/?609B>U03oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Q024L8@24PX@0oooo0?ooo`3oool0oooo0?ooo`36`lH0
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08B2Q00Q72408B0Y0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJZ]024P8@0Q7240
g]_N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K;aP0Q82408A`Q0=KGeP3oool0oooo0=KGeP0Q8240
61`Q05YIFP3oool0oooo0?ooo`24OH@08A`Q05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`12@D80moOg0?ooo`3oool0oooo0?ooo`3oool0FUUJ01PL8@0Q8240c/_>0?ooo`3oool0
oooo0?ooo`3oool0oooo0491@P0Q72408B0Q0<K;aP3FemH0e]OF0=KGeP3FemH0in?W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP0H72408B0Q06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0<k?cP0H72408B0Q07]mN`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Ngek024P8@1kOG/0
oooo0?ooo`3oool0c/_>09bNY@3FemH0oooo0?ooo`3oool0Jfe[024P8@2<SX`0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0W9jU0;Ff]@3oool0oooo0?ooo`3oool0g]_N024L8@0Q82T0
@T520?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>blh08B0Q024L8@24PX@0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LVY`08B0Y024L8@2DSY@0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3Nhmh08B0Y024L8@0Q8240in_W0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`1JFEX08B0Q0;g3_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0in_W024P8@0H7240
in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kOgP1JFEX0oooo0?ooo`3oool0
oooo08b>S00Q82408A`Q0:fZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q8240
8A`Q08AmQ03Ngmh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ024L8@0Q8240
YJJU0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024L8@0Q824061`Q0<k?cP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0moOg08b>S00Q8240_L>m0?ooo`3oool0oooo0?ooo`3oool0
oooo09B>U00Q7240a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
Jfe[0=kKgP3oool0oooo0?Ogm`0Q82T08A`Q024P8@3WjnL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0c/_>05YIFP0Q72408B0Y06]]J`2DSY@0W9ZL09bNY@3gmoL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo024L8@0Q82408A`Q0;ff_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q8240
8A`Q024P:@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF0;Ff]@3oool0
oooo0?ooo`3oool0oooo0?Ogm`2e][D0in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0<K3aP2<SX`0oooo0?ooo`2e][D08A`Q024P8@1kOG/0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0c/o>09bJW01[KG<0@T52024P8@0Q7240
e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`0Q824061`Q024P8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool061`Q024P8@0Q7240]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q7240g]_N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0@TE:0?Ogm`3oool0FUUJ024P8@0Q7240
c/o>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ024P:@0Q7240[JZ]0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024P:@0Q72408B0Q0<k;cP3oool0oooo0:fZ[@0Q7240
8B0Y024L8@0Q82408A`Q024P:@12@D80oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0c/o>01PL8@0Q82408A`Q024P8@0H72408B0Q0:FVY@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bJW02e][D0
e]OF024P8@0H7240FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S00Q7240
8B0Q08b>S03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`0Q72408B0Q01PL8@3_jnl0
oooo0?ooo`3oool0[JZ]024L8@0Q824061`Q024P8@0Q72408B0Q0<K;aP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>clh0Ngek0?ooo`3oool0oooo0?Ogm`2DSY@0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3WhnL0FUUJ09B>U00Q72408B0Y0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH08B0Q024L8@1JFF<0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH0
8B0Q024L8@24PX@0oooo0?ooo`3oool0oooo0?ooo`2]ZZd0]KJe0?ooo`3oool0oooo0=kSgP2]ZZd0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP2<SX`0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]L`0H72408B0Q024L8@3_jnl0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`0Q82408A`Q0=KGeP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0Q7f401PL8@0Q8240g]oN0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@2e][D0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0_KJm0:FZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2m`kd08B0Y024L8@2DSY@0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0FUUJ024P8@1[KF/0
oooo0?ooo`3oool0oooo0?ooo`3oool0c/o>024L8@0Q82T0a/?60?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0U8jD0<k;cP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SY@0]KJe0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
moOg0491@P0Q8240e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`2<SX`08B0Q07]mN`3WhnL0oooo0?ooo`3gloL0]KJe01PL8@12@D80c/_>0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0e]OF0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg0495BP3Nhmh0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2LWY`0FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kSgP2LWY`0Jfe[05YIH`1JFEX0FUUJ07]mN`36`lH0
moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0c/_>05YIFP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`1kOG/0S8jD0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0in_W0=KGeP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`12@D80]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0[JZ]024L8@3_knl0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0Lfec05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D0oooo0?ooo`3oool0oooo08b>S00Q8240g]oN0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP3>clh0oooo0?ooo`3oool0oooo05YIFP12@D80oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJZU0:fZ[@3oool0oooo0;Ff]@0Q82T0
Jfe[0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Ngek0<K3aP3oool0oooo07]mN`0Q8240
W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
@T5201PL8@0Q8240W9ZL0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`2LVY`0
8B0Q024L8@12@D80]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo00h0000D000000000100000D0000
\>"], "PlacedGraphics",
  ImageSize->{432.563, 71.9375},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}],

Cell[GraphicsData["Metafile", "\<\
CF5dJ6E]HGAYHf4PEfU^I6mgLb15CDHPAVmbKF5d0@000oa`0@0006`000000000000008L2001T0000
00000000002T@P00D`X00215CDH00040L?`300<000010000000000000000000000@00003000>0@00
b`0000000000000000000;0N103h60<0D@000?3k0`000000000008L2001T00000000000000000000
000008P2001U0000D00002P0001h0000N?/30000000P0<`0R08006D0000X0000QP8006D000010200
000007Sk0`0000000000000000000000oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool08B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@0Q82T0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP0Q824061`Q024P8@0Q7240
8B0Q06]]J`36`lH0oooo0?ooo`1kOG/08B0Q024L8@2LWY`0oooo0?ooo`2LVY`08B0Q06]]J`2e][D0
Jfe[0<K3aP3oool0]KJe024L8@0Q8240]KJe05YIFP2m`kd0oooo0=KGeP1JFEX08A`Q024P8@2e][D0
oooo0?ooo`3oool0oooo0?ooo`1JFEX08B0Q01PL8@0Q82408A`Q024P8@0H7240Jfe[0?ooo`3oool0
YJJU024P8@0Q7240Jfec0>O[i`3FemH08A`Q024P8@0H7240kn__0?ooo`3_jnl0Jfe[024P8@0Q7240
Q7f40=KGeP0Q82408A`Q024P8@2LVY`08B0Q024L8@24OH@0ko?_05YIFP1JFEX0in?W01PL8@0Q8240
8A`Q024P8@0H7240Q7f40?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3Ngmh08B0Q01PL8@1JFEX0_L>m0?ooo`3oool0oooo024L8@2e][D0
oooo05YIFP0Q7240W9jL0>ock`3oool0oooo0?ooo`3oool0e]OF0491@P0Q8240Jfe[0?Ogm`3oool0
oooo0?ooo`12@D80_L>m0?ooo`3oool0oooo08b>S00Q824061`Q06]]J`3FemH0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0Ngek024P8@0H72408B0Q024L8@1[KG<0oooo0?ooo`3FemH0
8B0Q01PL8@0Q8240_L>m0?ooo`3oool0Q7f4024L8@0Q8240FUUJ0?ooo`3oool0@T5201PL8@0Q8240
Ngek0>o[k`3oool0oooo0;Ff]@0Q8240oooo0?ooo`3WjnL0Jfec01PL8@1JFEX0g]oN0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo07]mN`0Q824061`Q05YIFP3oool0oooo08b>S00Q82408A`Q08b>U03_lnl0oooo0?ooo`3FemH0
61`Q0=KGeP3WjnL08B0Q0491@P2e][D0oooo0?ooo`3oool0e]OF024L8@0Q824061`Q024P8@0Q7240
c/o>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0]KJe024P8@0H7240W9jL0?ooo`3oool0S8j<024P8@0Q7240S8jD0>ock`3oool0
oooo0?ooo`2e][D08B0Q024L8@36`lH0oooo0?ooo`2e][D08B0Q0?ooo`3oool0oooo08AmQ00H7240
FUUJ0=kOgP3oool0oooo0?ooo`2e][D0FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q82408A`Q024P8@2LVY`0oooo0?ooo`3oool0
]KJe024P8@0Q7240a/?60?ooo`3oool0oooo0?ooo`3oool0W9jL05YIFP3oool0oooo0;Ff]@0Q7240
oooo0?ooo`2e][D08A`Q024P8@12@D80]KJe0?ooo`3oool0oooo08AmQ00Q72408B0Q0;g3_@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0H72408B0Q024L8@3_jnl0oooo0=KGeP0Q7240
e]OF0?ooo`3oool0Ngek024P8@1JFEX0in?W0?ooo`3_jnl061`Q0491@P2e][D0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool061`Q024P8@0Q7240W9jL0?ooo`3oool0oooo0?ooo`3oool0W9jL05YIFP3oool0
oooo0;Ff]@0Q7240oooo0=kOgP0Q82408A`Q05YIFP2m`kd0oooo0?ooo`2e][D061`Q024P8@0Q7240
8B0Q01PL8@3gmoL0oooo0?ooo`0H72408B0Q024L8@2LWY`0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9ZL05YIH`3oool0
oooo0=kSgP1cKG<08A`Q0;Ff]@3oool0oooo06]]J`2DSY@0oooo0?ooo`3gmoL08B0Q0:FZY@3gmoL0
Jfe[024P8@0Q7240g]_N024L8@2LWY`0oooo09B>U00Q72408B0Q0?Ogm`12ADX0Jfe[0>o_k`3oool0
_KJm0?Ogm`3oool0oooo0?ooo`3oool0oooo0:FZY@0Q82T0moOg0?ooo`3oool0in?W05YIFP3gmoL0
_L>m024P:@2e][D0oooo0:FZY@24PX@0oooo0<k?cP0Q7240in?W0?ooo`3oool0Ngek0495BP3FemH0
oooo0=KGeP3>clh0c/_>024P8@3Nhmh0oooo09bJW012ADX0oooo0=kKgP0Q7240in?W0;Ff]@2]ZZd0
8A`Q0>OSi`3oool0oooo09bJW03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg07=]L`3>blh0oooo0?ooo`3oool0
oooo0?ooo`3oool0moOg09bJW01JFF<0kn__0?ooo`3oool0kno_0491@P2]ZZd0oooo09B>U01kOG/0
oooo0?ooo`3oool0YJZ]09B>U03oool0oooo0?ooo`3oool0oooo0?ooo`3FemH08B0Q0>O[i`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/0in?W0?ooo`3oool0]KJe0>OSi`3oool0
oooo0=kSgP0Q82T0c/_>0?ooo`3oool0oooo0?ooo`2DSY@0Jfe[0?ooo`3oool0oooo0?ooo`3oool0
oooo0;Ff]@1JFEX0oooo0?ooo`3oool0oooo0?ooo`3oool0Lfec06]]J`3oool0]KJe0495BP3gmoL0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0S8j<07=]L`3oool0oooo0?ooo`3oool0oooo0?ooo`2m`kd0Lfec0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3WhnL0Ngek08B2Q03oool0oooo0?ooo`3Nfmh0W9ZL0?ooo`3oool0
moOg0:FZY@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0c/_>024P8@3>blh0moOg05YIFP2e][D0oooo0?ooo`3oool0oooo0;g3_@1cKG<0
oooo0?ooo`3FemH08B0Y0=kSgP3Nfmh08A`Q0=kKgP3oool0oooo0?ooo`3oool0oooo0;Ff]@1JFEX0
moOg0;Ff]@12ADX0moOg0?ooo`3oool0moOg024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0495BP2UZZD0oooo0?ooo`3oool0
e]OF024P:@3Nhmh0g]_N024L8@3Nfmh0oooo0?ooo`3oool0oooo09bJW01JFEX0oooo0?ooo`3oool0
oooo0?ooo`3WhnL08A`Q0:fZ[@3oool0a/?6024L8@3Nfmh0oooo0:fZ[@12@D80kno_0=kSgP0Q82T0
e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0_KJm024L8@3gmoL0oooo0?ooo`3oool0
oooo0?ooo`3oool0]KJe05YIFP3gmoL0_KJm0491@P3gmoL0oooo0?ooo`3Nhmh0Q8:407]mN`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`12@D80[JZ]0?ooo`3oool0oooo0?ooo`3oool0oooo09bJW01JFEX0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0Lfec0<k;cP3oool0oooo0;ff_@36`lH0
oooo0?ooo`3WhnL0]KJe0?ooo`3oool0oooo0491@P2]ZZd0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@1JFEX0
oooo0?ooo`3oool0oooo06]]L`0H7240kn__0?ooo`24OH@0W9ZL0?ooo`3oool0kn__01PL8@36`lH0
oooo0<K3aP0H7240kn__0:FVY@12@D80oooo0?ooo`3FemH08B0Q0=kOgP2e][D08A`Q0=KGeP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D08A`Q0?ooo`3oool0oooo0?ooo`3FemH0
]KJe06]]J`1JFEX0oooo0?ooo`3WhnL08A`Q0>OSi`3FemH08B0Q0>O[i`3oool0g]oN024P8@2e][D0
oooo0?ooo`3oool0oooo0=KGeP0H7240kn__0?ooo`2LWY`0FUUJ0?ooo`3FemH08B0Q0>O[i`3oool0
oooo06]]L`1JFEX0moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3Ngmh0FUUJ0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0c/_>06]]L`3oool0oooo0:FVY@12@D80ko?_0?ooo`3Ngmh0
8B0Q0=kOgP3oool0oooo0=KGeP0H7240moOg0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S02]ZZd0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`3_jnl0oooo0?ooo`3oool0
oooo0?ooo`3_jnl08A`Q0=KGeP3oool0oooo0?ooo`3oool0W9ZL08AmQ03oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0@T520<k;cP3oool0oooo0?ooo`3oool0c/o>01PL8@3FemH0oooo0?ooo`12@D80
]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo09bNW01kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S036`lH0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2UYZD0[JZ]0?ooo`3oool0oooo0=KGeP2UYZD0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo06]]L`1kOG/0oooo0?ooo`2e][D0@T520?ooo`3oool0oooo0?ooo`3oool0
S8j<0<K3aP3oool0S8j<08b>S03oool0oooo08b>S02<SX`0oooo0?ooo`3oool0oooo0?ooo`12@D80
]KJe0?ooo`3oool0@T520;Ff]@3oool0oooo0?ooo`24OH@0YJJU0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0]KJe0?ooo`3oool0
oooo08b>S02<SX`0oooo0?ooo`2<SX`0S8j<0?ooo`3oool0oooo0?ooo`2LWY`0FUUJ0?ooo`3oool0
oooo0?ooo`3oool0c/_>024P8@3WjnL0oooo0=kOgP0Q8240_L>m0?Ogm`0Q7240]KJe0?ooo`3oool0
S8j<08b>U03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;g3_@0Q8240oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0491@P2e][D0oooo0?ooo`12@D80]KJe0?ooo`3oool0oooo0?ooo`2]ZZd0
YJJU0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`2LWY`0
FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kOgP1JFEX0oooo0?ooo`3oool0
]KJe0<K;aP3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0]KJe0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
FUUJ0?ooo`3oool0oooo0?ooo`3>blh08B0Y0:FZ[@3oool0Ngek09bNY@3oool0oooo0>O[i`0Q82T0
a/?60?ooo`2m`kd08B0Y0>o[k`2LWY`0FUUJ0?ooo`3oool0g]_N024L8@3_knl0]KJe024P8@3oool0
oooo0?ooo`3oool0in_W024P:@0Q72408B0Q07]mN`3oool0]KJe024P8@3oool0oooo0?ooo`3oool0
oooo0?ooo`1JFEX0W9jL0?ooo`3oool0kn__024P8@2m`kd0g]_N024L8@3_knl0oooo0=kKgP0Q7240
g]_N0?ooo`3oool0oooo0?ooo`3FemH08B0Y0>o[k`3oool0W9ZL05YIH`3oool0g]_N024L8@3_knl0
oooo0?ooo`3WjnL0@TE:08b>S03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0in?W024L8@3oool0
oooo0?ooo`3oool0oooo0>O[i`1cKG<08A`Q07=]L`12@D80a/?60?ooo`2DSY@0Ngek0?ooo`3oool0
kno_024L8@3>clh0oooo0?ooo`3oool0FUUS0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`1cKG<0
W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0Jfe[0>o_k`3oool0
oooo0?ooo`3oool0kn__024P8@3FemH0oooo0?ooo`3oool0oooo09bNY@1kOG/0oooo0?ooo`3oool0
oooo0?ooo`3oool0moOg024L8@36`lH0oooo0?ooo`3oool0oooo0;Ff]@0Q82T0oooo0?ooo`3oool0
FUUS0:FZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2LVY`0Q8:40?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2DSY@0
]KJe0?ooo`3oool0oooo0?ooo`3oool0e]OF0495BP0Q7240Q8:4024L8@3_knl0oooo0?ooo`3oool0
]KJe0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`12@D80_KJm0?ooo`3oool0_L>m024P:@3gmoL0oooo0?ooo`3oool0
oooo09B>U02e][D0oooo0491@P2e][D0oooo0?ooo`2e][D0@TE:0?ooo`3oool0oooo0?ooo`3Nhmh0
8B0Y0=kSgP3oool0oooo08B2Q02<SX`0oooo0?ooo`3oool0a/?605YIFP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUS0;Ff]@3oool0
oooo0?ooo`12@D80]KJe0?ooo`3oool0]KJe0495BP2m`kd08B0Y024L8@0Q82408A`Q024P:@0Q7240
g]_N0?ooo`3oool0oooo0?Ogm`1JFEX0c/o>0?Ogm`1cKG<0@T520>o_k`3Nhmh08B0Q0=KGeP3oool0
oooo09bNW01[KF/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH08A`Q0?ooo`3oool0
oooo0?ooo`3oool0oooo0>OSi`0Q7240in?W0?ooo`3oool0Ngek09B>U03oool0g]_N0491@P0Q82T0
Ngek024P8@3WjnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP2m][d0oooo0?ooo`2m`kd08B0Y024L8@0Q8240
8A`Q024P:@0Q7240g]_N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WhnL08A`Q0?ooo`3oool0
oooo0?ooo`2DSY@0e]OF0?ooo`3oool0oooo0?ooo`3oool0FUUJ0;ff_@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
]KJe05YIFP3oool0oooo0?ooo`3oool0kn__024L8@2<SY@0oooo08AmQ02LVY`0oooo0?ooo`3_jnl0
8A`Q0<K3aP3oool0a/?6024L8@3_jnl0]KJe0491@P3oool0oooo0<K;aP0Q8240in_W0<K3aP0H7240
8B0Q024L8@0Q824061`Q0>o[k`3oool0oooo0?ooo`3oool0oooo0;Ff]@0H7240oooo0?ooo`3oool0
oooo0?ooo`3oool0Jfec08b>S03oool0oooo0<k?cP0H7240c/o>0=KGeP0Q8240in_W0?ooo`3WjnL0
8B0Q01PL8@0Q82408A`Q024P8@2m`kd0c/o>024L8@3_jnl0oooo09bNW01JFEX0oooo0=KGeP0Q8240
in_W0?ooo`3oool0oooo0<k;cP0Q8240]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:FVY@1JFEX0
oooo0?ooo`3oool0oooo0?ooo`24OH@0@T520>o[k`3_lnl0Jfe[06]]J`3oool0S8j<08AmQ03oool0
oooo0<K;aP0Q8240e]OF0?ooo`3oool0oooo0:FVY@2<SY@0oooo0?ooo`3oool0oooo0?ooo`2]ZZd0
8A`Q0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP1kOG/0
oooo0?ooo`3oool0oooo0>o[k`0H7240e]OF0?ooo`3oool0oooo0?ooo`2LVY`0Q7f40?ooo`3oool0
oooo0?ooo`3oool0moOg07]mN`0Q8240g]oN0?ooo`3oool0oooo0?ooo`2e][D08A`Q0?ooo`3oool0
moOg024L8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0W9jL07]mN`3oool0oooo0?ooo`3oool0oooo0?ooo`3_jnl0
@T520<K3aP3oool0oooo0?ooo`3oool0mo?g0491@P1[KF/0oooo0=kOgP12@D80YJJU0?ooo`3oool0
oooo0?ooo`24OH@0e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0@T520;Ff]@3oool0oooo0:fZ[@0Q7240oooo0?ooo`3oool0
oooo0>o[k`12@D80a/?60?Ocm`0Q8240_L>m0?ooo`3oool0a/?601PL8@3oool0oooo0?ooo`3oool0
c/o>024L8@3_jnl0oooo0?ooo`2LVY`0Q7f40?ooo`3oool0oooo0?Ogm`0H7240in?W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP2e][D0
oooo0?ooo`3gloL08B0Q0;g3_@3oool0oooo0<K3aP0H7240moOg07]mN`3oool0oooo09bNW01JFEX0
oooo0?ooo`3oool0oooo0?ooo`3oool0kn__08b>S01JFEX0@T520=KGeP3oool0in?W01PL8@3FemH0
oooo0?ooo`1[KF/0Q7f40?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0_L>m024P8@3oool0
oooo0?ooo`3oool0oooo0?ooo`36blH08B0Q0>O[i`3oool0oooo09bNW01kOG/0moOg0491@P1[KF/0
oooo0>OSi`12@D80[JZ]0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0]KJe0?ooo`3oool0moOg07]mN`3oool0
oooo09bNW01JFEX0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJJU05YIFP3oool0
oooo0?ooo`3oool0mo?g05YIFP3_lnl0oooo0?ooo`3oool0oooo05YIFP2e][D0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0;Ff]@1JFF<0oooo0?ooo`3oool0oooo0>o[k`0Q8240S8j<09B>U00Q7240Lfec0?Ogm`3oool0
g^?N024P8@2m`kd0oooo0;Ff]@0Q8240in_W0?Ogm`12@D80]KJe0?ooo`24PX@08A`Q0>o_k`3gmoL0
@TE:0=KGeP3oool0Ngek05YIH`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Y0?ooo`3oool0
oooo0?ooo`3oool0oooo0;g3_@12ADX0g^?N0>o_k`1JFEX0FUUS0=kSgP0Q82408A`Q0<K3aP3oool0
oooo07]mN`2m][d0oooo0;Ff]@0Q7240in?W024L8@0Q8240W9ZL0?ooo`1kOG/0Lfec0?ooo`3Nfmh0
8A`Q0>o_k`3oool0W9jU0?Ogm`3oool0YJZU024P:@3Nhmh0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0in?W07]mN`0Q8240
c/_>0?ooo`3oool0oooo0?ooo`3oool08A`Q0:fZ[@3oool0oooo0;Ff]@12ADX0oooo0:fZ[@12@D80
g]_N0?Ogm`1cKG<0@T520?Ogm`3oool0oooo0?ooo`3WhnL08A`Q0?Ogm`3oool0oooo0=KGeP0Q82T0
8A`Q09bNW03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
Lfec0;g3_@3oool0oooo0?ooo`3WjnL08B0Y0=KGeP3oool0oooo0?ooo`3oool0W9jL07]mN`3oool0
oooo0?ooo`3oool0_KJm024L8@0Q8240_L>m0?ooo`3oool0oooo0?ooo`3oool0c/_>024P8@2e][D0
oooo09bJW00Q8240e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bJW024PX@0oooo0?ooo`3oool0oooo0?ooo`2e][D0
@T5207=]L`3oool0oooo0?ooo`3oool0oooo0=kKgP0Q7240c/o>0?ooo`3oool0Ngek09B>U03oool0
oooo0?ooo`3oool0c/_>05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`1cKG<0in_W0>OSi`12@D80Q8:40?ooo`3oool0
oooo0;Ff]@12@D80Lfec0?ooo`3gmoL08A`Q0<K3aP3oool0oooo0;g3_@0Q82T0oooo0?ooo`3oool0
oooo0<k;cP0Q8240in_W0?ooo`3oool0W9jL07]mN`3oool0oooo0?ooo`3oool0U8jD0:FZ[@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0
]KJe0?ooo`3oool0moOg024L8@36`lH0oooo0?ooo`2m`kd08B0Y0?ooo`3>clh0YJZU0?ooo`2LVY`0
FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP0Q8240@T5209bNY@3oool0oooo0?Ogm`0Q82T0
Ngek0?ooo`2m`kd08B0Y0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K3aP0Q7240
oooo0?ooo`3oool0oooo0?ooo`3oool0c/o>024L8@3_knl0oooo0?ooo`2LVY`0Q8:40=KGeP0Q82T0
c/_>0?ooo`3oool0Q8:408b>S03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ0;Ff]@3oool0oooo0?ooo`3>clh0
YJZU0?ooo`2LVY`0FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WhnL0Ngek024P:@3>blh0
oooo0?ooo`3oool0oooo0?ooo`2UZZD0W9jU0?ooo`3oool0oooo0?ooo`1JFEX0]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D0FUUJ0?ooo`3oool0oooo0?ooo`3>clh061`Q0:fZ[@3WjnL0Q7f406]]J`1JFEX0
8A`Q024P8@0H7240a/?60=KGeP0Q824061`Q0>o[k`3oool0e]OF05YIFP0Q8240S8j<024P8@3WjnL0
oooo0=KGeP1JFEX061`Q05YIFP3Ngmh0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe024L8@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe024P8@0H7240Jfe[0>O[i`3oool0]KJe0491@P1kOG/0
8B0Q0:FVY@3_jnl0Jfe[024P8@0H7240a/?60?ooo`2e][D0@T520<K3aP12@D808B0Q0<K;aP3_jnl0
@T52024P8@0H7240W9jL09bJW00Q824061`Q024P8@0Q7240Jfec0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0@T52024L8@24OH@0
e]OF0?ooo`3oool0oooo0?ooo`3oool0mo?g024P8@2m`kd0oooo0?ooo`2e][D0@T520?ooo`3Ngmh0
8B0Q06]]J`0Q8240@T520=KGeP3oool0oooo0?ooo`3oool0oooo05YIFP2m`kd0oooo0?ooo`3oool0
in_W06]]L`2UYZD0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo07]mN`2<SX`0oooo0?ooo`3oool0kn__024L8@3FemH0oooo0?ooo`3oool0oooo09bJW024OH@0
oooo0?ooo`3oool0oooo0?ooo`3FemH0FUUJ0<k?cP3oool0oooo0?ooo`3oool0oooo0?ooo`12@D80
Jfe[024L8@0Q8240]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LWY`0Ngek0?ooo`3oool0oooo0;Ff]@0Q8240
@T520:fZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`36blH08B0Q0>O[i`3oool0oooo08AmQ02<SX`0
oooo0?ooo`3oool0oooo0=KGeP0H7240in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D061`Q06]]J`0Q7240Jfec0>O[i`3oool0
]KJe024P8@12@D80[JZ]0?ooo`3oool0oooo0491@P2e][D0oooo0?ooo`2e][D0@T520?ooo`3oool0
oooo0?ooo`3WhnL061`Q0=KGeP3oool0oooo08b>S02<SX`0oooo0?ooo`3oool0oooo0;g3_@1JFEX0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
FUUJ0;Ff]@3oool0oooo0?ooo`12@D80]KJe0?ooo`3oool0]KJe0491@P3oool0oooo0:fZ[@3>blh0
W9jL05YIFP3oool0oooo0?ooo`3oool0oooo0?Ocm`12@D80Jfe[0?Ogm`2UYZD0S8jD0?ooo`3oool0
S8j<0491@P1JFEX08B0Q07]mN`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2m`kd0
8B0Q0?ooo`3oool0oooo0?ooo`3oool0oooo0=kOgP0Q8240e]OF0?ooo`3oool0S8jD08b>S03>clh0
8A`Q0>o[k`3oool0oooo07]mN`2<SY@0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP2e][D0oooo0?ooo`3oool0
oooo0:fZ[@3>blh0W9jL05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`12@D8061`Q08AmQ03FemH0
oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe0491@P3oool0oooo0?ooo`3oool0FUUJ0;Ff]@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0]KJe05YIFP3oool0oooo0?ooo`3oool0Jfe[024P:@3FemH0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP0Q7240
kno_0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@0Q8240
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0>OSi`0Q7240kno_0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@3Nfmh0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`12@D80_KJm0?ooo`3oool0Ngek08B2Q03oool0
oooo09bJW01cKG<0moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2UZZd0U8jD0?ooo`3oool0
oooo0?ooo`3gmoL08B0Y0=kSgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@3oool0
oooo0?ooo`1JFF<0S8j<0?ooo`3oool0oooo0>o[k`0Q8240e]OF0?ooo`3oool0oooo0?ooo`2LWZD0
Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP12@D80oooo0?ooo`3oool0oooo0?ooo`3oool0
a/?60491@P3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9ZL08B2Q03oool0oooo0?ooo`3gmoL0
S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0in?W024L8@3WhnL0oooo0?Ogm`12@D80
]KJe0=KGeP3>clh0oooo0?ooo`2m`kd08B0Y0=kSgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIH`2LVY`0oooo0?ooo`3oool0
oooo0?Ogm`2<SX`0oooo0?ooo`3oool0oooo0?ooo`2<SX`0U8jD0?ooo`3oool0S8j<09B>U03oool0
oooo0?ooo`3oool0oooo0495BP2e][D0oooo0?ooo`12ADX0]KJe0?ooo`3oool0oooo0?ooo`3gmoL0
8A`Q0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo05YIH`2e][D0oooo0?ooo`3oool0S8j<09B>U03oool0oooo08b>S02DSY@0oooo0?ooo`3oool0
Q8:408b>S01JFF<0oooo0?ooo`3oool0oooo0?ooo`3Nfmh08A`Q0<k?cP3oool0g]_N024L8@3WhnL0
oooo0>o_k`0Q7240a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
a/?6024L8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0@T520;ff_@3oool0oooo0491@P2m][d0
g^?N024P8@3Nhmh0oooo0?Ogm`12ADX0]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0_KJm0?ooo`3oool0
oooo0?ooo`3oool0Q8:408b>S01JFF<0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe0=kKgP3oool0
oooo0?ooo`3oool0]KJe0?Ogm`3oool0oooo09bJW012ADX0oooo0?ooo`3oool0oooo05YIFP2m][d0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo09bNW01JFEX0oooo0?ooo`3WhnL0Jfe[024P8@2UYZD0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`0H7240
8B0Q0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2]ZZd0
61`Q0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0]KJe0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0
Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe06]]J`3gmoL0a/_6024P8@3>blh0
oooo0?ooo`3oool0Ngek05YIFP2e][D0oooo0?ooo`2]ZZd0g]oN0?ooo`3oool0in?W01PL8@3gmoL0
]KJe0<K3aP3oool0a/?6024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2m`kd0
]KJe0?ooo`2e][D08A`Q0<K3aP3oool0oooo0;Ff]@0Q824061`Q0=KGeP3oool0oooo0?ooo`12@D80
8A`Q08AmQ03oool0oooo0?ooo`2LWY`0g]oN0?ooo`2LVY`0@T520?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D061`Q0:fZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0@T52024P8@1kOG/0oooo0?ooo`3oool0
oooo0:fZ[@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`24OH@0W9ZL0?ooo`2UYZD0
@T520>O[i`3oool0Ngek0>o[k`3WjnL0FUUJ05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL0FUUJ06]]J`3FemH0
oooo0?ooo`3oool0[JZ]0=KGeP3oool0oooo0?ooo`3oool0e]OF01PL8@3WhnL0g]oN024P8@3FemH0
oooo0?ooo`3oool0oooo0?ooo`2e][D0FUUJ0>ock`2e][D0@T520?Ogm`3WjnL0W9jL0?ooo`3oool0
oooo08b>S02UYZD0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
g]oN024P8@0Q7240]KJe0?ooo`3oool0oooo0=KGeP0H7240in?W0=kOgP0Q8240e]OF0?ooo`3oool0
oooo0=kOgP0Q8240FUUJ0?ooo`3oool0oooo0?ooo`3oool0ko?_024P8@2UYZD0oooo0:FVY@0Q8240
mo?g0?ooo`3oool0e]OF024L8@2<SY@0in_W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
S8jD01PL8@0Q8240oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;Ff]@1JFEX0moOg0;Ff]@12@D80
mo?g0?ooo`1kOG/0W9jL0?ooo`2]ZZd0@T520>o[k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`0H72408B0Q0;Ff]@3oool0
oooo0?ooo`3oool0oooo0=kOgP0Q8240FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`1kOG/0
oooo0?ooo`3oool0oooo0>o[k`1kOG/0oooo0=KGeP0Q8240W9ZL0?ooo`3oool0in?W01PL8@0Q8240
]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo05YIFP0Q72408B0Y024L8@0Q82408A`Q07=]L`2e][D0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
_KJm0491@P3_knl0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0
8A`Q024P:@0Q7240a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo09bNY@0Q72408B0Q05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`2DSY@08A`Q024P:@2e][D0
oooo0?ooo`3oool0oooo0?ooo`2e][D0@TE:024L8@3_knl0g^?N024P:@0Q72408B0Q024L8@0Q82T0
e]OF0?ooo`2LVY`08B0Y024L8@2LWY`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08b>S00Q82T08A`Q09bNW03oool0oooo0?ooo`3oool0]KJe05YIH`3FemH0oooo0?ooo`3oool0
g^?N09B>U01kOG/0oooo0?ooo`3oool0moOg07=]L`0Q72408B0Q0;g3_@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0<K3aP1[KF/08B0Q0;g3_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`2<SX`0Q8:40?ooo`3oool0
oooo0?ooo`3_jnl08B0Q024L8@0Q82T0a/?60?ooo`3oool0oooo0?ooo`3oool0in_W07=]L`0Q7240
@TE:0=KGeP3oool0oooo0=kKgP1JFEX08B0Y05YIFP3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`2LWZD0
8A`Q05YIFP3oool0oooo0>o[k`0Q82408A`Q024P:@36`lH0oooo0?ooo`36`lH08A`Q024P8@2e][D0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bJW00Q82T0@T520=kKgP3oool0oooo08b>S00Q8240
8A`Q024P:@0Q7240Q8:40?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`36`lH0Lfec0;Ff]@3oool0oooo0?ooo`3oool0a/?6024L8@0Q8240]KJe0?ooo`3oool0
oooo0?ooo`3oool0]KJe05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH0@TE:024L8@12ADX0
a/?60?ooo`3oool0oooo0?ooo`3WhnL0Ngek024P:@2LVY`0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?Ogm`2LWZD08A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9jL024L8@12ADX0
e]OF0?ooo`3oool0kno_06]]J`0Q8240@T520=kKgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0a/?606]]J`2e][D0
oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
W9jL024L8@0Q82T0FUUJ0?ooo`3oool0_KJm024L8@0Q8240Jfe[0?Ogm`3oool0oooo0?ooo`36`lH0
Jfe[0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0_L>m024P8@0H72408B0Q024L8@0Q8240
61`Q09bNW03gloL0oooo0;Ff]@0Q82408A`Q05YIFP3oool0oooo0;g3_@0Q8240@T520<K3aP12@D80
]KJe0?ooo`3_jnl0Jfe[024P8@1kOG/0Q7f408b>S03gmoL0in_W06]]J`0Q72408B0Q07]mN`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;g3_@0Q8240Jfe[0<K3aP3oool0
oooo0?ooo`3oool08A`Q0;Ff]@3oool0oooo0;Ff]@0Q824061`Q0<K3aP3oool0oooo0?ooo`3FemH0
@T52024P8@1[KF/0moOg0?ooo`3oool0S8j<024P8@0Q7240Jfec0=KGeP3oool0]KJe024P8@0H7240
Jfe[0=kOgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o[k`1JFEX08B0Q01PL8@0Q8240
8A`Q0:fZ[@3oool0oooo07]mN`0Q824061`Q05YIFP3oool0oooo0;Ff]@0Q82408A`Q06]]L`3Ngmh0
oooo0?ooo`2e][D061`Q024P8@1[KF/0in?W0?ooo`3oool0e]OF024P8@3FemH0oooo0?ooo`24OH@0
61`Q024P8@2m`kd0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0Ngek024P8@0H7240FUUJ0?ooo`3oool0in_W06]]J`0Q7240
8B0Q07]mN`3_jnl0oooo0=KGeP0H7240e]OF0=kOgP0Q824061`Q024P8@2LVY`0oooo0?ooo`3oool0
oooo08AmQ00H72408B0Q0;g3_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP2LWY`0oooo0?ooo`12@D80
8B0Q01PL8@24OH@0in_W0?ooo`3oool0oooo07]mN`0Q824061`Q05YIFP3oool0oooo0;Ff]@0Q8240
oooo0?ooo`3oool0oooo0>O[i`0Q8240_L>m0?ooo`3oool0kn__06]]J`0Q8240FUUJ0>OSi`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL08B0Y0<K3aP3oool0
oooo0;ff_@0Q7240FUUJ0>O[i`3oool0YJZ]0495BP3oool0oooo0?ooo`1cKG<0Jfe[0?ooo`2LVY`0
8B0Q024L8@3Nfmh0Ngek0495BP3Nhmh0in?W024L8@0Q8240YJZU08B2Q012@D80g]_N0?ooo`3Nfmh0
c/_>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K3aP1[KF/0
]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP0Q8240g^?N0=kKgP0Q7240g]_N0?ooo`3_knl0
@T520:fZ[@3oool0U8jD07]mN`3oool0]KJe024P:@3>blh0oooo09bJW012ADX0moOg0?ooo`3oool0
oooo0=kSgP1JFEX0in_W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0kno_07]mN`3gmoL0
oooo0?ooo`2LVY`0oooo0?ooo`3oool0S8j<07=]L`3oool0oooo0?ooo`3oool0oooo0?ooo`3Nhmh0
FUUS0>o[k`3oool0oooo0?ooo`3oool0in?W05YIFP3_knl0oooo0?ooo`3oool0oooo0?ooo`2]ZZd0
@T520>o_k`3Nhmh08B0Q0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2<SX`0Lfec0?ooo`3oool0oooo09B>U012@D80
kno_0?Ogm`1cKG<0Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Q8:409bJW03oool0
oooo0:fZ[@12@D80kno_0=kSgP0Q8240e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0W9ZL0?ooo`3oool0
oooo0?ooo`3oool0]KJe05YIFP3oool0oooo0?ooo`3oool0S8j<07=]L`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_knl08A`Q0<K3aP3oool0oooo06]]J`1cKG<0oooo0;Ff]@12@D80
moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024L8@36`lH0
oooo0?ooo`3oool0c/o>01PL8@2LWY`0oooo0;Ff]@1JFEX0oooo0?ooo`3oool0FUUJ09bNW03oool0
kn__01PL8@36`lH0mo?g024P8@2e][D0oooo0?ooo`24OH@0S8j<0>OSi`0Q7240]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo06]]J`3>clh0oooo0?ooo`3oool0oooo0?ooo`2<SY@0S8j<0?ooo`3oool0S8jD08b>S03oool0
YJJU0491@P3_lnl0oooo0=kOgP0Q8240g]oN0:fZ[@1JFEX0oooo0?ooo`2e][D08A`Q0>o[k`3oool0
oooo0?ooo`3oool0]KJe09bNW03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_jnl0
Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo09bNW01kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0;Ff]@2LWY`0oooo0?ooo`3oool0oooo0?ooo`2e][D0W9ZL0?ooo`3oool0oooo0?ooo`3gmoL0
61`Q0;Ff]@3oool0oooo08b>S02<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9jL07]mN`3oool0oooo0?ooo`1[KF/0
W9jL0?ooo`3oool0S8j<05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kOgP0Q8240
mo?g0?Ogm`0H7240]KJe0?ooo`3oool0S8j<08b>S03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ09bNW03oool0
oooo0?ooo`3oool0oooo0?ooo`12@D80c/o>0?ooo`3oool0oooo09bNW01kOG/0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0in_W024P8@2m`kd0oooo0<K;aP0Q8240e]OF0?ooo`3oool0
@T520;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q8240
_L>m0?ooo`3oool0oooo0?ooo`1cKG<08A`Q0?Ogm`2e][D0FUUS0?ooo`3oool0oooo05YIH`2LVY`0
oooo0>O[i`0Q82T0a/?60>o_k`0Q7240a/?60?ooo`3oool0Ngek09bNY@3FemH08B0Q0=KGeP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
8B0Q024L8@24PX@0FUUJ0?ooo`3oool0oooo0?ooo`3oool0@T520;ff_@3oool0oooo0;Ff]@12ADX0
oooo09B>U01kOG/0oooo0?ooo`3_knl08A`Q0<k?cP3Nhmh08B0Q0?Ogm`3WhnL0@T5207=]L`3oool0
oooo0?ooo`3oool0oooo0;ff_@1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0>OSi`1kOG/0oooo0?ooo`3oool0oooo0?ooo`2LVY`0Q8:40?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D0Ngek0?ooo`3oool0oooo0?ooo`3oool0]KJe08B2Q03oool0oooo0?ooo`3oool0
g^?N024P:@3FemH0oooo0?ooo`2LWZD0Jfe[0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bJW024PX@0oooo0?ooo`3oool0
]KJe06]]J`3oool0a/?6024P8@2UZZD0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>clh0
8A`Q0>o_k`3Nhmh08B0Y0=KGeP3oool0oooo09bNY@1[KF/0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9jU024L8@0Q82408A`Q024P:@0Q7240
8B0Q0?ooo`3oool0oooo0?ooo`3gmoL08B0Y0<K3aP3oool0oooo0?ooo`2LVY`0Q8:40?ooo`3oool0
oooo0?ooo`3oool0oooo05YIFP0Q82408A`Q024P:@0Q72408B0Q07]mN`2m][d08A`Q0?ooo`3oool0
oooo05YIFP2]ZZd0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
61`Q0<K3aP3oool0oooo0?ooo`3oool0W9ZL024P8@3Ngmh0]KJe05YIFP3oool0oooo0?ooo`1JFEX0
W9jL0?ooo`3_jnl08A`Q0<K3aP3oool0@T520;Ff]@3oool0oooo06]]J`2LVY`0kn__01PL8@0Q8240
8A`Q024P8@0H7240a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH0
8B0Q0:FVY@3oool0_L>m024P8@36blH0oooo0?ooo`3oool0ko?_024P8@2m`kd0oooo0?ooo`36`lH0
8A`Q0?ooo`2<SX`0Q7f40?ooo`3oool0a/_6024P8@3FemH0oooo0=KGeP2<SX`08A`Q06]]L`3WjnL0
oooo0?ooo`3oool0oooo0?ooo`1[KF/0W9jL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0]KJe0;Ff]@3oool0oooo0?ooo`3oool0W9jL07]mN`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0Jfe[09bNW03oool0oooo0?ooo`3oool0oooo06]]L`2LVY`0oooo0?ooo`3oool0
oooo0>OSi`0Q7240e]OF0?ooo`3oool0Jfe[08AmQ03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LWY`0Ngek0?ooo`3oool0
oooo0?ooo`2e][D0Jfe[024P8@1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WhnL0
Jfe[0491@P3oool0in?W024L8@3FemH0oooo0?ooo`1[KF/0Q7f40?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=kOgP2LWY`0oooo0?ooo`1JFEX0
W9jL0?ooo`3oool0oooo0?ooo`3_lnl0Q7f4024L8@3WhnL0oooo0?ooo`3oool0W9jL07]mN`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D0_L>m0?ooo`3WjnL08B0Q0;g3_@3oool0]KJe024P8@3oool0
oooo0?Ocm`0Q8240]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo024P:@36`lH0oooo0?ooo`3oool0oooo09bNW00Q7240_KJm024L8@12ADX0g^?N0?ooo`3oool0
@TE:09bJW03oool0g^?N024P8@2m`kd0oooo0;Ff]@1JFEX0in_W0=kKgP0Q7240W9jL0?ooo`24PX@0
]KJe0?ooo`2e][D08B0Y0>o[k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
]KJe024L8@3gmoL0oooo0?ooo`0Q7240_KJm0?ooo`3oool0oooo0?Ogm`0Q7240a/?60?ooo`3oool0
a/?6024P8@3oool0[JZ]0491@P3Nfmh0moOg07=]L`12@D80moOg0?ooo`2m][d08A`Q0495BP2m`kd0
oooo0?ooo`3oool0oooo0<K3aP1[KF/0@TE:0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ogm`0Q8240in_W0?ooo`3oool0oooo09bJW024PX@0oooo0?ooo`3oool0
oooo0?ooo`36`lH0Jfe[0495BP3_jnl0oooo0?ooo`3oool0a/?607=]L`12@D80kno_0?ooo`3oool0
oooo0?ooo`3gmoL08B0Q07]mN`3oool0a/?6024P8@2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9ZL08B2Q03oool0
oooo0?ooo`3oool0Ngek024P8@1[KF/0in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`24PX@0
8A`Q05YIFP3Nhmh0oooo0?Ogm`0Q8240Ngek0?ooo`36`lH08B0Q0;Ff]@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJZU0<k?cP3oool0
FUUJ09bJW03oool0oooo0?ooo`2e][D08B0Y024L8@36`lH0oooo0?ooo`3oool0oooo09bJW024PX@0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08B2Q03_jnl0kno_024L8@36`lH0oooo0<k?cP0Q7240
_KJm0?ooo`2LWY`08A`Q0=kKgP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`0Q7240a/?60?ooo`3oool0oooo0?ooo`1[KF/08B0Q0?Ocm`2LWY`0@T5206]]J`0Q7240
8B0Q01PL8@2LWY`0oooo024P8@0H7240a/?60?ooo`3oool0S8j<024P8@1JFEX0Jfec09bJW03oool0
in_W06]]L`0H72408B0Q0;g3_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0:FVY@1JFEX0oooo0?ooo`3oool08B0Q0;Ff]@3oool0oooo0?ooo`3oool0@T520;Ff]@3oool0
oooo0;Ff]@12@D80oooo0=kOgP0Q8240Jfe[024P8@12@D80e]OF0?ooo`3FemH08A`Q09bNW03oool0
Jfe[0;Ff]@3oool0g]oN024P8@0Q7240W9jL0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool061`Q0;Ff]@3oool0oooo0?ooo`2LWY`0Ngek0?ooo`3oool0
oooo0=kOgP0Q824061`Q09bNW03WjnL0oooo0?ooo`3WhnL08A`Q024P8@2LVY`0kn__0?ooo`3oool0
oooo0?ooo`3oool0oooo08b>S012@D80FUUJ024P8@1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNW01kOG/0
oooo0?ooo`3oool0]KJe024P8@2m`kd0kn__0491@P3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0;Ff]@12@D80kn__0?ooo`3oool0S8j<0491@P1JFEX08B0Q07]mN`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/0
kn__05YIFP2LWY`0oooo0?ooo`3oool0oooo0=KGeP1JFEX0a/_60?ooo`3oool0oooo0?ooo`2LWY`0
Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL0Q7f40>O[i`0Q8240_L>m0?ooo`3oool0
@T5206]]J`0Q824061`Q0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool08B0Q0;g3_@3oool0oooo0?ooo`3>blh08B0Y07]mN`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`2LWZD0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2m][d0@T520?ooo`3oool0g]_N024L8@3Nfmh0oooo0?ooo`3oool0oooo08b>S02DSY@0
oooo0?ooo`2<SX`0U8jD0?ooo`3oool0W9ZL07=]L`3gmoL0oooo0?ooo`3oool0]KJe024P8@3gmoL0
oooo0;Ff]@12ADX0oooo0?ooo`2<SX`0moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0[JZ]0?ooo`3oool0in_W024P:@2e][D0oooo0?ooo`3oool0W9ZL08B2Q03oool0
oooo0?ooo`3oool0S8j<0?Ogm`3oool0oooo0?ooo`3oool0oooo09B>U03gmoL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_knl08A`Q0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LVY`0
Q8:40?ooo`3oool0oooo08B2Q01[KF/0oooo0?ooo`24PX@0S8j<0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0[JZ]08b>S03oool0oooo0>o_k`0Q7240a/?60?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
kno_08b>S01JFF<0W9ZL0?ooo`3oool0oooo0?ooo`3oool0c/_>0495BP3oool0oooo0?ooo`3oool0
W9ZL08B2Q03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0;g3_@2]ZZd08A`Q0<K3aP3oool0
oooo0<K3aP12ADX0g^?N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0kn__01PL8@36`lH0oooo0?ooo`2e][D08B0Q0491@P3_jnl0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S00Q8240
W9ZL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0mo?g0491@P2m`kd0oooo06]]J`1[KG<0oooo0?ooo`3oool0oooo0?ooo`3FemH0
8A`Q0>OSi`3Ngmh08B0Q0=KGeP3oool0oooo0?ooo`1kOG/0FUUJ0;Ff]@3oool0oooo0<k?cP0H7240
c/o>0?ooo`1[KG<0Jfe[0?ooo`3oool0c/o>0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo09bJW03FemH0oooo08AmQ00Q7240kn__0?ooo`3oool0@T52024P8@1kOG/0
oooo0?ooo`3oool0oooo0<k?cP2e][D0oooo0?ooo`3oool0oooo0?ooo`36blH0]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP0H7240S8j<0>O[i`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`12@D80
8B0Q07]mN`3oool0oooo0?ooo`2UYZD0@T520>O[i`3_jnl0@T520:fZ[@3oool0oooo0?ooo`3oool0
mo?g08AmQ03_lnl0moOg06]]J`2<SY@0oooo0?ooo`3oool0e]OF01PL8@2<SX`0in_W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D08A`Q09bNW03oool0oooo09bJW03WhnL0oooo09bNW012@D80oooo0?ooo`3oool0
@T52024P8@1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0S8j<024P8@2m`kd0
oooo0?ooo`3oool0]KJe024P8@2UYZD0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0a/?6024L8@0Q82T08A`Q024P8@0Q72408B0Y09bJW03_knl0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_knl0
Ngek09bNW03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3FemH0FUUS024L8@1cKG<0in_W0?ooo`3oool0oooo0?ooo`3oool0
oooo0<K3aP0Q72408B0Y0;Ff]@3oool0oooo0?ooo`3oool0oooo0;Ff]@12ADX08A`Q0>o_k`3oool0
[JZ]024L8@0Q8240Jfe[0>OSi`3oool0oooo0?ooo`1JFF<08A`Q024P8@2LVY`0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_knl0Jfe[024P:@0Q7240a/?60?ooo`3oool0oooo0>OSi`2<SX`0
Q8:40?ooo`3oool0oooo0?ooo`3oool0FUUJ024L8@0Q82T0W9ZL0?ooo`3oool0oooo05YIFP0Q8240
8A`Q09bNY@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0in?W07]mN`0Q8240W9ZL0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
in?W08b>S024PX@0oooo0?ooo`3oool0moOg07]mN`0Q82408A`Q08B2Q03gmoL0oooo0?ooo`3oool0
oooo0?ooo`3Nhmh0FUUS024L8@1JFEX0g^?N0?ooo`3oool0oooo0?ooo`3WhnL0Ngek024P8@2LVY`0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo08B2Q02LVY`0oooo0?ooo`3gmoL0Jfe[024P:@0Q7240a/?60?ooo`3oool0
oooo0>OSi`2<SX`0Q8:40?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`12@D80
a/?60?ooo`3oool0oooo0?ooo`36`lH0Lfec024L8@36`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFF<08A`Q024P8@0Q7240
@TE:0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0in?W0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3Nfmh0g^?N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`12ADX0S8j<0?ooo`3oool0
kno_024L8@0Q82T08A`Q024P8@1[KF/0a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o_k`3>blh0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k;cP3_knl0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH08A`Q024P8@0Q72408B0Y06]]J`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3Nhmh0g]_N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k;cP3_knl0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9jL024L8@0Q82T0
8A`Q024P8@2UZZD0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_knl0c/_>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3>blh0kno_0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:FVY@12@D80
a/_60<K3aP0Q7240[JZ]0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>U02UYZD0
oooo0=KGeP1[KG<0S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0in?W06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo06]]J`3Ngmh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0W9ZL024P8@3gloL0
oooo0?ooo`3oool0@T520<k?cP3oool0]KJe0491@P2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o[k`1[KF/0moOg0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0Jfe[0>o[k`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`0H7240moOg0?ooo`2LWY`0
g]oN0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3Ngmh0Jfe[0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0
Jfe[0>o[k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
Jfe[0?ooo`3_lnl0Q7f40?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_jnl0Jfe[0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0moOg06]]J`3_jnl0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0=KGeP2e][D0
in_W0495BP1kOG/0oooo08b>S03gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0
Lfec0?ooo`3oool0moOg0495BP3>blh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL0oooo0?ooo`3gmoL0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0491@P3_knl0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3_jnl0@TE:0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`12@D80
]KJe0?ooo`3oool0oooo09B>U02UZZD0oooo0?ooo`3Nfmh08A`Q0?Ogm`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg0?ooo`3oool0
moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1JFEX0in?W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>OSi`1JFEX0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUS0<k;cP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ogm`3oool0oooo0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0@TE:0>o[k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0>OSi`1JFEX0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
kn__024P8@3gmoL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`3oool0oooo0?Ogm`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0FUUJ0>OSi`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3WhnL0FUUJ0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_jnl0Jfe[024P8@0H72408B0Q0>O[i`3>clh0
_L>m0?ooo`1kOG/0@T520?ooo`2e][D0mo?g0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`36`lH061`Q024P8@2m`kd0Jfec01PL8@1[KF/0g]oN0?ooo`3oool0
moOg024L8@3gmoL0oooo0?ooo`2LVY`0[JZ]0?ooo`2e][D0c/_>0?ooo`3oool0oooo0?ooo`24OH@0
61`Q024P8@0Q72408B0Q06]]J`3oool0oooo0?ooo`36blH08B0Q0;g3_@3oool061`Q09bNW03oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:FVY@24OH@0oooo0?ooo`3oool0W9jL01PL8@0Q8240
8A`Q09bNW03oool0oooo05YIFP0Q824061`Q024P8@3WjnL0kn__01PL8@0Q82408A`Q05YIFP2LVY`0
8B0Q024L8@0Q824061`Q024P8@3FemH0oooo07]mN`2]ZZd0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
a/?605YIFP3oool0oooo0?ooo`36blH0Jfe[0?ooo`3oool0oooo05YIFP3>blh0oooo07]mN`3oool0
oooo0?ooo`3oool0kn__05YIFP0Q824061`Q024P8@0Q7240[JZ]0?ooo`3oool0c/_>024P8@2m`kd0
oooo024L8@2LWY`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0Jfe[0?ooo`3oool0
oooo0?ooo`1JFEX08B0Q01PL8@0Q8240_L>m0?ooo`2m`kd08B0Q024L8@0Q8240W9ZL0?ooo`2LVY`0
8B0Q01PL8@0Q8240_L>m024P8@0H72408B0Q024L8@0Q8240Ngek0?ooo`3oool0Jfec0;Ff]@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0<k;cP0Q824061`Q024P8@2e][D0oooo07]mN`0Q82408A`Q024P8@3_lnl0oooo0:FVY@2]ZZd0
oooo0?ooo`3oool0oooo08b>S03gmoL0oooo0?ooo`3oool0oooo07]mN`0Q824061`Q024P8@0Q7240
Jfec0?ooo`3oool0oooo0<k?cP0H7240a/?60?ooo`0Q8240W9ZL0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0[JZ]07]mN`3oool0oooo0?ooo`1JFEX08B0Q01PL8@0Q8240_L>m0?ooo`3WjnL0
8B0Q024L8@0Q8240FUUJ0?ooo`2m`kd08B0Q01PL8@0Q8240W9ZL05YIFP0H72408B0Q024L8@0Q8240
61`Q0?ooo`3oool0Jfec0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP0Q824061`Q024P8@3>blh0oooo0?ooo`3oool0
oooo0?ooo`1[KF/0c/o>0?ooo`3oool0oooo0?ooo`1kOG/0oooo0?ooo`3oool0oooo0>o[k`1JFEX0
8B0Q024L8@0Q824061`Q0:fZ[@3oool0oooo0<K;aP0Q8240_L>m0?ooo`0H7240W9jL0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe06]]L`3oool0oooo0?ooo`3_jnl061`Q024P8@0Q7240
8B0Q0>O[i`3oool0_L>m024P8@0H72408B0Q09bJW03oool0W9ZL024P8@0Q72408B0Q0;g3_@0Q8240
8A`Q024P8@0H72408B0Q07]mN`3oool0oooo06]]J`2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@2LVY`0oooo0>O[i`1[KG<061`Q024P8@0Q7240
kn__0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`24OH@08A`Q024P8@0H7240a/?60?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Jfe[024P8@3>blh0oooo0?Ogm`3oool0
oooo05YIH`0Q7240a/?608b>S00Q82T0S8j<0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool08A`Q08B2Q03oool0FUUJ07]mN`3oool0Jfe[0495BP3gmoL0
oooo0?ooo`1cKG<0c/_>0?ooo`3oool0W9jL09bJW03oool0kn__09B>U03oool0oooo0?ooo`3oool0
oooo08B2Q03Nhmh0oooo0?ooo`2m][d0g^?N0?ooo`3oool0kno_024L8@24PX@0oooo024P:@0Q7240
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q8240]KJe0?ooo`3oool0oooo0?ooo`3>clh0
8A`Q0<k?cP3oool0oooo0?ooo`3oool0YJZU0495BP3oool0oooo0?ooo`3oool0@T5209B>U03oool0
kno_0491@P2DSY@0oooo0?ooo`36`lH0g]_N0?ooo`2m][d08A`Q0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0;g3_@0Q82T0W9ZL0?ooo`3oool0moOg024L8@3gmoL0oooo0?ooo`1JFEX0a/?60?ooo`2LWZD0
g^?N0?ooo`3oool0oooo0?ooo`3_knl0Ngek0?Ogm`3oool0oooo09bJW03oool0oooo0>o_k`0Q7240
Q8:40?ooo`0Q82408A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0FUUS09bJW03oool0
oooo0?ooo`3oool0oooo0:FZY@0Q82T0moOg0?ooo`3oool0oooo0?Ogm`0Q8240]KJe0?ooo`3oool0
oooo0;Ff]@0Q82T0g^?N0?ooo`2UZZD08B0Y0=kSgP3oool0oooo09bNY@3oool0oooo09bJW01JFF<0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`1cKG<0W9ZL08B2Q00Q7240oooo0?ooo`3WhnL08A`Q09bNW01kOG/0_KJm0?ooo`3>clh0
FUUJ0?ooo`3oool0oooo0?ooo`36`lH0]KJe0?ooo`3oool0oooo0?ooo`3oool0Ngek0>OSi`3oool0
oooo0;Ff]@3WhnL0oooo0?ooo`3WjnL08B0Y07]mN`3oool08A`Q024P:@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo024L8@2e][D0oooo0?ooo`3oool0oooo0:FZY@0Q82T0moOg0?ooo`3oool0
oooo0?ooo`1cKG<0S8j<0?ooo`3oool0oooo0=kSgP0Q82T0]KJe0?ooo`3>blh08B0Y0;Ff]@3oool0
oooo09bNY@3oool0oooo09bJW01JFF<0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool08A`Q024P:@3_jnl0oooo0?ooo`3oool0
oooo0?ooo`3oool0[JZ]09bJW03oool0oooo0?ooo`3oool0W9jL0=kSgP3oool0oooo0?ooo`3oool0
kno_07]mN`3gmoL0oooo0?ooo`2LVY`0oooo0?ooo`3_knl08A`Q08B2Q03oool08B0Y024L8@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP2LVY`0oooo0?ooo`3oool0oooo0?ooo`1[KF/0
Lfec0?ooo`3oool0oooo0?ooo`3gmoL08B0Y0;Ff]@3oool0oooo0?ooo`2e][D08B0Q0=kSgP3oool0
YJZ]024P8@3Nhmh0oooo0?ooo`2LWY`0oooo0?ooo`2LVY`0FUUJ0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08B2Q01[KF/0W9jL09bJW02LWZD0W9ZL024P8@0Q7240kno_0?ooo`1cKG<08A`Q0<k?cP3oool0
moOg0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024P8@0Q7240kno_0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024P8@0H7240a/?60;g3_@36`lH0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0ko?_024P8@2<SX`0oooo09bJW00Q8240oooo0=KGeP0H7240
[JZ]0?ooo`3oool0YJJU08b>S03oool0e]OF0491@P36`lH0oooo0?ooo`2<SX`0moOg0?ooo`3oool0
oooo0?ooo`3oool0Q7f40>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KG<0Jfe[0?ooo`2<SX`0
@T520?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gmoL061`Q0<k?cP3oool0oooo0?ooo`3oool0
e]OF024P8@3FemH0oooo0?ooo`3oool0oooo0;Ff]@1JFEX0oooo0?ooo`3oool0oooo05YIFP2LVY`0
oooo0?ooo`3>clh061`Q0;Ff]@3oool0oooo0?ooo`3oool0c/_>024P8@3_lnl0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2e][D0@T52024P8@3FemH0oooo0?ooo`1[KG<0]KJe0?ooo`2e][D08B0Q0>O[i`3oool0
e]OF0;Ff]@3oool0oooo0?ooo`3oool0oooo0>o[k`1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0
Jfe[06]]J`3oool0S8j<0491@P3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024L8@2e][D0
oooo0?ooo`3oool0oooo0?ooo`2e][D08A`Q0?ooo`3oool0oooo0?ooo`3oool061`Q0<K3aP3oool0
oooo0?ooo`36`lH08A`Q0>o[k`3oool0oooo06]]J`1JFEX0ko?_0?ooo`3oool0oooo0?ooo`2e][D0
8A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0Jfe[0?ooo`12@D80W9jL0?ooo`3oool0oooo05YIFP2LVY`0oooo07]mN`3oool0
oooo024P8@3FemH0oooo0?ooo`3oool0mo?g08b>U03oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/0
kn__0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`1[KG<0oooo08b>S012@D80oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?Ocm`0Q8240a/_60?ooo`3oool0oooo0?ooo`2e][D08A`Q0?ooo`3oool0
oooo0?ooo`3oool0Ngek09bNW03oool0oooo0?ooo`3_jnl08A`Q0<K3aP3oool0oooo0:FVY@0Q8240
g]oN0?ooo`3oool0oooo0?ooo`2e][D08A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024P8@0Q7240kn__0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP12@D80ko?_0?ooo`3oool0oooo0=KGeP2e][D0oooo0?ooo`3oool0
oooo0?ooo`3_jnl0Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo06]]L`1[KF/0oooo08b>S012@D80
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0H7240]KJe0?ooo`3oool0oooo0?ooo`3oool0
Q7f407]mN`3oool0oooo0?ooo`3oool0oooo024L8@36`lH0oooo0?ooo`3oool0a/?601PL8@3_jnl0
oooo0?ooo`1[KF/0FUUJ0?Ocm`3oool0oooo0?ooo`3oool0]KJe01PL8@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH0S8jD0?ooo`3oool0oooo0=KGeP0H7240Jfe[0?ooo`3oool061`Q024P8@2m`kd0
a/?60;g3_@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_jnl0Ngek0491@P0H72408B0Q024L8@12@D80
W9ZL0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/08B0Y024L8@0Q8240
8A`Q0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q7240Lfec0?ooo`2LWZD08A`Q0<K3aP3oool0
8B0Y07]mN`3oool0g^?N024P:@0Q72408B0Q024L8@2m][d0oooo0?ooo`3oool0_KJm0<k;cP3oool0
oooo0?ooo`3oool0oooo0?Ogm`1cKG<0kn__0?ooo`3oool0oooo0;Ff]@3oool0c/_>0495BP3oool0
c/o>024L8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0c/_>024P:@3_jnl0oooo0?ooo`3oool0
oooo0=kKgP0Q7240g]_N0?ooo`3oool0oooo0?ooo`2e][D0FUUJ0?ooo`3oool0oooo0?ooo`1JFEX0
W9jU0?ooo`3oool0oooo0:fZ[@0Q7240in?W0?ooo`3oool0oooo0>o_k`0Q7240c/o>0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0W9ZL05YIFP2e][D0@TE:0?ooo`2e][D08A`Q024P:@0Q7240FUUJ0=KGeP3oool0
oooo0?ooo`1kOG/0oooo0?ooo`3oool0oooo0?ooo`3oool0in?W07]mN`3oool0oooo0?ooo`2e][D0
oooo0<k;cP12ADX0oooo0<k?cP0Q7240oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>o[k`0Q8240
_L>m0?ooo`3oool0oooo0?ooo`3oool0]KJe024P8@3oool0oooo0?ooo`3oool0oooo024P:@36`lH0
oooo0?ooo`3oool0a/?6024P8@3WjnL0oooo0?ooo`3_knl0@T5209B>U03oool0oooo0?ooo`3oool0
a/?6024P8@3WjnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2UZZd08B0Q0<k;cP3oool0oooo0?ooo`2UZZD08B0Y0=KGeP3oool0
oooo08B2Q00Q72408B0Q0<k;cP3oool0oooo0?ooo`2<SX`0moOg0?ooo`3oool0oooo0?ooo`3oool0
moOg06]]J`3_knl0oooo0?ooo`3oool0_KJm0?ooo`3>clh0@T520?ooo`3>blh08B0Q0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3>clh08A`Q0>o_k`3oool0oooo0?ooo`3oool0]KJe024P8@3oool0
oooo0?ooo`3oool0oooo08B2Q02LVY`0oooo0?ooo`3oool0kn__024P8@2m`kd0oooo0?ooo`3oool0
Jfe[05YIH`3gmoL0oooo0?ooo`3oool0a/?6024P8@3WjnL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q72408B0Q0>O[i`3oool0
oooo0?ooo`3oool0oooo0?ooo`0Q82408A`Q024P:@3gmoL0oooo0?ooo`3oool0Ngek0?ooo`3oool0
oooo0?ooo`3oool0oooo0>OSi`1kOG/0oooo0?ooo`3oool0]KJe0?ooo`3>blh0@TE:0?ooo`3>clh0
8A`Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL08B0Y0<K3aP3oool0oooo0?ooo`3oool0
oooo07]mN`24PX@0oooo0?ooo`3oool0oooo0?ooo`0Q8240_L>m0?ooo`3oool0oooo0;g3_@0Q82T0
kn__0?ooo`3oool0kno_0491@P2DSY@0oooo0?ooo`3oool0oooo0;g3_@0Q82T0kn__0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0491@P3oool0oooo0?ooo`1kOG/08B0Y0<K3aP3oool0oooo08B2Q00Q7240
8B0Q024L8@0Q82T0a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_knl0@T5208B2Q03oool08B0Y024L8@3_knl0
oooo07=]L`12@D80oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0in?W024L8@1[KG<0
a/_60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0[JZ]01PL8@3_jnl0e]OF024P8@36blH0
oooo024L8@2<SY@0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ocm`2<SY@0
oooo0?ooo`3oool0oooo0?ooo`3oool0e]OF08AmQ03oool0oooo0?ooo`1[KG<0_L>m0?ooo`0Q7240
oooo0?ooo`0Q8240oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K3aP0Q7240kn__0?ooo`3oool0
oooo0?ooo`3FemH08B0Q0=KGeP3oool0oooo0?ooo`3oool0]KJe05YIFP3oool0oooo0?ooo`3oool0
FUUJ09bJW03oool0oooo0?ooo`3oool0Jfec05YIFP3gmoL0oooo0?ooo`3WjnL08B0Q0;g3_@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08b>S036`lH0in_W0491@P2<SX`0oooo0491@P2]ZZd0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0[JZ]0=kOgP3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0]KJe0?ooo`3oool0
Jfe[0;g3_@3oool061`Q0?ooo`3oool08B0Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_jnl0
61`Q0<K3aP3oool0oooo0?ooo`3oool0oooo0;Ff]@0H7240oooo0?ooo`3oool0oooo0?ooo`0Q7240
a/?60?ooo`3oool0oooo0<K3aP0H7240kn__0?ooo`3oool0oooo0<k?cP0Q7240]KJe0?ooo`3oool0
oooo0<K3aP0H7240kn__0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool08B0Q01PL8@3oool0oooo0?ooo`3oool0a/?6024L8@24OH@0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0c/o>0;Ff]@3oool0oooo0?ooo`3oool0
oooo0?ooo`3FemH0Ngek0?ooo`3oool0oooo06]]J`36`lH0oooo024P8@3oool0oooo01PL8@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0_L>m024P8@3WjnL0oooo0?ooo`3oool0oooo0;Ff]@0H7240
oooo0?ooo`3oool0oooo0?ooo`1kOG/0W9jL0?ooo`3oool0oooo0>o[k`0H7240a/?60?ooo`3oool0
oooo0>o[k`12@D80S8jD0?ooo`3oool0oooo0<K3aP0H7240kn__0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool08B0Q01PL8@3_jnl0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@3Ngmh0
oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe0;Ff]@3oool0oooo06]]L`2m`kd0oooo024L8@3oool0
oooo024P8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0kn__024L8@36`lH0oooo0?ooo`3oool0
oooo0?ooo`24OH@0Ngek0?ooo`3oool0oooo0?ooo`3oool061`Q0<K3aP3oool0oooo0?ooo`36`lH0
8A`Q0>o[k`3oool0oooo0?ooo`3>clh061`Q0;Ff]@3oool0oooo0?ooo`36`lH08A`Q0>o[k`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2e][D0]KJe0?ooo`3oool08B0Q024L8@3oool0oooo0?ooo`3Ngmh0
8B0Q06]]J`3>clh0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0YJJU024P8@36blH0oooo024L8@0Q8240
in_W0?ooo`3>blh08B0Q0:FVY@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3_knl0
S8j<0495BP0Q72408B0Q0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1cKG<0Jfe[0?ooo`3oool0
oooo0=KGeP0Q8240g^?N0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
S8j<0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`1[KF/0a/?60?ooo`3oool0g^?N024P:@0Q7240
8B0Q024L8@0Q82T08A`Q024P8@0Q7240in?W0?ooo`3oool0oooo0?ooo`3FemH08B0Q0=KGeP3oool0
oooo0?ooo`3oool0g]_N024L8@3Nfmh0oooo0?ooo`3oool0oooo0;Ff]@0Q82T08A`Q024P8@0Q7240
8B0Y024L8@2LWY`0oooo0>OSi`2e][D0oooo0>O[i`12ADX0S8j<0?ooo`3oool0g]_N024L8@3Nfmh0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3_knl08A`Q024P8@0Q7240g]_N0?ooo`2e][D08A`Q0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0=KGeP2e][D0oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024P8@3WjnL0
oooo0=kSgP0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T08A`Q0>OSi`3oool0oooo0?ooo`3oool0
oooo024P:@36`lH0oooo0?ooo`3oool0oooo0?ooo`2e][D08B0Y0?ooo`3oool0oooo0?ooo`3oool0
8B0Q024L8@0Q82T08A`Q024P8@0Q72408B0Y0>o[k`3oool0W9ZL0?Ogm`3oool0[JZ]024L8@3WhnL0
oooo0?ooo`2m`kd08B0Y0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0moOg024L8@0Q82T0oooo0?ooo`3oool0oooo0>o[k`0Q8240
@T520?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`2DSY@0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo07=]L`2m`kd0oooo0?ooo`3WhnL08A`Q024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@3Nhmh0oooo0?ooo`3oool0oooo0=kKgP0Q7240g]_N0?ooo`3oool0oooo0?ooo`2e][D0
8B0Y0?ooo`3oool0oooo0?ooo`3oool0Q8:4024L8@0Q82T08A`Q024P8@0Q72408B0Y0<K3aP3oool0
]KJe0=kKgP3oool0c/o>024L8@2m][d0oooo0?ooo`2m`kd08B0Y0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo024L8@0Q82T0
kn__0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH0
_KJm0?ooo`3oool0oooo0?ooo`3oool0oooo0?Ogm`0Q82T0kn__0?ooo`3Nhmh08B0Y024L8@0Q8240
8A`Q024P:@0Q72408B0Q024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`0Q8240_L>m0?ooo`3oool0
oooo0?ooo`3oool0Ngek08B2Q03oool0oooo0?ooo`3oool0oooo024P:@0Q72408B0Q024L8@0Q82T0
8A`Q024P8@3WjnL0oooo09bJW03gmoL0oooo0:fZ[@0Q7240in?W0?ooo`3oool0a/?6024P8@3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo05YIFP3oool0_KJm024L8@2LWY`0oooo0?ooo`3oool0
oooo0>O[i`2DSY@0@T52024P8@0Q7240oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNW00Q7240kno_0?ooo`0Q8240
8A`Q0>o_k`3oool0kno_024L8@2LWZD0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<K3aP3FemH0
oooo0?ooo`36`lH0e]OF0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0;Ff]@2m`kd0oooo0?ooo`3oool0oooo0?ooo`3oool0Q7f408b>S03oool0oooo0?ooo`3WjnL0
]KJe0;Ff]@2e][D0]KJe0;Ff]@2e][D0]KJe0>O[i`3oool0oooo0?ooo`3oool0oooo01PL8@36`lH0
oooo0?ooo`3oool0oooo0=KGeP0Q8240e]OF0?ooo`3oool0oooo0?ooo`2e][D0FUUJ0?ooo`3oool0
oooo0?ooo`1JFEX0W9ZL0?ooo`3WjnL08B0Q01PL8@0Q82408A`Q024P8@36blH0oooo0;g3_@0Q8240
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>o[k`2e][D0e]OF0?ooo`3oool0oooo0<K3aP3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0Jfe[0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0H7240
]KJe0?ooo`3oool0in_W0;Ff]@2e][D0]KJe0;Ff]@2e][D0]KJe0;Ff]@3WjnL0oooo0?ooo`3oool0
oooo0?ooo`0Q7240[JZ]0?ooo`3oool0oooo0?ooo`3oool0]KJe024L8@3oool0oooo0?ooo`3oool0
oooo01PL8@36`lH0oooo0?ooo`3oool0a/?6024L8@3_jnl0oooo09bNW00Q72408B0Q01PL8@0Q8240
Jfe[0?ooo`3oool0[JZ]024L8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0>O[i`0Q82408A`Q0?ooo`3oool0oooo0?ooo`3_jnl0
61`Q024P8@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0S8j<0>o[k`3oool0
oooo0?ooo`3oool0oooo0?ooo`1kOG/0S8j<0?ooo`3oool0oooo0>o[k`2e][D0]KJe0;Ff]@2e][D0
]KJe0;Ff]@2e][D0kn__0?ooo`3oool0oooo0?ooo`3oool08B0Q0;g3_@3oool0oooo0?ooo`3oool0
]KJe024L8@3oool0oooo0?ooo`3oool0oooo07]mN`2LWY`0oooo0?ooo`3oool0kn__024L8@36`lH0
oooo0<K3aP0Q72408B0Q01PL8@0Q82408A`Q0?Ogm`3oool0[JZ]024L8@3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q8240
8A`Q0>o[k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo06]]J`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool08A`Q0;Ff]@3oool0oooo0>O[i`2e][D0
]KJe0;Ff]@2e][D0]KJe0;Ff]@2e][D0in_W0?ooo`3oool0oooo0?ooo`3oool061`Q0:fZ[@3oool0
oooo0?ooo`3oool0oooo08AmQ01kOG/0oooo0?ooo`3oool0oooo0?ooo`0Q7240a/?60?ooo`3oool0
oooo0<K3aP0H7240kn__0?ooo`2LWY`061`Q024P8@0Q72408B0Q06]]J`3oool0oooo0:fZ[@0H7240
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/0in?W05YIFP0Q8240in_W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36blH08B0Q0;Ff]@3oool0
61`Q024P8@3WjnL0oooo0;Ff]@0Q8240c/_>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3gmoL0U8jD0?ooo`3oool0YJZ]0?ooo`3oool0oooo05YIFP2DSY@0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`24PX@0
W9ZL0?ooo`3FemH0moOg0?ooo`3Nfmh08A`Q0=kKgP3oool0oooo0<k;cP3oool0]KJe05YIFP3oool0
oooo0?ooo`3oool0FUUJ09bNY@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2LWY`0
Ngek0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@3Nhmh0oooo0?ooo`2]ZZd0oooo0?ooo`3WjnL0
8B0Y0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0W9jL07]mN`3oool0oooo0<k?cP3oool0oooo0;Ff]@0Q8240oooo0?ooo`3_jnl0
in?W0?ooo`0Q82T0a/?60?ooo`3oool0oooo0<K3aP0Q8240in_W0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo07]mN`2LWY`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool08A`Q024P8@3oool0oooo0?ooo`3oool0
c/_>024P:@1[KF/0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0<k?cP2e][D0
oooo0?ooo`2]ZZd0oooo0?ooo`3oool0FUUS08b>S03oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`2LWZD0oooo0<k?cP3oool0
oooo0;Ff]@0Q8240oooo0?ooo`3_jnl0in?W0?ooo`24PX@0W9ZL0?ooo`3oool0oooo0>o[k`0Q8240
_L>m0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo07]mN`2LWY`0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
8A`Q024P8@3WjnL0oooo0?ooo`3oool0W9ZL0?Ogm`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2]ZZd0g^?N0?ooo`3oool0[JZ]0?ooo`3oool0kn__024P8@2e][D0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNY@1kOG/0
oooo0?ooo`3>clh0oooo0?ooo`1kOG/0Q8:40?ooo`3oool0e]OF0?Ogm`3oool08B0Q0;g3_@3oool0
oooo0?ooo`2m`kd08B0Y0>o[k`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`1kOG/0
W9jU0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0g]_N0491@P0Q8240Ngek0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo08b>S012ADX0
e]OF024P:@0Q7240kno_0;Ff]@0Q82T0YJZ]0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo07]mN`3gmoL0oooo0<K3aP2e][D0oooo0;Ff]@0Q8240_L>m0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
e]OF0491@P3oool0a/?60;Ff]@3oool0e]OF024P8@3FemH0oooo0;g3_@2]ZZd0oooo0:fZ[@12@D80
oooo0?ooo`3oool0oooo0491@P2<SX`0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
@T520=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH0[JZ]0?ooo`3oool0W9ZL0=KGeP3oool0
Q7f4024L8@3_jnl0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0>O[i`0Q8240oooo0?ooo`2LVY`0in?W0?ooo`2e][D061`Q0?ooo`3oool0
W9jL0<K;aP3gmoL08A`Q0;Ff]@3oool0oooo0?ooo`2e][D061`Q0>OSi`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`0Q8240in_W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo09bNW00H7240a/?60?ooo`3oool0
oooo08b>S00Q7240c/o>0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3gloL0
Q7f40?ooo`3oool0_L>m0;Ff]@3oool0]KJe024L8@36`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH0@T520?ooo`2LVY`0
in?W0?ooo`2e][D061`Q0?ooo`3oool0W9jL0<K;aP3oool0Jfe[08b>U03oool0oooo0?ooo`3WhnL0
61`Q0;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q8240in_W0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo024P8@0H7240kn__0?ooo`3oool0ko?_0491@P3WjnL0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0e]OF0:fZ[@3oool0oooo09bJW03FemH0oooo08AmQ00H7240kn__0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3WjnL0
8B0Q0?ooo`3oool0Ngek0?Ogm`3oool0Q7f407]mN`3oool0ko?_06]]J`3gloL0moOg01PL8@2e][D0
oooo0?ooo`3oool0]KJe024L8@3WhnL0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
8B0Q0>O[i`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`0Q824061`Q0<k?cP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
e]OF09bNW00Q72408B0Q05YIFP2]ZZd0in_W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2m][d0a/?60?ooo`3oool0U8jD024L8@0Q8240W9ZL0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`2UZZD0[JZ]0<K3aP0Q82408A`Q024P:@0Q72408B0Q024L8@0Q82T0]KJe05YIFP0Q7240
8B0Y024L8@3_knl0in_W024P:@0Q72408B0Q05YIFP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
YJZU0:fZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo06]]J`3oool0oooo0>o_k`1[KF/0
8B0Y024L8@36`lH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0a/?609B>U03oool0W9jU024L8@0Q82408A`Q024P:@0Q7240
8B0Q024L8@36`lH08A`Q024P8@0Q7240W9jU0?ooo`2LWY`08A`Q024P:@0Q7240a/?60?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`2DSY@0_L>m0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0Q8:40491@P3Nfmh0
oooo0<K3aP0Q7240[JZ]0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo08b>S03_knl0oooo0?ooo`2<SX`08B0Y024L8@2LWY`0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0:fZ[@2UZZD0
W9jU024L8@0Q82408A`Q024P:@0Q72408B0Q024L8@36`lH08A`Q024P8@0Q7240FUUS0?ooo`36`lH0
8A`Q024P:@0Q7240W9jL0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2DSY@0_L>m0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo05YIFP0Q72408B0Y024L8@1JFEX0@T52024P:@0Q7240kno_0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`1[KF/0oooo0?ooo`3_knl0Jfe[024P8@0Q7240a/?60?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0;g3_@2DSY@0oooo05YIFP0Q72408B0Y024L8@0Q82408A`Q024P:@0Q7240a/?6024L8@0Q82T0
8A`Q09bNW03oool0W9jU024L8@0Q82408A`Q0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
U8jD0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0]KJe0495BP3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`1kOG/08B0Q024L8@0Q82T0a/?60?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0:FVY@3FemH0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
e]OF0:fZ[@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`2e][D0a/?60?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`36`lH0]KJe0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3>clh0
Ngek05YIFP2LVY`0in?W0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
[JZ]0=KGeP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`36`lH0]KJe0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0]KJe0<K3aP3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
a/?60;Ff]@3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3FemH0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H7240
8B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q8240
61`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q7240
8B0Q01PL8@0Q82408A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q8240
8A`Q024P8@0H72408B0Q024L8@0Q824061`Q024P8@0Q72408B0Q01PL8@0Q82408A`Q0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0oooo0?ooo`3oool0
oooo0?ooo`0>000050000000000@00005000
\>"], "PlacedGraphics",
  ImageSize->{483.563, 74.9375},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Block[{epsilon, epsList, dList, \[Theta]in, 
          pol, }, \[IndentingNewLine]epsinf = 
          5.9673; \[IndentingNewLine]\[HBar]\[Omega]D\  = \ 
          2113.6*10\^12*\((2*\[Pi])\)*
            hbar/q; \[IndentingNewLine]\[HBar]\[Gamma]D\  = 
          15.92*10\^12*\((2*\[Pi])\)*
            hbar/q; \[IndentingNewLine]\[HBar]\[CapitalOmega]L = 
          650.07*10\^12*\((2*\[Pi])\)*
            hbar/q; \[IndentingNewLine]\[HBar]\[CapitalGamma]L\  = 
          104.86*10\^12*\((2*\[Pi])\)*
            hbar/q; \[IndentingNewLine]\[CapitalDelta]eps\  = 
          1.09; \[IndentingNewLine]epsilon = 
          epsinf\  - \ \[HBar]\[Omega]D\^2\/\(\[HBar]\[Omega] \((\[HBar]\
\[Omega] - \[ImaginaryI]\ \[HBar]\[Gamma]D)\)\) - \
\(\[CapitalDelta]eps*\[HBar]\[CapitalOmega]L\^2\)\/\(\((\[HBar]\[Omega]\^2 - \
\[HBar]\[CapitalOmega]L\^2)\) - \[ImaginaryI]*\[HBar]\[CapitalGamma]L*\[HBar]\
\[Omega]\); \[IndentingNewLine]Print["\<epsilon=\>", 
          epsilon]; \[IndentingNewLine]\[IndentingNewLine]<< 
          Graphics`Graphics`; \[IndentingNewLine]$TextStyle = {FontFamily \
\[Rule] "\<Times\>", FontSize \[Rule] 12}; \[IndentingNewLine]$FormatType\  = 
          TraditionalForm; \[IndentingNewLine]plotOps = {Frame -> True, 
            ImageSize \[Rule] \ 72\ 6, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Permittivity\>", "\<\>", "\<\
\>"}, Axes \[Rule] False, 
            PlotRange \[Rule] {{1.24, 2.48}, {\(-43\), \(-3\)}}, 
            DisplayFunction \[Rule] Identity}; \[IndentingNewLine]realPlot = 
          Plot[Re[epsilon], {\[HBar]\[Omega], 1.24, 2.48}, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Permittivity\>", "\<Figure 1\
\>", "\<\>"}, Evaluate[plotOps], 
            PlotStyle \[Rule] {Thickness[0.005], 
                RGBColor[0, 0, 0]}]; \[IndentingNewLine]imagPlot = 
          Plot[Im[epsilon], {\[HBar]\[Omega], 1.24, 2.48}, 
            PlotRange \[Rule] {{1.24, 2.48}, {\(-4\), \(-0.5\)}}, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Permittivity\>", "\<Figure 2\
\>", "\<\>"}, Evaluate[plotOps], 
            PlotStyle \[Rule] {Thickness[0.005], RGBColor[0, 0, 0], 
                Dashing[{0.02}]}]; \
\[IndentingNewLine]\[IndentingNewLine]epsList = {1, epsilon, \ 1}; \  (*\ 
          
          List\ of\ values\ for\ permittivity\ *) \[IndentingNewLine]dList = \
{0, 20*10\^\(-9\), 0}; \  (*\ 
          List\ of\ values\ for\ layer\ thicknesses\ *) \[IndentingNewLine]\
\[Theta]in\  = \ 
          0; \[IndentingNewLine]pol\  = \ "\<TM\>"; \[IndentingNewLine]nm20 = 
          Plot[{Abs[
                TransmissionCoef[epsList, dList, \[Theta]in, pol, en]]}, {en, 
              1.24, 2.48}, Frame -> True, ImageSize \[Rule] \ 72\ 6, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Transmission\>", "\<Figure 3\
\>", "\<\>"}, Axes \[Rule] False, 
            PlotRange \[Rule] {{1.24, 2.48}, {0.3, 0.8}}, 
            DisplayFunction \[Rule] Identity]; \[IndentingNewLine]dList = {0, 
            50*10\^\(-9\), 0}; \[IndentingNewLine]nm50 = 
          Plot[{Abs[
                TransmissionCoef[epsList, dList, \[Theta]in, pol, en]]}, {en, 
              1.24, 2.48}, Frame -> True, ImageSize \[Rule] \ 72\ 6, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Transmission\>", "\<Figure 4\
\>", "\<\>"}, Axes \[Rule] False, 
            PlotRange \[Rule] {{1.24, 2.48}, {0.05, 0.45}}, 
            DisplayFunction \[Rule] Identity]; \[IndentingNewLine]Show[
          GraphicsArray[{{realPlot, imagPlot}, {nm20, nm50}}], 
          DisplayFunction \[Rule] $DisplayFunction, 
          ImageSize \[Rule] \ 72*12]\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("epsilon="\[InvisibleSpace]\(\(\(5.9673`\)\(\
\[InvisibleSpace]\)\) - 
          76.40764511865186`\/\(\[HBar]\[Omega]\ \((\(-0.06583982541517296`\)\
\ \[ImaginaryI] + \[HBar]\[Omega])\)\) - 
          7.878398803264086`\/\(\(-7.227888810334023`\) - 0.4336660862459194`\
\ \[ImaginaryI]\ \[HBar]\[Omega] + \[HBar]\[Omega]\^2\)\)\),
      SequenceForm[ "epsilon=", 
        Plus[ 5.9672999999999998, 
          Times[ -76.407645118651857, 
            Power[ \[HBar]\[Omega], -1], 
            Power[ 
              Plus[ 
                Complex[ 0, -.065839825415172956], \[HBar]\[Omega]], -1]], 
          Times[ -7.8783988032640861, 
            Power[ 
              Plus[ -7.2278888103340231, 
                Times[ 
                  Complex[ 0, -.4336660862459194], \[HBar]\[Omega]], 
                Power[ \[HBar]\[Omega], 2]], -1]]]],
      Editable->False]], "Print"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 864 533.981 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0147151 0.47619 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0147151 0.477324 0.295003 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1 0.806452 -0.37082 1.23607 [
[.12903 -0.0125 -9.5 -15.6875 ]
[.12903 -0.0125 9.5 0 ]
[.29032 -0.0125 -9.5 -15.6875 ]
[.29032 -0.0125 9.5 0 ]
[.45161 -0.0125 -9.5 -15.6875 ]
[.45161 -0.0125 9.5 0 ]
[.6129 -0.0125 -5 -15.6875 ]
[.6129 -0.0125 5 0 ]
[.77419 -0.0125 -9.5 -15.6875 ]
[.77419 -0.0125 9.5 0 ]
[.93548 -0.0125 -9.5 -15.6875 ]
[.93548 -0.0125 9.5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9063 -31.375 ]
[.5 -0.0125 17.9063 -15.6875 ]
[-0.0125 0 -19 -7.84375 ]
[-0.0125 0 0 7.84375 ]
[-0.0125 .12361 -19 -7.84375 ]
[-0.0125 .12361 0 7.84375 ]
[-0.0125 .24721 -19 -7.84375 ]
[-0.0125 .24721 0 7.84375 ]
[-0.0125 .37082 -19 -7.84375 ]
[-0.0125 .37082 0 7.84375 ]
[-0.0125 .49443 -19 -7.84375 ]
[-0.0125 .49443 0 7.84375 ]
[-0.0125 .61803 -19 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -34.6875 -34.2813 ]
[-0.01875 .30902 -19 34.2813 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -21.9688 0 ]
[.5 .63053 21.9688 15.6875 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.12903 0 m
.12903 .00625 L
s
gsave
.12903 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.4) show
1.000 setlinewidth
grestore
.29032 0 m
.29032 .00625 L
s
gsave
.29032 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.6) show
1.000 setlinewidth
grestore
.45161 0 m
.45161 .00625 L
s
gsave
.45161 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.8) show
1.000 setlinewidth
grestore
.6129 0 m
.6129 .00625 L
s
gsave
.6129 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.77419 0 m
.77419 .00625 L
s
gsave
.77419 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.2) show
1.000 setlinewidth
grestore
.93548 0 m
.93548 .00625 L
s
gsave
.93548 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16935 0 m
.16935 .00375 L
s
.20968 0 m
.20968 .00375 L
s
.25 0 m
.25 .00375 L
s
.33065 0 m
.33065 .00375 L
s
.37097 0 m
.37097 .00375 L
s
.41129 0 m
.41129 .00375 L
s
.49194 0 m
.49194 .00375 L
s
.53226 0 m
.53226 .00375 L
s
.57258 0 m
.57258 .00375 L
s
.65323 0 m
.65323 .00375 L
s
.69355 0 m
.69355 .00375 L
s
.73387 0 m
.73387 .00375 L
s
.81452 0 m
.81452 .00375 L
s
.85484 0 m
.85484 .00375 L
s
.89516 0 m
.89516 .00375 L
s
.08871 0 m
.08871 .00375 L
s
.04839 0 m
.04839 .00375 L
s
.00806 0 m
.00806 .00375 L
s
.97581 0 m
.97581 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9063 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
73.313 15.813 moveto
(H) show
77.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.000 15.813 moveto
(L) show
94.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
.00625 0 L
s
gsave
-0.0125 0 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.3) show
1.000 setlinewidth
grestore
0 .12361 m
.00625 .12361 L
s
gsave
-0.0125 .12361 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
0 .24721 m
.00625 .24721 L
s
gsave
-0.0125 .24721 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
0 .37082 m
.00625 .37082 L
s
gsave
-0.0125 .37082 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
0 .49443 m
.00625 .49443 L
s
gsave
-0.0125 .49443 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.7) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .02472 m
.00375 .02472 L
s
0 .04944 m
.00375 .04944 L
s
0 .07416 m
.00375 .07416 L
s
0 .09889 m
.00375 .09889 L
s
0 .14833 m
.00375 .14833 L
s
0 .17305 m
.00375 .17305 L
s
0 .19777 m
.00375 .19777 L
s
0 .22249 m
.00375 .22249 L
s
0 .27193 m
.00375 .27193 L
s
0 .29666 m
.00375 .29666 L
s
0 .32138 m
.00375 .32138 L
s
0 .3461 m
.00375 .3461 L
s
0 .39554 m
.00375 .39554 L
s
0 .42026 m
.00375 .42026 L
s
0 .44498 m
.00375 .44498 L
s
0 .46971 m
.00375 .46971 L
s
0 .51915 m
.00375 .51915 L
s
0 .54387 m
.00375 .54387 L
s
0 .56859 m
.00375 .56859 L
s
0 .59331 m
.00375 .59331 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -122.125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
95.2813 11.8438 translate
90 rotate
-95.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Transmission) show
127.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.12903 .61178 m
.12903 .61803 L
s
.29032 .61178 m
.29032 .61803 L
s
.45161 .61178 m
.45161 .61803 L
s
.6129 .61178 m
.6129 .61803 L
s
.77419 .61178 m
.77419 .61803 L
s
.93548 .61178 m
.93548 .61803 L
s
.125 Mabswid
.16935 .61428 m
.16935 .61803 L
s
.20968 .61428 m
.20968 .61803 L
s
.25 .61428 m
.25 .61803 L
s
.33065 .61428 m
.33065 .61803 L
s
.37097 .61428 m
.37097 .61803 L
s
.41129 .61428 m
.41129 .61803 L
s
.49194 .61428 m
.49194 .61803 L
s
.53226 .61428 m
.53226 .61803 L
s
.57258 .61428 m
.57258 .61803 L
s
.65323 .61428 m
.65323 .61803 L
s
.69355 .61428 m
.69355 .61803 L
s
.73387 .61428 m
.73387 .61803 L
s
.81452 .61428 m
.81452 .61803 L
s
.85484 .61428 m
.85484 .61803 L
s
.89516 .61428 m
.89516 .61803 L
s
.08871 .61428 m
.08871 .61803 L
s
.04839 .61428 m
.04839 .61803 L
s
.00806 .61428 m
.00806 .61803 L
s
.97581 .61428 m
.97581 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -82.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Figure) show
96.938 15.813 moveto
(3) show
102.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 0 m
1 0 L
s
.99375 .12361 m
1 .12361 L
s
.99375 .24721 m
1 .24721 L
s
.99375 .37082 m
1 .37082 L
s
.99375 .49443 m
1 .49443 L
s
.125 Mabswid
.99625 .02472 m
1 .02472 L
s
.99625 .04944 m
1 .04944 L
s
.99625 .07416 m
1 .07416 L
s
.99625 .09889 m
1 .09889 L
s
.99625 .14833 m
1 .14833 L
s
.99625 .17305 m
1 .17305 L
s
.99625 .19777 m
1 .19777 L
s
.99625 .22249 m
1 .22249 L
s
.99625 .27193 m
1 .27193 L
s
.99625 .29666 m
1 .29666 L
s
.99625 .32138 m
1 .32138 L
s
.99625 .3461 m
1 .3461 L
s
.99625 .39554 m
1 .39554 L
s
.99625 .42026 m
1 .42026 L
s
.99625 .44498 m
1 .44498 L
s
.99625 .46971 m
1 .46971 L
s
.99625 .51915 m
1 .51915 L
s
.99625 .54387 m
1 .54387 L
s
.99625 .56859 m
1 .56859 L
s
.99625 .59331 m
1 .59331 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .0081 m
.04057 .02675 L
.08481 .04759 L
.12636 .06767 L
.16632 .08747 L
.20885 .1091 L
.2498 .13048 L
.29331 .15385 L
.33524 .17699 L
.37557 .19987 L
.41848 .22486 L
.4598 .24954 L
.49953 .27381 L
.54183 .30016 L
.58254 .32586 L
.62583 .35328 L
.66752 .37937 L
.70762 .40354 L
.7503 .42725 L
.79139 .4466 L
.81387 .45501 L
.82417 .45819 L
.83505 .46102 L
.8451 .4631 L
.85042 .46398 L
.85613 .46474 L
.85883 .46502 L
.86138 .46526 L
.86386 .46544 L
.86616 .46558 L
.86834 .46567 L
.86944 .46571 L
.87065 .46574 L
.87186 .46576 L
.87316 .46577 L
.8744 .46577 L
.87553 .46576 L
.87687 .46573 L
.87813 .4657 L
.87927 .46566 L
.8805 .4656 L
.88327 .46544 L
.88582 .46523 L
.89047 .46473 L
.8954 .46401 L
.90074 .46301 L
.90575 .46185 L
.91713 .45839 L
.92806 .45395 L
.93843 .44867 L
Mistroke
.9579 .43588 L
.97874 .41793 L
.99806 .39748 L
1 .39523 L
Mfstroke
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.522676 0.0147151 0.97619 0.295003 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1 0.806452 -0.0772542 1.54508 [
[.12903 -0.0125 -9.5 -15.6875 ]
[.12903 -0.0125 9.5 0 ]
[.29032 -0.0125 -9.5 -15.6875 ]
[.29032 -0.0125 9.5 0 ]
[.45161 -0.0125 -9.5 -15.6875 ]
[.45161 -0.0125 9.5 0 ]
[.6129 -0.0125 -5 -15.6875 ]
[.6129 -0.0125 5 0 ]
[.77419 -0.0125 -9.5 -15.6875 ]
[.77419 -0.0125 9.5 0 ]
[.93548 -0.0125 -9.5 -15.6875 ]
[.93548 -0.0125 9.5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9063 -31.375 ]
[.5 -0.0125 17.9063 -15.6875 ]
[-0.0125 .07725 -19 -7.84375 ]
[-0.0125 .07725 0 7.84375 ]
[-0.0125 .23176 -19 -7.84375 ]
[-0.0125 .23176 0 7.84375 ]
[-0.0125 .38627 -19 -7.84375 ]
[-0.0125 .38627 0 7.84375 ]
[-0.0125 .54078 -19 -7.84375 ]
[-0.0125 .54078 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -34.6875 -34.2813 ]
[-0.01875 .30902 -19 34.2813 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -21.9688 0 ]
[.5 .63053 21.9688 15.6875 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.12903 0 m
.12903 .00625 L
s
gsave
.12903 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.4) show
1.000 setlinewidth
grestore
.29032 0 m
.29032 .00625 L
s
gsave
.29032 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.6) show
1.000 setlinewidth
grestore
.45161 0 m
.45161 .00625 L
s
gsave
.45161 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.8) show
1.000 setlinewidth
grestore
.6129 0 m
.6129 .00625 L
s
gsave
.6129 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.77419 0 m
.77419 .00625 L
s
gsave
.77419 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.2) show
1.000 setlinewidth
grestore
.93548 0 m
.93548 .00625 L
s
gsave
.93548 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16935 0 m
.16935 .00375 L
s
.20968 0 m
.20968 .00375 L
s
.25 0 m
.25 .00375 L
s
.33065 0 m
.33065 .00375 L
s
.37097 0 m
.37097 .00375 L
s
.41129 0 m
.41129 .00375 L
s
.49194 0 m
.49194 .00375 L
s
.53226 0 m
.53226 .00375 L
s
.57258 0 m
.57258 .00375 L
s
.65323 0 m
.65323 .00375 L
s
.69355 0 m
.69355 .00375 L
s
.73387 0 m
.73387 .00375 L
s
.81452 0 m
.81452 .00375 L
s
.85484 0 m
.85484 .00375 L
s
.89516 0 m
.89516 .00375 L
s
.08871 0 m
.08871 .00375 L
s
.04839 0 m
.04839 .00375 L
s
.00806 0 m
.00806 .00375 L
s
.97581 0 m
.97581 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9063 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
73.313 15.813 moveto
(H) show
77.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.000 15.813 moveto
(L) show
94.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .07725 m
.00625 .07725 L
s
gsave
-0.0125 .07725 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
0 .23176 m
.00625 .23176 L
s
gsave
-0.0125 .23176 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 .38627 m
.00625 .38627 L
s
gsave
-0.0125 .38627 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.3) show
1.000 setlinewidth
grestore
0 .54078 m
.00625 .54078 L
s
gsave
-0.0125 .54078 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .10816 m
.00375 .10816 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .20086 m
.00375 .20086 L
s
0 .26266 m
.00375 .26266 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .35537 m
.00375 .35537 L
s
0 .41717 m
.00375 .41717 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .50988 m
.00375 .50988 L
s
0 .04635 m
.00375 .04635 L
s
0 .01545 m
.00375 .01545 L
s
0 .57168 m
.00375 .57168 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -122.125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
95.2813 11.8438 translate
90 rotate
-95.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Transmission) show
127.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.12903 .61178 m
.12903 .61803 L
s
.29032 .61178 m
.29032 .61803 L
s
.45161 .61178 m
.45161 .61803 L
s
.6129 .61178 m
.6129 .61803 L
s
.77419 .61178 m
.77419 .61803 L
s
.93548 .61178 m
.93548 .61803 L
s
.125 Mabswid
.16935 .61428 m
.16935 .61803 L
s
.20968 .61428 m
.20968 .61803 L
s
.25 .61428 m
.25 .61803 L
s
.33065 .61428 m
.33065 .61803 L
s
.37097 .61428 m
.37097 .61803 L
s
.41129 .61428 m
.41129 .61803 L
s
.49194 .61428 m
.49194 .61803 L
s
.53226 .61428 m
.53226 .61803 L
s
.57258 .61428 m
.57258 .61803 L
s
.65323 .61428 m
.65323 .61803 L
s
.69355 .61428 m
.69355 .61803 L
s
.73387 .61428 m
.73387 .61803 L
s
.81452 .61428 m
.81452 .61803 L
s
.85484 .61428 m
.85484 .61803 L
s
.89516 .61428 m
.89516 .61803 L
s
.08871 .61428 m
.08871 .61803 L
s
.04839 .61428 m
.04839 .61803 L
s
.00806 .61428 m
.00806 .61803 L
s
.97581 .61428 m
.97581 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -82.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Figure) show
96.938 15.813 moveto
(4) show
102.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .07725 m
1 .07725 L
s
.99375 .23176 m
1 .23176 L
s
.99375 .38627 m
1 .38627 L
s
.99375 .54078 m
1 .54078 L
s
.125 Mabswid
.99625 .10816 m
1 .10816 L
s
.99625 .13906 m
1 .13906 L
s
.99625 .16996 m
1 .16996 L
s
.99625 .20086 m
1 .20086 L
s
.99625 .26266 m
1 .26266 L
s
.99625 .29357 m
1 .29357 L
s
.99625 .32447 m
1 .32447 L
s
.99625 .35537 m
1 .35537 L
s
.99625 .41717 m
1 .41717 L
s
.99625 .44807 m
1 .44807 L
s
.99625 .47898 m
1 .47898 L
s
.99625 .50988 m
1 .50988 L
s
.99625 .04635 m
1 .04635 L
s
.99625 .01545 m
1 .01545 L
s
.99625 .57168 m
1 .57168 L
s
.99625 .60258 m
1 .60258 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .04344 m
.04057 .05091 L
.08481 .05954 L
.12636 .06815 L
.16632 .07696 L
.20885 .08696 L
.2498 .09727 L
.29331 .10906 L
.33524 .12135 L
.37557 .13413 L
.41848 .14893 L
.4598 .16451 L
.49953 .18091 L
.54183 .20011 L
.58254 .22055 L
.62583 .24466 L
.66752 .27048 L
.70762 .29786 L
.7503 .32962 L
.79139 .36202 L
.83505 .39622 L
.85696 .41192 L
.87712 .4245 L
.8968 .43416 L
.90672 .43775 L
.91234 .43934 L
.9176 .4405 L
.92015 .44095 L
.92285 .44135 L
.92426 .44152 L
.92582 .44167 L
.92726 .44179 L
.92857 .44187 L
.92983 .44193 L
.93115 .44197 L
.93248 .44199 L
.93323 .44199 L
.93393 .44198 L
.93521 .44195 L
.93637 .4419 L
.9377 .44182 L
.93897 .44172 L
.94133 .44148 L
.94383 .44114 L
.94832 .44029 L
.95314 .43904 L
.95845 .43726 L
.96892 .43248 L
.97848 .4266 L
1 .40821 L
Mistroke
Mfstroke
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.323031 0.477324 0.603319 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1 0.806452 0.664387 0.0154508 [
[.12903 -0.0125 -9.5 -15.6875 ]
[.12903 -0.0125 9.5 0 ]
[.29032 -0.0125 -9.5 -15.6875 ]
[.29032 -0.0125 9.5 0 ]
[.45161 -0.0125 -9.5 -15.6875 ]
[.45161 -0.0125 9.5 0 ]
[.6129 -0.0125 -5 -15.6875 ]
[.6129 -0.0125 5 0 ]
[.77419 -0.0125 -9.5 -15.6875 ]
[.77419 -0.0125 9.5 0 ]
[.93548 -0.0125 -9.5 -15.6875 ]
[.93548 -0.0125 9.5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9063 -31.375 ]
[.5 -0.0125 17.9063 -15.6875 ]
[-0.0125 .04635 -24.3125 -7.84375 ]
[-0.0125 .04635 0 7.84375 ]
[-0.0125 .12361 -24.3125 -7.84375 ]
[-0.0125 .12361 0 7.84375 ]
[-0.0125 .20086 -24.3125 -7.84375 ]
[-0.0125 .20086 0 7.84375 ]
[-0.0125 .27812 -24.3125 -7.84375 ]
[-0.0125 .27812 0 7.84375 ]
[-0.0125 .35537 -24.3125 -7.84375 ]
[-0.0125 .35537 0 7.84375 ]
[-0.0125 .43262 -24.3125 -7.84375 ]
[-0.0125 .43262 0 7.84375 ]
[-0.0125 .50988 -24.3125 -7.84375 ]
[-0.0125 .50988 0 7.84375 ]
[-0.0125 .58713 -18.3125 -7.84375 ]
[-0.0125 .58713 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40 -30.375 ]
[-0.01875 .30902 -24.3125 30.375 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -21.9688 0 ]
[.5 .63053 21.9688 15.6875 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.12903 0 m
.12903 .00625 L
s
gsave
.12903 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.4) show
1.000 setlinewidth
grestore
.29032 0 m
.29032 .00625 L
s
gsave
.29032 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.6) show
1.000 setlinewidth
grestore
.45161 0 m
.45161 .00625 L
s
gsave
.45161 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.8) show
1.000 setlinewidth
grestore
.6129 0 m
.6129 .00625 L
s
gsave
.6129 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.77419 0 m
.77419 .00625 L
s
gsave
.77419 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.2) show
1.000 setlinewidth
grestore
.93548 0 m
.93548 .00625 L
s
gsave
.93548 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16935 0 m
.16935 .00375 L
s
.20968 0 m
.20968 .00375 L
s
.25 0 m
.25 .00375 L
s
.33065 0 m
.33065 .00375 L
s
.37097 0 m
.37097 .00375 L
s
.41129 0 m
.41129 .00375 L
s
.49194 0 m
.49194 .00375 L
s
.53226 0 m
.53226 .00375 L
s
.57258 0 m
.57258 .00375 L
s
.65323 0 m
.65323 .00375 L
s
.69355 0 m
.69355 .00375 L
s
.73387 0 m
.73387 .00375 L
s
.81452 0 m
.81452 .00375 L
s
.85484 0 m
.85484 .00375 L
s
.89516 0 m
.89516 .00375 L
s
.08871 0 m
.08871 .00375 L
s
.04839 0 m
.04839 .00375 L
s
.00806 0 m
.00806 .00375 L
s
.97581 0 m
.97581 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9063 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
73.313 15.813 moveto
(H) show
77.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.000 15.813 moveto
(L) show
94.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .04635 m
.00625 .04635 L
s
gsave
-0.0125 .04635 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
0 .12361 m
.00625 .12361 L
s
gsave
-0.0125 .12361 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(35) show
1.000 setlinewidth
grestore
0 .20086 m
.00625 .20086 L
s
gsave
-0.0125 .20086 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(30) show
1.000 setlinewidth
grestore
0 .27812 m
.00625 .27812 L
s
gsave
-0.0125 .27812 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(25) show
1.000 setlinewidth
grestore
0 .35537 m
.00625 .35537 L
s
gsave
-0.0125 .35537 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .43262 m
.00625 .43262 L
s
gsave
-0.0125 .43262 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(15) show
1.000 setlinewidth
grestore
0 .50988 m
.00625 .50988 L
s
gsave
-0.0125 .50988 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
0 .58713 m
.00625 .58713 L
s
gsave
-0.0125 .58713 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0618 m
.00375 .0618 L
s
0 .07725 m
.00375 .07725 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .13906 m
.00375 .13906 L
s
0 .15451 m
.00375 .15451 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .21631 m
.00375 .21631 L
s
0 .23176 m
.00375 .23176 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .29357 m
.00375 .29357 L
s
0 .30902 m
.00375 .30902 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .37082 m
.00375 .37082 L
s
0 .38627 m
.00375 .38627 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .44807 m
.00375 .44807 L
s
0 .46353 m
.00375 .46353 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .52533 m
.00375 .52533 L
s
0 .54078 m
.00375 .54078 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .0309 m
.00375 .0309 L
s
0 .01545 m
.00375 .01545 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -123.531 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
91.375 11.8438 translate
90 rotate
-91.375 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Permittivity) show
119.750 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.12903 .61178 m
.12903 .61803 L
s
.29032 .61178 m
.29032 .61803 L
s
.45161 .61178 m
.45161 .61803 L
s
.6129 .61178 m
.6129 .61803 L
s
.77419 .61178 m
.77419 .61803 L
s
.93548 .61178 m
.93548 .61803 L
s
.125 Mabswid
.16935 .61428 m
.16935 .61803 L
s
.20968 .61428 m
.20968 .61803 L
s
.25 .61428 m
.25 .61803 L
s
.33065 .61428 m
.33065 .61803 L
s
.37097 .61428 m
.37097 .61803 L
s
.41129 .61428 m
.41129 .61803 L
s
.49194 .61428 m
.49194 .61803 L
s
.53226 .61428 m
.53226 .61803 L
s
.57258 .61428 m
.57258 .61803 L
s
.65323 .61428 m
.65323 .61803 L
s
.69355 .61428 m
.69355 .61803 L
s
.73387 .61428 m
.73387 .61803 L
s
.81452 .61428 m
.81452 .61803 L
s
.85484 .61428 m
.85484 .61803 L
s
.89516 .61428 m
.89516 .61803 L
s
.08871 .61428 m
.08871 .61803 L
s
.04839 .61428 m
.04839 .61803 L
s
.00806 .61428 m
.00806 .61803 L
s
.97581 .61428 m
.97581 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -82.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Figure) show
96.938 15.813 moveto
(1) show
102.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .04635 m
1 .04635 L
s
.99375 .12361 m
1 .12361 L
s
.99375 .20086 m
1 .20086 L
s
.99375 .27812 m
1 .27812 L
s
.99375 .35537 m
1 .35537 L
s
.99375 .43262 m
1 .43262 L
s
.99375 .50988 m
1 .50988 L
s
.99375 .58713 m
1 .58713 L
s
.125 Mabswid
.99625 .0618 m
1 .0618 L
s
.99625 .07725 m
1 .07725 L
s
.99625 .09271 m
1 .09271 L
s
.99625 .10816 m
1 .10816 L
s
.99625 .13906 m
1 .13906 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .16996 m
1 .16996 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .23176 m
1 .23176 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26266 m
1 .26266 L
s
.99625 .29357 m
1 .29357 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .32447 m
1 .32447 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .38627 m
1 .38627 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .41717 m
1 .41717 L
s
.99625 .44807 m
1 .44807 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .47898 m
1 .47898 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .54078 m
1 .54078 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57168 m
1 .57168 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .01545 m
1 .01545 L
s
.99625 .60258 m
1 .60258 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 0 r
.005 w
0 .01215 m
.04057 .07099 L
.08481 .12792 L
.12636 .17551 L
.16632 .21667 L
.20885 .25622 L
.2498 .29066 L
.29331 .32391 L
.33524 .35307 L
.37557 .37881 L
.41848 .40401 L
.4598 .42641 L
.49953 .44643 L
.54183 .46632 L
.58254 .48425 L
.62583 .50218 L
.66752 .51849 L
.70762 .53342 L
.7503 .54862 L
.79139 .56265 L
.83505 .57698 L
.87712 .59016 L
.9176 .60206 L
.96065 .61333 L
s
.96065 .61333 m
.98369 .61803 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.522676 0.323031 0.97619 0.603319 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1 0.806452 0.706325 0.176581 [
[.12903 -0.0125 -9.5 -15.6875 ]
[.12903 -0.0125 9.5 0 ]
[.29032 -0.0125 -9.5 -15.6875 ]
[.29032 -0.0125 9.5 0 ]
[.45161 -0.0125 -9.5 -15.6875 ]
[.45161 -0.0125 9.5 0 ]
[.6129 -0.0125 -5 -15.6875 ]
[.6129 -0.0125 5 0 ]
[.77419 -0.0125 -9.5 -15.6875 ]
[.77419 -0.0125 9.5 0 ]
[.93548 -0.0125 -9.5 -15.6875 ]
[.93548 -0.0125 9.5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9063 -31.375 ]
[.5 -0.0125 17.9063 -15.6875 ]
[-0.0125 .08829 -27.3125 -7.84375 ]
[-0.0125 .08829 0 7.84375 ]
[-0.0125 .17658 -18.3125 -7.84375 ]
[-0.0125 .17658 0 7.84375 ]
[-0.0125 .26487 -27.3125 -7.84375 ]
[-0.0125 .26487 0 7.84375 ]
[-0.0125 .35316 -18.3125 -7.84375 ]
[-0.0125 .35316 0 7.84375 ]
[-0.0125 .44145 -27.3125 -7.84375 ]
[-0.0125 .44145 0 7.84375 ]
[-0.0125 .52974 -18.3125 -7.84375 ]
[-0.0125 .52974 0 7.84375 ]
[-0.0125 .61803 -27.3125 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -43 -30.375 ]
[-0.01875 .30902 -27.3125 30.375 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -21.9688 0 ]
[.5 .63053 21.9688 15.6875 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.12903 0 m
.12903 .00625 L
s
gsave
.12903 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.4) show
1.000 setlinewidth
grestore
.29032 0 m
.29032 .00625 L
s
gsave
.29032 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.6) show
1.000 setlinewidth
grestore
.45161 0 m
.45161 .00625 L
s
gsave
.45161 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.8) show
1.000 setlinewidth
grestore
.6129 0 m
.6129 .00625 L
s
gsave
.6129 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.77419 0 m
.77419 .00625 L
s
gsave
.77419 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.2) show
1.000 setlinewidth
grestore
.93548 0 m
.93548 .00625 L
s
gsave
.93548 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16935 0 m
.16935 .00375 L
s
.20968 0 m
.20968 .00375 L
s
.25 0 m
.25 .00375 L
s
.33065 0 m
.33065 .00375 L
s
.37097 0 m
.37097 .00375 L
s
.41129 0 m
.41129 .00375 L
s
.49194 0 m
.49194 .00375 L
s
.53226 0 m
.53226 .00375 L
s
.57258 0 m
.57258 .00375 L
s
.65323 0 m
.65323 .00375 L
s
.69355 0 m
.69355 .00375 L
s
.73387 0 m
.73387 .00375 L
s
.81452 0 m
.81452 .00375 L
s
.85484 0 m
.85484 .00375 L
s
.89516 0 m
.89516 .00375 L
s
.08871 0 m
.08871 .00375 L
s
.04839 0 m
.04839 .00375 L
s
.00806 0 m
.00806 .00375 L
s
.97581 0 m
.97581 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9063 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
73.313 15.813 moveto
(H) show
77.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.000 15.813 moveto
(L) show
94.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .08829 m
.00625 .08829 L
s
gsave
-0.0125 .08829 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.5) show
1.000 setlinewidth
grestore
0 .17658 m
.00625 .17658 L
s
gsave
-0.0125 .17658 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .26487 m
.00625 .26487 L
s
gsave
-0.0125 .26487 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.5) show
1.000 setlinewidth
grestore
0 .35316 m
.00625 .35316 L
s
gsave
-0.0125 .35316 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
0 .44145 m
.00625 .44145 L
s
gsave
-0.0125 .44145 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.5) show
1.000 setlinewidth
grestore
0 .52974 m
.00625 .52974 L
s
gsave
-0.0125 .52974 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .01766 m
.00375 .01766 L
s
0 .03532 m
.00375 .03532 L
s
0 .05297 m
.00375 .05297 L
s
0 .07063 m
.00375 .07063 L
s
0 .10595 m
.00375 .10595 L
s
0 .12361 m
.00375 .12361 L
s
0 .14126 m
.00375 .14126 L
s
0 .15892 m
.00375 .15892 L
s
0 .19424 m
.00375 .19424 L
s
0 .2119 m
.00375 .2119 L
s
0 .22956 m
.00375 .22956 L
s
0 .24721 m
.00375 .24721 L
s
0 .28253 m
.00375 .28253 L
s
0 .30019 m
.00375 .30019 L
s
0 .31785 m
.00375 .31785 L
s
0 .3355 m
.00375 .3355 L
s
0 .37082 m
.00375 .37082 L
s
0 .38848 m
.00375 .38848 L
s
0 .40614 m
.00375 .40614 L
s
0 .42379 m
.00375 .42379 L
s
0 .45911 m
.00375 .45911 L
s
0 .47677 m
.00375 .47677 L
s
0 .49443 m
.00375 .49443 L
s
0 .51209 m
.00375 .51209 L
s
0 .5474 m
.00375 .5474 L
s
0 .56506 m
.00375 .56506 L
s
0 .58272 m
.00375 .58272 L
s
0 .60038 m
.00375 .60038 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -126.531 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
91.375 11.8438 translate
90 rotate
-91.375 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Permittivity) show
119.750 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.12903 .61178 m
.12903 .61803 L
s
.29032 .61178 m
.29032 .61803 L
s
.45161 .61178 m
.45161 .61803 L
s
.6129 .61178 m
.6129 .61803 L
s
.77419 .61178 m
.77419 .61803 L
s
.93548 .61178 m
.93548 .61803 L
s
.125 Mabswid
.16935 .61428 m
.16935 .61803 L
s
.20968 .61428 m
.20968 .61803 L
s
.25 .61428 m
.25 .61803 L
s
.33065 .61428 m
.33065 .61803 L
s
.37097 .61428 m
.37097 .61803 L
s
.41129 .61428 m
.41129 .61803 L
s
.49194 .61428 m
.49194 .61803 L
s
.53226 .61428 m
.53226 .61803 L
s
.57258 .61428 m
.57258 .61803 L
s
.65323 .61428 m
.65323 .61803 L
s
.69355 .61428 m
.69355 .61803 L
s
.73387 .61428 m
.73387 .61803 L
s
.81452 .61428 m
.81452 .61803 L
s
.85484 .61428 m
.85484 .61803 L
s
.89516 .61428 m
.89516 .61803 L
s
.08871 .61428 m
.08871 .61803 L
s
.04839 .61428 m
.04839 .61803 L
s
.00806 .61428 m
.00806 .61803 L
s
.97581 .61428 m
.97581 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -82.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Figure) show
96.938 15.813 moveto
(2) show
102.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 0 m
1 0 L
s
.99375 .08829 m
1 .08829 L
s
.99375 .17658 m
1 .17658 L
s
.99375 .26487 m
1 .26487 L
s
.99375 .35316 m
1 .35316 L
s
.99375 .44145 m
1 .44145 L
s
.99375 .52974 m
1 .52974 L
s
.125 Mabswid
.99625 .01766 m
1 .01766 L
s
.99625 .03532 m
1 .03532 L
s
.99625 .05297 m
1 .05297 L
s
.99625 .07063 m
1 .07063 L
s
.99625 .10595 m
1 .10595 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14126 m
1 .14126 L
s
.99625 .15892 m
1 .15892 L
s
.99625 .19424 m
1 .19424 L
s
.99625 .2119 m
1 .2119 L
s
.99625 .22956 m
1 .22956 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .28253 m
1 .28253 L
s
.99625 .30019 m
1 .30019 L
s
.99625 .31785 m
1 .31785 L
s
.99625 .3355 m
1 .3355 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .38848 m
1 .38848 L
s
.99625 .40614 m
1 .40614 L
s
.99625 .42379 m
1 .42379 L
s
.99625 .45911 m
1 .45911 L
s
.99625 .47677 m
1 .47677 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .51209 m
1 .51209 L
s
.99625 .5474 m
1 .5474 L
s
.99625 .56506 m
1 .56506 L
s
.99625 .58272 m
1 .58272 L
s
.99625 .60038 m
1 .60038 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 0 r
.005 w
[ .02 ] 0 setdash
0 .21882 m
.04057 .26898 L
.08481 .31491 L
.12636 .35114 L
.16632 .38068 L
.20885 .40725 L
.2498 .42871 L
.29331 .44766 L
.33524 .46253 L
.37557 .47398 L
.39599 .47877 L
.41848 .48326 L
.44006 .48683 L
.4598 .48946 L
.48018 .49152 L
.49142 .49237 L
.49702 .49272 L
.5021 .49299 L
.50694 .49321 L
.51217 .4934 L
.51505 .49349 L
.5177 .49356 L
.52039 .49361 L
.5229 .49365 L
.52506 .49368 L
.52627 .49369 L
.52738 .4937 L
.52858 .49371 L
.5299 .49371 L
.53114 .49371 L
.53229 .49371 L
.53347 .49371 L
.53457 .4937 L
.53579 .49369 L
.5371 .49368 L
.53976 .49365 L
.54221 .4936 L
.54718 .49348 L
.5525 .4933 L
.56208 .49285 L
.57346 .49208 L
.58381 .49117 L
.60396 .48875 L
.62579 .48513 L
.64666 .48059 L
.66876 .47454 L
.70854 .4599 L
.7509 .43787 L
.79166 .40857 L
.835 .36587 L
Mistroke
.87675 .30957 L
.91691 .23655 L
.95964 .13211 L
1 .00256 L
Mfstroke
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{864, 533.938},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000=P00025B000`400?l00000o`00003oo`3clo?o
0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00
o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S
0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?c
o`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??c
lol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c
003o0??clol0lo?co`3clo=S0??cl`00j@3clo<00`000000lo?c0??cl`0<0??cl`030000003clo<0
lo?c0?l0lo?cWP3clo<00`000000lo?c0??cl`0;0??cl`030000003clo<0lo?c0;L0lo?c003X0??c
l`030000003clo<0lo?c00h0lo?c00<000000??cl`3clo<0o`3clo>L0??cl`030000003clo<0lo?c
00d0lo?c00<000000??cl`3clo<0]P3clo<00=`0lo?c1P0000050??cl`040000003clo<0lo?c0??c
l`8000001@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0?l0lo?cSP3clo<60000
00H0lo?c00<000000??cl`3clo<00P0000050??cl`030000003clo<0lo?c00D0lo?c00<000000??c
l`3clo<0]@3clo<00=d0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`020??cl`04
0000003clo<0lo?c00000080lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`050??c
l`030000003clo<0lo?c0?l0lo?cS`3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c
00<0lo?c00<000000??cl`0000000P3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c
00D0lo?c00<000000??cl`3clo<0]@3clo<00=d0lo?c00<000000??cl`3clo<01`3clo<010000000
lo?c0??cl`0000060??cl`030000003clo<0000000H0lo?c00<000000??cl`3clo<0o`3clo>?0??c
l`030000003clo<0lo?c00P0lo?c00<000000??cl`0000001P3clo<00`000000lo?c000000060??c
l`030000003clo<0lo?c0;D0lo?c003M0??cl`040000003clo<0lo?c000000H0lo?c00<000000??c
l`3clo<0100000030??cl`030000003clo<0000000H0lo?c00<000000??cl`3clo<0o`3clo>?0??c
l`040000003clo<0lo?c000000L0lo?c00<000000??cl`0000000`0000030??cl`030000003clo<0
000000H0lo?c00<000000??cl`3clo<0]@3clo<00=d0lo?c100000060??cl`040000003clo<0lo?c
00000080lo?c00L000000??cl`3clo<0lo?c0000003clo<0000000H0lo?c00<000000??cl`3clo<0
o`3clo>?0??cl`@000001`3clo<00`000000lo?c000000020??cl`070000003clo<0lo?c0??cl`00
0000lo?c000000060??cl`030000003clo<0lo?c0;D0lo?c003M0??cl`040000003clo<0lo?c0000
00H0lo?c00@000000??cl`3clo<0lo?c0P0000030??cl`050000003clo<0lo?c0??cl`0000001@3c
lo<00`000000lo?c0??cl`3o0??clhl0lo?c00@000000??cl`3clo<000001`3clo<00`000000lo?c
0??cl`02000000<0lo?c00D000000??cl`3clo<0lo?c000000050??cl`030000003clo<0lo?c0;D0
lo?c003M0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<01P3clo<01@000000lo?c
0??cl`3clo<0000000D0lo?c00<000000??cl`3clo<0o`3clo>?0??cl`030000003clo<0lo?c00P0
lo?c00<000000??cl`3clo<01@3clo<01@000000lo?c0??cl`3clo<0000000D0lo?c00<000000??c
l`3clo<0]@3clo<00=d0lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000003clo<0lo?c
00D0lo?c00D000000??cl`3clo<0lo?c000000040??cl`030000003clo<0lo?c0?l0lo?cT03clo<0
1@000000lo?c0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<0103clo<01@000000lo?c0??c
l`3clo<0000000@0lo?c00<000000??cl`3clo<0]P3clo<00=`0lo?c1P0000070??cl`030000003c
lo<0lo?c00<0lo?c0`000000103clo<000000000000000020??cl`030000003clo<0lo?c0?l0lo?c
T03clo<6000000P0lo?c00<000000??cl`3clo<00P3clo<3000000040??cl`000000000000000080
lo?c00<000000??cl`3clo<0]`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3c
lo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0
lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c001e
0??cl`<000000`3clo<00`000000lo?c0??cl`030??cl`030000003clo<0lo?c0280lo?c0`000003
0??cl`030000003clo<0lo?c00<000009@3clo<3000000<0lo?c00<000000??cl`3clo<00`00000X
0??cl`D00000:03clo<500000080lo?c00<000000??cl`0000001000000R0??cl`D000000P3clo<0
0`000000lo?c0??cl`030??cl`030000003clo<0lo?c0:/0lo?c0`0000030??cl`030000003clo<0
lo?c00<0lo?c00<000000??cl`3clo<08P3clo<3000000<0lo?c00<000000??cl`3clo<00`00000U
0??cl`<000000`3clo<00`000000lo?c0??cl`03000002P0lo?c1@00000X0??cl`D000000P3clo<0
0`000000lo?c0000000400000280lo?c1@0000020??cl`030000003clo<0lo?c00<0lo?c00<00000
0??cl`3clo<0>P3clo<007H0lo?c00<000000??cl`3clo<0203clo<00`000000lo?c0??cl`0S0??c
l`030000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000U0??cl`030000003clo<0
lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000X0??cl`040000003clo<0lo?c000002T0lo?c
00@000000??cl`3clo<000001@3clo<010000000lo?c0??cl`00000S0??cl`040000003clo<0lo?c
000000P0lo?c00<000000??cl`3clo<0[03clo<00`000000lo?c0??cl`080??cl`030000003clo<0
lo?c02<0lo?c00<000000??cl`3clo<0103clo<01@000000lo?c0??cl`3clo<0000002D0lo?c00<0
00000??cl`3clo<0103clo<01@000000lo?c0??cl`3clo<0000002P0lo?c00@000000??cl`3clo<0
0000:@3clo<010000000lo?c0??cl`0000050??cl`040000003clo<0lo?c000002<0lo?c00@00000
0??cl`3clo<00000203clo<00`000000lo?c0??cl`0j0??cl`00MP3clo<00`000000lo?c0??cl`04
0??cl`H00000903clo<00`000000lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`0000009@3c
lo<00`000000lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`000000:@3clo<00`000000lo?c
0??cl`0Z0??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<0903clo<00`000000lo?c
0??cl`040??cl`H00000[@3clo<00`000000lo?c0??cl`040??cl`H00000903clo<00`000000lo?c
0??cl`040??cl`050000003clo<0lo?c0??cl`0000009@3clo<00`000000lo?c0??cl`040??cl`05
0000003clo<0lo?c0??cl`000000:@3clo<00`000000lo?c0??cl`0Z0??cl`030000003clo<0lo?c
00H0lo?c00<000000??cl`3clo<0903clo<00`000000lo?c0??cl`040??cl`H00000>`3clo<007H0
lo?c00<000000??cl`3clo<0103clo<01@000000lo?c0??cl`3clo<0000002D0lo?c00<000000??c
l`3clo<0103clo<01@000000lo?c0??cl`3clo<0000002D0lo?c00<000000??cl`3clo<01@3clo<0
0`000000lo?c0000000[0??cl`030000003clo<0lo?c02X0lo?c00<000000??cl`3clo<01P3clo<0
0`000000lo?c0??cl`0T0??cl`030000003clo<0lo?c00<0lo?c00D000000??cl`3clo<0lo?c0000
002^0??cl`030000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000U0??cl`030000
003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000U0??cl`030000003clo<0lo?c00D0
lo?c00<000000??cl`000000:`3clo<00`000000lo?c0??cl`0Z0??cl`030000003clo<0lo?c00H0
lo?c00<000000??cl`3clo<0903clo<00`000000lo?c0??cl`030??cl`050000003clo<0lo?c0??c
l`000000?03clo<007H0lo?c00<000000??cl`3clo<01@3clo<010000000lo?c0??cl`00000U0??c
l`030000003clo<0lo?c00@0lo?c0P0000020??cl`030000003clo<0lo?c02<0lo?c00<000000??c
l`3clo<01P3clo<00`000000lo?c0??cl`0[0??cl`030000003clo<0lo?c02X0lo?c00<000000??c
l`3clo<01P3clo<00`000000lo?c0??cl`0T0??cl`030000003clo<0lo?c00<0lo?c00@000000??c
l`3clo<00000[P3clo<00`000000lo?c0??cl`050??cl`040000003clo<0lo?c000002D0lo?c00<0
00000??cl`3clo<0103clo<200000080lo?c00<000000??cl`3clo<08`3clo<00`000000lo?c0??c
l`060??cl`030000003clo<0lo?c02/0lo?c00<000000??cl`3clo<0:P3clo<00`000000lo?c0??c
l`060??cl`030000003clo<0lo?c02@0lo?c00<000000??cl`3clo<00`3clo<010000000lo?c0??c
l`00000l0??cl`00MP3clo<00`000000lo?c0??cl`060??cl`030000003clo<0000002D0lo?c00<0
00000??cl`3clo<0103clo<010000000lo?c00000000000V0??cl`030000003clo<0lo?c00D0lo?c
00<000000??cl`000000;03clo<00`000000lo?c0??cl`0Z0??cl`030000003clo<0lo?c00H0lo?c
00<000000??cl`3clo<0903clo<00`000000lo?c0??cl`040??cl`030000003clo<000000:h0lo?c
00<000000??cl`3clo<01P3clo<00`000000lo?c0000000U0??cl`030000003clo<0lo?c00@0lo?c
00@000000??cl`00000000009P3clo<00`000000lo?c0??cl`050??cl`030000003clo<0000002`0
lo?c00<000000??cl`3clo<0:P3clo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c02@0
lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0000000l0??cl`00MP3clo<00`000000lo?c
0??cl`060??cl`030000003clo<0000002D0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c
0??cl`0V0??cl`030000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000[0??cl`03
0000003clo<0lo?c02X0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`0T0??cl`03
0000003clo<0lo?c00@0lo?c00<000000??cl`000000[P3clo<00`000000lo?c0??cl`060??cl`03
0000003clo<0000002D0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0V0??cl`03
0000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000[0??cl`030000003clo<0lo?c
02X0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`0T0??cl`030000003clo<0lo?c
00@0lo?c00<000000??cl`000000?03clo<007D0lo?c0P0000090??cl`800000903clo<2000000P0
lo?c00<000000??cl`3clo<0903clo<2000000H0lo?c00D000000??cl`3clo<0lo?c0000000W0??c
l`050000003clo<0lo?c0??cl`000000:03clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D0
00000??cl`3clo<0lo?c0000000R0??cl`050000003clo<0lo?c0??cl`0000001`3clo<200000:d0
lo?c0P0000090??cl`800000903clo<2000000P0lo?c00<000000??cl`3clo<0903clo<2000000H0
lo?c00D000000??cl`3clo<0lo?c0000000W0??cl`050000003clo<0lo?c0??cl`000000:03clo<0
1@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c0000000R0??cl`050000
003clo<0lo?c0??cl`0000001`3clo<2000003`0lo?c001f0??cl`030000003clo<0lo?c00P0lo?c
00<000000??cl`3clo<08`3clo<00`000000lo?c0??cl`070??cl`8000009@3clo<00`000000lo?c
0??cl`050??cl`<00000:@3clo<3000002X0lo?c0`0000060??cl`<00000903clo<3000000T0lo?c
00<000000??cl`3clo<0[03clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c02<0lo?c
00<000000??cl`3clo<01`3clo<2000002D0lo?c00<000000??cl`3clo<01@3clo<3000002T0lo?c
0`00000Z0??cl`<000001P3clo<3000002@0lo?c0`0000090??cl`030000003clo<0lo?c03X0lo?c
003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??c
lf<0lo?c00110??cl`<000000`3clo<00`000000lo?c0000000300000?l0lo?co`3clo?o0??claH0
lo?c00100??cl`050000003clo<0lo?c0??cl`000000203clo<00`000000lo?c0??cl`3o0??clol0
lo?co`3clo<C0??cl`00@03clo<01@000000lo?c0??cl`3clo<0000000P0lo?c00<000000??cl`3c
lo<01@3clo?o000003000000O`3clo?o000003000000;P3clo<00400lo?c00D000000??cl`3clo<0
lo?c000000080??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`0000002`3clo<00`000000
lo?c0??cl`0:0??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`0:0??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<0103clo<00`000000
lo?c0??cl`1m0??cl`030000003clo<0000000/0lo?c00<000000??cl`3clo<02P3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`0:0??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`090??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0;03clo<00400lo?c
00D000000??cl`3clo<0lo?c000000060??cl`800000203clo<00`000000lo?c0000000;0??cl`03
0000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`040??cl`03
0000003clo<0lo?c07d0lo?c00<000000??cl`0000002`3clo<00`000000lo?c0??cl`0:0??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`0/0??cl`00
@03clo<01@000000lo?c0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<01P3clo<00`000000
lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0
lo?c00<000000??cl`3clo<0;03clo<00400lo?c00D000000??cl`3clo<0lo?c000000080??cl`03
0000003clo<0lo?c00D0lo?c0`00003o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<00`000000
lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<00400lo?c00D000000??cl`3clo<0
lo?c000000040??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`020000
0?l0lo?c:P3clo<00`000000lo?c0??cl`1m0??cl`800000o`3clo</0??cl`800000;P3clo<00440
lo?c0`0000060??cl`<00000203clo<00`000000lo?c0??cl`020??cl`800000o`3clo<X0??cl`03
0000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0
lo?c001E0??cl`030000003clo<0lo?c00@0lo?c0P00003o0??clbH0lo?c00<000000??cl`3clo<0
O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c0P00
00070??cl`800000o`3clo<S0??cl`800000O`3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<0
00000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0203clo<300000?l0lo?c8@3clo<0
0`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`0/
0??cl`00E@3clo<00`000000lo?c0??cl`0;0??cl`800000o`3clo<O0??cl`030000003clo<0lo?c
07d0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`03
0000003clo<0lo?c00d0lo?c0P00003o0??clad0lo?c00<000000??cl`3clo<0O@3clo<00`000000
lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0
3`3clo<200000?l0lo?c6`3clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c
;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`0A0??cl`800000o`3c
lo<I0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003c
lo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c01<0lo?c0P00003o0??claL0lo?c00<00000
0??cl`3clo<0O@3clo<300000?l0lo?c:`3clo<2000002h0lo?c001E0??cl`8000005P3clo<30000
0?l0lo?c4`3clo<2000007l0lo?c00<000000??cl`3clo<01P00003o0??clbH0lo?c00<000000??c
l`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0603clo<200000?l0lo?c4P3clo<00`000000
lo?c0??cl`1m0??cl`030000003clo<0lo?c00H0lo?c1`00003o0??clal0lo?c00<000000??cl`3c
lo<0;03clo<005D0lo?c00<000000??cl`3clo<06P3clo<200000?l0lo?c403clo<00`000000lo?c
0??cl`1m0??cl`030000003clo<0lo?c00d0lo?c1P00003o0??claT0lo?c00<000000??cl`3clo<0
;03clo<005D0lo?c00<000000??cl`3clo<0703clo<200000?l0lo?c3P3clo<00`000000lo?c0??c
l`1m0??cl`030000003clo<0lo?c01<0lo?c1P00003o0??cla<0lo?c00<000000??cl`3clo<0;03c
lo<005D0lo?c00<000000??cl`3clo<07P3clo<200000?l0lo?c303clo<00`000000lo?c0??cl`1m
0??cl`030000003clo<0lo?c01T0lo?c1000003o0??cl`l0lo?c00<000000??cl`3clo<0;03clo<0
05D0lo?c00<000000??cl`3clo<0803clo<200000?l0lo?c2P3clo<00`000000lo?c0??cl`1m0??c
l`030000003clo<0lo?c01d0lo?c1000003o0??cl`/0lo?c00<000000??cl`3clo<0;03clo<005D0
lo?c00<000000??cl`3clo<08P3clo<200000?l0lo?c203clo<00`000000lo?c0??cl`1Y0??cl`<0
00000`3clo<00`000000lo?c0??cl`03000000P0lo?c00<000000??cl`3clo<08@3clo<400000?l0
lo?c1`3clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`0T0??cl`800000
o`3clo<60??cl`030000003clo<0lo?c06P0lo?c00D000000??cl`3clo<0lo?c000000060??cl`03
0000003clo<0lo?c00L0lo?c00<000000??cl`3clo<09@3clo<400000?l0lo?c0`3clo<00`000000
lo?c0??cl`0/0??cl`00E@3clo<2000002L0lo?c0P00003o0??cl`<0lo?c0P00001Z0??cl`050000
003clo<0lo?c0??cl`0000001P3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c02T0
lo?c1000003n0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c02P0lo?c
0P00003o0??cl`80lo?c00<000000??cl`3clo<0J03clo<01@000000lo?c0??cl`3clo<0000000H0
lo?c00<000000??cl`3clo<01`3clo<3000002d0lo?c1@00003h0??cl`800000;P3clo<005D0lo?c
00<000000??cl`3clo<0:P3clo<200000?l0lo?c00<000000??cl`3clo<0J03clo<01@000000lo?c
0??cl`3clo<0000000H0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`0b0??cl`@0
0000m@3clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`0/0??cl`800000
o@3clo<00`000000lo?c0??cl`1X0??cl`050000003clo<0lo?c0??cl`0000001P3clo<00`000000
lo?c0??cl`070??cl`030000003clo<0lo?c03H0lo?c1000003a0??cl`030000003clo<0lo?c02`0
lo?c001E0??cl`030000003clo<0lo?c02h0lo?c0`00003j0??cl`030000003clo<0lo?c06P0lo?c
00D000000??cl`3clo<0lo?c000000060??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3c
lo<0>P3clo<400000>d0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0
<@3clo<200000?P0lo?c00<000000??cl`3clo<0J03clo<01@000000lo?c0??cl`3clo<0000000D0
lo?c0P0000090??cl`030000003clo<0lo?c03h0lo?c1000003Y0??cl`030000003clo<0lo?c02`0
lo?c001E0??cl`030000003clo<0lo?c03<0lo?c0P00003f0??cl`030000003clo<0lo?c06T0lo?c
0`0000070??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0@P3clo<400000>D0lo?c
00<000000??cl`3clo<0;03clo<005D0lo?c0P00000f0??cl`800000l`3clo<2000007l0lo?c00<0
00000??cl`3clo<0AP3clo<500000>00lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<00000
0??cl`3clo<0=`3clo<200000?80lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`1;
0??cl`@00000g03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`0i0??c
l`800000l03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c04l0lo?c1@00003G0??c
l`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c03/0lo?c0P00003^0??cl`03
0000003clo<0lo?c07d0lo?c0P00001E0??cl`@00000dP3clo<2000002h0lo?c001E0??cl`030000
003clo<0lo?c03d0lo?c0P00003/0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0
F03clo<300000=00lo?c00<000000??cl`3clo<0;03clo<00440lo?c0`0000030??cl`030000003c
lo<0lo?c00<0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0o0??cl`800000jP3c
lo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c05/0lo?c0`00003=0??cl`030000003c
lo<0lo?c02`0lo?c00100??cl`050000003clo<0lo?c0??cl`000000203clo<00`000000lo?c0??c
l`050??cl`030000003clo<0lo?c0440lo?c00<000000??cl`3clo<0i`3clo<00`000000lo?c0??c
l`1m0??cl`030000003clo<0lo?c05h0lo?c0`00003:0??cl`030000003clo<0lo?c02`0lo?c0010
0??cl`050000003clo<0lo?c0??cl`000000103clo<6000000H0lo?c00<000000??cl`3clo<0@P3c
lo<200000>L0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`1Q0??cl`<00000a`3c
lo<00`000000lo?c0??cl`0/0??cl`00@03clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D0
00000??cl`3clo<0lo?c000000070??cl`<00000A03clo<200000><0lo?c0`00001o0??cl`030000
003clo<0lo?c06@0lo?c0`0000340??cl`030000003clo<0lo?c02`0lo?c00100??cl`050000003c
lo<0lo?c0??cl`0000001@3clo<010000000lo?c0??cl`0000070??cl`030000003clo<0lo?c04H0
lo?c0P00003S0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0I`3clo<300000<40
lo?c00<000000??cl`3clo<0;03clo<00400lo?c00D000000??cl`3clo<0lo?c000000060??cl`03
0000003clo<0000000L0lo?c00<000000??cl`3clo<0B03clo<00`000000lo?c0??cl`3P0??cl`03
0000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0JP3clo<300000;h0lo?c00<000000??cl`3c
lo<0;03clo<00400lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000003clo<0000000L0
lo?c00<000000??cl`3clo<0B@3clo<200000>00lo?c00<000000??cl`3clo<0O@3clo<00`000000
lo?c0??cl`1]0??cl`<00000^`3clo<00`000000lo?c0??cl`0/0??cl`00@03clo<01@000000lo?c
0??cl`3clo<0000000L0lo?c0P0000070??cl`030000003clo<0lo?c04/0lo?c0P00003N0??cl`03
0000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0L03clo<200000;T0lo?c00<000000??cl`3c
lo<0;03clo<00440lo?c0`0000090??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0
C@3clo<200000=`0lo?c00<000000??cl`3clo<0O@3clo<2000007<0lo?c0`00002e0??cl`800000
;P3clo<005D0lo?c00<000000??cl`3clo<0C`3clo<200000=X0lo?c00<000000??cl`3clo<0O@3c
lo<00`000000lo?c0??cl`1e0??cl`<00000/`3clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<2
00000580lo?c0P00003G0??cl`800000O`3clo<00`000000lo?c0??cl`1h0??cl`800000/@3clo<0
0`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`1C0??cl`800000eP3clo<00`00
0000lo?c0??cl`1m0??cl`030000003clo<0lo?c07X0lo?c0`00002^0??cl`030000003clo<0lo?c
02`0lo?c001E0??cl`030000003clo<0lo?c05D0lo?c00<000000??cl`3clo<0d`3clo<00`000000
lo?c0??cl`1m0??cl`030000003clo<0lo?c07d0lo?c0`00002[0??cl`030000003clo<0lo?c02`0
lo?c001E0??cl`030000003clo<0lo?c05H0lo?c0P00003C0??cl`030000003clo<0lo?c07d0lo?c
00<000000??cl`3clo<0P03clo<400000:L0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<0
00000??cl`3clo<0F03clo<200000=40lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??c
l`240??cl`<00000Y03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`1J
0??cl`800000c`3clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c08L0lo?c0`00002Q
0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c05`0lo?c0P00003=0??c
l`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0RP3clo<2000009l0lo?c00<000000??c
l`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0GP3clo<200000</0lo?c00<000000??cl`3c
lo<0O@3clo<2000008d0lo?c0P00002L0??cl`800000;P3clo<005D0lo?c0P00001Q0??cl`800000
b03clo<2000007l0lo?c00<000000??cl`3clo<0SP3clo<3000009X0lo?c00<000000??cl`3clo<0
;03clo<005D0lo?c00<000000??cl`3clo<0HP3clo<00`000000lo?c0??cl`360??cl`030000003c
lo<0lo?c07d0lo?c00<000000??cl`3clo<0T@3clo<2000009P0lo?c00<000000??cl`3clo<0;03c
lo<005D0lo?c00<000000??cl`3clo<0H`3clo<200000<H0lo?c00<000000??cl`3clo<0O@3clo<0
0`000000lo?c0??cl`2C0??cl`<00000U@3clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`00
0000lo?c0??cl`1U0??cl`800000a03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c
09H0lo?c0P00002C0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c06L0
lo?c00<000000??cl`3clo<0`@3clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c09P0
lo?c0P00002A0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c06P0lo?c
0P0000310??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0VP3clo<2000008l0lo?c
00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0JP3clo<200000;l0lo?c00<0
00000??cl`3clo<0O@3clo<00`000000lo?c0??cl`2L0??cl`800000S@3clo<00`000000lo?c0??c
l`0/0??cl`00E@3clo<2000006d0lo?c0P00002l0??cl`800000O`3clo<00`000000lo?c0??cl`2N
0??cl`800000R`3clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`1^0??c
l`030000003clo<0lo?c0;X0lo?c00<000000??cl`3clo<0O@3clo<200000:40lo?c0P0000280??c
l`800000;P3clo<005D0lo?c00<000000??cl`3clo<0K`3clo<200000;X0lo?c00<000000??cl`3c
lo<0O@3clo<00`000000lo?c0??cl`2R0??cl`800000Q`3clo<00`000000lo?c0??cl`0/0??cl`00
;`3clo<2000002@0lo?c00<000000??cl`3clo<0L@3clo<200000;P0lo?c00<000000??cl`3clo<0
E`3clo<2000002@0lo?c00<000000??cl`3clo<0Y03clo<2000008D0lo?c00<000000??cl`3clo<0
;03clo<002l0lo?c00<000000??cl`3clo<08`3clo<00`000000lo?c0??cl`1c0??cl`030000003c
lo<0lo?c0;D0lo?c00<000000??cl`3clo<0E`3clo<00`000000lo?c0??cl`0S0??cl`030000003c
lo<0lo?c0:H0lo?c0P0000230??cl`030000003clo<0lo?c02`0lo?c000_0??cl`030000003clo<0
lo?c00D0lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??cl`1d0??cl`800000]@3clo<0
0`000000lo?c0??cl`1G0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<06`3clo<0
0`000000lo?c0??cl`2X0??cl`800000P@3clo<00`000000lo?c0??cl`0/0??cl`00;`3clo<90000
01d0lo?c00<000000??cl`3clo<0MP3clo<00`000000lo?c0??cl`2b0??cl`030000003clo<0lo?c
05L0lo?c2@00000M0??cl`030000003clo<0lo?c0:X0lo?c0P00001o0??cl`030000003clo<0lo?c
02`0lo?c000_0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<06`3clo<00`000000
lo?c0??cl`1g0??cl`800000/P3clo<00`000000lo?c0??cl`1G0??cl`030000003clo<0lo?c00D0
lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??cl`2/0??cl`800000O@3clo<00`000000
lo?c0??cl`0/0??cl`00;`3clo<00`000000lo?c0??cl`0S0??cl`800000NP3clo<200000:l0lo?c
0P00001I0??cl`030000003clo<0lo?c00l0lo?c0`0000030??cl`030000003clo<0000000@00000
1`3clo<00`000000lo?c0??cl`2^0??cl`800000N`3clo<00`000000lo?c0??cl`0/0??cl`00;`3c
lo<2000002@0lo?c00<000000??cl`3clo<0N`3clo<00`000000lo?c0??cl`2]0??cl`030000003c
lo<0lo?c05L0lo?c0P00000?0??cl`050000003clo<0lo?c0??cl`0000001@3clo<010000000lo?c
0??cl`0000070??cl`030000003clo<0lo?c0;00lo?c0P00001i0??cl`030000003clo<0lo?c02`0
lo?c000b0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c
0??cl`1l0??cl`800000[@3clo<00`000000lo?c0??cl`1J0??cl`030000003clo<0lo?c0080lo?c
00<000000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`3c
lo<01`3clo<00`000000lo?c0??cl`2b0??cl`030000003clo<0lo?c07H0lo?c00<000000??cl`3c
lo<0;03clo<00380lo?c1P00000M0??cl`030000003clo<0lo?c07h0lo?c0P00002[0??cl`030000
003clo<0lo?c05X0lo?c1P0000080??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000
lo?c0??cl`060??cl`<00000/`3clo<2000007D0lo?c0P00000^0??cl`00<P3clo<00`000000lo?c
0??cl`020??cl`030000003clo<0lo?c00L0lo?c0`0000030??cl`030000003clo<0000000800000
2@3clo<00`000000lo?c0??cl`200??cl`800000Z@3clo<00`000000lo?c0??cl`1J0??cl`030000
003clo<0lo?c0080lo?c00<000000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000P0
lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`2e0??cl`030000003clo<0lo?c07<0
lo?c00<000000??cl`3clo<0;03clo<00380lo?c00<000000??cl`3clo<02`3clo<01@000000lo?c
0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`220??cl`80
0000Y`3clo<00`000000lo?c0??cl`1J0??cl`030000003clo<0lo?c00/0lo?c00D000000??cl`3c
lo<0lo?c000000080??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0]P3clo<20000
07<0lo?c00<000000??cl`3clo<0;03clo<003<0lo?c00@000000??cl`00000000002@3clo<01@00
0000lo?c0??cl`3clo<0000000P0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`24
0??cl`800000Y@3clo<00`000000lo?c0??cl`1K0??cl`040000003clo<00000000000T0lo?c00D0
00000??cl`3clo<0lo?c000000080??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0
^03clo<200000740lo?c00<000000??cl`3clo<0;03clo<00380lo?c00<000000??cl`0000000P3c
lo<00`000000lo?c0??cl`060??cl`050000003clo<0lo?c0??cl`000000203clo<00`000000lo?c
0??cl`050??cl`<00000QP3clo<200000:40lo?c0`00001L0??cl`030000003clo<000000080lo?c
00<000000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3c
lo<0lo?c000000070??cl`030000003clo<0lo?c0;X0lo?c00<000000??cl`3clo<0KP3clo<00`00
0000lo?c0??cl`0/0??cl`00<P3clo<01@000000lo?c0000003clo<0000000T0lo?c00D000000??c
l`3clo<0lo?c000000080??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0R03clo<0
0`000000lo?c0??cl`2P0??cl`030000003clo<0lo?c05X0lo?c00D000000??cl`000000lo?c0000
000:0??cl`<000001P3clo<3000000P0lo?c00<000000??cl`3clo<0^`3clo<2000006h0lo?c00<0
00000??cl`3clo<0;03clo<003<0lo?c1@0000080??cl`050000003clo<0lo?c0??cl`0000001`3c
lo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c08T0lo?c0P00002P0??cl`030000003c
lo<0lo?c05/0lo?c1@00000M0??cl`030000003clo<0lo?c0;d0lo?c0P00001/0??cl`030000003c
lo<0lo?c02`0lo?c000g0??cl`030000003clo<0lo?c00H0lo?c00D000000??cl`3clo<0lo?c0000
00050??cl`8000002@3clo<00`000000lo?c0??cl`2;0??cl`800000WP3clo<00`000000lo?c0??c
l`1O0??cl`030000003clo<0lo?c01/0lo?c00<000000??cl`3clo<0_`3clo<2000006X0lo?c00<0
00000??cl`3clo<0;03clo<00380lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`06
0??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0??cl`080??cl`030000003c
lo<0lo?c08d0lo?c00<000000??cl`3clo<0V`3clo<00`000000lo?c0??cl`1J0??cl`030000003c
lo<0lo?c0080lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??cl`310??cl`800000J03c
lo<00`000000lo?c0??cl`0/0??cl`00<P3clo<6000000T0lo?c0`0000070??cl`<000001`3clo<0
0`000000lo?c0??cl`2>0??cl`800000V`3clo<00`000000lo?c0??cl`1J0??cl`H000007@3clo<2
00000<@0lo?c0P00001U0??cl`800000;P3clo<00380lo?c00<000000??cl`3clo<00P3clo<00`00
0000lo?c0??cl`0K0??cl`030000003clo<0lo?c0900lo?c0P00002I0??cl`030000003clo<0lo?c
05X0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0K0??cl`030000003clo<0lo?c
0<D0lo?c0P00001T0??cl`030000003clo<0lo?c02`0lo?c000b0??cl`030000003clo<0lo?c0200
lo?c00<000000??cl`3clo<0TP3clo<2000009L0lo?c00<000000??cl`3clo<0FP3clo<00`000000
lo?c0??cl`0P0??cl`030000003clo<0lo?c0<L0lo?c00<000000??cl`3clo<0H@3clo<00`000000
lo?c0??cl`0/0??cl`00<`3clo<5000001d0lo?c0P00002E0??cl`030000003clo<0lo?c09<0lo?c
0P00001M0??cl`D000007@3clo<00`000000lo?c0??cl`380??cl`030000003clo<0lo?c0600lo?c
00<000000??cl`3clo<0;03clo<003L0lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??c
l`2E0??cl`800000U03clo<00`000000lo?c0??cl`1O0??cl`030000003clo<0lo?c01/0lo?c00<0
00000??cl`3clo<0b@3clo<200000600lo?c00<000000??cl`3clo<0;03clo<003<0lo?c0P000000
0`3clo<000000000000M0??cl`030000003clo<0lo?c09L0lo?c0P00002B0??cl`030000003clo<0
lo?c05/0lo?c0P0000000`3clo<000000000000M0??cl`030000003clo<0lo?c0</0lo?c00<00000
0??cl`3clo<0G@3clo<00`000000lo?c0??cl`0/0??cl`00<P3clo<01@000000lo?c0000003clo<0
000001h0lo?c00<000000??cl`3clo<0V@3clo<00`000000lo?c0??cl`2?0??cl`030000003clo<0
lo?c05X0lo?c00D000000??cl`000000lo?c0000000N0??cl`030000003clo<0lo?c0<`0lo?c00<0
00000??cl`3clo<0G03clo<00`000000lo?c0??cl`0/0??cl`00<P3clo<01P000000lo?c0??cl`00
0000lo?c000001d0lo?c00<000000??cl`3clo<0VP3clo<2000008l0lo?c00<000000??cl`3clo<0
FP3clo<01P000000lo?c0??cl`000000lo?c000001d0lo?c00<000000??cl`3clo<0c@3clo<20000
05`0lo?c00<000000??cl`3clo<0;03clo<00380lo?c0P0000000`3clo<000000000000N0??cl`03
0000003clo<0lo?c09`0lo?c00<000000??cl`3clo<0S03clo<00`000000lo?c0??cl`1J0??cl`80
000000<0lo?c0000000000007P3clo<00`000000lo?c0??cl`3?0??cl`030000003clo<0lo?c05T0
lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0W@3clo<2000008`0lo?c
00<000000??cl`3clo<0O@3clo<200000=40lo?c00<000000??cl`3clo<0E`3clo<2000002h0lo?c
000b0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<06`3clo<200000:00lo?c0P00
00290??cl`800000G03clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c01/0lo?c00<0
00000??cl`3clo<0d@3clo<2000005P0lo?c00<000000??cl`3clo<0;03clo<00380lo?c1P00000M
0??cl`030000003clo<0lo?c0:40lo?c00<000000??cl`3clo<0Q`3clo<00`000000lo?c0??cl`1J
0??cl`H000007@3clo<00`000000lo?c0??cl`3C0??cl`030000003clo<0lo?c05D0lo?c00<00000
0??cl`3clo<0;03clo<00380lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0K0??c
l`030000003clo<0lo?c0:80lo?c0P0000270??cl`030000003clo<0lo?c05X0lo?c00<000000??c
l`3clo<00P3clo<00`000000lo?c0??cl`0K0??cl`030000003clo<0lo?c0=@0lo?c0P00001E0??c
l`030000003clo<0lo?c02`0lo?c000b0??cl`030000003clo<0lo?c0200lo?c00<000000??cl`3c
lo<0Y03clo<00`000000lo?c0??cl`240??cl`030000003clo<0lo?c05X0lo?c00<000000??cl`3c
lo<0803clo<00`000000lo?c0??cl`3F0??cl`030000003clo<0lo?c0580lo?c00<000000??cl`3c
lo<0;03clo<003<0lo?c1@00000M0??cl`030000003clo<0lo?c0:D0lo?c0P0000240??cl`030000
003clo<0lo?c05/0lo?c1@00000M0??cl`030000003clo<0lo?c0=L0lo?c0P00001B0??cl`030000
003clo<0lo?c02`0lo?c000b0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<06`3c
lo<00`000000lo?c0??cl`2W0??cl`030000003clo<0lo?c0840lo?c00<000000??cl`3clo<0FP3c
lo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c01/0lo?c00<000000??cl`3clo<0f@3c
lo<00`000000lo?c0??cl`1?0??cl`030000003clo<0lo?c02`0lo?c000b0??cl`030000003clo<0
lo?c0200lo?c00<000000??cl`3clo<0Z03clo<200000840lo?c00<000000??cl`3clo<0FP3clo<0
0`000000lo?c0??cl`0P0??cl`030000003clo<0lo?c0=X0lo?c00<000000??cl`3clo<0CP3clo<0
0`000000lo?c0??cl`0/0??cl`00<`3clo<5000001d0lo?c00<000000??cl`3clo<0ZP3clo<00`00
0000lo?c0??cl`1n0??cl`030000003clo<0lo?c05/0lo?c1@00000M0??cl`030000003clo<0lo?c
0=/0lo?c0P00001>0??cl`030000003clo<0lo?c02`0lo?c000g0??cl`030000003clo<0lo?c01/0
lo?c0P00002/0??cl`800000O@3clo<200000640lo?c00<000000??cl`3clo<06`3clo<00`000000
lo?c0??cl`3M0??cl`030000003clo<0lo?c04/0lo?c00<000000??cl`3clo<0;03clo<00380lo?c
00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0K0??cl`030000003clo<0lo?c0:d0lo?c
00<000000??cl`3clo<0N`3clo<00`000000lo?c0??cl`1J0??cl`030000003clo<0lo?c0080lo?c
00<000000??cl`3clo<06`3clo<200000=l0lo?c0P00001:0??cl`800000;P3clo<002l0lo?c00<0
00000??cl`3clo<01P00000M0??cl`030000003clo<0lo?c0:h0lo?c0P00001k0??cl`030000003c
lo<0lo?c05L0lo?c00<000000??cl`3clo<01P00000M0??cl`030000003clo<0lo?c0>00lo?c00<0
00000??cl`3clo<0B03clo<00`000000lo?c0??cl`0/0??cl`00=`3clo<00`000000lo?c0??cl`0K
0??cl`030000003clo<0lo?c0;00lo?c0P00001i0??cl`030000003clo<0lo?c05l0lo?c00<00000
0??cl`3clo<06`3clo<00`000000lo?c0??cl`3Q0??cl`030000003clo<0lo?c04L0lo?c00<00000
0??cl`3clo<0;03clo<003<0lo?c0P0000000`3clo<000000000000M0??cl`030000003clo<0lo?c
0;80lo?c00<000000??cl`3clo<0MP3clo<00`000000lo?c0??cl`1K0??cl`80000000<0lo?c0000
000000007@3clo<00`000000lo?c0??cl`3R0??cl`030000003clo<0lo?c04H0lo?c00<000000??c
l`3clo<0;03clo<00380lo?c00D000000??cl`000000lo?c0000000N0??cl`030000003clo<0lo?c
0;<0lo?c0P00001f0??cl`030000003clo<0lo?c05X0lo?c00D000000??cl`000000lo?c0000000N
0??cl`030000003clo<0lo?c0><0lo?c0P0000160??cl`030000003clo<0lo?c02`0lo?c000b0??c
l`060000003clo<0lo?c0000003clo<000007@3clo<00`000000lo?c0??cl`2e0??cl`030000003c
lo<0lo?c07<0lo?c00<000000??cl`3clo<0FP3clo<01P000000lo?c0??cl`000000lo?c000001d0
lo?c00<000000??cl`3clo<0i@3clo<00`000000lo?c0??cl`130??cl`030000003clo<0lo?c02`0
lo?c000b0??cl`80000000<0lo?c0000000000007P3clo<200000;L0lo?c0P00001b0??cl`800000
G03clo<2000000030??cl`000000000001h0lo?c00<000000??cl`3clo<0iP3clo<00`000000lo?c
0??cl`120??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0;P0lo?c0P00
001a0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0i`3clo<00`000000lo?c0??c
l`110??cl`030000003clo<0lo?c02`0lo?c000c0??cl`80000000<0lo?c0000000000007@3clo<0
0`000000lo?c0??cl`2j0??cl`030000003clo<0lo?c06h0lo?c00<000000??cl`3clo<0F`3clo<2
000000030??cl`000000000001d0lo?c00<000000??cl`3clo<0j03clo<00`000000lo?c0??cl`10
0??cl`030000003clo<0lo?c02`0lo?c000b0??cl`050000003clo<000000??cl`0000007P3clo<0
0`000000lo?c0??cl`2k0??cl`800000KP3clo<00`000000lo?c0??cl`1J0??cl`050000003clo<0
00000??cl`0000007P3clo<200000>X0lo?c0P00000o0??cl`800000;P3clo<00380lo?c00H00000
0??cl`3clo<000000??cl`00000M0??cl`030000003clo<0lo?c0;d0lo?c0P00001/0??cl`030000
003clo<0lo?c05X0lo?c00H000000??cl`3clo<000000??cl`00000M0??cl`030000003clo<0lo?c
0>/0lo?c00<000000??cl`3clo<0?@3clo<00`000000lo?c0??cl`0/0??cl`00<P3clo<200000003
0??cl`000000000000X0lo?c0`0000030??cl`030000003clo<0lo?c00<00000203clo<00`000000
lo?c0??cl`2o0??cl`030000003clo<0lo?c06T0lo?c00<000000??cl`3clo<0FP3clo<200000003
0??cl`000000000001h0lo?c00<000000??cl`3clo<0k03clo<00`000000lo?c0??cl`0l0??cl`03
0000003clo<0lo?c02`0lo?c00100??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000
lo?c0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<0`03clo<2000006T0lo?c00<000000??c
l`3clo<0O@3clo<00`000000lo?c0??cl`3]0??cl`800000?03clo<00`000000lo?c0??cl`0/0??c
l`00<P3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c00H0lo?c00D000000??cl`3c
lo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`32
0??cl`030000003clo<0lo?c06H0lo?c00<000000??cl`3clo<0FP3clo<00`000000lo?c0??cl`02
0??cl`030000003clo<0lo?c01/0lo?c00<000000??cl`3clo<0k`3clo<00`000000lo?c0??cl`0i
0??cl`030000003clo<0lo?c02`0lo?c000_0??cl`030000003clo<0lo?c00H00000203clo<01@00
0000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c000000070??cl`<00000``3c
lo<2000006@0lo?c0`00001I0??cl`030000003clo<0lo?c00H000007@3clo<00`000000lo?c0??c
l`3`0??cl`030000003clo<0lo?c03P0lo?c00<000000??cl`3clo<0;03clo<003L0lo?c00<00000
0??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c0P0000020??cl`030000003c
lo<0lo?c00D0lo?c00<000000??cl`3clo<0a@3clo<2000006@0lo?c00<000000??cl`3clo<0G`3c
lo<00`000000lo?c0??cl`070??cl`<000000`3clo<00`000000lo?c00000003000000P0lo?c00<0
00000??cl`3clo<0l@3clo<2000003P0lo?c00<000000??cl`3clo<0;03clo<003<0lo?c10000009
0??cl`050000003clo<0lo?c0??cl`000000103clo<010000000lo?c0000000000080??cl`030000
003clo<0lo?c0<L0lo?c00<000000??cl`3clo<0H@3clo<00`000000lo?c0??cl`1K0??cl`@00000
2@3clo<01@000000lo?c0??cl`3clo<0000000P0lo?c00<000000??cl`3clo<01@3clo<00`000000
lo?c0??cl`3c0??cl`030000003clo<0lo?c03D0lo?c00<000000??cl`3clo<0;03clo<00380lo?c
00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`060??cl`050000003clo<0lo?c0??cl`00
00001@3clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c0<P0lo?c0P00001Q0??cl`03
0000003clo<0lo?c05X0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`060??cl`05
0000003clo<0lo?c0??cl`000000203clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c
0?@0lo?c0P00000e0??cl`030000003clo<0lo?c02`0lo?c000b0??cl`030000003clo<0lo?c0080
lo?c00<000000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c00<000000??c
l`3clo<01`3clo<00`000000lo?c0??cl`3:0??cl`030000003clo<0lo?c05h0lo?c00<000000??c
l`3clo<0FP3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c00H0lo?c00D000000??c
l`3clo<0lo?c000000080??cl`030000003clo<0lo?c00D0lo?c0`00003f0??cl`030000003clo<0
lo?c0340lo?c0P00000^0??cl`00<P3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c
00L0lo?c0`0000080??cl`8000001`3clo<00`000000lo?c0??cl`3;0??cl`800000GP3clo<00`00
0000lo?c0??cl`1J0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<01P3clo<01@00
0000lo?c0??cl`3clo<0000000H0lo?c0P0000080??cl`030000003clo<0lo?c0?L0lo?c00<00000
0??cl`3clo<0<@3clo<00`000000lo?c0??cl`0/0??cl`00<`3clo<4000001h0lo?c00<000000??c
l`3clo<0c@3clo<00`000000lo?c0??cl`1K0??cl`030000003clo<0lo?c05/0lo?c100000090??c
l`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`060??cl`030000003clo<0
lo?c0?P0lo?c0P00000a0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`800000c`3clo<20000
05X0lo?c0P00001Z0??cl`050000003clo<0lo?c0??cl`000000203clo<00`000000lo?c0??cl`05
0??cl`030000003clo<0lo?c0?X0lo?c00<000000??cl`3clo<0;P3clo<00`000000lo?c0??cl`0/
0??cl`00<P3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c01/0lo?c00<000000??c
l`3clo<0d03clo<00`000000lo?c0??cl`1H0??cl`030000003clo<0lo?c05X0lo?c00<000000??c
l`3clo<00P3clo<00`000000lo?c0??cl`060??cl`050000003clo<0lo?c0??cl`000000103clo<0
1@000000lo?c0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<0n`3clo<00`000000lo?c0??c
l`0]0??cl`030000003clo<0lo?c02`0lo?c000b0??cl`H000007@3clo<00`000000lo?c0??cl`3A
0??cl`800000E`3clo<2000005`0lo?c1P0000090??cl`<000001P3clo<3000000P0lo?c00<00000
0??cl`3clo<0o03clo<00`000000lo?c0??cl`0/0??cl`030000003clo<0lo?c02`0lo?c000b0??c
l`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??cl`3C0??c
l`030000003clo<0lo?c05<0lo?c00<000000??cl`000000G03clo<00`000000lo?c0??cl`020??c
l`030000003clo<0lo?c01/0lo?c00<000000??cl`3clo<0o@3clo<2000002`0lo?c00<000000??c
l`3clo<0;03clo<00380lo?c00<000000??cl`3clo<0803clo<00`000000lo?c0??cl`3D0??cl`80
0000DP3clo<010000000lo?c0??cl`00001L0??cl`030000003clo<0lo?c0200lo?c00<000000??c
l`3clo<0o`3clo<00`000000lo?c0??cl`0Y0??cl`030000003clo<0lo?c02`0lo?c000c0??cl`D0
00007@3clo<00`000000lo?c0??cl`3F0??cl`800000C`3clo<01@000000lo?c0??cl`3clo<00000
05d0lo?c1@00000M0??cl`030000003clo<0lo?c0?l0lo?c0@3clo<00`000000lo?c0??cl`0W0??c
l`800000;P3clo<003L0lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??cl`3H0??cl`80
0000C03clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c05l0lo?c00<000000??cl`3c
lo<06`3clo<00`000000lo?c0??cl`3o0??cl`80lo?c0P00000V0??cl`030000003clo<0000002h0
lo?c001E0??cl`030000003clo<0lo?c0=X0lo?c0P0000190??cl`030000003clo<0lo?c00<0lo?c
00<000000??cl`3clo<0O@3clo<200000?l0lo?c1@3clo<00`000000lo?c0??cl`0R0??cl`040000
003clo<00000000002h0lo?c001E0??cl`800000g@3clo<2000004H0lo?c00<000000??cl`3clo<0
0`3clo<2000007l0lo?c00<000000??cl`3clo<0o`3clo<50??cl`8000008@3clo<01@000000lo?c
0??cl`3clo<0000002h0lo?c001E0??cl`030000003clo<0lo?c0=h0lo?c0P0000130??cl`030000
003clo<0lo?c00D0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??cl`L0lo?c
00<000000??cl`3clo<07@3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c02`0lo?c
001E0??cl`030000003clo<0lo?c0>00lo?c00<000000??cl`3clo<0?P3clo<2000000P0lo?c00<0
00000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??cl`P0lo?c0P00000K0??cl`8000001@3c
lo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`3Q0??cl`800000?@3clo<0
0`000000lo?c0??cl`080??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0o`3clo<:
0??cl`8000005`3clo<2000000L0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??c
l`3clo<0h`3clo<2000003X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`1m0??c
l`030000003clo<0lo?c0?l0lo?c303clo<300000180lo?c0P0000090??cl`030000003clo<0lo?c
02`0lo?c001E0??cl`030000003clo<0lo?c0>D0lo?c0P00000f0??cl`800000303clo<00`000000
lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c3`3clo<4000000/0lo?c0`00000;0??cl`03
0000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0>L0lo?c0P00000c0??cl`030000
003clo<0lo?c00`0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??cla<0lo?c
2`00000>0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`800000jP3clo<2000002l0lo?c0P00
000>0??cl`800000O`3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03c
lo<005D0lo?c00<000000??cl`3clo<0j`3clo<3000002/0lo?c00<000000??cl`3clo<03`3clo<0
0`000000lo?c0??cl`1m0??cl`800000o`3clo</0??cl`800000;P3clo<005D0lo?c00<000000??c
l`3clo<0kP3clo<4000002D0lo?c0P00000B0??cl`030000003clo<0lo?c07d0lo?c00<000000??c
l`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0?80
lo?c0`00000P0??cl`800000503clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0
lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`3e0??cl`<00000
6@3clo<4000001H0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c
00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0n03clo<3000001@0lo?c0P00
000J0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003c
lo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0?/0lo?c5000000L0??cl`030000003clo<0
lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c001E0??c
l`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c
0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<200000?l0lo?c;03clo<2000007l0
lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000
003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c
;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<0
00000??cl`3clo<0O@3clo<200000?l0lo?c;03clo<2000002h0lo?c001E0??cl`030000003clo<0
lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c;03clo<0
0`000000lo?c0??cl`0/0??cl`00@@3clo<3000000<0lo?c00D000000??cl`3clo<0lo?c00000009
0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0
lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00@03clo<01@000000lo?c0??cl`3clo<0
000000H0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<00000
0??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<0
0400lo?c00D000000??cl`3clo<0lo?c000000070??cl`030000003clo<0lo?c00H0lo?c00<00000
0??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</
0??cl`030000003clo<0lo?c02`0lo?c00100??cl`050000003clo<0lo?c0??cl`0000001`3clo<0
0`000000lo?c0??cl`060??cl`<00000o`3clo<Z0??cl`<00000O`3clo<00`000000lo?c0??cl`3o
0??clb`0lo?c00<000000??cl`3clo<0;03clo<00400lo?c00D000000??cl`3clo<0lo?c00000007
0??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0
lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c00100??c
l`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`060??cl`030000003clo<0
lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c;03clo<0
0`000000lo?c0??cl`0/0??cl`00@03clo<01@000000lo?c0??cl`3clo<0000000P0lo?c00<00000
0??cl`3clo<01@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<0
0`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<00400lo?c00D000000??c
l`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??c
l`3o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<0
00000??cl`3clo<0;03clo<00440lo?c0`0000060??cl`@000001`3clo<00`000000lo?c0??cl`3o
0??clb`0lo?c00<000000??cl`3clo<0O@3clo<200000?l0lo?c;03clo<2000002h0lo?c001E0??c
l`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c
0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`3o0??clb`0
lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3c
lo<0;03clo<005D0lo?c0P00003o0??clb`0lo?c0P00001o0??cl`030000003clo<0lo?c0?l0lo?c
;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<0
00000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03c
lo<005D0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c07d0lo?c00<00000
0??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c
0?l0lo?c;03clo<00`000000lo?c0??cl`1Y0??cl`<000000`3clo<00`000000lo?c0??cl`030??c
l`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c
02`0lo?c001E0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1X0??cl`05
0000003clo<0lo?c0??cl`000000203clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c
0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`3o0??clb`0
lo?c00<000000??cl`3clo<0J03clo<01@000000lo?c0??cl`3clo<0000000@0lo?c1P0000060??c
l`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000
lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0J03clo<01@000000lo?c0??cl`3clo<00000
00@0lo?c00D000000??cl`3clo<0lo?c000000070??cl`<00000o`3clo<[0??cl`800000;P3clo<0
05D0lo?c0P00003o0??clb`0lo?c0P00001Z0??cl`050000003clo<0lo?c0??cl`0000001@3clo<0
10000000lo?c0??cl`0000070??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??c
l`0/0??cl`00E@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0J03clo<0
1@000000lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`0000001`3clo<00`000000lo?c0??c
l`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0o`3clo</
0??cl`030000003clo<0lo?c06P0lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000003c
lo<0000000L0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c001E
0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1X0??cl`050000003clo<0
lo?c0??cl`0000001`3clo<2000000L0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003c
lo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1Y
0??cl`<000002@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0?l0lo?c;03clo<0
0`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??c
l`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<005D0
lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3c
lo<0o`3clo</0??cl`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0?l0lo?c
;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c
0??cl`0/0??cl`00E@3clo<200000?l0lo?c;03clo<2000007l0lo?c0P00003o0??clb`0lo?c0P00
000^0??cl`00E@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<0
0`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??c
l`3clo<0o`3clo</0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</0??c
l`030000003clo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000
lo?c0??cl`1m0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00
E@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c
0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0o`3c
lo</0??cl`030000003clo<0lo?c07d0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003c
lo<0lo?c02`0lo?c001E0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m
0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<20000
0?l0lo?c;03clo<2000007l0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003clo<0lo?c
02`0lo?c001E0??cl`030000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`1m0??cl`03
0000003clo<0lo?c0?l0lo?c;03clo<00`000000lo?c0??cl`0/0??cl`00E@3clo<00`000000lo?c
0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c
00<000000??cl`3clo<0;03clo<005D0lo?c00<000000??cl`3clo<0o`3clo</0??cl`030000003c
lo<0lo?c07d0lo?c0P00003o0??clb`0lo?c0P00000^0??cl`00E@3clo<00`000000lo?c0??cl`3o
0??clb`0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??clb`0lo?c00<00000
0??cl`3clo<0;03clo<00440lo?c0`0000030??cl`030000003clo<0lo?c00<00000203clo<00`00
0000lo?c0??cl`3o0??clb`0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`3o0??c
lb`0lo?c00<000000??cl`3clo<0;03clo<00400lo?c00D000000??cl`3clo<0lo?c000000040??c
l`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0000000;0??cl`030000003clo<0
lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`040??cl`030000003clo<0
lo?c07d0lo?c00<000000??cl`0000002`3clo<00`000000lo?c0??cl`0:0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`0/0??cl`00@03clo<01@00
0000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c000000070??cl`030000003c
lo<0000000/0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00@0lo?c00<000000??cl`3clo<0O@3clo<00`000000lo?c0000000;0??cl`030000003c
lo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`040??cl`030000003c
lo<0lo?c02`0lo?c00100??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0000
00080??clol00000<000001o0??clol00000<000000^0??cl`00@03clo<01@000000lo?c0??cl`3c
lo<0000000H0lo?c00<000000??cl`3clo<0o`3clo?o0??clol0lo?c5@3clo<00400lo?c00D00000
0??cl`3clo<0lo?c000000050??cl`030000003clo<000000?l0lo?co`3clo?o0??claH0lo?c0010
0??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<000000?l0lo?c
o`3clo?o0??claD0lo?c00100??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c
0??cl`3clo<000000?l0lo?co`3clo?o0??claD0lo?c00110??cl`<000001P3clo<3000009H0lo?c
0`00003o0??clj`0lo?c0`00003<0??cl`00hP3clo<01@000000lo?c0??cl`3clo<000000?l0lo?c
ZP3clo<01@000000lo?c0??cl`3clo<000000</0lo?c003R0??cl`050000003clo<0lo?c0??cl`00
0000o`3clo>Z0??cl`050000003clo<0lo?c0??cl`000000b`3clo<00=P0lo?c0`0000040??cl`<0
000000@0lo?c00000000000000000`3clo<700000080lo?c0P0000060??cl`@00000o`3clo>90??c
l`<00000103clo<3000000040??cl`0000000000000000<0lo?c1`0000020??cl`8000002P3clo<0
0`000000lo?c0??cl`2a0??cl`00f@3clo<00`000000lo?c0??cl`040??cl`030000003clo<00000
00D0lo?c00@000000??cl`3clo<000000P3clo<010000000lo?c0??cl`0000020??cl`030000003c
lo<0lo?c00L0lo?c00<000000??cl`3clo<0o`3clo>70??cl`030000003clo<0lo?c00@0lo?c00<0
00000??cl`0000001@3clo<010000000lo?c0??cl`0000020??cl`040000003clo<0lo?c00000080
lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`2a0??cl`00f@3clo<00`000000lo?c
0??cl`040??cl`030000003clo<0lo?c008000000`3clo<010000000lo?c0??cl`0000020??cl`04
0000003clo<0lo?c000000`0lo?c00<000000??cl`3clo<0o`3clo>70??cl`030000003clo<0lo?c
00@0lo?c00<000000??cl`3clo<00P0000030??cl`040000003clo<0lo?c00000080lo?c00@00000
0??cl`3clo<00000203clo<600000;80lo?c003I0??cl`040000003clo<0lo?c000000<0lo?c00<0
00000??cl`0000000P3clo<010000000lo?c0??cl`0000020??cl`040000003clo<0lo?c00000080
lo?c100000090??cl`030000003clo<0lo?c0?l0lo?cQ`3clo<010000000lo?c0??cl`0000030??c
l`030000003clo<000000080lo?c00@000000??cl`3clo<000000P3clo<010000000lo?c0??cl`00
00020??cl`@000001@3clo<01@000000lo?c0??cl`3clo<000000;<0lo?c003I0??cl`@000000`3c
lo<00`000000lo?c000000020??cl`040000003clo<0lo?c00000080lo?c00<000000??cl`3clo<0
0P0000001@3clo<000000??cl`3clo<0000000L0lo?c0P00003o0??clhX0lo?c100000030??cl`03
0000003clo<000000080lo?c00@000000??cl`3clo<000000P3clo<00`000000lo?c0??cl`020000
00050??cl`000000lo?c0??cl`0000001P3clo<010000000lo?c0??cl`00002c0??cl`00f@3clo<0
10000000lo?c0??cl`0000020??cl`8000000P3clo<6000000040??cl`00000000000??cl`800000
00<0lo?c0000003clo<00P0000090??cl`030000003clo<0lo?c0?l0lo?cR03clo<010000000lo?c
0??cl`0000020??cl`8000000P3clo<6000000040??cl`00000000000??cl`80000000<0lo?c0000
003clo<00P0000080??cl`030000003clo<000000;<0lo?c003I0??cl`030000003clo<0lo?c0280
lo?c00<000000??cl`3clo<0o`3clo>70??cl`030000003clo<0lo?c0200lo?c00<000000??cl`00
0000/`3clo<00=T0lo?c00D000000??cl`3clo<0lo?c0000000L0??cl`050000003clo<0lo?c0??c
l`000000o`3clo>90??cl`050000003clo<0lo?c0??cl`0000007`3clo<200000;<0lo?c003H0??c
l`H000000P3clo<00`000000lo?c0??cl`0H0??cl`<00000o`3clo>90??cl`H000000P3clo<00`00
0000lo?c0??cl`0K0??cl`030000003clo<0lo?c0;40lo?c003o0??clol0lo?co`3clo=S0??cl`00
o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S
0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?c
o`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??c
lol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c
003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??c
lf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3c
lo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0
lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3c
lo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0
lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o
0??clol0lo?cH`3clo<00>`0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`3o0??c
lil0lo?c00<000000??cl`3clo<02`3clo<00`000000lo?c0??cl`2c0??cl`00j`3clo<00`000000
lo?c0??cl`0>0??cl`030000003clo<0lo?c0?l0lo?cW@3clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c0;80lo?c003O0??cl`H000001@3clo<010000000lo?c0??cl`3clo<2000000D0
lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`3o0??clhl0lo?c1P0000060??cl`04
0000003clo<0lo?c0??cl`8000001@3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c
0;40lo?c003P0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<00P3clo<010000000
lo?c0??cl`0000020??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<01@3clo<00`00
0000lo?c0??cl`3o0??cli00lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`030??c
l`040000003clo<0lo?c00000080lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`04
0??cl`030000003clo<0lo?c0;40lo?c003P0??cl`030000003clo<0lo?c00L0lo?c00@000000??c
l`3clo<000001P3clo<00`000000lo?c000000060??cl`030000003clo<0lo?c0?l0lo?cT03clo<0
0`000000lo?c0??cl`080??cl`040000003clo<0lo?c000000H0lo?c00<000000??cl`0000001@3c
lo<00`000000lo?c0??cl`2a0??cl`00h03clo<010000000lo?c0??cl`0000060??cl`030000003c
lo<0lo?c00@000000`3clo<00`000000lo?c000000060??cl`030000003clo<0lo?c0?l0lo?cT03c
lo<010000000lo?c0??cl`0000070??cl`030000003clo<0lo?c00@000000`3clo<00`000000lo?c
000000050??cl`030000003clo<0lo?c0;40lo?c003P0??cl`@000001P3clo<010000000lo?c0??c
l`0000020??cl`070000003clo<0lo?c0??cl`000000lo?c000000060??cl`030000003clo<0lo?c
0?l0lo?cT03clo<4000000L0lo?c00@000000??cl`3clo<000000P3clo<01`000000lo?c0??cl`3c
lo<000000??cl`0000001@3clo<00`000000lo?c0??cl`2a0??cl`00h03clo<010000000lo?c0??c
l`0000060??cl`040000003clo<0lo?c0??cl`8000000`3clo<01@000000lo?c0??cl`3clo<00000
00D0lo?c00<000000??cl`3clo<0o`3clo>@0??cl`040000003clo<0lo?c000000L0lo?c00@00000
0??cl`3clo<0lo?c0P0000030??cl`050000003clo<0lo?c0??cl`000000103clo<00`000000lo?c
0??cl`2a0??cl`00h03clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00H0lo?c00D0
00000??cl`3clo<0lo?c000000050??cl`030000003clo<0lo?c0?l0lo?cT03clo<00`000000lo?c
0??cl`080??cl`030000003clo<0lo?c00H0lo?c00D000000??cl`3clo<0lo?c000000040??cl`03
0000003clo<0lo?c0;40lo?c003P0??cl`050000003clo<0lo?c0??cl`0000001P3clo<00`000000
lo?c0??cl`050??cl`050000003clo<0lo?c0??cl`000000103clo<00`000000lo?c0??cl`3o0??c
li40lo?c00D000000??cl`3clo<0lo?c000000070??cl`030000003clo<0lo?c00D0lo?c00D00000
0??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c0;80lo?c003O0??cl`H000001`3clo<0
0`000000lo?c0??cl`030??cl`<0000000@0lo?c00000000000000000P3clo<00`000000lo?c0??c
l`3o0??cli40lo?c1P0000080??cl`030000003clo<0lo?c00<0lo?c0`0000001P3clo<000000000
00000000lo?c00000;D0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3c
lo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0
lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00N03clo<3
000000<0lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`0R0??cl`<000000`3clo<0
0`000000lo?c0??cl`03000002D0lo?c0`0000030??cl`030000003clo<0lo?c00<00000:03clo<5
000002L0lo?c1@0000020??cl`030000003clo<0000000@000008`3clo<500000080lo?c00<00000
0??cl`3clo<00`3clo<00`000000lo?c0??cl`2]0??cl`<000000`3clo<00`000000lo?c0??cl`03
0??cl`030000003clo<0lo?c0240lo?c0`0000030??cl`030000003clo<0lo?c00<000009@3clo<3
000000<0lo?c00<000000??cl`3clo<00`00000X0??cl`D00000:03clo<500000080lo?c00<00000
0??cl`0000001000000S0??cl`D000000P3clo<00`000000lo?c0??cl`030??cl`030000003clo<0
lo?c03D0lo?c001i0??cl`030000003clo<0lo?c00P0lo?c00<000000??cl`3clo<08`3clo<00`00
0000lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`0000009@3clo<00`000000lo?c0??cl`04
0??cl`050000003clo<0lo?c0??cl`000000:03clo<010000000lo?c0??cl`00000X0??cl`040000
003clo<0lo?c000000D0lo?c00@000000??cl`3clo<00000903clo<010000000lo?c0??cl`000008
0??cl`030000003clo<0lo?c0:h0lo?c00<000000??cl`3clo<0203clo<00`000000lo?c0??cl`0R
0??cl`030000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000U0??cl`030000003c
lo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000X0??cl`040000003clo<0lo?c000002T0
lo?c00@000000??cl`3clo<000001@3clo<010000000lo?c0??cl`00000T0??cl`040000003clo<0
lo?c000000P0lo?c00<000000??cl`3clo<0=@3clo<007T0lo?c00<000000??cl`3clo<0103clo<6
000002@0lo?c00<000000??cl`3clo<0103clo<01@000000lo?c0??cl`3clo<0000002D0lo?c00<0
00000??cl`3clo<0103clo<01@000000lo?c0??cl`3clo<0000002T0lo?c00<000000??cl`3clo<0
:@3clo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c02D0lo?c00<000000??cl`3clo<0
103clo<600000:l0lo?c00<000000??cl`3clo<0103clo<6000002<0lo?c00<000000??cl`3clo<0
103clo<01@000000lo?c0??cl`3clo<0000002D0lo?c00<000000??cl`3clo<0103clo<01@000000
lo?c0??cl`3clo<0000002T0lo?c00<000000??cl`3clo<0:P3clo<00`000000lo?c0??cl`060??c
l`030000003clo<0lo?c02D0lo?c00<000000??cl`3clo<0103clo<6000003H0lo?c001i0??cl`03
0000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c0000000U0??cl`030000003clo<0lo?c
00@0lo?c00D000000??cl`3clo<0lo?c0000000U0??cl`030000003clo<0lo?c00D0lo?c00<00000
0??cl`000000:`3clo<00`000000lo?c0??cl`0Y0??cl`030000003clo<0lo?c00H0lo?c00<00000
0??cl`3clo<09@3clo<00`000000lo?c0??cl`030??cl`050000003clo<0lo?c0??cl`000000/03c
lo<00`000000lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`000000903clo<00`000000lo?c
0??cl`040??cl`050000003clo<0lo?c0??cl`0000009@3clo<00`000000lo?c0??cl`050??cl`03
0000003clo<0000002/0lo?c00<000000??cl`3clo<0:P3clo<00`000000lo?c0??cl`060??cl`03
0000003clo<0lo?c02D0lo?c00<000000??cl`3clo<00`3clo<01@000000lo?c0??cl`3clo<00000
03L0lo?c001i0??cl`030000003clo<0lo?c00D0lo?c00@000000??cl`3clo<000009@3clo<00`00
0000lo?c0??cl`040??cl`8000000P3clo<00`000000lo?c0??cl`0S0??cl`030000003clo<0lo?c
00H0lo?c00<000000??cl`3clo<0:`3clo<00`000000lo?c0??cl`0Y0??cl`030000003clo<0lo?c
00H0lo?c00<000000??cl`3clo<09@3clo<00`000000lo?c0??cl`030??cl`040000003clo<0lo?c
00000;00lo?c00<000000??cl`3clo<01@3clo<010000000lo?c0??cl`00000T0??cl`030000003c
lo<0lo?c00@0lo?c0P0000020??cl`030000003clo<0lo?c02<0lo?c00<000000??cl`3clo<01P3c
lo<00`000000lo?c0??cl`0[0??cl`030000003clo<0lo?c02X0lo?c00<000000??cl`3clo<01P3c
lo<00`000000lo?c0??cl`0U0??cl`030000003clo<0lo?c00<0lo?c00@000000??cl`3clo<00000
=`3clo<007T0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0000000U0??cl`030000003c
lo<0lo?c00@0lo?c00@000000??cl`00000000009P3clo<00`000000lo?c0??cl`050??cl`030000
003clo<0000002`0lo?c00<000000??cl`3clo<0:@3clo<00`000000lo?c0??cl`060??cl`030000
003clo<0lo?c02D0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0000002`0??cl`030000
003clo<0lo?c00H0lo?c00<000000??cl`000000903clo<00`000000lo?c0??cl`040??cl`040000
003clo<00000000002H0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0000000/0??cl`03
0000003clo<0lo?c02X0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`0U0??cl`03
0000003clo<0lo?c00@0lo?c00<000000??cl`000000=`3clo<007T0lo?c00<000000??cl`3clo<0
1P3clo<00`000000lo?c0000000U0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0
9P3clo<00`000000lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`000000:`3clo<00`000000
lo?c0??cl`0Y0??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<09@3clo<00`000000
lo?c0??cl`040??cl`030000003clo<000000;00lo?c00<000000??cl`3clo<01P3clo<00`000000
lo?c0000000T0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<09P3clo<00`000000
lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`000000:`3clo<00`000000lo?c0??cl`0Z0??c
l`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<09@3clo<00`000000lo?c0??cl`040??c
l`030000003clo<0000003L0lo?c001h0??cl`8000002@3clo<2000002@0lo?c0P0000080??cl`03
0000003clo<0lo?c02@0lo?c0P0000060??cl`050000003clo<0lo?c0??cl`0000009`3clo<01@00
0000lo?c0??cl`3clo<0000002L0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003c
lo<0lo?c0??cl`0000008`3clo<01@000000lo?c0??cl`3clo<0000000L0lo?c0P00002_0??cl`80
00002@3clo<2000002<0lo?c0P0000080??cl`030000003clo<0lo?c02@0lo?c0P0000060??cl`05
0000003clo<0lo?c0??cl`0000009`3clo<01@000000lo?c0??cl`3clo<0000002P0lo?c00D00000
0??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000008`3clo<01@000000lo?c
0??cl`3clo<0000000L0lo?c0P00000g0??cl`00N@3clo<00`000000lo?c0??cl`080??cl`030000
003clo<0lo?c02<0lo?c00<000000??cl`3clo<01`3clo<2000002D0lo?c00<000000??cl`3clo<0
1@3clo<3000002T0lo?c0`00000Y0??cl`<000001P3clo<3000002D0lo?c0`0000090??cl`030000
003clo<0lo?c0:h0lo?c00<000000??cl`3clo<0203clo<00`000000lo?c0??cl`0R0??cl`030000
003clo<0lo?c00L0lo?c0P00000U0??cl`030000003clo<0lo?c00D0lo?c0`00000Y0??cl`<00000
:P3clo<3000000H0lo?c0`00000U0??cl`<000002@3clo<00`000000lo?c0??cl`0e0??cl`00o`3c
lo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??c
l`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o0??clf<0lo?c001G0??clol00000<@00
00200??clol00000<@00000Y0??cl`00E`3clo<010000000lo?c0??cl`00000;0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`040??cl`030000003c
lo<0lo?c07h0lo?c00<000000??cl`000000303clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<0
0`000000lo?c0??cl`0T0??cl`030000003clo<0lo?c02h0lo?c00<000000??cl`3clo<0;P3clo<0
0`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c02h0lo?c00<000000??cl`3clo<0;@3clo<0
0`000000lo?c0??cl`0A0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0903clo<0
0`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c02h0lo?c00<000000??cl`3clo<0;@3clo<0
0`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c02h0lo?c00<000000??cl`3clo<0403clo<2
000002T0lo?c001G0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`1n0??c
l`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000
lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`3o0??clbd0
lo?c00<000000??cl`3clo<09`3clo<005L0lo?c0P00003o0??clbd0lo?c0P0000200??cl`030000
003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<200000?l0lo?c;P3c
lo<00`000000lo?c0??cl`1n0??cl`800000o`3clo<]0??cl`800000:@3clo<005L0lo?c00<00000
0??cl`000000o`3clo<]0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]
0??cl`030000003clo<0lo?c02L0lo?c001G0??cl`040000003clo<0lo?c00000?l0lo?c;03clo<0
0`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c:`3clo<00`000000lo?c0000000Y
0??cl`00E`3clo<01@000000lo?c0??cl`3clo<000000?l0lo?c:`3clo<00`000000lo?c0??cl`1n
0??cl`030000003clo<0lo?c0?l0lo?c:`3clo<00`000000lo?c0000000Y0??cl`00E`3clo<20000
0080lo?c00<000000??cl`3clo<0o`3clo<X0??cl`800000P03clo<00`000000lo?c0??cl`3o0??c
lbX0lo?c00@000000??cl`3clo<00000:@3clo<004X0lo?c00<000000??cl`3clo<00`0000070??c
l`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0o`3clo<X0??cl`030000003clo<0lo?c
07h0lo?c0P00003o0??clb/0lo?c00@000000??cl`0000000000:@3clo<004X0lo?c00<000000??c
l`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c00<0lo?c00<000000??c
l`3clo<0o`3clo<W0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<Z0??c
l`040000003clo<0lo?c000002T0lo?c00160??cl`L000000`3clo<00`000000lo?c0??cl`040??c
l`030000003clo<0lo?c00<0lo?c00<000000??cl`3clo<0o`3clo<W0??cl`030000003clo<0lo?c
07h0lo?c00<000000??cl`3clo<0o`3clo<Y0??cl`050000003clo<0lo?c0??cl`000000:@3clo<0
03h0lo?c1`0000000`3clo<000000??cl`020??cl`030000003clo<0000000<0lo?c00<000000??c
l`3clo<0103clo<3000000@0lo?c00<000000??cl`3clo<0o`3clo<T0??cl`<00000P03clo<00`00
0000lo?c0??cl`3o0??clbT0lo?c00D000000??cl`3clo<0lo?c0000000Y0??cl`00A`3clo<01P00
0000lo?c0??cl`000000lo?c000000<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??c
l`050??cl`030000003clo<0lo?c0?l0lo?c9@3clo<00`000000lo?c0??cl`1n0??cl`030000003c
lo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00B03clo<01@000000lo?c0000003c
lo<0000000<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`060??cl`030000003c
lo<0lo?c0?l0lo?c903clo<00`000000lo?c0??cl`1n0??cl`800000o`3clo<]0??cl`800000:@3c
lo<004P0lo?c00D000000??cl`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<0
00000??cl`3clo<01P3clo<00`000000lo?c0??cl`3o0??clb@0lo?c00<000000??cl`3clo<0OP3c
lo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<004T0lo?c0P000000
0`3clo<000000??cl`020??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<01`3clo<0
0`000000lo?c0??cl`3o0??clb<0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`3o
0??clbd0lo?c00<000000??cl`3clo<09`3clo<004X0lo?c00<000000??cl`3clo<00`0000070??c
l`8000002@3clo<00`000000lo?c0??cl`3o0??clb40lo?c0P0000200??cl`030000003clo<0lo?c
0?l0lo?c9`3clo<00`000000lo?c0??cl`030??cl`030000003clo<0lo?c02L0lo?c001G0??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<0o`3clo<Q0??cl`030000003clo<0lo?c07h0
lo?c00<000000??cl`3clo<0o`3clo<W0??cl`030000003clo<0lo?c00<0lo?c00<000000??cl`3c
lo<09`3clo<005L0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`3o0??clb40lo?c
00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`3o0??clbH0lo?c00<000000??cl`3clo<0
103clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`0:0??cl`030000003c
lo<0lo?c0?l0lo?c803clo<00`000000lo?c0??cl`1n0??cl`800000o`3clo<W0??cl`030000003c
lo<0lo?c00<0lo?c0P00000Y0??cl`00E`3clo<00`000000lo?c0??cl`0;0??cl`030000003clo<0
lo?c0?l0lo?c7`3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c9P3clo<0
0`000000lo?c0??cl`040??cl`030000003clo<0lo?c02L0lo?c001G0??cl`8000003@3clo<00`00
0000lo?c0??cl`3o0??clad0lo?c0P00001[0??cl`@000000`3clo<00`000000lo?c000000020000
00T0lo?c00<000000??cl`3clo<0o`3clo<U0??cl`030000003clo<0lo?c00D0lo?c00<000000??c
l`3clo<09`3clo<005L0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`3o0??clah0
lo?c00<000000??cl`3clo<0K@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c00H0
lo?c00<000000??cl`3clo<0o`3clo<U0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3c
lo<09`3clo<005L0lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`3o0??clad0lo?c
00<000000??cl`3clo<0K@3clo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c00D0lo?c
00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c001G0??cl`030000003c
lo<0lo?c00h0lo?c00<000000??cl`3clo<0o`3clo<L0??cl`030000003clo<0lo?c0680lo?c1`00
00040??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<01@3clo<300000?l0lo?c:`3c
lo<3000002T0lo?c001G0??cl`800000403clo<00`000000lo?c0??cl`3o0??claX0lo?c0P00001]
0??cl`8000002@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0?l0lo?c;@3clo<0
0`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`0?0??cl`030000003clo<0lo?c
0?l0lo?c6`3clo<00`000000lo?c0??cl`1/0??cl`030000003clo<0lo?c00H0lo?c00<000000??c
l`3clo<01P3clo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0
lo?c00<000000??cl`3clo<0403clo<00`000000lo?c0??cl`3o0??claX0lo?c00<000000??cl`3c
lo<0K@3clo<00`000000lo?c0??cl`030??cl`8000002@3clo<00`000000lo?c0??cl`3o0??clbd0
lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<000000??cl`3clo<04@3clo<00`000000lo?c
0??cl`3o0??claT0lo?c00<000000??cl`3clo<0J@3clo<01@000000lo?c0??cl`3clo<0000000D0
lo?c00<000000??cl`3clo<0203clo<00`000000lo?c0??cl`3o0??clb<0lo?c00<000000??cl`3c
lo<01`3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`0B0??cl`030000
003clo<0lo?c0?l0lo?c603clo<00`000000lo?c0??cl`1Z0??cl`<000001`3clo<3000000L0lo?c
0P00003o0??clb@0lo?c00<000000??cl`3clo<01P3clo<2000002T0lo?c001G0??cl`8000004`3c
lo<00`000000lo?c0??cl`3o0??claL0lo?c0P0000200??cl`030000003clo<0lo?c0?l0lo?c8`3c
lo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c02L0lo?c001G0??cl`030000003clo<0
lo?c01<0lo?c00<000000??cl`3clo<0o`3clo<G0??cl`030000003clo<0lo?c07h0lo?c00<00000
0??cl`3clo<0o`3clo<R0??cl`030000003clo<0lo?c00P0lo?c00<000000??cl`3clo<09`3clo<0
04H0lo?c100000020??cl`<00000203clo<00`000000lo?c0??cl`0D0??cl`030000003clo<0lo?c
0?l0lo?c5P3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c8P3clo<00`00
0000lo?c0??cl`080??cl`030000003clo<0lo?c02L0lo?c001:0??cl`030000003clo<0lo?c0080
lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0E0??cl`030000003clo<0lo?c0?l0
lo?c5@3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c8P3clo<00`000000
lo?c0??cl`080??cl`030000003clo<0lo?c02L0lo?c001:0??cl`030000003clo<0lo?c00<0lo?c
00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`0E0??cl`030000003clo<0lo?c0?l0lo?c
5@3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c8P3clo<00`000000lo?c
0??cl`080??cl`030000003clo<0lo?c02L0lo?c000n0??cl`L000001@3clo<00`000000lo?c0??c
l`030??cl`030000003clo<0lo?c00@0lo?c0`00000F0??cl`030000003clo<0lo?c0?l0lo?c4P3c
lo<300000800lo?c0P00003o0??clbd0lo?c0P00000Y0??cl`00B03clo<2000000H0lo?c00<00000
0??cl`3clo<0103clo<00`000000lo?c0??cl`0G0??cl`030000003clo<0lo?c0?l0lo?c4`3clo<0
0`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W
0??cl`00B@3clo<00`000000lo?c0??cl`030??cl`030000003clo<0lo?c00D0lo?c00<000000??c
l`3clo<0603clo<00`000000lo?c0??cl`3o0??cla80lo?c00<000000??cl`3clo<0OP3clo<00`00
0000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<004X0lo?c00<000000??cl`3c
lo<00P0000080??cl`030000003clo<0lo?c01T0lo?c00<000000??cl`3clo<0o`3clo<A0??cl`03
0000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0
lo?c00160??cl`050000003clo<0lo?c0??cl`0000000P3clo<00`000000lo?c0??cl`070??cl`80
00006`3clo<00`000000lo?c0??cl`3o0??cl`l0lo?c0P0000200??cl`030000003clo<0lo?c0?l0
lo?c803clo<00`000000lo?c0??cl`0:0??cl`030000003clo<0lo?c02L0lo?c00170??cl`<00000
103clo<3000000H0lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??cl`3o0??cl`l0lo?c
00<000000??cl`3clo<0OP3clo<200000?l0lo?c803clo<00`000000lo?c0??cl`0:0??cl`800000
:@3clo<005L0lo?c00<000000??cl`3clo<0703clo<00`000000lo?c0??cl`3o0??cl`h0lo?c00<0
00000??cl`3clo<0OP3clo<00`000000lo?c0??cl`3o0??clal0lo?c00<000000??cl`3clo<02`3c
lo<00`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`0M0??cl`030000003clo<0
lo?c0?l0lo?c3@3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c7P3clo<0
0`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c02L0lo?c001G0??cl`030000003clo<0lo?c
01d0lo?c00<000000??cl`3clo<0o`3clo<=0??cl`030000003clo<0lo?c07h0lo?c00<000000??c
l`3clo<0o`3clo<N0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<09`3clo<005L0
lo?c0P00000O0??cl`030000003clo<0lo?c0?l0lo?c2`3clo<200000800lo?c00<000000??cl`3c
lo<0o`3clo<M0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c
00<000000??cl`3clo<07`3clo<00`000000lo?c0??cl`3o0??cl`/0lo?c00<000000??cl`3clo<0
OP3clo<200000?l0lo?c7P3clo<00`000000lo?c0??cl`0<0??cl`800000:@3clo<005L0lo?c00<0
00000??cl`3clo<0803clo<00`000000lo?c0??cl`3o0??cl`X0lo?c00<000000??cl`3clo<0OP3c
lo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<00000
0??cl`3clo<08@3clo<00`000000lo?c0??cl`3o0??cl`T0lo?c00<000000??cl`3clo<0OP3clo<0
0`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<000000??c
l`3clo<08P3clo<00`000000lo?c0??cl`3o0??cl`P0lo?c00<000000??cl`3clo<0LP3clo<40000
00P0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c001G0??cl`80
0000903clo<00`000000lo?c0??cl`3o0??cl`H0lo?c0P00001h0??cl`030000003clo<0lo?c00D0
lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c001G0??cl`030000
003clo<0lo?c02@0lo?c00<000000??cl`3clo<0o`3clo<60??cl`030000003clo<0lo?c07H0lo?c
00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<0
9`3clo<005L0lo?c00<000000??cl`3clo<09@3clo<00`000000lo?c0??cl`3o0??cl`D0lo?c00<0
00000??cl`3clo<0J`3clo<7000000@0lo?c00<000000??cl`3clo<01@3clo<300000?l0lo?c6`3c
lo<00`000000lo?c0??cl`0=0??cl`<00000:@3clo<005L0lo?c00<000000??cl`3clo<09P3clo<0
0`000000lo?c0??cl`3o0??cl`@0lo?c00<000000??cl`3clo<0M03clo<2000000P0lo?c00<00000
0??cl`3clo<0o`3clo<K0??cl`030000003clo<0lo?c00l0lo?c00<000000??cl`3clo<09`3clo<0
05L0lo?c0P00000X0??cl`030000003clo<0lo?c0?l0lo?c0P3clo<2000007L0lo?c00<000000??c
l`3clo<01P3clo<00`000000lo?c0??cl`3o0??claX0lo?c00<000000??cl`3clo<0403clo<00`00
0000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c0?l0
lo?c0P3clo<00`000000lo?c0??cl`1f0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3c
lo<0o`3clo<J0??cl`030000003clo<0lo?c0100lo?c00<000000??cl`3clo<09`3clo<004H0lo?c
100000030??cl`<000001`3clo<00`000000lo?c0??cl`0Y0??cl`030000003clo<0lo?c0?l0lo?c
0@3clo<00`000000lo?c0??cl`1b0??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000
lo?c0??cl`3o0??claT0lo?c00<000000??cl`3clo<04@3clo<00`000000lo?c0??cl`0W0??cl`00
BP3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0
:P3clo<00`000000lo?c0??cl`3o0??cl`030000003clo<0lo?c07<0lo?c0`0000080??cl`800000
o`3clo<J0??cl`030000003clo<0lo?c0100lo?c0P00000Y0??cl`00BP3clo<00`000000lo?c0000
00030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0:P3clo<00`000000lo?c0??c
l`3o0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003c
lo<0lo?c02L0lo?c000n0??cl`L000001@3clo<00`000000lo?c000000030??cl`030000003clo<0
lo?c00@0lo?c0`00000[0??cl`030000003clo<0lo?c0?`0lo?c0`0000200??cl`030000003clo<0
lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00B03clo<200000080lo?c00D000000??c
l`3clo<0lo?c000000060??cl`030000003clo<0lo?c02`0lo?c00<000000??cl`3clo<0o@3clo<0
0`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W
0??cl`00B@3clo<010000000lo?c0??cl`0000030??cl`030000003clo<0lo?c00@0lo?c00<00000
0??cl`3clo<0;@3clo<00`000000lo?c0??cl`3l0??cl`030000003clo<0lo?c07h0lo?c00<00000
0??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c001:0??cl`030000003clo<00000
00<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c
0?/0lo?c00<000000??cl`3clo<0OP3clo<200000?l0lo?c;@3clo<2000002T0lo?c00160??cl`07
0000003clo<0lo?c0??cl`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<00000
0??cl`3clo<0;`3clo<00`000000lo?c0??cl`3j0??cl`030000003clo<0lo?c07h0lo?c00<00000
0??cl`3clo<0o`3clo<F0??cl`030000003clo<0lo?c01@0lo?c00<000000??cl`3clo<09`3clo<0
02d0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0?0??cl`<000000`3clo<30000
00L0lo?c0P00000a0??cl`030000003clo<0lo?c0?P0lo?c0P00001B0??cl`030000003clo<0lo?c
00D0lo?c00<000000??cl`3clo<08`3clo<00`000000lo?c0??cl`3o0??claH0lo?c00<000000??c
l`3clo<0503clo<00`000000lo?c0??cl`0W0??cl`00;@3clo<900000240lo?c00<000000??cl`3c
lo<0<@3clo<00`000000lo?c0??cl`3h0??cl`030000003clo<0lo?c0500lo?c2@00000U0??cl`03
0000003clo<0lo?c0?l0lo?c5@3clo<00`000000lo?c0??cl`0E0??cl`030000003clo<0lo?c02L0
lo?c000]0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`0O0??cl`03
0000003clo<0lo?c0380lo?c00<000000??cl`3clo<0m`3clo<00`000000lo?c0??cl`1@0??cl`05
0000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`0S0??cl`800000o`3clo<F0??c
l`030000003clo<0lo?c01D0lo?c00<000000??cl`3clo<09`3clo<002d0lo?c00D000000??cl`3c
lo<0lo?c0000000U0??cl`030000003clo<0lo?c03<0lo?c00<000000??cl`3clo<0mP3clo<00`00
0000lo?c0??cl`1@0??cl`050000003clo<0lo?c0??cl`000000:@3clo<00`000000lo?c0000003o
0??cla@0lo?c00<000000??cl`3clo<05P3clo<00`000000lo?c0??cl`0W0??cl`00;@3clo<01@00
0000lo?c0??cl`3clo<0000002D0lo?c0P00000e0??cl`030000003clo<0lo?c0?@0lo?c0P00001B
0??cl`050000003clo<0lo?c0??cl`000000:@3clo<2000000030??cl`000000lo?c0?l0lo?c4P3c
lo<00`000000lo?c0??cl`0E0??cl`800000:@3clo<002h0lo?c0`00000V0??cl`030000003clo<0
lo?c03D0lo?c00<000000??cl`3clo<0m03clo<00`000000lo?c0??cl`1A0??cl`<00000:P3clo<0
1@000000lo?c0??cl`3clo<000000?l0lo?c:`3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<0
0`000000lo?c0??cl`0f0??cl`800000m03clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0
lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00<@3clo<400000280lo?c00<000000??c
l`3clo<0>03clo<00`000000lo?c0??cl`3a0??cl`030000003clo<0lo?c05@0lo?c1000000V0??c
l`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00<03clo<00`000000
lo?c000000020??cl`030000003clo<0lo?c01l0lo?c00<000000??cl`3clo<0>@3clo<00`000000
lo?c0??cl`3`0??cl`030000003clo<0lo?c05<0lo?c00<000000??cl`0000000P3clo<00`000000
lo?c0??cl`0S0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00
<03clo<00`000000lo?c000000020??cl`030000003clo<0lo?c01l0lo?c0P00000k0??cl`030000
003clo<0lo?c0>h0lo?c0P00001E0??cl`030000003clo<000000080lo?c00<000000??cl`3clo<0
8`3clo<200000?l0lo?c;@3clo<2000002T0lo?c000a0??cl`80000000<0lo?c0000003clo<08@3c
lo<00`000000lo?c0??cl`0k0??cl`030000003clo<0lo?c0>h0lo?c00<000000??cl`3clo<0E03c
lo<2000000030??cl`000000lo?c02D0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??c
l`3o0??cl`X0lo?c00<000000??cl`3clo<06@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<0
0`000000lo?c0??cl`0l0??cl`030000003clo<0lo?c0>d0lo?c00<000000??cl`3clo<0J@3clo<5
00000080lo?c00<000000??cl`0000000P0000090??cl`030000003clo<0lo?c00D0lo?c00<00000
0??cl`3clo<0o`3clo<80??cl`030000003clo<0lo?c01X0lo?c00<000000??cl`3clo<09`3clo<0
0300lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0O0??cl`030000003clo<0lo?c
03d0lo?c00<000000??cl`3clo<0k03clo<00`000000lo?c0??cl`1C0??cl`030000003clo<0lo?c
0080lo?c00<000000??cl`3clo<03`3clo<010000000lo?c0??cl`0000070??cl`030000003clo<0
lo?c00H0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`3o0??cl`L0lo?c00<00000
0??cl`3clo<06P3clo<00`000000lo?c0??cl`0W0??cl`00<03clo<600000240lo?c00<000000??c
l`3clo<0?P3clo<00`000000lo?c0??cl`3[0??cl`030000003clo<0lo?c05<0lo?c1P00000B0??c
l`030000003clo<0lo?c00P0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`070??c
l`030000003clo<0lo?c0?l0lo?c1@3clo<00`000000lo?c0??cl`0K0??cl`030000003clo<0lo?c
02L0lo?c000`0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<07`3clo<200000400
lo?c00<000000??cl`3clo<0j@3clo<2000005D0lo?c00<000000??cl`3clo<00P3clo<00`000000
lo?c0??cl`070??cl`L000000`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00D0
lo?c0`0000080??cl`030000003clo<0lo?c0?l0lo?c103clo<00`000000lo?c0??cl`0I0??cl`<0
0000:@3clo<00300lo?c00<000000??cl`3clo<04`3clo<5000000040??cl`0000000000000000P0
lo?c00<000000??cl`3clo<0@03clo<200000>X0lo?c00<000000??cl`3clo<0D`3clo<00`000000
lo?c0??cl`0G0??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<01@3clo<00`000000
lo?c0??cl`3o0??cl`h0lo?c00<000000??cl`3clo<0703clo<00`000000lo?c0??cl`0W0??cl`00
<03clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c00l0lo?c00@000000??cl`3clo<0
0000103clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0480lo?c00<000000??cl`3c
lo<0i`3clo<00`000000lo?c0??cl`1C0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3c
lo<04P3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3c
lo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c000`0??cl`H000004P3clo<00`000000lo?c
0??cl`050??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0@`3clo<00`000000lo?c
0??cl`3V0??cl`030000003clo<0lo?c05<0lo?c1P00000D0??cl`030000003clo<0lo?c00<0lo?c
0P0000090??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00<03c
lo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c00H0lo?c1`0000040??cl`030000003c
lo<0lo?c00@0lo?c00<000000??cl`3clo<0103clo<3000004@0lo?c00<000000??cl`3clo<0h`3c
lo<3000005D0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0>0??cl`050000003c
lo<0lo?c0??cl`0000001@3clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c0?l0lo?c
;@3clo<00`000000lo?c0??cl`0W0??cl`00<03clo<00`000000lo?c0??cl`0G0??cl`030000003c
lo<0lo?c00<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`150??cl`030000003c
lo<0lo?c0>@0lo?c00<000000??cl`3clo<0D`3clo<00`000000lo?c0??cl`0D0??cl`<000001`3c
lo<3000000L0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`3o0??clah0lo?c00<0
00000??cl`3clo<09`3clo<00340lo?c1@00000D0??cl`030000003clo<0lo?c0080lo?c00<00000
0??cl`3clo<01@3clo<00`000000lo?c0??cl`160??cl`800000i03clo<00`000000lo?c0??cl`1D
0??cl`D000009@3clo<2000000h0lo?c00<000000??cl`3clo<0n`3clo<00`000000lo?c0??cl`0M
0??cl`800000:@3clo<00300lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0B0??c
l`030000003clo<0lo?c00800000203clo<00`000000lo?c0??cl`180??cl`030000003clo<0lo?c
0>40lo?c00<000000??cl`3clo<0D`3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c
02<0lo?c00<000000??cl`3clo<03P3clo<00`000000lo?c0??cl`3i0??cl`030000003clo<0lo?c
01l0lo?c00<000000??cl`3clo<09`3clo<00300lo?c00<000000??cl`3clo<04`3clo<01@000000
lo?c0??cl`3clo<000000080lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`190??c
l`030000003clo<0lo?c0>00lo?c00<000000??cl`3clo<0D`3clo<00`000000lo?c0??cl`0X0??c
l`030000003clo<0lo?c00l0lo?c00<000000??cl`3clo<0n03clo<00`000000lo?c0??cl`0O0??c
l`030000003clo<0lo?c02L0lo?c000a0??cl`D000004@3clo<3000000@0lo?c0`0000060??cl`80
0000B`3clo<00`000000lo?c0??cl`3N0??cl`800000EP3clo<5000002D0lo?c00<000000??cl`3c
lo<0403clo<00`000000lo?c0??cl`3f0??cl`030000003clo<0lo?c0200lo?c00<000000??cl`3c
lo<09`3clo<003D0lo?c00<000000??cl`3clo<07`3clo<00`000000lo?c0??cl`1;0??cl`800000
g`3clo<00`000000lo?c0??cl`1H0??cl`030000003clo<0lo?c02<0lo?c00<000000??cl`3clo<0
o`3clo<:0??cl`030000003clo<0lo?c0200lo?c00<000000??cl`3clo<09`3clo<00300lo?c00<0
00000??cl`3clo<00P3clo<00`000000lo?c0??cl`0O0??cl`030000003clo<0lo?c04d0lo?c00<0
00000??cl`3clo<0g03clo<00`000000lo?c0??cl`1C0??cl`030000003clo<0lo?c0080lo?c00<0
00000??cl`3clo<08`3clo<200000?l0lo?c2P3clo<00`000000lo?c0??cl`0P0??cl`800000:@3c
lo<002d0lo?c00<000000??cl`3clo<01P00000Q0??cl`030000003clo<0lo?c04h0lo?c00<00000
0??cl`3clo<0f`3clo<00`000000lo?c0??cl`1@0??cl`030000003clo<0lo?c00H000009@3clo<0
0`000000lo?c0??cl`3o0??cl`T0lo?c00<000000??cl`3clo<08@3clo<00`000000lo?c0??cl`0W
0??cl`00=@3clo<00`000000lo?c0??cl`0O0??cl`030000003clo<0lo?c04l0lo?c0P00003K0??c
l`030000003clo<0lo?c05P0lo?c00<000000??cl`3clo<08`3clo<00`000000lo?c0??cl`3o0??c
lbd0lo?c00<000000??cl`3clo<09`3clo<00300lo?c00<000000??cl`3clo<0903clo<200000580
lo?c00<000000??cl`3clo<0e`3clo<2000005D0lo?c00<000000??cl`3clo<0:03clo<00`000000
lo?c0??cl`0E0??cl`030000003clo<0lo?c0?l0lo?c5@3clo<00`000000lo?c0??cl`0W0??cl`00
;P3clo<800000240lo?c00<000000??cl`3clo<0DP3clo<00`000000lo?c0??cl`3G0??cl`030000
003clo<0lo?c0540lo?c2000000U0??cl`030000003clo<0lo?c01H0lo?c00<000000??cl`3clo<0
o`3clo<D0??cl`030000003clo<0lo?c02L0lo?c000`0??cl`030000003clo<0lo?c0080lo?c00<0
00000??cl`3clo<07`3clo<00`000000lo?c0??cl`1C0??cl`800000e`3clo<00`000000lo?c0??c
l`1C0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<08`3clo<2000001P0lo?c00<0
00000??cl`3clo<0o`3clo<B0??cl`800000:@3clo<00300lo?c00<000000??cl`3clo<0903clo<0
0`000000lo?c0??cl`1E0??cl`030000003clo<0lo?c0=@0lo?c00<000000??cl`3clo<0D`3clo<0
0`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c01P0lo?c00<000000??cl`3clo<0j@3clo<0
0`000000lo?c0??cl`0U0??cl`030000003clo<0lo?c02L0lo?c000^0??cl`P000008@3clo<20000
05L0lo?c00<000000??cl`3clo<0dP3clo<2000005<0lo?c2000000U0??cl`030000003clo<0lo?c
01T0lo?c00<000000??cl`3clo<0i`3clo<00`000000lo?c0??cl`0V0??cl`030000003clo<0lo?c
02L0lo?c000`0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<07`3clo<00`000000
lo?c0??cl`1G0??cl`800000d`3clo<00`000000lo?c0??cl`1C0??cl`030000003clo<0lo?c0080
lo?c00<000000??cl`3clo<08`3clo<00`000000lo?c0??cl`3o0??cl`@0lo?c00<000000??cl`3c
lo<09P3clo<00`000000lo?c0??cl`0W0??cl`00<03clo<00`000000lo?c0??cl`020??cl`030000
003clo<0lo?c01l0lo?c00<000000??cl`3clo<0F@3clo<00`000000lo?c0??cl`3@0??cl`030000
003clo<0lo?c05<0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0S0??cl`030000
003clo<0lo?c0?l0lo?c0`3clo<00`000000lo?c0??cl`0W0??cl`030000003clo<0lo?c02L0lo?c
000]0??cl`030000003clo<0lo?c00H000008@3clo<00`000000lo?c0??cl`1J0??cl`800000d03c
lo<00`000000lo?c0??cl`1@0??cl`030000003clo<0lo?c00H000009@3clo<00`000000lo?c0??c
l`3o0??cl`<0lo?c00<000000??cl`3clo<09`3clo<00`000000lo?c0??cl`0W0??cl`00<03clo<0
0`000000lo?c0??cl`020??cl`030000003clo<0lo?c01l0lo?c00<000000??cl`3clo<0G03clo<0
0`000000lo?c0??cl`3=0??cl`030000003clo<0lo?c05<0lo?c00<000000??cl`3clo<00P3clo<0
0`000000lo?c0??cl`0S0??cl`8000007`3clo<00`000000lo?c0??cl`3P0??cl`030000003clo<0
lo?c02L0lo?c0P00000Y0??cl`00<03clo<2000002D0lo?c0P00001N0??cl`030000003clo<0lo?c
0</0lo?c0P00001E0??cl`800000:@3clo<00`000000lo?c0??cl`0O0??cl`030000003clo<0lo?c
0?l0lo?c2`3clo<00`000000lo?c0??cl`0W0??cl`00<03clo<010000000lo?c00000000000S0??c
l`030000003clo<0lo?c05h0lo?c0P00003<0??cl`030000003clo<0lo?c05<0lo?c00@000000??c
l`00000000006`3clo<5000000L0lo?c00<000000??cl`3clo<0803clo<00`000000lo?c0??cl`3o
0??cl`X0lo?c00<000000??cl`3clo<09`3clo<003@0lo?c0P00000@0??cl`D000000P3clo<30000
00L0lo?c00<000000??cl`3clo<0H03clo<00`000000lo?c0??cl`390??cl`030000003clo<0lo?c
05L0lo?c0P00000J0??cl`040000003clo<0lo?c000000L0lo?c00<000000??cl`3clo<08@3clo<0
0`000000lo?c0??cl`3o0??cl`T0lo?c00<000000??cl`3clo<09`3clo<00380lo?c0P00000C0??c
l`060000003clo<0lo?c0000003clo<000000`3clo<00`000000lo?c0??cl`040??cl`030000003c
lo<0lo?c0640lo?c0P0000390??cl`030000003clo<0lo?c05D0lo?c0P00000M0??cl`030000003c
lo<0lo?c00L0lo?c00<000000??cl`3clo<08P3clo<00`000000lo?c0??cl`3H0??cl`030000003c
lo<0lo?c02`0lo?c00<000000??cl`3clo<09`3clo<00300lo?c0P00000F0??cl`050000003clo<0
lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c06<0lo?c00<0
00000??cl`3clo<0aP3clo<00`000000lo?c0??cl`1C0??cl`8000005P3clo<7000000<0lo?c00<0
00000??cl`3clo<01P3clo<300000?`0lo?c00<000000??cl`3clo<0:`3clo<3000002T0lo?c000`
0??cl`030000003clo<0lo?c00/0lo?c1`0000040??cl`040000003clo<0lo?c000000<0lo?c00<0
00000??cl`3clo<0103clo<3000006@0lo?c0P0000340??cl`<00000E@3clo<00`000000lo?c0??c
l`0P0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0o03clo<00`000000lo?c0??c
l`0]0??cl`030000003clo<0lo?c02L0lo?c000`0??cl`030000003clo<0lo?c0080lo?c00<00000
0??cl`3clo<04P3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<00000
0??cl`3clo<0IP3clo<00`000000lo?c0??cl`330??cl`030000003clo<0lo?c05<0lo?c00<00000
0??cl`3clo<00P3clo<00`000000lo?c0??cl`0K0??cl`030000003clo<0lo?c00D0lo?c00<00000
0??cl`3clo<09`3clo<00`000000lo?c0??cl`3A0??cl`030000003clo<0lo?c02h0lo?c00<00000
0??cl`3clo<09`3clo<002d0lo?c00<000000??cl`3clo<01P00000D0??cl`030000003clo<00000
00<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`1W0??cl`800000``3clo<00`00
0000lo?c0??cl`1@0??cl`030000003clo<0lo?c00H000007@3clo<00`000000lo?c0??cl`050??c
l`030000003clo<0lo?c02P0lo?c00<000000??cl`3clo<0d03clo<00`000000lo?c0??cl`0^0??c
l`030000003clo<0lo?c02L0lo?c000e0??cl`030000003clo<0lo?c0180lo?c00<000000??cl`00
00000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c06T0lo?c00<000000??cl`3c
lo<0`03clo<00`000000lo?c0??cl`1H0??cl`030000003clo<0lo?c01L0lo?c00D000000??cl`3c
lo<0lo?c000000070??cl`030000003clo<0lo?c02T0lo?c0P00003?0??cl`030000003clo<0lo?c
02l0lo?c00<000000??cl`3clo<09`3clo<00300lo?c00<000000??cl`3clo<04`3clo<01`000000
lo?c0??cl`3clo<000000??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0
lo?c06X0lo?c0P0000300??cl`030000003clo<0lo?c05<0lo?c00<000000??cl`3clo<07@3clo<3
000000P0lo?c00<000000??cl`3clo<0:`3clo<00`000000lo?c0??cl`3;0??cl`030000003clo<0
lo?c0300lo?c00<000000??cl`3clo<09`3clo<002h0lo?c2000000A0??cl`<000000`3clo<30000
00L0lo?c0P00001]0??cl`030000003clo<0lo?c0;`0lo?c0P00001C0??cl`P000009@3clo<20000
02d0lo?c00<000000??cl`3clo<0o03clo<2000002T0lo?c000`0??cl`030000003clo<0lo?c0080
lo?c00<000000??cl`3clo<07`3clo<00`000000lo?c0??cl`1]0??cl`800000_@3clo<00`000000
lo?c0??cl`1C0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<08`3clo<00`000000
lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<00300lo?c00<000000??cl`3clo<0
1@3clo<00`000000lo?c0??cl`0L0??cl`030000003clo<0lo?c06l0lo?c00<000000??cl`3clo<0
^P3clo<00`000000lo?c0??cl`1C0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0
803clo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<00300lo?c0`00
00050??cl`030000003clo<0lo?c01`0lo?c00<000000??cl`3clo<0L03clo<200000;X0lo?c00<0
00000??cl`3clo<0D`3clo<3000000D0lo?c00<000000??cl`3clo<0803clo<00`000000lo?c0??c
l`0a0??cl`030000003clo<0lo?c0<40lo?c00<000000??cl`3clo<0=03clo<00`000000lo?c0??c
l`0W0??cl`00<03clo<00`000000lo?c0??cl`0200000080lo?c00<000000??cl`3clo<07@3clo<2
000007<0lo?c0P00002g0??cl`800000E@3clo<00`000000lo?c0??cl`0200000080lo?c00<00000
0??cl`3clo<08@3clo<00`000000lo?c0??cl`0b0??cl`030000003clo<0lo?c0;l0lo?c00<00000
0??cl`3clo<0=@3clo<00`000000lo?c0??cl`0W0??cl`00=@3clo<200000200lo?c00<000000??c
l`3clo<0M03clo<00`000000lo?c0??cl`2e0??cl`030000003clo<0lo?c05P0lo?c0P00000T0??c
l`800000=03clo<200000;h0lo?c00<000000??cl`3clo<0=@3clo<2000002T0lo?c000`0??cl`03
0000003clo<00000008000008P3clo<00`000000lo?c0??cl`1e0??cl`800000]@3clo<00`000000
lo?c0??cl`1C0??cl`030000003clo<00000008000009P3clo<00`000000lo?c0??cl`0e0??cl`03
0000003clo<0lo?c0;X0lo?c00<000000??cl`3clo<0=`3clo<00`000000lo?c0??cl`0W0??cl`00
<03clo<2000002D0lo?c00<000000??cl`3clo<0M`3clo<00`000000lo?c0??cl`2b0??cl`030000
003clo<0lo?c05<0lo?c0P00000Y0??cl`030000003clo<0lo?c03H0lo?c00<000000??cl`3clo<0
^03clo<00`000000lo?c0??cl`0h0??cl`030000003clo<0lo?c02L0lo?c000`0??cl`030000003c
lo<0lo?c02@0lo?c00<000000??cl`3clo<0N03clo<200000;80lo?c00<000000??cl`3clo<0D`3c
lo<00`000000lo?c0??cl`0X0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??c
l`0W0??cl`00E`3clo<2000007/0lo?c0P00002_0??cl`800000P03clo<00`000000lo?c0??cl`3o
0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<000000??cl`3clo<0O03clo<20000
0:h0lo?c00<000000??cl`3clo<0OP3clo<2000003`0lo?c00<000000??cl`3clo<0k@3clo<20000
02T0lo?c001G0??cl`030000003clo<0lo?c07h0lo?c0P00002/0??cl`030000003clo<0lo?c07h0
lo?c00<000000??cl`3clo<0?03clo<00`000000lo?c0??cl`2]0??cl`030000003clo<0lo?c03d0
lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<000000??cl`3clo<0P03clo<200000:X0lo?c
00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`0m0??cl`800000[03clo<00`000000lo?c
0??cl`0n0??cl`030000003clo<0lo?c02L0lo?c001G0??cl`030000003clo<0lo?c0880lo?c0P00
002X0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0?`3clo<200000:P0lo?c0P00
00110??cl`030000003clo<0lo?c02L0lo?c001G0??cl`800000Q@3clo<200000:D0lo?c0P000020
0??cl`030000003clo<0lo?c0>P0lo?c00<000000??cl`3clo<0@@3clo<00`000000lo?c0??cl`0W
0??cl`00A`3clo<300000080lo?c0`0000080??cl`030000003clo<0lo?c08H0lo?c0P00002T0??c
l`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0i`3clo<00`000000lo?c0??cl`120??c
l`030000003clo<0lo?c02L0lo?c00180??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3c
lo<01@3clo<00`000000lo?c0??cl`280??cl`800000XP3clo<00`000000lo?c0??cl`1n0??cl`80
0000A`3clo<200000><0lo?c0P00000Y0??cl`00B03clo<00`000000lo?c0??cl`050??cl`030000
003clo<0lo?c00@0lo?c00<000000??cl`3clo<0RP3clo<200000:00lo?c00<000000??cl`3clo<0
OP3clo<00`000000lo?c0??cl`180??cl`800000hP3clo<00`000000lo?c0??cl`0W0??cl`00?P3c
lo<7000000<0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`040??cl`<00000S03c
lo<2000009`0lo?c0`00001/0??cl`<000000`3clo<00`000000lo?c00000002000000T0lo?c00<0
00000??cl`3clo<0BP3clo<2000009H0lo?c00<000000??cl`3clo<0A`3clo<00`000000lo?c0??c
l`0W0??cl`00B03clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c00@0lo?c00<00000
0??cl`3clo<0SP3clo<2000009`0lo?c00<000000??cl`3clo<0J`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<0h@3clo<00`000000lo?c0??cl`18
0??cl`030000003clo<0lo?c02L0lo?c00180??cl`030000003clo<0lo?c00@0lo?c00<000000??c
l`3clo<01@3clo<00`000000lo?c0??cl`2@0??cl`800000VP3clo<00`000000lo?c0??cl`1[0??c
l`030000003clo<0lo?c00P0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`3O0??c
l`800000B`3clo<00`000000lo?c0??cl`0W0??cl`00B03clo<00`000000lo?c0??cl`020??cl`80
0000203clo<00`000000lo?c0??cl`2B0??cl`800000V03clo<00`000000lo?c0??cl`1R0??cl`L0
00000P3clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c00D0lo?c0`00001A0??cl`80
0000RP3clo<2000004/0lo?c0`00000Y0??cl`00A`3clo<2000000@0lo?c00<000000??cl`3clo<0
1`3clo<00`000000lo?c0??cl`2D0??cl`800000UP3clo<00`000000lo?c0??cl`1[0??cl`030000
003clo<0lo?c00P0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`1C0??cl`800000
e`3clo<00`000000lo?c0??cl`0W0??cl`00B03clo<00`000000lo?c0??cl`030??cl`<000001P3c
lo<2000009L0lo?c0P00002C0??cl`800000K@3clo<00`000000lo?c0??cl`070??cl`030000003c
lo<0lo?c00H0lo?c00<000000??cl`3clo<0E@3clo<200000=D0lo?c00<000000??cl`3clo<09`3c
lo<005L0lo?c00<000000??cl`3clo<0V03clo<200000980lo?c00<000000??cl`3clo<0J`3clo<0
0`000000lo?c0??cl`050??cl`8000002@3clo<00`000000lo?c0??cl`3G0??cl`800000D`3clo<0
0`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`2J0??cl`800000T03clo<00`00
0000lo?c0??cl`1Z0??cl`8000001`3clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c
05`0lo?c0P00001g0??cl`800000E@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000
lo?c0??cl`2L0??cl`800000SP3clo<00`000000lo?c0??cl`1[0??cl`030000003clo<0lo?c00H0
lo?c0`0000070??cl`800000G`3clo<300000780lo?c0P00001F0??cl`800000:@3clo<005L0lo?c
00<000000??cl`3clo<0WP3clo<2000008`0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c
0??cl`1Q0??cl`800000K`3clo<00`000000lo?c0??cl`1G0??cl`030000003clo<0lo?c02L0lo?c
001G0??cl`800000X@3clo<3000008P0lo?c0P0000200??cl`030000003clo<0lo?c0?l0lo?c;@3c
lo<00`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`2S0??cl`800000Q`3clo<0
0`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c06P0lo?c0`00001Q0??cl`800000GP3clo<0
0`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`2U0??cl`800000Q@3clo<00`00
0000lo?c0??cl`1n0??cl`030000003clo<0lo?c06/0lo?c0`00001K0??cl`<00000H03clo<00`00
0000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`2W0??cl`800000P`3clo<00`000000
lo?c0??cl`1n0??cl`030000003clo<0lo?c0<L0lo?c0P00001S0??cl`030000003clo<0lo?c02L0
lo?c001G0??cl`800000ZP3clo<200000800lo?c0P0000200??cl`800000M03clo<400000;@0lo?c
0P00000Y0??cl`00E`3clo<00`000000lo?c0??cl`2[0??cl`800000O`3clo<00`000000lo?c0??c
l`1n0??cl`030000003clo<0lo?c07L0lo?c0`0000150??cl`<00000JP3clo<00`000000lo?c0??c
l`0W0??cl`00E`3clo<00`000000lo?c0??cl`2]0??cl`<00000O03clo<00`000000lo?c0??cl`1n
0??cl`030000003clo<0lo?c07l0lo?c0P00000j0??cl`@00000K@3clo<00`000000lo?c0??cl`0W
0??cl`00E`3clo<00`000000lo?c0??cl`2`0??cl`800000NP3clo<00`000000lo?c0??cl`1n0??c
l`030000003clo<0lo?c0840lo?c1@00000/0??cl`@00000MP3clo<00`000000lo?c0??cl`0W0??c
l`00E`3clo<00`000000lo?c0??cl`2b0??cl`<00000M`3clo<00`000000lo?c0??cl`1n0??cl`03
0000003clo<0lo?c08/0lo?c1`00000D0??cl`@000001@3clo<3000007X0lo?c00<000000??cl`3c
lo<09`3clo<005L0lo?c0P00002f0??cl`<00000L`3clo<200000800lo?c0P00002H0??cl`L00000
1@3clo<3000008D0lo?c0P00000Y0??cl`00E`3clo<00`000000lo?c0??cl`2h0??cl`800000LP3c
lo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??c
l`0W0??cl`00A`3clo<3000000<0lo?c0`0000070??cl`030000003clo<0lo?c0;X0lo?c0`00001_
0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0
lo?c02L0lo?c00180??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`04
0??cl`030000003clo<0lo?c0;d0lo?c0P00001]0??cl`030000003clo<0lo?c07h0lo?c00<00000
0??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c00180??cl`050000003clo<0lo?c
0??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c0;l0lo?c0P00001[
0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0
lo?c02L0lo?c000n0??cl`L000000`3clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<00000
0??cl`3clo<0103clo<300000<40lo?c0`00001V0??cl`<00000P03clo<200000?l0lo?c;@3clo<2
000002T0lo?c00180??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`04
0??cl`030000003clo<0lo?c0<@0lo?c0P00001V0??cl`030000003clo<0lo?c07h0lo?c00<00000
0??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c00180??cl`050000003clo<0lo?c
0??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c0<H0lo?c0`00001S
0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0
lo?c02L0lo?c00180??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`04
0??cl`030000003clo<0lo?c0<T0lo?c0P00001Q0??cl`030000003clo<0lo?c07<0lo?c0`000008
0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00A`3clo<20000
00<0lo?c00D000000??cl`3clo<0lo?c000000060??cl`800000c03clo<3000005d0lo?c0P00001f
0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0
lo?c02L0lo?c00180??cl`030000003clo<0lo?c0080lo?c0`0000070??cl`030000003clo<0lo?c
0<h0lo?c0`00001K0??cl`030000003clo<0lo?c07@0lo?c00<000000??cl`3clo<01`3clo<00`00
0000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<000000??cl`3c
lo<0d@3clo<2000005T0lo?c00<000000??cl`3clo<0J`3clo<700000080lo?c00<000000??cl`3c
lo<01`3clo<300000?l0lo?c:`3clo<3000002T0lo?c001G0??cl`030000003clo<0lo?c0=<0lo?c
0`00001F0??cl`030000003clo<0lo?c07@0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c
0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<000000??cl`3clo<0eP3c
lo<3000005<0lo?c00<000000??cl`3clo<0M03clo<00`000000lo?c0??cl`070??cl`030000003c
lo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<200000=X0lo?c0`00001?
0??cl`800000MP3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c0?l0lo?c;@3clo<0
0`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`3L0??cl`<00000C@3clo<00`00
0000lo?c0??cl`1c0??cl`8000002@3clo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??c
l`3clo<09`3clo<005L0lo?c00<000000??cl`3clo<0g`3clo<3000004X0lo?c00<000000??cl`3c
lo<0M03clo<00`000000lo?c0??cl`070??cl`800000o`3clo<]0??cl`800000:@3clo<005L0lo?c
00<000000??cl`3clo<0hP3clo<2000004P0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c
0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<000000??cl`3clo<0i03c
lo<3000004D0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`3o0??clbd0lo?c00<0
00000??cl`3clo<09`3clo<005L0lo?c0P00003X0??cl`<00000@@3clo<200000800lo?c00<00000
0??cl`3clo<0o`3clo<]0??cl`030000003clo<0lo?c02L0lo?c001G0??cl`030000003clo<0lo?c
0>X0lo?c0P0000100??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??c
l`030000003clo<0lo?c02L0lo?c001G0??cl`030000003clo<0lo?c0>`0lo?c0`00000m0??cl`03
0000003clo<0lo?c07h0lo?c0P00003o0??clbd0lo?c0P00000Y0??cl`00E`3clo<00`000000lo?c
0??cl`3_0??cl`<00000>P3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c
;@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<200000?<0lo?c1000000e0??cl`800000P03c
lo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<005L0lo?c00<00000
0??cl`3clo<0mP3clo<3000003<0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`3o
0??clbd0lo?c00<000000??cl`3clo<09`3clo<004`0lo?c0`0000080??cl`030000003clo<0lo?c
0?T0lo?c0`00000`0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??c
l`030000003clo<0lo?c02L0lo?c001?0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3c
lo<0o03clo<3000002d0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`3o0??clbd0
lo?c00<000000??cl`3clo<09`3clo<00500lo?c00<000000??cl`3clo<0103clo<00`000000lo?c
0??cl`3o0??cl`@00000:@3clo<00`000000lo?c0??cl`1n0??cl`800000o`3clo<]0??cl`800000
:@3clo<004@0lo?c1`0000050??cl`030000003clo<0lo?c00@0lo?c0`00003o0??cl`@0lo?c0`00
000T0??cl`<00000P03clo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3c
lo<00500lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`3o0??cl`L0lo?c0`00000S
0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<0o`3clo<]0??cl`030000003clo<0
lo?c02L0lo?c001?0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0o`3clo<:0??c
l`<00000803clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`00
0000lo?c0??cl`0W0??cl`00C@3clo<2000000P0lo?c00<000000??cl`3clo<0o`3clo<=0??cl`<0
00007@3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c0?l0lo?c;@3clo<00`000000
lo?c0??cl`0W0??cl`00C@3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c0?l0lo?c
403clo<3000001X0lo?c00<000000??cl`3clo<0OP3clo<200000?l0lo?c;@3clo<2000002T0lo?c
001>0??cl`<000001P3clo<200000?l0lo?c503clo<4000001D0lo?c0P0000200??cl`030000003c
lo<0lo?c0?l0lo?c;@3clo<00`000000lo?c0??cl`0W0??cl`00E`3clo<00`000000lo?c0??cl`3o
0??claL0lo?c1000000B0??cl`030000003clo<0lo?c06X0lo?c0`0000030??cl`030000003clo<0
0000008000002@3clo<00`000000lo?c0??cl`3o0??clbd0lo?c00<000000??cl`3clo<09`3clo<0
05L0lo?c00<000000??cl`3clo<0903clo<00`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c
02h0lo?c00<000000??cl`3clo<0;P3clo<00`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c
02d0lo?c00<000000??cl`0000000`00000>0??cl`030000003clo<0lo?c06T0lo?c00D000000??c
l`3clo<0lo?c000000070??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<0903clo<0
0`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c02h0lo?c00<000000??cl`3clo<0;@3clo<0
0`000000lo?c0??cl`0^0??cl`030000003clo<0lo?c02h0lo?c00<000000??cl`3clo<04@3clo<0
0`000000lo?c0??cl`0W0??cl`00E`3clo<010000000lo?c0??cl`00000;0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02P3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00X0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`0:0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<02@3clo<00`000000lo?c0??cl`090??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<00`3clo<6000000030??cl`000000lo?c00D0lo?c00<00000
0??cl`3clo<0J@3clo<01@000000lo?c0??cl`3clo<0000000P0lo?c00<000000??cl`3clo<01@3c
lo<00`000000lo?c0000000<0??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3c
lo<00`000000lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02P3c
lo<00`000000lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3c
lo<00`000000lo?c0??cl`090??cl`030000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3c
lo<00`000000lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3c
lo<00`000000lo?c0??cl`0:0??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3c
lo<00`000000lo?c0??cl`090??cl`030000003clo<0lo?c00X0lo?c00<000000??cl`3clo<02@3c
lo<00`000000lo?c0??cl`090??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<02@3c
lo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c02L0lo?c001G0??clol00000<@00001T
0??cl`P000000`3clo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c00D0lo?co`00000a
000002T0lo?c003o0??clo<0lo?c00D000000??cl`3clo<0lo?c000000080??cl`030000003clo<0
lo?c0?l0lo?cG`3clo<00?l0lo?cl`3clo<01@000000lo?c0??cl`3clo<0000000L0lo?c00<00000
0??cl`3clo<0o`3clo=P0??cl`00o`3clo?c0??cl`050000003clo<0lo?c0??cl`0000001@3clo<2
00000?l0lo?cH`3clo<00?l0lo?cl`3clo<01@000000lo?c0??cl`3clo<0000000D0lo?c00<00000
0??cl`3clo<0o`3clo=R0??cl`00o`3clo?d0??cl`<000001`3clo<300000?l0lo?cH@3clo<00>H0
lo?c0`00003o0??cljd0lo?c0`0000380??cl`00i@3clo<01@000000lo?c0??cl`3clo<000000?l0
lo?cZ`3clo<01@000000lo?c0??cl`3clo<000000<L0lo?c003U0??cl`050000003clo<0lo?c0??c
l`000000o`3clo>[0??cl`050000003clo<0lo?c0??cl`000000a`3clo<00=/0lo?c0`0000040??c
l`<0000000@0lo?c00000000000000000`3clo<700000080lo?c0P0000070??cl`<00000o`3clo>:
0??cl`<00000103clo<3000000040??cl`0000000000000000<0lo?c1`0000020??cl`8000001P3c
lo<500000:l0lo?c003L0??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`0000001@3clo<0
10000000lo?c0??cl`0000020??cl`040000003clo<0lo?c00000080lo?c00<000000??cl`3clo<0
1@3clo<00`000000lo?c0??cl`3o0??clhX0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c
000000050??cl`040000003clo<0lo?c00000080lo?c00@000000??cl`3clo<000000P3clo<00`00
0000lo?c0??cl`040??cl`040000003clo<0lo?c00000:l0lo?c003L0??cl`030000003clo<0lo?c
00@0lo?c00<000000??cl`3clo<00P0000030??cl`040000003clo<0lo?c00000080lo?c00@00000
0??cl`3clo<000002P3clo<00`000000lo?c0??cl`3o0??clhX0lo?c00<000000??cl`3clo<0103c
lo<00`000000lo?c0??cl`02000000<0lo?c00@000000??cl`3clo<000000P3clo<010000000lo?c
0??cl`00000:0??cl`030000003clo<0lo?c0:l0lo?c003L0??cl`040000003clo<0lo?c000000<0
lo?c00<000000??cl`0000000P3clo<010000000lo?c0??cl`0000020??cl`040000003clo<0lo?c
00000080lo?c100000070??cl`030000003clo<0lo?c0?l0lo?cRP3clo<010000000lo?c0??cl`00
00030??cl`030000003clo<000000080lo?c00@000000??cl`3clo<000000P3clo<010000000lo?c
0??cl`0000020??cl`@00000203clo<00`000000lo?c0??cl`2^0??cl`00g03clo<4000000<0lo?c
00<000000??cl`0000000P3clo<010000000lo?c0??cl`0000020??cl`030000003clo<0lo?c0080
000000D0lo?c0000003clo<0lo?c000000070??cl`030000003clo<0lo?c0?l0lo?cRP3clo<40000
00<0lo?c00<000000??cl`0000000P3clo<010000000lo?c0??cl`0000020??cl`030000003clo<0
lo?c0080000000D0lo?c0000003clo<0lo?c000000090??cl`030000003clo<0lo?c0:d0lo?c003L
0??cl`040000003clo<0lo?c00000080lo?c0P0000020??cl`H0000000@0lo?c000000000000lo?c
0P0000000`3clo<000000??cl`02000000P0lo?c00<000000??cl`3clo<0o`3clo>:0??cl`040000
003clo<0lo?c00000080lo?c0P0000020??cl`H0000000@0lo?c000000000000lo?c0P0000000`3c
lo<000000??cl`02000000X0lo?c00<000000??cl`3clo<0[@3clo<00=`0lo?c00<000000??cl`3c
lo<0803clo<00`000000lo?c0??cl`3o0??clhX0lo?c00<000000??cl`3clo<08P3clo<00`000000
lo?c0??cl`2]0??cl`00g03clo<01@000000lo?c0??cl`3clo<0000001d0lo?c0P00003o0??clh`0
lo?c00D000000??cl`3clo<0lo?c0000000L0??cl`050000003clo<0lo?c0??cl`000000[`3clo<0
0=/0lo?c1P0000020??cl`030000003clo<0lo?c01T0lo?c00<000000??cl`3clo<0o`3clo>90??c
l`H000000P3clo<00`000000lo?c0??cl`0H0??cl`<00000/03clo<00?l0lo?co`3clo?o0??clf<0
lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?co`3clo?o
0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<00?l0lo?c
o`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?cH`3clo<0
0?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??clol0lo?c
H`3clo<00?l0lo?co`3clo?o0??clf<0lo?c003o0??clol0lo?co`3clo=S0??cl`00o`3clo?o0??c
lol0lo?cH`3clo<00001\
\>"],
  ImageRangeCache->{{{0, 863}, {532.938, 0}} -> {-0.0508427, -0.0309082, \
0.00243533, 0.00243533}, {{20.875, 411.938}, {520.188, 278.5}} -> {0.890973, \
0.171939, 0.00409693, 0.00267297}, {{451, 842.063}, {520.188, 278.5}} -> \
{-0.871219, -0.0524484, 0.00409693, 0.00213839}, {{20.875, 411.938}, \
{254.375, 12.6875}} -> {0.880091, -110.086, 0.00409693, 0.213839}, {{451, \
842.063}, {254.375, 12.6875}} -> {-0.888246, -9.87, 0.00409693, 0.0187108}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Case 2: Five Layers with a Transmission Peak", "Subsection"],

Cell["\<\
Here I will test using the fit made to Au above. The transmission of pulses \
in the FDTD domain is compared with that of the theoretical results.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Block[{epsList, dList, \[Theta]in, 
          pol}, \[IndentingNewLine]epsilon\  = \ 
          epsilon\  /. \ 
            solnFinal[\([2]\)]; \[IndentingNewLine]Print["\<epsilon=\>", 
          epsilon]; \[IndentingNewLine]\[IndentingNewLine]<< 
          Graphics`Graphics`; \[IndentingNewLine]$TextStyle = {FontFamily \
\[Rule] "\<Times\>", FontSize \[Rule] 12}; \[IndentingNewLine]$FormatType\  = 
          TraditionalForm; \[IndentingNewLine]plotOps = {Frame -> True, 
            ImageSize \[Rule] \ 72\ 6, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Permittivity\>", "\<\>", "\<\
\>"}, Axes \[Rule] False, 
            DisplayFunction \[Rule] Identity}; \[IndentingNewLine]realPlot = 
          Plot[Re[epsilon], {\[HBar]\[Omega], enEnd, enStart}, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Permittivity\>", "\<Figure 1\
\>", "\<\>"}, Evaluate[plotOps], 
            PlotStyle \[Rule] {Thickness[0.005], RGBColor[0, 0, 0]}, 
            PlotLabel -> \*"\"\<Re(\!\(\[Epsilon]\_r\))\>\""]; \
\[IndentingNewLine]imagPlot = 
          Plot[Im[epsilon], {\[HBar]\[Omega], enEnd, enStart}, 
            FrameLabel \[Rule] {"\<E (eV)\>", "\<Permittivity\>", "\<Figure 2\
\>", "\<\>"}, Evaluate[plotOps], 
            PlotStyle \[Rule] {Thickness[0.005], RGBColor[1, 0, 0]}, 
            PlotLabel -> \*"\"\<Im(\!\(\[Epsilon]\_r\))\>\""]; \
\[IndentingNewLine]Show[GraphicsArray[{realPlot, imagPlot}], 
          DisplayFunction \[Rule] $DisplayFunction, 
          ImageSize \[Rule] \ 
            72*10]; \[IndentingNewLine]\[IndentingNewLine]epsList = {1, 
            epsilon, 2, epsilon, 1}; \  (*\ 
          List\ of\ values\ for\ permittivity\ *) \[IndentingNewLine]dList = \
{0, 50*10\^\(-9\), 510*10\^\(-9\), 50*10\^\(-9\), 0}; \  (*\ 
          List\ of\ values\ for\ layer\ thicknesses\ *) \[IndentingNewLine]\
\[Theta]in\  = \ 
          0; \[IndentingNewLine]pol\  = \ "\<TM\>"; \
\[IndentingNewLine]Plot[{Abs[
              TransmissionCoef[epsList, dList, \[Theta]in, pol, en]]}, {en, 
            enEnd, enStart}, Frame -> True, ImageSize \[Rule] \ 72\ 6, 
          FrameLabel \[Rule] {"\<E (eV)\>", "\<Transmission\>", "\<Figure \
3\>", "\<\>"}, Axes \[Rule] False]; \[IndentingNewLine]enList\  = \ 
          Table[
            enEnd\  + \ i*\((enStart - enEnd)\)/1000, {i, 0, 
              1000}]; \[IndentingNewLine]tranList\  = \ 
          Table[Abs[
              TransmissionCoef[epsList, dList, \[Theta]in, pol, 
                enList[\([i]\)]]], {i, 1, 
              Length[enList]}]; \[IndentingNewLine]Export["\<enList.mat\>", 
          enList]; \[IndentingNewLine]Export["\<tranList.mat\>", 
          tranList];\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("epsilon="\[InvisibleSpace]\(1 + 
          44.55261933048567`\/\(0.`\ \[ImaginaryI]\ \[HBar]\[Omega] - \[HBar]\
\[Omega]\^2\) + 
          20.39759963659135`\/\(1.8139151456597415`*^-10 + \
0.2890102765721252`\ \[ImaginaryI]\ \[HBar]\[Omega] - \[HBar]\[Omega]\^2\) + 
          1.4543481244082495`\/\(\(\(0.29310265440177746`\)\(\[InvisibleSpace]\
\)\) + 2.331464640953873`\ \[ImaginaryI]\ \[HBar]\[Omega] - \
\[HBar]\[Omega]\^2\)\)\),
      SequenceForm[ "epsilon=", 
        Plus[ 1, 
          Times[ 44.552619330485669, 
            Power[ 
              Plus[ 
                Times[ 
                  Complex[ 0, 0.0], \[HBar]\[Omega]], 
                Times[ -1, 
                  Power[ \[HBar]\[Omega], 2]]], -1]], 
          Times[ 20.397599636591352, 
            Power[ 
              Plus[ .18139151456597415*^-9, 
                Times[ 
                  Complex[ 0, .28901027657212519], \[HBar]\[Omega]], 
                Times[ -1, 
                  Power[ \[HBar]\[Omega], 2]]], -1]], 
          Times[ 1.4543481244082495, 
            Power[ 
              Plus[ .29310265440177746, 
                Times[ 
                  Complex[ 0, 2.3314646409538731], \[HBar]\[Omega]], 
                Times[ -1, 
                  Power[ \[HBar]\[Omega], 2]]], -1]]]],
      Editable->False]], "Print"],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .2943 
%%ImageSize: 720 211.897 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.00700719 0.47619 [
[ 0 0 0 0 ]
[ 1 .2943 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .2943 L
0 .2943 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00700719 0.477324 0.287295 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.40476 2.30444 0.948559 0.00588823 [
[.20835 -0.0125 -9.5 -15.6875 ]
[.20835 -0.0125 9.5 0 ]
[.43879 -0.0125 -9.5 -15.6875 ]
[.43879 -0.0125 9.5 0 ]
[.66924 -0.0125 -9.5 -15.6875 ]
[.66924 -0.0125 9.5 0 ]
[.89968 -0.0125 -5 -15.6875 ]
[.89968 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9063 -31.375 ]
[.5 -0.0125 17.9063 -15.6875 ]
[-0.0125 .00644 -30.3125 -7.84375 ]
[-0.0125 .00644 0 7.84375 ]
[-0.0125 .12421 -30.3125 -7.84375 ]
[-0.0125 .12421 0 7.84375 ]
[-0.0125 .24197 -30.3125 -7.84375 ]
[-0.0125 .24197 0 7.84375 ]
[-0.0125 .35974 -30.3125 -7.84375 ]
[-0.0125 .35974 0 7.84375 ]
[-0.0125 .4775 -24.3125 -7.84375 ]
[-0.0125 .4775 0 7.84375 ]
[-0.0125 .59527 -24.3125 -7.84375 ]
[-0.0125 .59527 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -46 -30.375 ]
[-0.01875 .30902 -30.3125 30.375 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -21.9688 0 ]
[.5 .63053 21.9688 15.6875 ]
[ 1 0 .125 0 ]
[.5 .63053 -16.6875 15.6875 ]
[.5 .63053 16.6875 31.375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.20835 0 m
.20835 .00625 L
s
gsave
.20835 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.7) show
1.000 setlinewidth
grestore
.43879 0 m
.43879 .00625 L
s
gsave
.43879 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.66924 0 m
.66924 .00625 L
s
gsave
.66924 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.9) show
1.000 setlinewidth
grestore
.89968 0 m
.89968 .00625 L
s
gsave
.89968 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.25444 0 m
.25444 .00375 L
s
.30052 0 m
.30052 .00375 L
s
.34661 0 m
.34661 .00375 L
s
.3927 0 m
.3927 .00375 L
s
.48488 0 m
.48488 .00375 L
s
.53097 0 m
.53097 .00375 L
s
.57706 0 m
.57706 .00375 L
s
.62315 0 m
.62315 .00375 L
s
.71532 0 m
.71532 .00375 L
s
.76141 0 m
.76141 .00375 L
s
.8075 0 m
.8075 .00375 L
s
.85359 0 m
.85359 .00375 L
s
.16226 0 m
.16226 .00375 L
s
.11617 0 m
.11617 .00375 L
s
.07008 0 m
.07008 .00375 L
s
.02399 0 m
.02399 .00375 L
s
.94577 0 m
.94577 .00375 L
s
.99186 0 m
.99186 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9063 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
73.313 15.813 moveto
(H) show
77.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.000 15.813 moveto
(L) show
94.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .00644 m
.00625 .00644 L
s
gsave
-0.0125 .00644 -91.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(160) show
1.000 setlinewidth
grestore
0 .12421 m
.00625 .12421 L
s
gsave
-0.0125 .12421 -91.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(140) show
1.000 setlinewidth
grestore
0 .24197 m
.00625 .24197 L
s
gsave
-0.0125 .24197 -91.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(120) show
1.000 setlinewidth
grestore
0 .35974 m
.00625 .35974 L
s
gsave
-0.0125 .35974 -91.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
0 .4775 m
.00625 .4775 L
s
gsave
-0.0125 .4775 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(80) show
1.000 setlinewidth
grestore
0 .59527 m
.00625 .59527 L
s
gsave
-0.0125 .59527 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(60) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .03588 m
.00375 .03588 L
s
0 .06532 m
.00375 .06532 L
s
0 .09477 m
.00375 .09477 L
s
0 .15365 m
.00375 .15365 L
s
0 .18309 m
.00375 .18309 L
s
0 .21253 m
.00375 .21253 L
s
0 .27141 m
.00375 .27141 L
s
0 .30085 m
.00375 .30085 L
s
0 .3303 m
.00375 .3303 L
s
0 .38918 m
.00375 .38918 L
s
0 .41862 m
.00375 .41862 L
s
0 .44806 m
.00375 .44806 L
s
0 .50694 m
.00375 .50694 L
s
0 .53638 m
.00375 .53638 L
s
0 .56582 m
.00375 .56582 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -129.531 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
91.375 11.8438 translate
90 rotate
-91.375 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Permittivity) show
119.750 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.20835 .61178 m
.20835 .61803 L
s
.43879 .61178 m
.43879 .61803 L
s
.66924 .61178 m
.66924 .61803 L
s
.89968 .61178 m
.89968 .61803 L
s
.125 Mabswid
.25444 .61428 m
.25444 .61803 L
s
.30052 .61428 m
.30052 .61803 L
s
.34661 .61428 m
.34661 .61803 L
s
.3927 .61428 m
.3927 .61803 L
s
.48488 .61428 m
.48488 .61803 L
s
.53097 .61428 m
.53097 .61803 L
s
.57706 .61428 m
.57706 .61803 L
s
.62315 .61428 m
.62315 .61803 L
s
.71532 .61428 m
.71532 .61803 L
s
.76141 .61428 m
.76141 .61803 L
s
.8075 .61428 m
.8075 .61803 L
s
.85359 .61428 m
.85359 .61803 L
s
.16226 .61428 m
.16226 .61803 L
s
.11617 .61428 m
.11617 .61803 L
s
.07008 .61428 m
.07008 .61803 L
s
.02399 .61428 m
.02399 .61803 L
s
.94577 .61428 m
.94577 .61803 L
s
.99186 .61428 m
.99186 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -82.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Figure) show
96.938 15.813 moveto
(1) show
102.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .00644 m
1 .00644 L
s
.99375 .12421 m
1 .12421 L
s
.99375 .24197 m
1 .24197 L
s
.99375 .35974 m
1 .35974 L
s
.99375 .4775 m
1 .4775 L
s
.99375 .59527 m
1 .59527 L
s
.125 Mabswid
.99625 .03588 m
1 .03588 L
s
.99625 .06532 m
1 .06532 L
s
.99625 .09477 m
1 .09477 L
s
.99625 .15365 m
1 .15365 L
s
.99625 .18309 m
1 .18309 L
s
.99625 .21253 m
1 .21253 L
s
.99625 .27141 m
1 .27141 L
s
.99625 .30085 m
1 .30085 L
s
.99625 .3303 m
1 .3303 L
s
.99625 .38918 m
1 .38918 L
s
.99625 .41862 m
1 .41862 L
s
.99625 .44806 m
1 .44806 L
s
.99625 .50694 m
1 .50694 L
s
.99625 .53638 m
1 .53638 L
s
.99625 .56582 m
1 .56582 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
gsave
.5 .63053 -77.6875 11.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Re) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
76.250 15.813 moveto
(H) show
80.063 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(e) show
84.750 17.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 8.500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(r) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
88.563 15.813 moveto
(L) show
92.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 0 r
.005 w
.02381 .01472 m
.06244 .06119 L
.10458 .10808 L
.14415 .14885 L
.18221 .18536 L
.22272 .22158 L
.26171 .25409 L
.30316 .28634 L
.34309 .31537 L
.3815 .34155 L
.42237 .36768 L
.46172 .39131 L
.49955 .41271 L
.53984 .43419 L
.57861 .45369 L
.61984 .47324 L
.65954 .49102 L
.69774 .50721 L
.73838 .52353 L
.77751 .53841 L
.81909 .5534 L
.85916 .5671 L
.89771 .57962 L
.93871 .59229 L
.97619 .60332 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.522676 0.00700719 0.97619 0.287295 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.40476 2.30444 0.808047 0.0371949 [
[.20835 -0.0125 -9.5 -15.6875 ]
[.20835 -0.0125 9.5 0 ]
[.43879 -0.0125 -9.5 -15.6875 ]
[.43879 -0.0125 9.5 0 ]
[.66924 -0.0125 -9.5 -15.6875 ]
[.66924 -0.0125 9.5 0 ]
[.89968 -0.0125 -5 -15.6875 ]
[.89968 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9063 -31.375 ]
[.5 -0.0125 17.9063 -15.6875 ]
[-0.0125 .06415 -24.3125 -7.84375 ]
[-0.0125 .06415 0 7.84375 ]
[-0.0125 .15714 -33.3125 -7.84375 ]
[-0.0125 .15714 0 7.84375 ]
[-0.0125 .25012 -24.3125 -7.84375 ]
[-0.0125 .25012 0 7.84375 ]
[-0.0125 .34311 -33.3125 -7.84375 ]
[-0.0125 .34311 0 7.84375 ]
[-0.0125 .4361 -24.3125 -7.84375 ]
[-0.0125 .4361 0 7.84375 ]
[-0.0125 .52909 -27.3125 -7.84375 ]
[-0.0125 .52909 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -49 -30.375 ]
[-0.01875 .30902 -33.3125 30.375 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -21.9688 0 ]
[.5 .63053 21.9688 15.6875 ]
[ 1 0 .125 0 ]
[.5 .63053 -16.5938 15.6875 ]
[.5 .63053 16.5938 31.375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.20835 0 m
.20835 .00625 L
s
gsave
.20835 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.7) show
1.000 setlinewidth
grestore
.43879 0 m
.43879 .00625 L
s
gsave
.43879 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.66924 0 m
.66924 .00625 L
s
gsave
.66924 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.9) show
1.000 setlinewidth
grestore
.89968 0 m
.89968 .00625 L
s
gsave
.89968 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.25444 0 m
.25444 .00375 L
s
.30052 0 m
.30052 .00375 L
s
.34661 0 m
.34661 .00375 L
s
.3927 0 m
.3927 .00375 L
s
.48488 0 m
.48488 .00375 L
s
.53097 0 m
.53097 .00375 L
s
.57706 0 m
.57706 .00375 L
s
.62315 0 m
.62315 .00375 L
s
.71532 0 m
.71532 .00375 L
s
.76141 0 m
.76141 .00375 L
s
.8075 0 m
.8075 .00375 L
s
.85359 0 m
.85359 .00375 L
s
.16226 0 m
.16226 .00375 L
s
.11617 0 m
.11617 .00375 L
s
.07008 0 m
.07008 .00375 L
s
.02399 0 m
.02399 .00375 L
s
.94577 0 m
.94577 .00375 L
s
.99186 0 m
.99186 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9063 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
73.313 15.813 moveto
(H) show
77.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.000 15.813 moveto
(L) show
94.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .06415 m
.00625 .06415 L
s
gsave
-0.0125 .06415 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .15714 m
.00625 .15714 L
s
gsave
-0.0125 .15714 -94.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(17.5) show
1.000 setlinewidth
grestore
0 .25012 m
.00625 .25012 L
s
gsave
-0.0125 .25012 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(15) show
1.000 setlinewidth
grestore
0 .34311 m
.00625 .34311 L
s
gsave
-0.0125 .34311 -94.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(12.5) show
1.000 setlinewidth
grestore
0 .4361 m
.00625 .4361 L
s
gsave
-0.0125 .4361 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
0 .52909 m
.00625 .52909 L
s
gsave
-0.0125 .52909 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .08275 m
.00375 .08275 L
s
0 .10134 m
.00375 .10134 L
s
0 .11994 m
.00375 .11994 L
s
0 .13854 m
.00375 .13854 L
s
0 .17573 m
.00375 .17573 L
s
0 .19433 m
.00375 .19433 L
s
0 .21293 m
.00375 .21293 L
s
0 .23153 m
.00375 .23153 L
s
0 .26872 m
.00375 .26872 L
s
0 .28732 m
.00375 .28732 L
s
0 .30592 m
.00375 .30592 L
s
0 .32451 m
.00375 .32451 L
s
0 .36171 m
.00375 .36171 L
s
0 .38031 m
.00375 .38031 L
s
0 .3989 m
.00375 .3989 L
s
0 .4175 m
.00375 .4175 L
s
0 .4547 m
.00375 .4547 L
s
0 .47329 m
.00375 .47329 L
s
0 .49189 m
.00375 .49189 L
s
0 .51049 m
.00375 .51049 L
s
0 .54768 m
.00375 .54768 L
s
0 .56628 m
.00375 .56628 L
s
0 .58488 m
.00375 .58488 L
s
0 .60348 m
.00375 .60348 L
s
0 .04555 m
.00375 .04555 L
s
0 .02695 m
.00375 .02695 L
s
0 .00836 m
.00375 .00836 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -132.531 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
91.375 11.8438 translate
90 rotate
-91.375 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Permittivity) show
119.750 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.20835 .61178 m
.20835 .61803 L
s
.43879 .61178 m
.43879 .61803 L
s
.66924 .61178 m
.66924 .61803 L
s
.89968 .61178 m
.89968 .61803 L
s
.125 Mabswid
.25444 .61428 m
.25444 .61803 L
s
.30052 .61428 m
.30052 .61803 L
s
.34661 .61428 m
.34661 .61803 L
s
.3927 .61428 m
.3927 .61803 L
s
.48488 .61428 m
.48488 .61803 L
s
.53097 .61428 m
.53097 .61803 L
s
.57706 .61428 m
.57706 .61803 L
s
.62315 .61428 m
.62315 .61803 L
s
.71532 .61428 m
.71532 .61803 L
s
.76141 .61428 m
.76141 .61803 L
s
.8075 .61428 m
.8075 .61803 L
s
.85359 .61428 m
.85359 .61803 L
s
.16226 .61428 m
.16226 .61803 L
s
.11617 .61428 m
.11617 .61803 L
s
.07008 .61428 m
.07008 .61803 L
s
.02399 .61428 m
.02399 .61803 L
s
.94577 .61428 m
.94577 .61803 L
s
.99186 .61428 m
.99186 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -82.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Figure) show
96.938 15.813 moveto
(2) show
102.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .06415 m
1 .06415 L
s
.99375 .15714 m
1 .15714 L
s
.99375 .25012 m
1 .25012 L
s
.99375 .34311 m
1 .34311 L
s
.99375 .4361 m
1 .4361 L
s
.99375 .52909 m
1 .52909 L
s
.125 Mabswid
.99625 .08275 m
1 .08275 L
s
.99625 .10134 m
1 .10134 L
s
.99625 .11994 m
1 .11994 L
s
.99625 .13854 m
1 .13854 L
s
.99625 .17573 m
1 .17573 L
s
.99625 .19433 m
1 .19433 L
s
.99625 .21293 m
1 .21293 L
s
.99625 .23153 m
1 .23153 L
s
.99625 .26872 m
1 .26872 L
s
.99625 .28732 m
1 .28732 L
s
.99625 .30592 m
1 .30592 L
s
.99625 .32451 m
1 .32451 L
s
.99625 .36171 m
1 .36171 L
s
.99625 .38031 m
1 .38031 L
s
.99625 .3989 m
1 .3989 L
s
.99625 .4175 m
1 .4175 L
s
.99625 .4547 m
1 .4547 L
s
.99625 .47329 m
1 .47329 L
s
.99625 .49189 m
1 .49189 L
s
.99625 .51049 m
1 .51049 L
s
.99625 .54768 m
1 .54768 L
s
.99625 .56628 m
1 .56628 L
s
.99625 .58488 m
1 .58488 L
s
.99625 .60348 m
1 .60348 L
s
.99625 .04555 m
1 .04555 L
s
.99625 .02695 m
1 .02695 L
s
.99625 .00836 m
1 .00836 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
gsave
.5 .63053 -77.5938 11.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Im) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
76.063 15.813 moveto
(H) show
79.875 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(e) show
84.563 17.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 8.500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(r) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
88.375 15.813 moveto
(L) show
92.188 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.005 w
.02381 .01472 m
.06244 .06739 L
.10458 .1197 L
.14415 .16446 L
.18221 .20394 L
.22272 .24251 L
.26171 .2766 L
.30316 .3099 L
.34309 .3394 L
.3815 .36562 L
.42237 .3914 L
.46172 .41436 L
.49955 .43486 L
.53984 .45515 L
.57861 .4733 L
.61984 .49124 L
.65954 .50732 L
.69774 .52177 L
.73838 .53613 L
.77751 .54906 L
.81909 .5619 L
.85916 .57347 L
.89771 .58392 L
.93871 .59435 L
.97619 .60332 L
s
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{720, 211.875},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000;@0000db000`400?l00000o`00003oo`3clo?o
0??clm80lo?c003o0??clol0lo?cdP3clo<00?l0lo?co`3clo?B0??cl`00o`3clo?o0??clm80lo?c
003o0??clol0lo?cdP3clo<00?l0lo?co`3clo?B0??cl`00o`3clo?o0??clm80lo?c003o0??clol0
lo?cdP3clo<00</0lo?c00<000000??cl`3clo<02`3clo<00`000000lo?c0??cl`3o0??cleL0lo?c
00<000000??cl`3clo<02`3clo<00`000000lo?c0??cl`2=0??cl`00bP3clo<00`000000lo?c0??c
l`0=0??cl`030000003clo<0lo?c0?l0lo?cE@3clo<00`000000lo?c0??cl`0=0??cl`030000003c
lo<0lo?c08`0lo?c002m0??cl`H000001P3clo<00`000000lo?c0??cl`02000000D0lo?c00<00000
0??cl`3clo<01@3clo<00`000000lo?c0??cl`3o0??cldL0lo?c1P0000060??cl`030000003clo<0
lo?c008000001@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c08/0lo?c002n0??c
l`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c000000020??c
l`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`3o0??c
ldP0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`030??cl`030000003clo<00000
0080lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c
08/0lo?c002n0??cl`030000003clo<0lo?c00P0lo?c00<000000??cl`0000001P3clo<00`000000
lo?c000000060??cl`030000003clo<0lo?c0?l0lo?cB03clo<00`000000lo?c0??cl`080??cl`03
0000003clo<0000000H0lo?c00<000000??cl`0000001P3clo<00`000000lo?c0??cl`2;0??cl`00
_P3clo<010000000lo?c0??cl`0000070??cl`030000003clo<0000000<000000`3clo<00`000000
lo?c000000060??cl`030000003clo<0lo?c0?l0lo?cB03clo<010000000lo?c0??cl`0000070??c
l`030000003clo<0000000<000000`3clo<00`000000lo?c000000060??cl`030000003clo<0lo?c
08/0lo?c002n0??cl`@000001`3clo<00`000000lo?c000000020??cl`070000003clo<0lo?c0??c
l`000000lo?c000000060??cl`030000003clo<0lo?c0?l0lo?cB03clo<4000000L0lo?c00<00000
0??cl`0000000P3clo<01`000000lo?c0??cl`3clo<000000??cl`0000001P3clo<00`000000lo?c
0??cl`2;0??cl`00_P3clo<010000000lo?c0??cl`0000070??cl`030000003clo<0lo?c00800000
0`3clo<01@000000lo?c0??cl`3clo<0000000D0lo?c00<000000??cl`3clo<0o`3clo=80??cl`04
0000003clo<0lo?c000000L0lo?c00<000000??cl`3clo<00P0000030??cl`050000003clo<0lo?c
0??cl`0000001@3clo<00`000000lo?c0??cl`2;0??cl`00_P3clo<00`000000lo?c0??cl`080??c
l`030000003clo<0lo?c00D0lo?c00D000000??cl`3clo<0lo?c000000050??cl`030000003clo<0
lo?c0?l0lo?cB03clo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c00D0lo?c00D00000
0??cl`3clo<0lo?c000000050??cl`030000003clo<0lo?c08/0lo?c002n0??cl`050000003clo<0
lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`040??cl`050000003clo<0lo?c0??cl`000000
103clo<00`000000lo?c0??cl`3o0??cldT0lo?c00D000000??cl`3clo<0lo?c000000070??cl`03
0000003clo<0lo?c00@0lo?c00D000000??cl`3clo<0lo?c000000040??cl`030000003clo<0lo?c
08`0lo?c002m0??cl`H00000203clo<00`000000lo?c0??cl`020??cl`<0000000@0lo?c00000000
000000000P3clo<00`000000lo?c0??cl`3o0??cldT0lo?c1P0000080??cl`030000003clo<0lo?c
0080lo?c0`000000103clo<000000000000000020??cl`030000003clo<0lo?c08d0lo?c003o0??c
lol0lo?cdP3clo<00?l0lo?co`3clo?B0??cl`00o`3clo?o0??clm80lo?c003o0??clol0lo?cdP3c
lo<00?l0lo?co`3clo?B0??cl`00o`3clo?o0??clm80lo?c00280??cl`<000000`3clo<01@000000
lo?c0??cl`3clo<0000002L0lo?c0`0000030??cl`030000003clo<0lo?c00<000009@3clo<30000
00<0lo?c00@000000??cl`0000000000;03clo<300000</0lo?c0`0000030??cl`050000003clo<0
lo?c0??cl`0000009`3clo<3000000<0lo?c00<000000??cl`3clo<00`00000U0??cl`<000000`3c
lo<010000000lo?c00000000000/0??cl`<00000A@3clo<008L0lo?c00D000000??cl`3clo<0lo?c
000000060??cl`030000003clo<0lo?c02@0lo?c00D000000??cl`3clo<0lo?c000000040??cl`05
0000003clo<0lo?c0??cl`0000008`3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c00<00000
0??cl`3clo<0:P3clo<00`000000lo?c0??cl`390??cl`050000003clo<0lo?c0??cl`0000001P3c
lo<00`000000lo?c0??cl`0T0??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c
0??cl`3clo<0000002<0lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000003clo<0lo?c
02X0lo?c00<000000??cl`3clo<0A03clo<008L0lo?c00D000000??cl`3clo<0lo?c000000070??c
l`030000003clo<0lo?c02<0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0
lo?c0??cl`0000008`3clo<01@000000lo?c0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<0
:@3clo<00`000000lo?c0??cl`390??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000
lo?c0??cl`0S0??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<0
000002<0lo?c00D000000??cl`3clo<0lo?c000000070??cl`030000003clo<0lo?c02T0lo?c00<0
00000??cl`3clo<0A03clo<008L0lo?c00D000000??cl`3clo<0lo?c000000070??cl`030000003c
lo<0lo?c02<0lo?c00D000000??cl`3clo<0lo?c000000050??cl`030000003clo<0000002@0lo?c
00D000000??cl`3clo<0lo?c000000050??cl`@00000:P3clo<00`000000lo?c0??cl`390??cl`05
0000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`0S0??cl`050000003clo<0lo?c
0??cl`0000001@3clo<00`000000lo?c0000000T0??cl`050000003clo<0lo?c0??cl`0000001@3c
lo<4000002X0lo?c00<000000??cl`3clo<0A03clo<008L0lo?c00D000000??cl`3clo<0lo?c0000
00070??cl`030000003clo<0lo?c02<0lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000
003clo<0lo?c02<0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??c
l`000000:P3clo<00`000000lo?c0??cl`390??cl`050000003clo<0lo?c0??cl`0000001`3clo<0
0`000000lo?c0??cl`0S0??cl`050000003clo<0lo?c0??cl`0000001P3clo<00`000000lo?c0??c
l`0S0??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<0000002X0
lo?c00<000000??cl`3clo<0A03clo<008L0lo?c00D000000??cl`3clo<0lo?c000000070??cl`03
0000003clo<0lo?c02<0lo?c00D000000??cl`3clo<0lo?c000000050??cl`030000003clo<00000
02@0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`000000:P3c
lo<00`000000lo?c0??cl`390??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c
0??cl`0S0??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0000000T0??cl`05
0000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<0000002X0lo?c00<00000
0??cl`3clo<0A03clo<008L0lo?c00D000000??cl`3clo<0lo?c000000080??cl`030000003clo<0
lo?c0280lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`000000
8`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c0000000Z0??c
l`030000003clo<0lo?c0<T0lo?c00D000000??cl`3clo<0lo?c000000080??cl`030000003clo<0
lo?c0280lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`000000
8`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c0000000Z0??c
l`030000003clo<0lo?c04@0lo?c00270??cl`050000003clo<0lo?c0??cl`000000103clo<01@00
0000lo?c0??cl`3clo<0000002@0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003c
lo<0lo?c0??cl`0000008`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3c
lo<0lo?c0000000Y0??cl`800000b`3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D00000
0??cl`3clo<0lo?c0000000T0??cl`050000003clo<0lo?c0??cl`000000103clo<01@000000lo?c
0??cl`3clo<0000002<0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c
0??cl`000000:@3clo<2000004H0lo?c00280??cl`<000001P3clo<4000002D0lo?c0`0000060??c
l`<000009@3clo<3000000H0lo?c0`00000[0??cl`030000003clo<0lo?c0<X0lo?c0`0000060??c
l`@000009@3clo<3000000H0lo?c0`00000U0??cl`<000001P3clo<3000002/0lo?c00<000000??c
l`3clo<0A03clo<00?l0lo?co`3clo?B0??cl`00o`3clo?o0??clm80lo?c003o0??clol0lo?cdP3c
lo<004`0lo?c0`0000030??cl`<000000`3clo<300000?l0lo?co`3clo=g0??cl`00C@3clo<01@00
0000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`3o
0??clol0lo?cM03clo<004d0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0
000000<0lo?c00<000000??cl`3clo<0103clo?H000008l0lo?cf000000_0??cl`00A03clo<70000
0080lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0000000<0lo?c00<00000
0??cl`3clo<0103clo<2000000<0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`06
0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00L0lo?c0`00002?0??cl`030000003clo<0lo?c0080lo?c00<00000
0??cl`3clo<01`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00L0lo?c00<00000
0??cl`3clo<01`3clo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c00L0lo?c00<00000
0??cl`3clo<01`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00L0lo?c00<00000
0??cl`3clo<01`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00L0lo?c00<00000
0??cl`3clo<01`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00H0lo?c00<00000
0??cl`3clo<01`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00L0lo?c00<00000
0??cl`3clo<01`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00L0lo?c00<00000
0??cl`000000;`3clo<004d0lo?c00@000000??cl`3clo<0lo?c0P0000020??cl`030000003clo<0
000000<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`3D0??cl`030000003clo<0
lo?c08d0lo?c0P00003D0??cl`800000;`3clo<004d0lo?c00H000000??cl`3clo<0lo?c0000003c
lo<200000080lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000003clo<0lo?c0080lo?c
00<000000??cl`3clo<0c`3clo<00`000000lo?c0??cl`2=0??cl`030000003clo<0lo?c0080lo?c
00<0o`000??cl`3clo<0c`3clo<00`000000lo?c0??cl`0]0??cl`00C@3clo<00`000000lo?c0??c
l`020??cl`030000003clo<0lo?c0080lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000
003clo<0lo?c00<0lo?c00<000000??cl`3clo<0cP3clo<00`000000lo?c0??cl`2=0??cl`030000
003clo<0lo?c00<0lo?c00<0o`000??cl`3clo<0cP3clo<00`000000lo?c0??cl`0]0??cl`00C03c
lo<2000000D0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00@0lo?c
00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`3=0??cl`030000003clo<0lo?c08d0lo?c
00<000000??cl`3clo<00`3clo<00`3o0000lo?c0??cl`3>0??cl`030000003clo<0lo?c02d0lo?c
001=0??cl`030000003clo<0lo?c00@0lo?c0P0000020??cl`<000001`3clo<00`000000lo?c0??c
l`040??cl`030000003clo<0lo?c0<d0lo?c00<000000??cl`3clo<0S@3clo<2000000D0lo?c00<0
o`000??cl`3clo<0c03clo<2000002l0lo?c001R0??cl`8000001P3clo<00`000000lo?c0??cl`3;
0??cl`800000S`3clo<00`000000lo?c0??cl`050??cl`030?l0003clo<0lo?c0<`0lo?c00<00000
0??cl`3clo<0;@3clo<00680lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`3;0??c
l`030000003clo<0lo?c08d0lo?c00<000000??cl`3clo<01P3clo<00`3o0000lo?c0??cl`3;0??c
l`030000003clo<0lo?c02d0lo?c001R0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3c
lo<0bP3clo<00`000000lo?c0??cl`2=0??cl`030000003clo<0lo?c00H0lo?c00<0o`000??cl`3c
lo<0b`3clo<00`000000lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c0??cl`080??cl`030000
003clo<0lo?c0<T0lo?c00<000000??cl`3clo<0S@3clo<2000000P0lo?c00<0o`000??cl`3clo<0
b@3clo<2000002l0lo?c001R0??cl`030000003clo<0lo?c00P0lo?c00<000000??cl`3clo<0b@3c
lo<00`000000lo?c0??cl`1l0??cl`D000000P3clo<3000000L0lo?c00<000000??cl`3clo<0203c
lo<00`3o0000lo?c0??cl`390??cl`030000003clo<0lo?c02d0lo?c001R0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<0b03clo<00`000000lo?c0??cl`1m0??cl`060000003clo<0
lo?c0000003clo<000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c00T0lo?c
00<0o`000??cl`3clo<0b03clo<00`000000lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c0??c
l`0:0??cl`030000003clo<0lo?c0<L0lo?c00<000000??cl`3clo<0OP3clo<01@000000lo?c0??c
l`3clo<0000000<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`090??cl`030?l0
003clo<0lo?c0<P0lo?c00<000000??cl`3clo<0;@3clo<00680lo?c0P00000<0??cl`030000003c
lo<0lo?c0<D0lo?c0P00001g0??cl`L000000`3clo<010000000lo?c0??cl`0000030??cl`030000
003clo<0lo?c00@0lo?c0P00000;0??cl`030?l0003clo<0lo?c0<D0lo?c0`00000_0??cl`00HP3c
lo<00`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c0<D0lo?c00<000000??cl`3clo<0P03c
lo<00`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<02`3c
lo<00`3o0000lo?c0??cl`360??cl`030000003clo<0lo?c02d0lo?c001R0??cl`030000003clo<0
lo?c00d0lo?c00<000000??cl`3clo<0a03clo<00`000000lo?c0??cl`200??cl`030000003clo<0
000000<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`0<0??cl`030?l0003clo<0
lo?c0<D0lo?c00<000000??cl`3clo<0;@3clo<00680lo?c00<000000??cl`3clo<03P3clo<00`00
0000lo?c0??cl`330??cl`030000003clo<0lo?c0800lo?c00<000000??cl`0000000`3clo<00`00
0000lo?c0??cl`040??cl`030000003clo<0lo?c00`0lo?c00<0o`000??cl`3clo<0a@3clo<00`00
0000lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c0??cl`0>0??cl`030000003clo<0lo?c0<<0
lo?c00<000000??cl`3clo<0O03clo<01`000000lo?c0??cl`3clo<000000??cl`0000000`3clo<0
0`000000lo?c0??cl`040??cl`8000003P3clo<00`3o0000lo?c0??cl`330??cl`800000;`3clo<0
0680lo?c00<000000??cl`3clo<03`3clo<00`000000lo?c0??cl`320??cl`030000003clo<0lo?c
07d0lo?c0`0000030??cl`<000001`3clo<00`000000lo?c0??cl`0>0??cl`030?l0003clo<0lo?c
0<<0lo?c00<000000??cl`3clo<0;@3clo<00680lo?c0P00000A0??cl`030000003clo<0lo?c0<00
lo?c0P00002?0??cl`030000003clo<0lo?c00l0lo?c00<0o`000??cl`3clo<0`P3clo<00`000000
lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c0??cl`0A0??cl`030000003clo<0lo?c0<00lo?c
00<000000??cl`3clo<0S@3clo<00`000000lo?c0??cl`0@0??cl`030?l0003clo<0lo?c0<40lo?c
00<000000??cl`3clo<0;@3clo<00680lo?c00<000000??cl`3clo<04P3clo<00`000000lo?c0??c
l`2o0??cl`030000003clo<0lo?c08d0lo?c0P00000B0??cl`030?l0003clo<0lo?c0;l0lo?c0P00
000_0??cl`00C03clo<3000000H0lo?c00<000000??cl`3clo<00`0000070??cl`030000003clo<0
lo?c01<0lo?c00<000000??cl`3clo<0_P3clo<00`000000lo?c0??cl`2=0??cl`030000003clo<0
lo?c0140lo?c00<0o`000??cl`3clo<0`03clo<00`000000lo?c0??cl`0]0??cl`00C@3clo<00`00
0000lo?c0??cl`050??cl`030000003clo<0000000<0lo?c00<000000??cl`3clo<0103clo<00`00
0000lo?c0??cl`0D0??cl`030000003clo<0lo?c0;d0lo?c00<000000??cl`3clo<0S@3clo<00`00
0000lo?c0??cl`0B0??cl`030?l0003clo<0lo?c0;l0lo?c00<000000??cl`3clo<0;@3clo<004d0
lo?c00@000000??cl`3clo<0lo?c1`0000030??cl`030000003clo<0lo?c00@0lo?c00<000000??c
l`3clo<05@3clo<00`000000lo?c0??cl`2l0??cl`030000003clo<0lo?c08d0lo?c0P00000D0??c
l`030?l0003clo<0lo?c0;d0lo?c0P00000_0??cl`00A03clo<700000080lo?c00D000000??cl`3c
lo<0lo?c000000030??cl`030000003clo<0000000<0lo?c00<000000??cl`3clo<0103clo<20000
01L0lo?c00<000000??cl`3clo<0^@3clo<3000008l0lo?c00<000000??cl`3clo<0503clo<00`3o
0000lo?c0??cl`2m0??cl`030000003clo<0lo?c02d0lo?c001=0??cl`030000003clo<0lo?c0080
lo?c00H000000??cl`3clo<000000??cl`0000030??cl`030000003clo<0lo?c00@0lo?c00<00000
0??cl`3clo<05`3clo<00`000000lo?c0??cl`2j0??cl`030000003clo<0lo?c08d0lo?c00<00000
0??cl`3clo<05@3clo<00`3o0000lo?c0??cl`2l0??cl`030000003clo<0lo?c02d0lo?c001=0??c
l`030000003clo<0lo?c00<0lo?c00D000000??cl`000000lo?c000000030??cl`030000003clo<0
lo?c00@0lo?c00<000000??cl`3clo<0603clo<00`000000lo?c0??cl`2i0??cl`030000003clo<0
lo?c08d0lo?c00<000000??cl`3clo<05P3clo<00`3o0000lo?c0??cl`2k0??cl`030000003clo<0
lo?c02d0lo?c001=0??cl`030000003clo<0lo?c00<0lo?c00D000000??cl`000000lo?c00000003
0??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<06@3clo<00`000000lo?c0??cl`2h
0??cl`030000003clo<0lo?c08d0lo?c0P00000H0??cl`030?l0003clo<0lo?c0;T0lo?c0P00000_
0??cl`00C03clo<2000000H0lo?c0P0000000`3clo<000000??cl`020??cl`030000003clo<0lo?c
00@0lo?c00<000000??cl`3clo<06P3clo<00`000000lo?c0??cl`2g0??cl`030000003clo<0lo?c
08d0lo?c00<000000??cl`3clo<05`3clo<00`3o0000lo?c0??cl`2j0??cl`030000003clo<0lo?c
02d0lo?c001=0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<00`0000070??cl`03
0000003clo<0lo?c01/0lo?c00<000000??cl`3clo<0]P3clo<00`000000lo?c0??cl`1d0??cl`<0
0000103clo<00`000000lo?c0??cl`020??cl`030000003clo<0000000800000203clo<00`000000
lo?c0??cl`0H0??cl`030?l0003clo<0lo?c0;T0lo?c00<000000??cl`3clo<0;@3clo<00680lo?c
00<000000??cl`3clo<0703clo<00`000000lo?c0??cl`2e0??cl`030000003clo<0lo?c07D0lo?c
00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00D0lo?c
00<000000??cl`3clo<06@3clo<00`3o0000lo?c0??cl`2h0??cl`030000003clo<0lo?c02d0lo?c
001R0??cl`8000007P3clo<00`000000lo?c0??cl`2c0??cl`800000M`3clo<00`000000lo?c0??c
l`040??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0103clo<2000001/0lo?c00<0
o`000??cl`3clo<0]@3clo<3000002l0lo?c001R0??cl`030000003clo<0lo?c01h0lo?c00<00000
0??cl`3clo<0/`3clo<00`000000lo?c0??cl`1/0??cl`L000000P3clo<00`000000lo?c0??cl`04
0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`0K
0??cl`030?l0003clo<0lo?c0;H0lo?c00<000000??cl`3clo<0;@3clo<00680lo?c00<000000??c
l`3clo<07`3clo<00`000000lo?c0??cl`2b0??cl`030000003clo<0lo?c07D0lo?c00<000000??c
l`3clo<0103clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c00@0lo?c00<000000??c
l`3clo<0703clo<00`3o0000lo?c0??cl`2e0??cl`030000003clo<0lo?c02d0lo?c001R0??cl`03
0000003clo<0lo?c0200lo?c00<000000??cl`3clo<0/@3clo<00`000000lo?c0??cl`1e0??cl`03
0000003clo<0lo?c00@0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`050??cl`03
0000003clo<0lo?c01d0lo?c00<0o`000??cl`3clo<0]03clo<00`000000lo?c0??cl`0]0??cl`00
HP3clo<00`000000lo?c0??cl`0Q0??cl`030000003clo<0lo?c0;00lo?c00<000000??cl`3clo<0
M@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c00<0lo?c0P0000080??cl`800000
7`3clo<00`3o0000lo?c0??cl`2b0??cl`800000;`3clo<00680lo?c00<000000??cl`3clo<08P3c
lo<00`000000lo?c0??cl`2_0??cl`030000003clo<0lo?c07@0lo?c0P0000030??cl`050000003c
lo<0lo?c0??cl`0000001@3clo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c01l0lo?c
00<0o`000??cl`3clo<0/P3clo<00`000000lo?c0??cl`0]0??cl`00<`3clo<00`000000lo?c0??c
l`050??cl`030000003clo<0lo?c02@0lo?c0P00000T0??cl`030000003clo<0lo?c0:d0lo?c0P00
001M0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<03`3clo<00`000000lo?c0??c
l`020??cl`@000001P3clo<3000000H0lo?c00<000000??cl`3clo<0803clo<00`3o0000lo?c0??c
l`2a0??cl`030000003clo<0lo?c02d0lo?c000c0??cl`T000009P3clo<00`000000lo?c0??cl`0T
0??cl`030000003clo<0lo?c0:d0lo?c00<000000??cl`3clo<0F`3clo<9000002T0lo?c00<00000
0??cl`3clo<0803clo<00`3o0000lo?c0??cl`2a0??cl`030000003clo<0lo?c02d0lo?c000c0??c
l`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`0T0??cl`030000003clo<0
lo?c02D0lo?c00<000000??cl`3clo<0[03clo<00`000000lo?c0??cl`1K0??cl`050000003clo<0
lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`0W0??cl`8000008P3clo<00`3o0000lo?c0??c
l`2_0??cl`800000;`3clo<003<0lo?c00D000000??cl`3clo<0lo?c0000000Z0??cl`030000003c
lo<0lo?c02H0lo?c0P00002/0??cl`030000003clo<0lo?c05/0lo?c00D000000??cl`3clo<0lo?c
0000000]0??cl`030000003clo<0lo?c0280lo?c00<0o`000??cl`3clo<0[`3clo<00`000000lo?c
0??cl`0]0??cl`00<`3clo<01@000000lo?c0??cl`3clo<0000002X0lo?c00<000000??cl`3clo<0
:03clo<00`000000lo?c0??cl`2Y0??cl`030000003clo<0lo?c05/0lo?c00D000000??cl`3clo<0
lo?c0000000]0??cl`030000003clo<0lo?c02<0lo?c00<0o`000??cl`3clo<0[P3clo<00`000000
lo?c0??cl`0]0??cl`00=03clo<3000002/0lo?c00<000000??cl`3clo<0:@3clo<00`000000lo?c
0??cl`2X0??cl`030000003clo<0lo?c05`0lo?c0`00000^0??cl`030000003clo<0lo?c02@0lo?c
00<0o`000??cl`3clo<0[@3clo<00`000000lo?c0??cl`0]0??cl`00HP3clo<2000002/0lo?c0P00
002W0??cl`800000S`3clo<2000002H0lo?c00<0o`000??cl`3clo<0Z`3clo<2000002l0lo?c000g
0??cl`@000009`3clo<00`000000lo?c0??cl`0/0??cl`030000003clo<0lo?c0:D0lo?c00<00000
0??cl`3clo<0G`3clo<4000002X0lo?c00<000000??cl`3clo<09P3clo<00`3o0000lo?c0??cl`2[
0??cl`030000003clo<0lo?c02d0lo?c000f0??cl`030000003clo<000000080lo?c00<000000??c
l`3clo<0903clo<00`000000lo?c0??cl`0]0??cl`030000003clo<0lo?c0:@0lo?c00<000000??c
l`3clo<0GP3clo<00`000000lo?c000000020??cl`030000003clo<0lo?c02L0lo?c00<000000??c
l`3clo<09`3clo<00`3o0000lo?c0??cl`2Z0??cl`030000003clo<0lo?c02d0lo?c000f0??cl`03
0000003clo<000000080lo?c00<000000??cl`3clo<0903clo<00`000000lo?c0??cl`0^0??cl`03
0000003clo<0lo?c0:<0lo?c00<000000??cl`3clo<0GP3clo<00`000000lo?c000000020??cl`03
0000003clo<0lo?c02L0lo?c00<000000??cl`3clo<0:03clo<20?l00:X0lo?c00<000000??cl`3c
lo<0;@3clo<003L0lo?c0P0000000`3clo<000000??cl`0@0??cl`<000000P3clo<500000080lo?c
0`0000070??cl`030000003clo<0lo?c02l0lo?c00<000000??cl`3clo<0XP3clo<00`000000lo?c
0??cl`1O0??cl`80000000<0lo?c0000003clo<0:@3clo<2000002/0lo?c00<0o`000??cl`3clo<0
YP3clo<2000002l0lo?c001=0??cl`030000003clo<0lo?c0080lo?c00H000000??cl`3clo<00000
0??cl`0000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0<03clo<00`000000
lo?c0??cl`2Q0??cl`030000003clo<0lo?c07d0lo?c0`0000020??cl`<00000203clo<00`000000
lo?c0??cl`0[0??cl`030?l0003clo<0lo?c0:H0lo?c00<000000??cl`3clo<0;@3clo<003H0lo?c
00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0?0??cl`030000003clo<0lo?c00<0lo?c
00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3c
lo<0<@3clo<200000:40lo?c00<000000??cl`3clo<0GP3clo<00`000000lo?c0??cl`020??cl`03
0000003clo<0lo?c01P0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`050??cl`03
0000003clo<0lo?c02`0lo?c00<0o`000??cl`3clo<0Y@3clo<00`000000lo?c0??cl`0]0??cl`00
=P3clo<6000000P0lo?c1`0000020??cl`030000003clo<0lo?c00@0lo?c00@000000??cl`3clo<0
00000`3clo<00`000000lo?c0??cl`040??cl`800000=03clo<00`000000lo?c0??cl`2L0??cl`<0
0000H03clo<6000001X0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`040??cl`03
0000003clo<0lo?c02d0lo?c00<0o`000??cl`3clo<0Y03clo<00`000000lo?c0??cl`0]0??cl`00
=P3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c00l0lo?c00<000000??cl`3clo<0
1@3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0
=03clo<00`000000lo?c0??cl`2M0??cl`030000003clo<0lo?c05h0lo?c00<000000??cl`3clo<0
0P3clo<00`000000lo?c0??cl`0?0??cl`L000000P3clo<00`000000lo?c0??cl`050??cl`030000
003clo<0lo?c00@0lo?c0P00000_0??cl`030?l0003clo<0lo?c0:40lo?c0`00000_0??cl`00=P3c
lo<00`000000lo?c0??cl`0D0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`0000000`3c
lo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c03D0lo?c00<000000??cl`3clo<0W03c
lo<00`000000lo?c0??cl`1N0??cl`030000003clo<0lo?c01d0lo?c00<000000??cl`3clo<01@3c
lo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c02l0lo?c00<0o`000??cl`3clo<0XP3c
lo<00`000000lo?c0??cl`0]0??cl`00=P3clo<00`000000lo?c0??cl`020??cl`030000003clo<0
lo?c00l0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c000000030??cl`030000003clo<0
lo?c00@0lo?c00<000000??cl`3clo<0=P3clo<00`000000lo?c0??cl`2K0??cl`030000003clo<0
lo?c05h0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0H0??cl`030000003clo<0
lo?c00@0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0`0??cl`030?l0003clo<0
lo?c0:40lo?c00<000000??cl`3clo<0;@3clo<003H0lo?c1P00000@0??cl`8000000`3clo<01`00
0000lo?c0??cl`3clo<000000??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003c
lo<0lo?c03L0lo?c0P00002K0??cl`030000003clo<0lo?c05h0lo?c1P00000J0??cl`030000003c
lo<0lo?c0080lo?c0P0000080??cl`030000003clo<0lo?c0340lo?c0P3o002Q0??cl`030000003c
lo<0lo?c02d0lo?c000f0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<03`3clo<0
0`000000lo?c0??cl`020??cl`<000000`3clo<3000000L0lo?c00<000000??cl`3clo<0>@3clo<0
0`000000lo?c0??cl`2H0??cl`030000003clo<0lo?c05h0lo?c00<000000??cl`3clo<00P3clo<0
0`000000lo?c0??cl`0G0??cl`800000103clo<00`000000lo?c0??cl`070??cl`800000=03clo<0
0`3o0000lo?c0??cl`2M0??cl`800000;`3clo<003H0lo?c00<000000??cl`3clo<0:@3clo<20000
03/0lo?c00<000000??cl`3clo<0UP3clo<200000600lo?c00<000000??cl`3clo<07@3clo<00`00
0000lo?c0??cl`030??cl`<000001P3clo<00`000000lo?c0??cl`0d0??cl`030?l0003clo<0lo?c
09d0lo?c00<000000??cl`3clo<0;@3clo<003L0lo?c1@00000V0??cl`030000003clo<0lo?c03/0
lo?c0P00002G0??cl`030000003clo<0lo?c05l0lo?c1@00000Y0??cl`030000003clo<0lo?c03D0
lo?c00<0o`000??cl`3clo<0W03clo<00`000000lo?c0??cl`0]0??cl`00=P3clo<00`000000lo?c
0??cl`020??cl`030000003clo<0lo?c02@0lo?c00<000000??cl`3clo<0?@3clo<00`000000lo?c
0??cl`2D0??cl`030000003clo<0lo?c05h0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c
0??cl`0W0??cl`030000003clo<0lo?c03H0lo?c00<0o`000??cl`3clo<0V`3clo<00`000000lo?c
0??cl`0]0??cl`00=P3clo<00`000000lo?c0??cl`0Y0??cl`030000003clo<0lo?c03h0lo?c00<0
00000??cl`3clo<0T`3clo<00`000000lo?c0??cl`1N0??cl`030000003clo<0lo?c02`0lo?c0P00
000h0??cl`80o`00VP3clo<2000002l0lo?c000g0??cl`D000009P3clo<00`000000lo?c0??cl`0o
0??cl`800000T`3clo<00`000000lo?c0??cl`1O0??cl`D00000:@3clo<00`000000lo?c0??cl`0i
0??cl`030?l0003clo<0lo?c09P0lo?c00<000000??cl`3clo<0;@3clo<003/0lo?c00<000000??c
l`3clo<0903clo<00`000000lo?c0??cl`110??cl`030000003clo<0lo?c0900lo?c00<000000??c
l`3clo<0H`3clo<00`000000lo?c0??cl`0W0??cl`030000003clo<0lo?c03X0lo?c00<0o`000??c
l`3clo<0U`3clo<00`000000lo?c0??cl`0]0??cl`00=P3clo<00`000000lo?c0??cl`020??cl`03
0000003clo<0lo?c02@0lo?c0P0000130??cl`800000S`3clo<200000600lo?c00<000000??cl`3c
lo<00P3clo<00`000000lo?c0??cl`0W0??cl`030000003clo<0lo?c03/0lo?c0P3o002G0??cl`03
0000003clo<0lo?c02d0lo?c000c0??cl`030000003clo<0lo?c00H000009P3clo<00`000000lo?c
0??cl`140??cl`030000003clo<0lo?c08d0lo?c00<000000??cl`3clo<0F`3clo<00`000000lo?c
0??cl`06000002T0lo?c0P00000n0??cl`030?l0003clo<0lo?c09<0lo?c0P00000_0??cl`00>`3c
lo<00`000000lo?c0??cl`0T0??cl`030000003clo<0lo?c04D0lo?c0P00002=0??cl`030000003c
lo<0lo?c06<0lo?c00<000000??cl`3clo<09`3clo<00`000000lo?c0??cl`0n0??cl`030?l0003c
lo<0lo?c09<0lo?c00<000000??cl`3clo<0;@3clo<003H0lo?c00<000000??cl`3clo<0:@3clo<0
0`000000lo?c0??cl`170??cl`030000003clo<0lo?c08X0lo?c00<000000??cl`3clo<0GP3clo<0
0`000000lo?c0??cl`0/0??cl`030000003clo<0lo?c03l0lo?c00<0o`000??cl`3clo<0TP3clo<0
0`000000lo?c0??cl`0]0??cl`00=03clo<8000002H0lo?c00<000000??cl`3clo<0B03clo<00`00
0000lo?c0??cl`290??cl`030000003clo<0lo?c05`0lo?c2000000Y0??cl`030000003clo<0lo?c
0400lo?c00<0o`000??cl`3clo<0T@3clo<00`000000lo?c0??cl`0]0??cl`00=P3clo<00`000000
lo?c0??cl`020??cl`030000003clo<0lo?c02@0lo?c00<000000??cl`3clo<0B@3clo<2000008T0
lo?c00<000000??cl`3clo<0GP3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c02L0
lo?c0P0000120??cl`030?l0003clo<0lo?c08l0lo?c0P00000_0??cl`00=P3clo<00`000000lo?c
0??cl`0Y0??cl`030000003clo<0lo?c04/0lo?c00<000000??cl`3clo<0QP3clo<00`000000lo?c
0??cl`1N0??cl`030000003clo<0lo?c01<0lo?c0`0000020??cl`D000000P3clo<00`000000lo?c
00000002000000P0lo?c00<000000??cl`3clo<0@P3clo<20?l00900lo?c00<000000??cl`3clo<0
;@3clo<003@0lo?c2000000V0??cl`800000C@3clo<00`000000lo?c0??cl`240??cl`800000GP3c
lo<800000140lo?c00<000000??cl`3clo<00P3clo<010000000lo?c0??cl`0000070??cl`030000
003clo<0lo?c00D0lo?c00<000000??cl`3clo<0A03clo<00`3o0000lo?c0??cl`2=0??cl`030000
003clo<0lo?c02d0lo?c000f0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0903c
lo<00`000000lo?c0??cl`1=0??cl`800000Q@3clo<00`000000lo?c0??cl`1N0??cl`030000003c
lo<0lo?c0080lo?c00<000000??cl`3clo<03`3clo<00`000000lo?c0??cl`030??cl`030000003c
lo<0lo?c00P0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`150??cl`030?l0003c
lo<0lo?c08`0lo?c00<000000??cl`3clo<0;@3clo<003H0lo?c00<000000??cl`3clo<00P3clo<0
0`000000lo?c0??cl`0T0??cl`030000003clo<0lo?c04l0lo?c00<000000??cl`3clo<0PP3clo<0
0`000000lo?c0??cl`1N0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<01P3clo<7
00000080lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`070??cl`030000003clo<0
lo?c00@0lo?c0P0000170??cl`030?l0003clo<0lo?c08T0lo?c0`00000_0??cl`00<`3clo<00`00
0000lo?c0??cl`0600000100lo?c0`0000030??cl`<000000`3clo<3000000L0lo?c00<000000??c
l`3clo<0D03clo<200000880lo?c00<000000??cl`3clo<0F`3clo<00`000000lo?c0??cl`060000
0140lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c
00@0lo?c00<000000??cl`3clo<0A`3clo<20?l008/0lo?c00<000000??cl`3clo<0;@3clo<003H0
lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0?0??cl`050000003clo<0lo?c0??c
l`0000000`3clo<00`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??c
l`3clo<0DP3clo<00`000000lo?c0??cl`1o0??cl`030000003clo<0lo?c05h0lo?c00<000000??c
l`3clo<00P3clo<00`000000lo?c0??cl`0?0??cl`030000003clo<0lo?c00D0lo?c00<000000??c
l`3clo<01@3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c04T0lo?c0P3o00290??c
l`030000003clo<0lo?c02d0lo?c000f0??cl`8000005@3clo<01@000000lo?c0??cl`3clo<00000
00<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c
05<0lo?c0P00001o0??cl`030000003clo<0lo?c05h0lo?c0P00000E0??cl`030000003clo<0lo?c
00D0lo?c00<000000??cl`3clo<00`3clo<2000000P0lo?c00<000000??cl`3clo<0B`3clo<20?l0
08L0lo?c00<000000??cl`3clo<0;@3clo<003H0lo?c00@000000??cl`00000000002P3clo<70000
0080lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0000000<0lo?c00<00000
0??cl`3clo<0103clo<2000005H0lo?c00<000000??cl`3clo<0NP3clo<300000600lo?c00@00000
0??cl`00000000004P3clo<2000000<0lo?c00D000000??cl`3clo<0lo?c000000050??cl`030000
003clo<0lo?c00L0lo?c0P00001>0??cl`80o`00Q03clo<2000002l0lo?c000j0??cl`8000004@3c
lo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`0000000`3clo<00`000000lo?c
0??cl`040??cl`030000003clo<0lo?c05H0lo?c0P00001l0??cl`030000003clo<0lo?c0680lo?c
0P00000A0??cl`030000003clo<0lo?c0080lo?c0`0000070??cl`<000001P3clo<00`000000lo?c
0??cl`1?0??cl`030?l0003clo<0lo?c0880lo?c00<000000??cl`3clo<0;@3clo<003P0lo?c0P00
000C0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c000000030??cl`030000
003clo<0lo?c00@0lo?c00<000000??cl`3clo<0F03clo<00`000000lo?c0??cl`1i0??cl`030000
003clo<0lo?c0600lo?c0P00000[0??cl`030000003clo<0lo?c0500lo?c0P3o00220??cl`030000
003clo<0lo?c02d0lo?c000f0??cl`8000005@3clo<01@000000lo?c0??cl`3clo<0000000<0lo?c
00<000000??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c05T0lo?c
0P00001i0??cl`030000003clo<0lo?c05h0lo?c0P00000]0??cl`030000003clo<0lo?c0580lo?c
00<0o`000??cl`3clo<0O`3clo<00`000000lo?c0??cl`0]0??cl`00=P3clo<00`000000lo?c0??c
l`0C0??cl`8000000`3clo<01`000000lo?c0??cl`3clo<000000??cl`0000000`3clo<00`000000
lo?c0??cl`040??cl`030000003clo<0lo?c05/0lo?c00<000000??cl`3clo<0MP3clo<00`000000
lo?c0??cl`1N0??cl`030000003clo<0lo?c02`0lo?c0P00001D0??cl`80o`00OP3clo<2000002l0
lo?c000f0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<03`3clo<00`000000lo?c
0??cl`020??cl`<000000`3clo<3000000L0lo?c00<000000??cl`3clo<0G03clo<2000007H0lo?c
00<000000??cl`3clo<0GP3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c02L0lo?c
00<000000??cl`3clo<0E@3clo<00`3o0000lo?c0??cl`1l0??cl`030000003clo<0lo?c02d0lo?c
000c0??cl`030000003clo<0lo?c00H000009P3clo<2000005l0lo?c0P00001c0??cl`800000G@3c
lo<00`000000lo?c0??cl`06000002T0lo?c00<000000??cl`3clo<0EP3clo<20?l007`0lo?c00<0
00000??cl`3clo<0;@3clo<003/0lo?c00<000000??cl`3clo<0903clo<00`000000lo?c0??cl`1P
0??cl`030000003clo<0lo?c0740lo?c00<000000??cl`3clo<0H`3clo<00`000000lo?c0??cl`0W
0??cl`030000003clo<0lo?c05P0lo?c00<0o`000??cl`3clo<0N@3clo<00`000000lo?c0??cl`0]
0??cl`00=P3clo<00`000000lo?c0??cl`0Y0??cl`030000003clo<0lo?c0640lo?c0P00001a0??c
l`030000003clo<0lo?c05h0lo?c00<000000??cl`3clo<0;03clo<2000005X0lo?c0P3o001h0??c
l`800000;`3clo<003@0lo?c2000000V0??cl`030000003clo<0lo?c06<0lo?c0P00001_0??cl`03
0000003clo<0lo?c05`0lo?c2000000Y0??cl`030000003clo<0lo?c05/0lo?c00<0o`000??cl`3c
lo<0MP3clo<00`000000lo?c0??cl`0]0??cl`00=P3clo<00`000000lo?c0??cl`020??cl`030000
003clo<0lo?c02@0lo?c00<000000??cl`3clo<0I@3clo<2000006d0lo?c00<000000??cl`3clo<0
GP3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c02L0lo?c00<000000??cl`3clo<0
G03clo<20?l007H0lo?c00<000000??cl`3clo<0;@3clo<003H0lo?c00<000000??cl`3clo<01@3c
lo<00`000000lo?c0??cl`0Q0??cl`030000003clo<0lo?c06L0lo?c0P00001[0??cl`030000003c
lo<0lo?c05h0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`0T0??cl`030000003c
lo<0lo?c05h0lo?c0P3o001d0??cl`030000003clo<0lo?c02d0lo?c000f0??cl`<000001@3clo<0
0`000000lo?c0??cl`0Q0??cl`030000003clo<0lo?c06T0lo?c0P00001Y0??cl`030000003clo<0
lo?c05h0lo?c0`0000050??cl`030000003clo<0lo?c02@0lo?c0P00001Q0??cl`030?l0003clo<0
lo?c0700lo?c0P00000_0??cl`00=P3clo<00`000000lo?c0??cl`0200000080lo?c00<000000??c
l`3clo<08P3clo<2000006`0lo?c0P00001V0??cl`800000H03clo<00`000000lo?c0??cl`020000
0080lo?c00<000000??cl`3clo<05@3clo<3000000<0lo?c0`0000070??cl`030000003clo<0lo?c
0640lo?c0P3o001a0??cl`030000003clo<0lo?c02d0lo?c000k0??cl`8000009@3clo<00`000000
lo?c0??cl`1]0??cl`800000I@3clo<00`000000lo?c0??cl`1S0??cl`8000006@3clo<01@000000
lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??cl`1S0??c
l`80o`00K`3clo<00`000000lo?c0??cl`0]0??cl`00=P3clo<00`000000lo?c00000002000002L0
lo?c00<000000??cl`3clo<0K`3clo<2000006<0lo?c00<000000??cl`3clo<0GP3clo<00`000000
lo?c00000002000001/0lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c
00@0lo?c00<000000??cl`3clo<0I@3clo<20?l006d0lo?c00<000000??cl`3clo<0;@3clo<003H0
lo?c0P00000Z0??cl`030000003clo<0lo?c0740lo?c00<000000??cl`3clo<0H03clo<00`000000
lo?c0??cl`1N0??cl`8000005@3clo<700000080lo?c00D000000??cl`3clo<0lo?c000000030??c
l`030000003clo<0lo?c00@0lo?c0P00001X0??cl`<0o`00J03clo<3000002l0lo?c000f0??cl`03
0000003clo<0lo?c02T0lo?c00<000000??cl`3clo<0LP3clo<200000600lo?c00<000000??cl`3c
lo<0GP3clo<00`000000lo?c0??cl`0M0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`00
0000lo?c0??cl`040??cl`030000003clo<0lo?c06X0lo?c0P3o001X0??cl`030000003clo<0lo?c
02d0lo?c001R0??cl`030000003clo<0lo?c07@0lo?c0P00001N0??cl`030000003clo<0lo?c07h0
lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??c
l`3clo<0K03clo<20?l006H0lo?c00<000000??cl`3clo<0;@3clo<00680lo?c0P00001g0??cl`80
0000F`3clo<200000800lo?c00D000000??cl`3clo<0lo?c000000030??cl`030000003clo<0lo?c
00@0lo?c00<000000??cl`3clo<0KP3clo<20?l006@0lo?c00<000000??cl`3clo<0;@3clo<00680
lo?c00<000000??cl`3clo<0N03clo<3000005T0lo?c00<000000??cl`3clo<0O@3clo<2000000<0
lo?c00D000000??cl`3clo<0lo?c000000060??cl`800000L@3clo<20?l00640lo?c0P00000_0??c
l`00HP3clo<00`000000lo?c0??cl`1k0??cl`800000E`3clo<00`000000lo?c0??cl`1n0??cl`03
0000003clo<0lo?c0080lo?c0`0000070??cl`030000003clo<0lo?c0780lo?c0P3o001P0??cl`03
0000003clo<0lo?c02d0lo?c001B0??cl`<000000`3clo<3000000L0lo?c00<000000??cl`3clo<0
O@3clo<2000005D0lo?c00<000000??cl`3clo<0S@3clo<00`000000lo?c0??cl`1d0??cl`80o`00
GP3clo<00`000000lo?c0??cl`0]0??cl`00D@3clo<01`000000lo?c0??cl`3clo<000000??cl`00
00000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c07l0lo?c0P00001C0??cl`03
0000003clo<0lo?c08d0lo?c00<000000??cl`3clo<0MP3clo<20?l005`0lo?c00<000000??cl`3c
lo<0;@3clo<00540lo?c00L000000??cl`3clo<0lo?c0000003clo<0000000<0lo?c00<000000??c
l`3clo<0103clo<00`000000lo?c0??cl`210??cl`800000D@3clo<00`000000lo?c0??cl`2=0??c
l`800000N@3clo<30?l005P0lo?c0P00000_0??cl`00BP3clo<7000000040??cl`000000lo?c0000
0080lo?c00D000000??cl`3clo<0lo?c000000060??cl`800000Q03clo<2000004d0lo?c0`00002?
0??cl`030000003clo<0lo?c07/0lo?c0P3o001G0??cl`030000003clo<0lo?c02d0lo?c001C0??c
l`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0
lo?c08D0lo?c0P00001=0??cl`030000003clo<0lo?c08d0lo?c00<000000??cl`3clo<0O@3clo<2
0?l005D0lo?c00<000000??cl`3clo<0;@3clo<00580lo?c00<000000??cl`0000000P3clo<01@00
0000lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`3clo<0Q`3clo<2000004/0lo?c00<00000
0??cl`3clo<0S@3clo<00`000000lo?c0??cl`1o0??cl`80o`00D`3clo<00`000000lo?c0??cl`0]
0??cl`00D@3clo<01`000000lo?c0??cl`3clo<000000??cl`0000000`3clo<00`000000lo?c0??c
l`040??cl`030000003clo<0lo?c08T0lo?c0`0000180??cl`030000003clo<0lo?c08d0lo?c0P00
00220??cl`80o`00D03clo<2000002l0lo?c001A0??cl`070000003clo<0lo?c0??cl`000000lo?c
000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0S03clo<2000004H0lo?c
00<000000??cl`3clo<0S@3clo<00`000000lo?c0??cl`230??cl`80o`00C`3clo<00`000000lo?c
0??cl`0]0??cl`00DP3clo<3000000<0lo?c0`0000070??cl`030000003clo<0lo?c08h0lo?c0P00
00140??cl`030000003clo<0lo?c08d0lo?c00<000000??cl`3clo<0Q@3clo<20?l004d0lo?c00<0
00000??cl`3clo<0;@3clo<00680lo?c00<000000??cl`3clo<0T03clo<200000480lo?c00<00000
0??cl`3clo<0S@3clo<00`000000lo?c0??cl`270??cl`80o`00B`3clo<00`000000lo?c0??cl`0]
0??cl`00HP3clo<2000009<0lo?c0`00000n0??cl`800000S`3clo<2000008X0lo?c0`3o00170??c
l`800000;`3clo<00680lo?c00<000000??cl`3clo<0U@3clo<2000003d0lo?c00<000000??cl`3c
lo<0N`3clo<00`000000lo?c0??cl`020??cl`030000003clo<0000000800000203clo<00`000000
lo?c0??cl`2<0??cl`<0o`00A@3clo<00`000000lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c
0??cl`2G0??cl`<00000>P3clo<00`000000lo?c0??cl`1k0??cl`030000003clo<0lo?c00L0lo?c
00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`2?0??cl`<0o`00@P3clo<00`000000lo?c
0??cl`0]0??cl`00HP3clo<00`000000lo?c0??cl`2J0??cl`<00000=`3clo<00`000000lo?c0??c
l`1l0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c0??c
l`2B0??cl`<0o`00?`3clo<00`000000lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c0??cl`2M
0??cl`<00000=03clo<00`000000lo?c0??cl`1b0??cl`L000000`3clo<00`000000lo?c0??cl`07
0??cl`030000003clo<0lo?c00@0lo?c0P00002F0??cl`80o`00>`3clo<3000002l0lo?c001R0??c
l`030000003clo<0lo?c0:00lo?c0`00000a0??cl`030000003clo<0lo?c07`0lo?c00<000000??c
l`3clo<01`3clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c09L0lo?c0`3o000j0??c
l`030000003clo<0lo?c02d0lo?c001R0??cl`800000Y03clo<3000002d0lo?c0P00001n0??cl`03
0000003clo<0lo?c00H0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`2J0??cl`<0
o`00=`3clo<00`000000lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c0??cl`2V0??cl`800000
;03clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c00<0lo?c0P0000080??cl`030000
003clo<0lo?c09d0lo?c0`3o000d0??cl`030000003clo<0lo?c02d0lo?c001R0??cl`030000003c
lo<0lo?c0:P0lo?c0P00000Z0??cl`030000003clo<0lo?c07T0lo?c00D000000??cl`3clo<0lo?c
000000050??cl`030000003clo<0lo?c00L0lo?c0P00002Q0??cl`<0o`00<03clo<2000002l0lo?c
001R0??cl`030000003clo<0lo?c0:X0lo?c0P00000X0??cl`030000003clo<0lo?c07X0lo?c1000
00060??cl`<000001P3clo<00`000000lo?c0??cl`2S0??cl`@0o`00;@3clo<00`000000lo?c0??c
l`0]0??cl`00HP3clo<00`000000lo?c0??cl`2/0??cl`<000009@3clo<00`000000lo?c0??cl`2=
0??cl`030000003clo<0lo?c0:L0lo?c103o000Y0??cl`030000003clo<0lo?c02d0lo?c001R0??c
l`030000003clo<0lo?c0:l0lo?c0P00000S0??cl`030000003clo<0lo?c08d0lo?c00<000000??c
l`3clo<0Z`3clo<40?l002D0lo?c00<000000??cl`3clo<0;@3clo<00680lo?c0P00002b0??cl`<0
00007`3clo<2000008l0lo?c0P00002`0??cl`80o`008P3clo<2000002l0lo?c001R0??cl`030000
003clo<0lo?c0;@0lo?c1000000L0??cl`030000003clo<0lo?c08d0lo?c00<000000??cl`3clo<0
/@3clo<30?l00200lo?c00<000000??cl`3clo<0;@3clo<00680lo?c00<000000??cl`3clo<0^03c
lo<4000001P0lo?c00<000000??cl`3clo<0S@3clo<00`000000lo?c0??cl`2d0??cl`@0o`00703c
lo<00`000000lo?c0??cl`0]0??cl`00HP3clo<00`000000lo?c0??cl`2l0??cl`@00000503clo<0
0`000000lo?c0??cl`2=0??cl`030000003clo<0lo?c0;P0lo?c103o000H0??cl`030000003clo<0
lo?c02d0lo?c001B0??cl`<000000`3clo<3000000L0lo?c00<000000??cl`3clo<0`03clo<20000
0180lo?c00<000000??cl`3clo<0S@3clo<200000;d0lo?c103o000C0??cl`800000;`3clo<00540
lo?c00L000000??cl`3clo<0lo?c0000003clo<0000000<0lo?c00<000000??cl`3clo<0103clo<0
0`000000lo?c0??cl`320??cl`<000003`3clo<00`000000lo?c0??cl`2=0??cl`030000003clo<0
lo?c0<00lo?c103o000@0??cl`030000003clo<0lo?c02d0lo?c001A0??cl`070000003clo<0lo?c
0??cl`000000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0a@3c
lo<3000000`0lo?c00<000000??cl`3clo<0S@3clo<00`000000lo?c0??cl`340??cl`@0o`00303c
lo<00`000000lo?c0??cl`0]0??cl`00BP3clo<8000000<0lo?c00<000000??cl`0000000`3clo<0
0`000000lo?c0??cl`040??cl`800000b@3clo<4000000H0lo?c0`00002?0??cl`030000003clo<0
lo?c0<P0lo?c103o00080??cl`030000003clo<0lo?c02d0lo?c001A0??cl`8000000P3clo<00`00
0000lo?c000000030??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0c03clo<30000
00D0lo?c00<000000??cl`3clo<0S@3clo<200000<d0lo?c0`3o00040??cl`800000;`3clo<00540
lo?c00@000000??cl`00000000000P3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c00<00000
0??cl`3clo<0e03clo<00`000000lo?c0??cl`2=0??cl`030000003clo<0lo?c0=@0lo?c00<00000
0??cl`3clo<0;@3clo<00580lo?c00<000000??cl`3clo<00P3clo<01@000000lo?c0??cl`3clo<0
000000H0lo?c00<000000??cl`3clo<0:P3clo<00`000000lo?c0??cl`0^0??cl`030000003clo<0
lo?c02l0lo?c00<000000??cl`3clo<0;P3clo<00`000000lo?c0??cl`0C0??cl`030000003clo<0
lo?c08d0lo?c00<000000??cl`3clo<0:P3clo<00`000000lo?c0??cl`0^0??cl`030000003clo<0
lo?c02l0lo?c00<000000??cl`3clo<0;P3clo<00`000000lo?c0??cl`0C0??cl`030000003clo<0
lo?c02d0lo?c001C0??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c0??cl`04
0??clmP00000S`3clo?H000002l0lo?c001D0??cl`8000000P3clo<300000?l0lo?co`3clo=g0??c
l`00o`3clo?o0??clm80lo?c003o0??clol0lo?cdP3clo<00?l0lo?co`3clo?B0??cl`00a03clo<3
00000?l0lo?cI@3clo<300000:80lo?c00330??cl`050000003clo<0lo?c0??cl`000000o`3clo=S
0??cl`050000003clo<0lo?c0??cl`000000X@3clo<00<<0lo?c00D000000??cl`3clo<0lo?c0000
003o0??clf<0lo?c00D000000??cl`3clo<0lo?c0000002Q0??cl`00^@3clo<3000000@0lo?c0`00
0000103clo<000000000000000030??cl`L000000P3clo<2000000L0lo?c0`00003o0??cld80lo?c
0`0000040??cl`<0000000@0lo?c00000000000000000`3clo<700000080lo?c0P0000060??cl`D0
0000R@3clo<00;X0lo?c00<000000??cl`3clo<0103clo<00`000000lo?c000000050??cl`040000
003clo<0lo?c00000080lo?c00@000000??cl`3clo<000000P3clo<00`000000lo?c0??cl`050??c
l`030000003clo<0lo?c0?l0lo?c@P3clo<00`000000lo?c0??cl`040??cl`030000003clo<00000
00D0lo?c00@000000??cl`3clo<000000P3clo<010000000lo?c0??cl`0000020??cl`030000003c
lo<0lo?c00@0lo?c00@000000??cl`3clo<00000R@3clo<00;X0lo?c00<000000??cl`3clo<0103c
lo<00`000000lo?c0??cl`02000000<0lo?c00@000000??cl`3clo<000000P3clo<010000000lo?c
0??cl`00000:0??cl`030000003clo<0lo?c0?l0lo?c@P3clo<00`000000lo?c0??cl`040??cl`03
0000003clo<0lo?c008000000`3clo<010000000lo?c0??cl`0000020??cl`040000003clo<0lo?c
000000X0lo?c00<000000??cl`3clo<0R@3clo<00;X0lo?c00@000000??cl`3clo<000000`3clo<0
0`000000lo?c000000020??cl`040000003clo<0lo?c00000080lo?c00@000000??cl`3clo<00000
0P3clo<4000000L0lo?c00<000000??cl`3clo<0o`3clo=20??cl`040000003clo<0lo?c000000<0
lo?c00<000000??cl`0000000P3clo<010000000lo?c0??cl`0000020??cl`040000003clo<0lo?c
00000080lo?c100000080??cl`030000003clo<0lo?c08P0lo?c002j0??cl`@000000`3clo<00`00
0000lo?c000000020??cl`040000003clo<0lo?c00000080lo?c00<000000??cl`3clo<00P000000
1@3clo<000000??cl`3clo<0000000L0lo?c00<000000??cl`3clo<0o`3clo=20??cl`@000000`3c
lo<00`000000lo?c000000020??cl`040000003clo<0lo?c00000080lo?c00<000000??cl`3clo<0
0P0000001@3clo<000000??cl`3clo<0000000T0lo?c00<000000??cl`3clo<0Q`3clo<00;X0lo?c
00@000000??cl`3clo<000000P3clo<200000080lo?c1P000000103clo<000000000003clo<20000
00030??cl`000000lo?c00800000203clo<00`000000lo?c0??cl`3o0??cld80lo?c00@000000??c
l`3clo<000000P3clo<200000080lo?c1P000000103clo<000000000003clo<2000000030??cl`00
0000lo?c008000002P3clo<00`000000lo?c0??cl`270??cl`00^P3clo<00`000000lo?c0??cl`0P
0??cl`030000003clo<0lo?c0?l0lo?c@P3clo<00`000000lo?c0??cl`0R0??cl`030000003clo<0
lo?c08L0lo?c002j0??cl`050000003clo<0lo?c0??cl`0000007@3clo<200000?l0lo?cA03clo<0
1@000000lo?c0??cl`3clo<0000001`0lo?c00D000000??cl`3clo<0lo?c000000290??cl`00^@3c
lo<600000080lo?c00<000000??cl`3clo<06@3clo<00`000000lo?c0??cl`3o0??cld40lo?c1P00
00020??cl`030000003clo<0lo?c01P0lo?c0`00002:0??cl`00o`3clo?o0??clm80lo?c003o0??c
lol0lo?cdP3clo<00?l0lo?co`3clo?B0??cl`00o`3clo?o0??clm80lo?c003o0??clol0lo?cdP3c
lo<00<l0lo?c00<000000??cl`3clo<00P3clo<01@000000lo?c0??cl`3clo<000000?l0lo?cGP3c
lo<00`000000lo?c0??cl`020??cl`050000003clo<0lo?c0??cl`000000T03clo<00<h0lo?c00<0
00000??cl`3clo<00`3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c0?l0lo?cFP3c
lo<00`000000lo?c0??cl`030??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0S@3c
lo<00;l0lo?c0`0000030??cl`80000000<0lo?c0000000000000`3clo<010000000lo?c0??cl`3c
lo<3000000030??cl`000000lo?c00<0lo?c00<000000??cl`3clo<0o`3clo=:0??cl`<0000000D0
lo?c00000000000000000??cl`02000000050??cl`00000000000??cl`0000000P3clo<300000080
lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`2<0??cl`00`03clo<01@000000lo?c
0??cl`3clo<000000080lo?c00@000000??cl`3clo<000000P3clo<010000000lo?c0??cl`000004
0??cl`8000000`3clo<00`000000lo?c0??cl`3o0??cld/0lo?c00D000000??cl`3clo<0lo?c0000
00020??cl`040000003clo<0lo?c00000080lo?c00<000000??cl`0000001@3clo<2000000<0lo?c
00<000000??cl`3clo<0S03clo<00<00lo?c00@000000??cl`3clo<000000`3clo<00`000000lo?c
0??cl`030??cl`040000003clo<0lo?c000000<0lo?c0P0000001@3clo<000000??cl`3clo<00000
0?l0lo?cC@3clo<01@000000lo?c0??cl`3clo<000000080lo?c00@000000??cl`3clo<000000P3c
lo<00`000000lo?c000000040??cl`80000000D0lo?c0000003clo<0lo?c0000002>0??cl`00`03c
lo<010000000lo?c0??cl`0000030??cl`@000000P3clo<00`000000lo?c0??cl`04000000H0lo?c
00<000000??cl`3clo<0o`3clo=;0??cl`050000003clo<0lo?c0??cl`0000000P3clo<010000000
lo?c0??cl`0000020??cl`030000003clo<0000000<000001`3clo<00`000000lo?c0??cl`2<0??c
l`00`03clo<4000000<0lo?c00@000000??cl`3clo<000000P3clo<01@000000lo?c0??cl`3clo<0
000000P0lo?c00<000000??cl`3clo<0o`3clo=;0??cl`040000003clo<0lo?c0??cl`80000000D0
lo?c000000000000lo?c000000020??cl`040000003clo<0lo?c000000T0lo?c00<000000??cl`3c
lo<0S03clo<00<00lo?c00D000000??cl`3clo<0lo?c000000030??cl`8000000`3clo<00`000000
lo?c0??cl`020??cl`<000001@3clo<00`000000lo?c0??cl`3o0??cld/0lo?c00<000000??cl`3c
lo<00P0000001@3clo<000000??cl`3clo<0000000<0lo?c00@000000??cl`3clo<0lo?c0`000006
0??cl`030000003clo<0lo?c08`0lo?c00300??cl`050000003clo<0lo?c0??cl`000000203clo<0
0`000000lo?c0??cl`0:0??cl`030000003clo<0lo?c0?l0lo?cB`3clo<00`000000lo?c0??cl`0:
0??cl`030000003clo<0lo?c00X0lo?c00<000000??cl`3clo<0S03clo<00<00lo?c00D000000??c
l`3clo<0lo?c000000090??cl`030000003clo<0lo?c00P0lo?c00<000000??cl`3clo<0o`3clo=<
0??cl`030000003clo<0lo?c00/0lo?c00<000000??cl`3clo<0203clo<00`000000lo?c0??cl`2=
0??cl`00_`3clo<5000000/0lo?c00<000000??cl`3clo<01P3clo<00`000000lo?c0??cl`3o0??c
ld`0lo?c0`00000=0??cl`030000003clo<0lo?c00H0lo?c00<000000??cl`3clo<0SP3clo<00?l0
lo?co`3clo?B0??cl`00o`3clo?o0??clm80lo?c003o0??clol0lo?cdP3clo<00?l0lo?co`3clo?B
0??cl`00o`3clo?o0??clm80lo?c003o0??clol0lo?cdP3clo<00?l0lo?co`3clo?B0??cl`00\
\>"],
  ImageRangeCache->{{{0, 719}, {210.875, 0}} -> {-0.0536184, -0.0147151, \
0.00293079, 0.00293079}, {{18.25, 343.25}, {205.813, 5}} -> {0.410814, \
-192.039, 0.00202113, 0.790997}, {{375.75, 700.688}, {205.813, 5}} -> \
{-0.314709, -26.6236, 0.00202113, 0.125221}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 432 266.991 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.40476 2.30444 0.0147151 5.95199 [
[.20835 -0.0125 -9.5 -15.6875 ]
[.20835 -0.0125 9.5 0 ]
[.43879 -0.0125 -9.5 -15.6875 ]
[.43879 -0.0125 9.5 0 ]
[.66924 -0.0125 -9.5 -15.6875 ]
[.66924 -0.0125 9.5 0 ]
[.89968 -0.0125 -5 -15.6875 ]
[.89968 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9063 -31.375 ]
[.5 -0.0125 17.9063 -15.6875 ]
[-0.0125 .01472 -10 -7.84375 ]
[-0.0125 .01472 0 7.84375 ]
[-0.0125 .13375 -25 -7.84375 ]
[-0.0125 .13375 0 7.84375 ]
[-0.0125 .25279 -25 -7.84375 ]
[-0.0125 .25279 0 7.84375 ]
[-0.0125 .37183 -25 -7.84375 ]
[-0.0125 .37183 0 7.84375 ]
[-0.0125 .49087 -25 -7.84375 ]
[-0.0125 .49087 0 7.84375 ]
[-0.0125 .60991 -19 -7.84375 ]
[-0.0125 .60991 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40.6875 -34.2813 ]
[-0.01875 .30902 -25 34.2813 ]
[ 0 .61803 .125 0 ]
[.5 .63053 -21.9688 0 ]
[.5 .63053 21.9688 15.6875 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.20835 0 m
.20835 .00625 L
s
gsave
.20835 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.7) show
1.000 setlinewidth
grestore
.43879 0 m
.43879 .00625 L
s
gsave
.43879 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.66924 0 m
.66924 .00625 L
s
gsave
.66924 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.9) show
1.000 setlinewidth
grestore
.89968 0 m
.89968 .00625 L
s
gsave
.89968 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.25444 0 m
.25444 .00375 L
s
.30052 0 m
.30052 .00375 L
s
.34661 0 m
.34661 .00375 L
s
.3927 0 m
.3927 .00375 L
s
.48488 0 m
.48488 .00375 L
s
.53097 0 m
.53097 .00375 L
s
.57706 0 m
.57706 .00375 L
s
.62315 0 m
.62315 .00375 L
s
.71532 0 m
.71532 .00375 L
s
.76141 0 m
.76141 .00375 L
s
.8075 0 m
.8075 .00375 L
s
.85359 0 m
.85359 .00375 L
s
.16226 0 m
.16226 .00375 L
s
.11617 0 m
.11617 .00375 L
s
.07008 0 m
.07008 .00375 L
s
.02399 0 m
.02399 .00375 L
s
.94577 0 m
.94577 .00375 L
s
.99186 0 m
.99186 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9063 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(E) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
73.313 15.813 moveto
(H) show
77.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(eV) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.000 15.813 moveto
(L) show
94.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .01472 m
.00625 .01472 L
s
gsave
-0.0125 .01472 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .13375 m
.00625 .13375 L
s
gsave
-0.0125 .13375 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.02) show
1.000 setlinewidth
grestore
0 .25279 m
.00625 .25279 L
s
gsave
-0.0125 .25279 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.04) show
1.000 setlinewidth
grestore
0 .37183 m
.00625 .37183 L
s
gsave
-0.0125 .37183 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.06) show
1.000 setlinewidth
grestore
0 .49087 m
.00625 .49087 L
s
gsave
-0.0125 .49087 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.08) show
1.000 setlinewidth
grestore
0 .60991 m
.00625 .60991 L
s
gsave
-0.0125 .60991 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .04448 m
.00375 .04448 L
s
0 .07424 m
.00375 .07424 L
s
0 .104 m
.00375 .104 L
s
0 .16351 m
.00375 .16351 L
s
0 .19327 m
.00375 .19327 L
s
0 .22303 m
.00375 .22303 L
s
0 .28255 m
.00375 .28255 L
s
0 .31231 m
.00375 .31231 L
s
0 .34207 m
.00375 .34207 L
s
0 .40159 m
.00375 .40159 L
s
0 .43135 m
.00375 .43135 L
s
0 .46111 m
.00375 .46111 L
s
0 .52063 m
.00375 .52063 L
s
0 .55039 m
.00375 .55039 L
s
0 .58015 m
.00375 .58015 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -128.125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
95.2813 11.8438 translate
90 rotate
-95.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Transmission) show
127.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.20835 .61178 m
.20835 .61803 L
s
.43879 .61178 m
.43879 .61803 L
s
.66924 .61178 m
.66924 .61803 L
s
.89968 .61178 m
.89968 .61803 L
s
.125 Mabswid
.25444 .61428 m
.25444 .61803 L
s
.30052 .61428 m
.30052 .61803 L
s
.34661 .61428 m
.34661 .61803 L
s
.3927 .61428 m
.3927 .61803 L
s
.48488 .61428 m
.48488 .61803 L
s
.53097 .61428 m
.53097 .61803 L
s
.57706 .61428 m
.57706 .61803 L
s
.62315 .61428 m
.62315 .61803 L
s
.71532 .61428 m
.71532 .61803 L
s
.76141 .61428 m
.76141 .61803 L
s
.8075 .61428 m
.8075 .61803 L
s
.85359 .61428 m
.85359 .61803 L
s
.16226 .61428 m
.16226 .61803 L
s
.11617 .61428 m
.11617 .61803 L
s
.07008 .61428 m
.07008 .61803 L
s
.02399 .61428 m
.02399 .61803 L
s
.94577 .61428 m
.94577 .61803 L
s
.99186 .61428 m
.99186 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
gsave
.5 .63053 -82.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Figure) show
96.938 15.813 moveto
(3) show
102.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.99375 .01472 m
1 .01472 L
s
.99375 .13375 m
1 .13375 L
s
.99375 .25279 m
1 .25279 L
s
.99375 .37183 m
1 .37183 L
s
.99375 .49087 m
1 .49087 L
s
.99375 .60991 m
1 .60991 L
s
.125 Mabswid
.99625 .04448 m
1 .04448 L
s
.99625 .07424 m
1 .07424 L
s
.99625 .104 m
1 .104 L
s
.99625 .16351 m
1 .16351 L
s
.99625 .19327 m
1 .19327 L
s
.99625 .22303 m
1 .22303 L
s
.99625 .28255 m
1 .28255 L
s
.99625 .31231 m
1 .31231 L
s
.99625 .34207 m
1 .34207 L
s
.99625 .40159 m
1 .40159 L
s
.99625 .43135 m
1 .43135 L
s
.99625 .46111 m
1 .46111 L
s
.99625 .52063 m
1 .52063 L
s
.99625 .55039 m
1 .55039 L
s
.99625 .58015 m
1 .58015 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .02802 m
.06244 .03005 L
.10458 .0329 L
.14415 .03648 L
.18221 .04123 L
.20178 .04441 L
.22272 .04864 L
.24402 .05414 L
.26416 .061 L
.28414 .07026 L
.30258 .08231 L
.31224 .09073 L
.3213 .10055 L
.33124 .11441 L
.33678 .12408 L
.34193 .13476 L
.35203 .16249 L
.35737 .18245 L
.36306 .20991 L
.36849 .24475 L
.3736 .28899 L
.37819 .34231 L
.38315 .41913 L
.38774 .50617 L
.39026 .55322 L
.39151 .57292 L
.39266 .58759 L
.39331 .59403 L
.39401 .59911 L
.39471 .60224 L
.39548 .60332 L
.39612 .60229 L
.39681 .59928 L
.39805 .58914 L
.39933 .57336 L
.40068 .55236 L
.40309 .50864 L
.40795 .41983 L
.41315 .34374 L
.41885 .28341 L
.42415 .24292 L
.42948 .21248 L
.4344 .1907 L
.43968 .17204 L
.4455 .15565 L
.45529 .13479 L
.46565 .11873 L
.47581 .10686 L
.4851 .09831 L
.49594 .09028 L
Mistroke
.50599 .08424 L
.52502 .07534 L
.54575 .06822 L
.56632 .06288 L
.58494 .05907 L
.60584 .05562 L
.62505 .05303 L
.66366 .04901 L
.7032 .04603 L
.74122 .04389 L
.7817 .04217 L
.82066 .04091 L
.86207 .03989 L
.90197 .03917 L
.92059 .0389 L
.94035 .03866 L
.97619 .03834 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{432, 266.938},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3clo>a
0??cl`00o`3clo>a0??cl`00k03clo<00`000000lo?c0??cl`0<0??cl`030000003clo<0lo?c0;80
lo?c003[0??cl`030000003clo<0lo?c00h0lo?c00<000000??cl`3clo<0/@3clo<00=l0lo?c1P00
00050??cl`040000003clo<0lo?c0??cl`8000001@3clo<00`000000lo?c0??cl`050??cl`030000
003clo<0lo?c0;00lo?c003P0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<00P3c
lo<010000000lo?c0??cl`0000020??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0
1@3clo<00`000000lo?c0??cl`2`0??cl`00h03clo<00`000000lo?c0??cl`070??cl`040000003c
lo<0lo?c000000H0lo?c00<000000??cl`0000001P3clo<00`000000lo?c0??cl`2`0??cl`00h03c
lo<010000000lo?c0??cl`0000060??cl`030000003clo<0lo?c00@000000`3clo<00`000000lo?c
000000060??cl`030000003clo<0lo?c0;00lo?c003P0??cl`@000001P3clo<010000000lo?c0??c
l`0000020??cl`070000003clo<0lo?c0??cl`000000lo?c000000060??cl`030000003clo<0lo?c
0;00lo?c003P0??cl`040000003clo<0lo?c000000H0lo?c00@000000??cl`3clo<0lo?c0P000003
0??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0??cl`2`0??cl`00h03clo<0
0`000000lo?c0??cl`070??cl`030000003clo<0lo?c00H0lo?c00D000000??cl`3clo<0lo?c0000
00050??cl`030000003clo<0lo?c0;00lo?c003P0??cl`050000003clo<0lo?c0??cl`0000001P3c
lo<00`000000lo?c0??cl`050??cl`050000003clo<0lo?c0??cl`000000103clo<00`000000lo?c
0??cl`2a0??cl`00g`3clo<6000000L0lo?c00<000000??cl`3clo<00`3clo<3000000040??cl`00
0000000000000080lo?c00<000000??cl`3clo<0/P3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<0
0?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<0
08D0lo?c0`0000030??cl`050000003clo<0lo?c0??cl`000000@`3clo<3000000<0lo?c00<00000
0??cl`3clo<00`0000130??cl`<000000`3clo<010000000lo?c0000000000190??cl`<00000>03c
lo<008@0lo?c00D000000??cl`3clo<0lo?c000000060??cl`030000003clo<0lo?c0400lo?c00D0
00000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`000000@@3clo<01@000000
lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`3clo<0A`3clo<00`000000lo?c0??cl`0g0??c
l`00Q03clo<01@000000lo?c0??cl`3clo<0000000L0lo?c00<000000??cl`3clo<0?`3clo<01@00
0000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c000000110??cl`050000003c
lo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`160??cl`030000003clo<0lo?c03L0lo?c
00240??cl`050000003clo<0lo?c0??cl`0000001`3clo<00`000000lo?c0??cl`0o0??cl`050000
003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c000000120??cl`050000003clo<0lo?c0??c
l`0000001@3clo<4000004L0lo?c00<000000??cl`3clo<0=`3clo<008@0lo?c00D000000??cl`3c
lo<0lo?c000000070??cl`030000003clo<0lo?c03l0lo?c00D000000??cl`3clo<0lo?c00000006
0??cl`030000003clo<0lo?c0440lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003c
lo<0lo?c0??cl`000000A`3clo<00`000000lo?c0??cl`0g0??cl`00Q03clo<01@000000lo?c0??c
l`3clo<0000000L0lo?c00<000000??cl`3clo<0?`3clo<01@000000lo?c0??cl`3clo<0000000D0
lo?c00<000000??cl`000000@P3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??c
l`3clo<0lo?c000000170??cl`030000003clo<0lo?c03L0lo?c00240??cl`050000003clo<0lo?c
0??cl`000000203clo<00`000000lo?c0??cl`0n0??cl`050000003clo<0lo?c0??cl`000000103c
lo<01@000000lo?c0??cl`3clo<000000440lo?c00D000000??cl`3clo<0lo?c000000040??cl`05
0000003clo<0lo?c0??cl`000000A`3clo<00`000000lo?c0??cl`0g0??cl`00Q03clo<01@000000
lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c000000100??cl`050000003clo<0
lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<000000440lo?c00D000000??cl`3clo<0
lo?c000000040??cl`050000003clo<0lo?c0??cl`000000AP3clo<2000003T0lo?c00250??cl`<0
00001P3clo<400000440lo?c0`0000060??cl`<00000@`3clo<3000000H0lo?c0`0000180??cl`03
0000003clo<0lo?c03L0lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??clk40lo?c003o0??c
lk40lo?c003o0??clk40lo?c003o0??clk40lo?c00140??clol00000E`00000F0??cl`00A03clo<0
0`000000lo?c0??cl`050??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<03@3clo<0
0`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<0
0`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<0
0`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<0
0`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<0
0`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c00<000000??cl`3clo<03@3clo<0
0`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<0
0`000000lo?c0??cl`0=0??cl`040000003clo<0lo?c000001H0lo?c000i0??cl`<00000203clo<0
0`000000lo?c0??cl`140??cl`030000003clo<0lo?c04`0lo?c00<000000??cl`3clo<0B`3clo<0
0`000000lo?c0??cl`1<0??cl`030000003clo<0lo?c01l0lo?c00<000000??cl`3clo<0503clo<0
03P0lo?c00D000000??cl`3clo<0lo?c000000070??cl`030000003clo<0lo?c0?l0lo?cD`3clo<0
0`000000lo?c0??cl`0D0??cl`00>03clo<01@000000lo?c0??cl`3clo<0000000L0lo?c00<00000
0??cl`3clo<0o`3clo=C0??cl`030000003clo<0lo?c01@0lo?c000h0??cl`050000003clo<0lo?c
0??cl`0000001`3clo<300000?l0lo?cD@3clo<3000001H0lo?c000h0??cl`050000003clo<0lo?c
0??cl`0000001`3clo<00`000000lo?c0??cl`3o0??cle<0lo?c00<000000??cl`3clo<0503clo<0
03P0lo?c00D000000??cl`3clo<0lo?c000000070??cl`030000003clo<0lo?c0?l0lo?cD`3clo<0
0`000000lo?c0??cl`0D0??cl`00>03clo<01@000000lo?c0??cl`3clo<0000000L0lo?c00<00000
0??cl`3clo<0o`3clo=C0??cl`030000003clo<0lo?c01@0lo?c000h0??cl`050000003clo<0lo?c
0??cl`0000001`3clo<00`000000lo?c0??cl`3o0??cle<0lo?c00<000000??cl`3clo<0503clo<0
03T0lo?c0`0000080??cl`030000003clo<0lo?c00D0lo?c5@00003o0??clcT0lo?c00<000000??c
l`3clo<0503clo<004@0lo?c00<000000??cl`3clo<06P3clo<;00000?l0lo?c;P3clo<00`000000
lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`0U0??cl`H00000o`3clo<X0??cl`030000
003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c02/0lo?c2P00003o0??claL000001`3c
lo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`0e0??cl`X00000d03clo<U
000001h0lo?c00<000000??cl`3clo<0503clo<004@0lo?c0`00000o0??cl`D00000/03clo<K0000
0440lo?c0`00000F0??cl`00A03clo<00`000000lo?c0??cl`140??cl`@00000W`3clo<=000005h0
lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0B03clo<3000008h0lo?c
3P00001[0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c04/0lo?c1000
00200??cl`X00000N@3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1?
0??cl`<00000MP3clo<7000008<0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??c
l`3clo<0DP3clo<4000006d0lo?c1@00002:0??cl`030000003clo<0lo?c01@0lo?c00140??cl`03
0000003clo<0lo?c05H0lo?c0`00001W0??cl`<00000S`3clo<00`000000lo?c0??cl`0D0??cl`00
A03clo<00`000000lo?c0??cl`1I0??cl`800000H03clo<500000980lo?c00<000000??cl`3clo<0
503clo<004@0lo?c00<000000??cl`3clo<0F`3clo<2000005T0lo?c1@00002G0??cl`030000003c
lo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c05d0lo?c0P00001E0??cl`800000W03clo<0
0`000000lo?c0??cl`0D0??cl`00A03clo<3000005l0lo?c0P00001A0??cl`800000W03clo<30000
01H0lo?c00140??cl`030000003clo<0lo?c0640lo?c00<000000??cl`3clo<0C03clo<200000:00
lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0HP3clo<2000004X0lo?c
0P00002R0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c06@0lo?c00<0
00000??cl`3clo<0A@3clo<200000:@0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<00000
0??cl`3clo<0I@3clo<00`000000lo?c0??cl`120??cl`800000YP3clo<00`000000lo?c0??cl`0D
0??cl`00A03clo<00`000000lo?c0??cl`1V0??cl`800000@03clo<200000:P0lo?c00<000000??c
l`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0J03clo<00`000000lo?c0??cl`0k0??cl`80
0000ZP3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1Y0??cl`030000
003clo<0lo?c03T0lo?c00<000000??cl`3clo<0ZP3clo<00`000000lo?c0??cl`0D0??cl`00A03c
lo<00`000000lo?c0??cl`1Z0??cl`030000003clo<0lo?c03H0lo?c0P00002]0??cl`030000003c
lo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c06/0lo?c00<000000??cl`3clo<0=03clo<0
0`000000lo?c0??cl`2]0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c
06`0lo?c00<000000??cl`3clo<0<P3clo<00`000000lo?c0??cl`2^0??cl`030000003clo<0lo?c
01@0lo?c00140??cl`<00000K03clo<00`000000lo?c0??cl`0a0??cl`030000003clo<0lo?c0:d0
lo?c0`00000F0??cl`00A03clo<00`000000lo?c0??cl`1]0??cl`030000003clo<0lo?c02l0lo?c
00<000000??cl`3clo<0/03clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??c
l`1]0??cl`030000003clo<0lo?c02h0lo?c00<000000??cl`3clo<0/@3clo<00`000000lo?c0??c
l`0D0??cl`00A03clo<00`000000lo?c0??cl`1^0??cl`030000003clo<0lo?c02`0lo?c00<00000
0??cl`3clo<0/P3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1_0??c
l`030000003clo<0lo?c02/0lo?c00<000000??cl`3clo<0/P3clo<00`000000lo?c0??cl`0D0??c
l`00A03clo<00`000000lo?c0??cl`1_0??cl`030000003clo<0lo?c02X0lo?c00<000000??cl`3c
lo<0/`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1`0??cl`030000
003clo<0lo?c02P0lo?c00<000000??cl`3clo<0]03clo<00`000000lo?c0??cl`0D0??cl`00:P3c
lo<3000000<0lo?c00<000000??cl`3clo<00`0000020??cl`D000001`3clo<00`000000lo?c0??c
l`1`0??cl`030000003clo<0lo?c02L0lo?c00<000000??cl`3clo<0]@3clo<00`000000lo?c0??c
l`0D0??cl`00:@3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3clo<0lo?c
000000020??cl`040000003clo<0lo?c000000L0lo?c00<000000??cl`3clo<0L@3clo<00`000000
lo?c0??cl`0V0??cl`030000003clo<0lo?c0;D0lo?c00<000000??cl`3clo<0503clo<002T0lo?c
00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`00
0000lo?c0??cl`070??cl`030000003clo<0lo?c0740lo?c00<000000??cl`3clo<09@3clo<00`00
0000lo?c0??cl`2f0??cl`030000003clo<0lo?c01@0lo?c000Y0??cl`050000003clo<0lo?c0??c
l`000000103clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00<000000??cl`3clo<01P3clo<3
00000780lo?c00<000000??cl`3clo<08`3clo<00`000000lo?c0??cl`2e0??cl`<000005P3clo<0
02T0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000001@3c
lo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0780lo?c00<000000??cl`3clo<08`3c
lo<00`000000lo?c0??cl`2g0??cl`030000003clo<0lo?c01@0lo?c000Y0??cl`050000003clo<0
lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<0000000D0lo?c00<000000??cl`3clo<0
1@3clo<00`000000lo?c0??cl`1c0??cl`030000003clo<0lo?c0240lo?c00<000000??cl`3clo<0
^03clo<00`000000lo?c0??cl`0D0??cl`00:@3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c
00D000000??cl`3clo<0lo?c000000050??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3c
lo<0L`3clo<00`000000lo?c0??cl`0Q0??cl`030000003clo<0lo?c0;P0lo?c00<000000??cl`3c
lo<0503clo<002T0lo?c00D000000??cl`3clo<0lo?c000000040??cl`070000003clo<0lo?c0??c
l`000000lo?c000000030??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0L`3clo<0
0`000000lo?c0??cl`0P0??cl`030000003clo<0lo?c0;T0lo?c00<000000??cl`3clo<0503clo<0
02X0lo?c0`0000060??cl`<000000`3clo<3000000P0lo?c00<000000??cl`3clo<0M03clo<00`00
0000lo?c0??cl`0O0??cl`030000003clo<0lo?c0;T0lo?c00<000000??cl`3clo<0503clo<004@0
lo?c00<000000??cl`3clo<0M03clo<00`000000lo?c0??cl`0N0??cl`030000003clo<0lo?c0;X0
lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0M03clo<00`000000lo?c
0??cl`0N0??cl`030000003clo<0lo?c0;X0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<0
00000??cl`3clo<0M@3clo<00`000000lo?c0??cl`0M0??cl`030000003clo<0lo?c0;X0lo?c00<0
00000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0M@3clo<00`000000lo?c0??cl`0L
0??cl`030000003clo<0lo?c0;/0lo?c00<000000??cl`3clo<0503clo<004@0lo?c0`00001e0??c
l`030000003clo<0lo?c01`0lo?c00<000000??cl`3clo<0^@3clo<3000001H0lo?c00140??cl`03
0000003clo<0lo?c07H0lo?c00<000000??cl`3clo<06`3clo<00`000000lo?c0??cl`2k0??cl`03
0000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c07H0lo?c00<000000??cl`3clo<0
6P3clo<00`000000lo?c0??cl`2l0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003c
lo<0lo?c07H0lo?c00<000000??cl`3clo<06P3clo<00`000000lo?c0??cl`2l0??cl`030000003c
lo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c07H0lo?c00<000000??cl`3clo<06P3clo<0
0`000000lo?c0??cl`2l0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c
07L0lo?c00<000000??cl`3clo<0603clo<00`000000lo?c0??cl`2m0??cl`030000003clo<0lo?c
01@0lo?c00140??cl`030000003clo<0lo?c07L0lo?c00<000000??cl`3clo<0603clo<00`000000
lo?c0??cl`2m0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c07L0lo?c
00<000000??cl`3clo<0603clo<00`000000lo?c0??cl`2m0??cl`030000003clo<0lo?c01@0lo?c
00140??cl`030000003clo<0lo?c07L0lo?c00<000000??cl`3clo<05`3clo<00`000000lo?c0??c
l`2n0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c07P0lo?c00<00000
0??cl`3clo<05P3clo<00`000000lo?c0??cl`2n0??cl`030000003clo<0lo?c01@0lo?c00140??c
l`<00000N03clo<00`000000lo?c0??cl`0F0??cl`030000003clo<0lo?c0;`0lo?c0`00000F0??c
l`00A03clo<00`000000lo?c0??cl`1h0??cl`030000003clo<0lo?c01D0lo?c00<000000??cl`3c
lo<0_`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1h0??cl`030000
003clo<0lo?c01D0lo?c00<000000??cl`3clo<0_`3clo<00`000000lo?c0??cl`0D0??cl`00A03c
lo<00`000000lo?c0??cl`1h0??cl`030000003clo<0lo?c01D0lo?c00<000000??cl`3clo<0_`3c
lo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1i0??cl`030000003clo<0
lo?c01@0lo?c00<000000??cl`3clo<0_`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`00
0000lo?c0??cl`1i0??cl`030000003clo<0lo?c01<0lo?c00<000000??cl`3clo<0`03clo<00`00
0000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1i0??cl`030000003clo<0lo?c01<0
lo?c00<000000??cl`3clo<0`03clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c
0??cl`1i0??cl`030000003clo<0lo?c01<0lo?c00<000000??cl`3clo<0`03clo<00`000000lo?c
0??cl`0D0??cl`00603clo<2000002X0lo?c00<000000??cl`3clo<0N@3clo<00`000000lo?c0??c
l`0C0??cl`030000003clo<0lo?c0<00lo?c00<000000??cl`3clo<0503clo<001P0lo?c00<00000
0??cl`3clo<0:@3clo<00`000000lo?c0??cl`1j0??cl`030000003clo<0lo?c0180lo?c00<00000
0??cl`3clo<0`03clo<00`000000lo?c0??cl`0D0??cl`00603clo<00`000000lo?c0??cl`050??c
l`030000003clo<0lo?c0240lo?c0`00001j0??cl`030000003clo<0lo?c0180lo?c00<000000??c
l`3clo<0_P3clo<3000001H0lo?c000H0??cl`T000008`3clo<00`000000lo?c0??cl`1j0??cl`03
0000003clo<0lo?c0180lo?c00<000000??cl`3clo<0`03clo<00`000000lo?c0??cl`0D0??cl`00
603clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0240lo?c00<000000??cl`3clo<0
NP3clo<00`000000lo?c0??cl`0A0??cl`030000003clo<0lo?c0<40lo?c00<000000??cl`3clo<0
503clo<001P0lo?c00<000000??cl`3clo<0:@3clo<00`000000lo?c0??cl`1j0??cl`030000003c
lo<0lo?c0140lo?c00<000000??cl`3clo<0`@3clo<00`000000lo?c0??cl`0D0??cl`00603clo<2
000002X0lo?c00<000000??cl`3clo<0NP3clo<00`000000lo?c0??cl`0A0??cl`030000003clo<0
lo?c0<40lo?c00<000000??cl`3clo<0503clo<001/0lo?c00<000000??cl`3clo<00P3clo<00`00
0000lo?c0??cl`0Q0??cl`030000003clo<0lo?c07/0lo?c00<000000??cl`3clo<0403clo<00`00
0000lo?c0??cl`310??cl`030000003clo<0lo?c01@0lo?c000K0??cl`H000008`3clo<00`000000
lo?c0??cl`1k0??cl`030000003clo<0lo?c0100lo?c00<000000??cl`3clo<0`@3clo<00`000000
lo?c0??cl`0D0??cl`006`3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c00L0lo?c
0`0000030??cl`030000003clo<0lo?c00<000001P3clo<00`000000lo?c0??cl`050??cl`030000
003clo<0lo?c07/0lo?c00<000000??cl`3clo<0403clo<00`000000lo?c0??cl`310??cl`030000
003clo<0lo?c01@0lo?c000K0??cl`030000003clo<0lo?c00/0lo?c00D000000??cl`3clo<0lo?c
000000040??cl`050000003clo<0lo?c0??cl`0000001@3clo<00`000000lo?c0??cl`050??cl`03
0000003clo<0lo?c07/0lo?c00<000000??cl`3clo<0403clo<00`000000lo?c0??cl`310??cl`03
0000003clo<0lo?c01@0lo?c000L0??cl`040000003clo<00000000000T0lo?c00D000000??cl`3c
lo<0lo?c000000040??cl`060000003clo<0lo?c0??cl`000000lo?c1P0000060??cl`030000003c
lo<0lo?c07/0lo?c00<000000??cl`3clo<0403clo<00`000000lo?c0??cl`310??cl`030000003c
lo<0lo?c01@0lo?c000K0??cl`030000003clo<000000080lo?c00<000000??cl`3clo<01P3clo<0
1@000000lo?c0??cl`3clo<0000000@0lo?c00L000000??cl`3clo<0lo?c0000003clo<0000000<0
lo?c00<000000??cl`3clo<01@3clo<3000007/0lo?c00<000000??cl`3clo<0403clo<00`000000
lo?c0??cl`2o0??cl`<000005P3clo<001/0lo?c00D000000??cl`000000lo?c000000090??cl`05
0000003clo<0lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<000000080lo?c00@00000
0??cl`3clo<000001`3clo<00`000000lo?c0??cl`1k0??cl`030000003clo<0lo?c00l0lo?c00<0
00000??cl`3clo<0`P3clo<00`000000lo?c0??cl`0D0??cl`00703clo<5000000P0lo?c00D00000
0??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000lo?c
000000070??cl`030000003clo<0lo?c07/0lo?c00<000000??cl`3clo<03`3clo<00`000000lo?c
0??cl`320??cl`030000003clo<0lo?c01@0lo?c000P0??cl`030000003clo<0lo?c00H0lo?c00D0
00000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000000`3clo<00`000000
lo?c000000070??cl`030000003clo<0lo?c07/0lo?c00<000000??cl`3clo<03`3clo<00`000000
lo?c0??cl`320??cl`030000003clo<0lo?c01@0lo?c000K0??cl`030000003clo<0lo?c0080lo?c
00<000000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00D000000??cl`3c
lo<0lo?c000000040??cl`8000001`3clo<00`000000lo?c0??cl`1k0??cl`030000003clo<0lo?c
00l0lo?c00<000000??cl`3clo<0`P3clo<00`000000lo?c0??cl`0D0??cl`006`3clo<6000000T0
lo?c0`0000060??cl`<000001P3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c07/0
lo?c00<000000??cl`3clo<03`3clo<00`000000lo?c0??cl`320??cl`030000003clo<0lo?c01@0
lo?c000K0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<08@3clo<00`000000lo?c
0??cl`1l0??cl`030000003clo<0lo?c00h0lo?c00<000000??cl`3clo<0`P3clo<00`000000lo?c
0??cl`0D0??cl`006`3clo<00`000000lo?c0??cl`0V0??cl`030000003clo<0lo?c07`0lo?c00<0
00000??cl`3clo<03P3clo<00`000000lo?c0??cl`320??cl`030000003clo<0lo?c01@0lo?c000L
0??cl`D000008`3clo<00`000000lo?c0??cl`1l0??cl`030000003clo<0lo?c00d0lo?c00<00000
0??cl`3clo<0``3clo<00`000000lo?c0??cl`0D0??cl`00803clo<00`000000lo?c0??cl`0Q0??c
l`030000003clo<0lo?c07`0lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`330??c
l`030000003clo<0lo?c01@0lo?c000L0??cl`80000000<0lo?c0000000000008`3clo<3000007`0
lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`310??cl`<000005P3clo<001/0lo?c
00D000000??cl`000000lo?c0000000T0??cl`030000003clo<0lo?c07`0lo?c00<000000??cl`3c
lo<03@3clo<00`000000lo?c0??cl`330??cl`030000003clo<0lo?c01@0lo?c000K0??cl`060000
003clo<0lo?c0000003clo<000008`3clo<00`000000lo?c0??cl`1l0??cl`030000003clo<0lo?c
00d0lo?c00<000000??cl`3clo<0``3clo<00`000000lo?c0??cl`0D0??cl`006`3clo<200000003
0??cl`000000000002@0lo?c00<000000??cl`3clo<0O03clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c0<<0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0
O03clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c0<<0lo?c00<000000??cl`3clo<0
503clo<001/0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`0Q0??cl`030000003c
lo<0lo?c07`0lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`330??cl`030000003c
lo<0lo?c01@0lo?c000K0??cl`H000008`3clo<00`000000lo?c0??cl`1l0??cl`030000003clo<0
lo?c00`0lo?c00<000000??cl`3clo<0a03clo<00`000000lo?c0??cl`0D0??cl`006`3clo<00`00
0000lo?c0??cl`020??cl`030000003clo<0lo?c0240lo?c00<000000??cl`3clo<0O03clo<00`00
0000lo?c0??cl`0<0??cl`030000003clo<0lo?c0<@0lo?c00<000000??cl`3clo<0503clo<001/0
lo?c00<000000??cl`3clo<09P3clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c00/0
lo?c00<000000??cl`3clo<0a03clo<00`000000lo?c0??cl`0D0??cl`00703clo<5000002<0lo?c
00<000000??cl`3clo<0O@3clo<00`000000lo?c0??cl`0;0??cl`030000003clo<0lo?c0<@0lo?c
00<000000??cl`3clo<0503clo<001/0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??c
l`0Q0??cl`<00000O@3clo<00`000000lo?c0??cl`0;0??cl`030000003clo<0lo?c0<80lo?c0`00
000F0??cl`006`3clo<00`000000lo?c0??cl`0V0??cl`030000003clo<0lo?c07d0lo?c00<00000
0??cl`3clo<02`3clo<00`000000lo?c0??cl`340??cl`030000003clo<0lo?c01@0lo?c000L0??c
l`D000008`3clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c00/0lo?c00<000000??c
l`3clo<0a03clo<00`000000lo?c0??cl`0D0??cl`00803clo<00`000000lo?c0??cl`0Q0??cl`03
0000003clo<0lo?c07d0lo?c00<000000??cl`3clo<02`3clo<00`000000lo?c0??cl`340??cl`03
0000003clo<0lo?c01@0lo?c000K0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0
8@3clo<00`000000lo?c0??cl`1m0??cl`030000003clo<0lo?c00/0lo?c00<000000??cl`3clo<0
a03clo<00`000000lo?c0??cl`0D0??cl`00603clo<00`000000lo?c0??cl`06000002<0lo?c00<0
00000??cl`3clo<0O@3clo<00`000000lo?c0??cl`0;0??cl`030000003clo<0lo?c0<@0lo?c00<0
00000??cl`3clo<0503clo<00200lo?c00<000000??cl`3clo<08@3clo<00`000000lo?c0??cl`1m
0??cl`030000003clo<0lo?c00X0lo?c00<000000??cl`3clo<0a@3clo<00`000000lo?c0??cl`0D
0??cl`00703clo<2000000030??cl`000000000002<0lo?c00<000000??cl`3clo<0OP3clo<00`00
0000lo?c0??cl`090??cl`030000003clo<0lo?c0<D0lo?c00<000000??cl`3clo<0503clo<001/0
lo?c00D000000??cl`000000lo?c0000000T0??cl`030000003clo<0lo?c07h0lo?c00<000000??c
l`3clo<02@3clo<00`000000lo?c0??cl`350??cl`030000003clo<0lo?c01@0lo?c000K0??cl`06
0000003clo<0lo?c0000003clo<000008`3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0
lo?c00T0lo?c00<000000??cl`3clo<0a@3clo<00`000000lo?c0??cl`0D0??cl`006`3clo<20000
00030??cl`000000000002@0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`090??c
l`030000003clo<0lo?c0<D0lo?c00<000000??cl`3clo<0503clo<004@0lo?c0`00001n0??cl`03
0000003clo<0lo?c00T0lo?c00<000000??cl`3clo<0``3clo<3000001H0lo?c000L0??cl`800000
00<0lo?c0000000000008`3clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c00T0lo?c
00<000000??cl`3clo<0a@3clo<00`000000lo?c0??cl`0D0??cl`006`3clo<01@000000lo?c0000
003clo<0000002@0lo?c00<000000??cl`3clo<0OP3clo<00`000000lo?c0??cl`090??cl`030000
003clo<0lo?c0<D0lo?c00<000000??cl`3clo<0503clo<001/0lo?c00H000000??cl`3clo<00000
0??cl`00000S0??cl`030000003clo<0lo?c07h0lo?c00<000000??cl`3clo<02@3clo<00`000000
lo?c0??cl`350??cl`030000003clo<0lo?c01@0lo?c000K0??cl`80000000<0lo?c000000000000
903clo<00`000000lo?c0??cl`1n0??cl`030000003clo<0lo?c00T0lo?c00<000000??cl`3clo<0
a@3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`1n0??cl`030000003c
lo<0lo?c00T0lo?c00<000000??cl`3clo<0a@3clo<00`000000lo?c0??cl`0D0??cl`006`3clo<0
0`000000lo?c0??cl`020??cl`030000003clo<0lo?c0240lo?c00<000000??cl`3clo<0OP3clo<0
0`000000lo?c0??cl`090??cl`030000003clo<0lo?c0<D0lo?c00<000000??cl`3clo<0503clo<0
01P0lo?c00<000000??cl`3clo<01P0000090??cl`<000000`3clo<00`000000lo?c0??cl`030000
00<0lo?c0`0000080??cl`030000003clo<0lo?c07l0lo?c00<000000??cl`3clo<01`3clo<00`00
0000lo?c0??cl`360??cl`030000003clo<0lo?c01@0lo?c000P0??cl`030000003clo<0lo?c00H0
lo?c00D000000??cl`3clo<0lo?c000000040??cl`070000003clo<0lo?c0??cl`000000lo?c0000
00030??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0O`3clo<00`000000lo?c0??c
l`070??cl`030000003clo<0lo?c0<H0lo?c00<000000??cl`3clo<0503clo<001`0lo?c10000009
0??cl`050000003clo<0lo?c0??cl`000000103clo<01`000000lo?c0??cl`3clo<000000??cl`00
00000`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c07l0lo?c00<000000??cl`3c
lo<01`3clo<00`000000lo?c0??cl`360??cl`030000003clo<0lo?c01@0lo?c000K0??cl`030000
003clo<0lo?c0080lo?c00<000000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000@0
lo?c00L000000??cl`3clo<0lo?c0000003clo<0000000<0lo?c00<000000??cl`3clo<01@3clo<3
000007l0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`340??cl`<000005P3clo<0
01/0lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`060??cl`050000003clo<0lo?c
0??cl`000000103clo<01P000000lo?c0??cl`3clo<000000??cl`8000000P3clo<00`000000lo?c
0??cl`050??cl`030000003clo<0lo?c07l0lo?c00<000000??cl`3clo<01`3clo<00`000000lo?c
0??cl`360??cl`030000003clo<0lo?c01@0lo?c000K0??cl`030000003clo<0lo?c0080lo?c00<0
00000??cl`3clo<01P3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00P000000??cl`3clo<0
lo?c0000003clo<000000??cl`800000203clo<00`000000lo?c0??cl`1o0??cl`030000003clo<0
lo?c00L0lo?c00<000000??cl`3clo<0aP3clo<00`000000lo?c0??cl`0D0??cl`00703clo<40000
00T0lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000000P3c
lo<00`000000lo?c0??cl`080??cl`030000003clo<0lo?c07l0lo?c00<000000??cl`3clo<01`3c
lo<00`000000lo?c0??cl`360??cl`030000003clo<0lo?c01@0lo?c000Y0??cl`050000003clo<0
lo?c0??cl`000000103clo<01@000000lo?c0??cl`3clo<0000000<0lo?c00<000000??cl`3clo<0
1`3clo<00`000000lo?c0??cl`1o0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0
aP3clo<00`000000lo?c0??cl`0D0??cl`006`3clo<00`000000lo?c0??cl`020??cl`030000003c
lo<0lo?c00L0lo?c0`0000060??cl`<000001@3clo<2000000L0lo?c00<000000??cl`3clo<0O`3c
lo<00`000000lo?c0??cl`070??cl`030000003clo<0lo?c0<H0lo?c00<000000??cl`3clo<0503c
lo<001/0lo?c1P00000S0??cl`030000003clo<0lo?c07l0lo?c00<000000??cl`3clo<01`3clo<0
0`000000lo?c0??cl`360??cl`030000003clo<0lo?c01@0lo?c000K0??cl`030000003clo<0lo?c
0080lo?c00<000000??cl`3clo<08@3clo<00`000000lo?c0??cl`1o0??cl`030000003clo<0lo?c
00L0lo?c00<000000??cl`3clo<0aP3clo<00`000000lo?c0??cl`0D0??cl`006`3clo<00`000000
lo?c0??cl`0V0??cl`030000003clo<0lo?c07l0lo?c00<000000??cl`3clo<01`3clo<00`000000
lo?c0??cl`360??cl`030000003clo<0lo?c01@0lo?c000L0??cl`D000008`3clo<00`000000lo?c
0??cl`1o0??cl`030000003clo<0lo?c00L0lo?c00<000000??cl`3clo<0aP3clo<00`000000lo?c
0??cl`0D0??cl`00803clo<00`000000lo?c0??cl`0Q0??cl`<00000P03clo<00`000000lo?c0??c
l`050??cl`030000003clo<0lo?c0<D0lo?c0`00000F0??cl`00A03clo<00`000000lo?c0??cl`20
0??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0a`3clo<00`000000lo?c0??cl`0D
0??cl`00A03clo<00`000000lo?c0??cl`200??cl`030000003clo<0lo?c00D0lo?c00<000000??c
l`3clo<0a`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`200??cl`03
0000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0a`3clo<00`000000lo?c0??cl`0D0??cl`00
A03clo<00`000000lo?c0??cl`200??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0
a`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`200??cl`030000003c
lo<0lo?c00D0lo?c00<000000??cl`3clo<0a`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<0
0`000000lo?c0??cl`200??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0a`3clo<0
0`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`200??cl`030000003clo<0lo?c
00D0lo?c00<000000??cl`3clo<0a`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000
lo?c0??cl`200??cl`030000003clo<0lo?c00D0lo?c00<000000??cl`3clo<0a`3clo<00`000000
lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`200??cl`030000003clo<0lo?c00D0lo?c
00<000000??cl`3clo<0a`3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<300000800lo?c00<0
00000??cl`3clo<01@3clo<00`000000lo?c0??cl`350??cl`<000005P3clo<004@0lo?c00<00000
0??cl`3clo<0P03clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0<L0lo?c00<00000
0??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0P03clo<00`000000lo?c0??cl`050??c
l`030000003clo<0lo?c0<L0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3c
lo<0P03clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0<L0lo?c00<000000??cl`3c
lo<0503clo<004@0lo?c00<000000??cl`3clo<0P03clo<00`000000lo?c0??cl`040??cl`030000
003clo<0lo?c0<P0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0P03c
lo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c0<P0lo?c00<000000??cl`3clo<0503c
lo<004@0lo?c00<000000??cl`3clo<0P03clo<00`000000lo?c0??cl`040??cl`030000003clo<0
lo?c0<P0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0P03clo<00`00
0000lo?c0??cl`040??cl`030000003clo<0lo?c0<P0lo?c00<000000??cl`3clo<0503clo<004@0
lo?c00<000000??cl`3clo<0P03clo<00`000000lo?c0??cl`040??cl`030000003clo<0lo?c0<P0
lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0P03clo<00`000000lo?c
0??cl`040??cl`030000003clo<0lo?c0<P0lo?c00<000000??cl`3clo<0503clo<004@0lo?c0`00
00200??cl`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<0aP3clo<3000001H0lo?c0014
0??cl`030000003clo<0lo?c0840lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`38
0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c0840lo?c00<000000??c
l`3clo<00`3clo<00`000000lo?c0??cl`380??cl`030000003clo<0lo?c01@0lo?c00140??cl`03
0000003clo<0lo?c0840lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`380??cl`03
0000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c0840lo?c00<000000??cl`3clo<0
0`3clo<00`000000lo?c0??cl`380??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003c
lo<0lo?c0840lo?c00<000000??cl`3clo<00`3clo<00`000000lo?c0??cl`380??cl`030000003c
lo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c0840lo?c00<000000??cl`3clo<00`3clo<0
0`000000lo?c0??cl`380??cl`030000003clo<0lo?c01@0lo?c000Z0??cl`<000000`3clo<00`00
0000lo?c0??cl`03000000<0lo?c0`0000080??cl`030000003clo<0lo?c0840lo?c00<000000??c
l`3clo<00`3clo<00`000000lo?c0??cl`380??cl`030000003clo<0lo?c01@0lo?c000Y0??cl`05
0000003clo<0lo?c0??cl`000000103clo<01`000000lo?c0??cl`3clo<000000??cl`0000000`3c
lo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c0840lo?c00<000000??cl`3clo<00`3c
lo<00`000000lo?c0??cl`380??cl`030000003clo<0lo?c01@0lo?c000Y0??cl`050000003clo<0
lo?c0??cl`000000103clo<01`000000lo?c0??cl`3clo<000000??cl`0000000`3clo<00`000000
lo?c0??cl`050??cl`030000003clo<0lo?c0840lo?c00<000000??cl`3clo<00P3clo<00`000000
lo?c0??cl`390??cl`030000003clo<0lo?c01@0lo?c000Y0??cl`050000003clo<0lo?c0??cl`00
0000103clo<01@000000lo?c0??cl`3clo<000000080lo?c00<000000??cl`000000203clo<30000
0840lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`370??cl`<000005P3clo<002T0
lo?c00D000000??cl`3clo<0lo?c000000040??cl`050000003clo<0lo?c0??cl`0000000`3clo<0
0`000000lo?c0??cl`070??cl`030000003clo<0lo?c0840lo?c00<000000??cl`3clo<00P3clo<0
0`000000lo?c0??cl`390??cl`030000003clo<0lo?c01@0lo?c000Y0??cl`050000003clo<0lo?c
0??cl`000000103clo<01@000000lo?c0??cl`3clo<000000080lo?c00<000000??cl`000000203c
lo<00`000000lo?c0??cl`210??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0b@3c
lo<00`000000lo?c0??cl`0D0??cl`00:@3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00L0
00000??cl`3clo<0lo?c0000003clo<0000000<0lo?c00<000000??cl`3clo<01@3clo<00`000000
lo?c0??cl`210??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0b@3clo<00`000000
lo?c0??cl`0D0??cl`00:@3clo<01@000000lo?c0??cl`3clo<0000000@0lo?c00L000000??cl`3c
lo<0lo?c0000003clo<0000000<0lo?c00<000000??cl`3clo<01@3clo<00`000000lo?c0??cl`21
0??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0b@3clo<00`000000lo?c0??cl`0D
0??cl`00:P3clo<3000000H0lo?c0`0000030??cl`<00000203clo<00`000000lo?c0??cl`210??c
l`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0b@3clo<00`000000lo?c0??cl`0D0??c
l`00A03clo<00`000000lo?c0??cl`210??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3c
lo<0b@3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`210??cl`030000
003clo<0lo?c0080lo?c00<000000??cl`3clo<0b@3clo<00`000000lo?c0??cl`0D0??cl`00A03c
lo<00`000000lo?c0??cl`210??cl`030000003clo<0lo?c0080lo?c00<000000??cl`3clo<0b@3c
lo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`210??cl`030000003clo<0
lo?c0080lo?c00<000000??cl`3clo<0b@3clo<00`000000lo?c0??cl`0D0??cl`00A03clo<30000
0840lo?c00<000000??cl`3clo<00P3clo<00`000000lo?c0??cl`370??cl`<000005P3clo<004@0
lo?c00<000000??cl`3clo<0P@3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c0<T0
lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0P@3clo<00`000000lo?c
0??cl`020??cl`030000003clo<0lo?c0<T0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<0
00000??cl`3clo<0P@3clo<00`000000lo?c0??cl`020??cl`030000003clo<0lo?c0<T0lo?c00<0
00000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0PP3clo<01@000000lo?c0??cl`3c
lo<000000</0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0PP3clo<0
1@000000lo?c0??cl`3clo<000000</0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<00000
0??cl`3clo<0PP3clo<01@000000lo?c0??cl`3clo<000000</0lo?c00<000000??cl`3clo<0503c
lo<004@0lo?c00<000000??cl`3clo<0PP3clo<01@000000lo?c0??cl`3clo<000000</0lo?c00<0
00000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0PP3clo<01@000000lo?c0??cl`3c
lo<000000</0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0PP3clo<0
1@000000lo?c0??cl`3clo<000000</0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<00000
0??cl`3clo<0PP3clo<01@000000lo?c0??cl`3clo<000000</0lo?c00<000000??cl`3clo<0503c
lo<004@0lo?c0`0000220??cl`050000003clo<0lo?c0??cl`000000b@3clo<3000001H0lo?c0014
0??cl`030000003clo<0lo?c0880lo?c00D000000??cl`3clo<0lo?c0000003;0??cl`030000003c
lo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c0880lo?c00D000000??cl`3clo<0lo?c0000
003;0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c0880lo?c00D00000
0??cl`3clo<0lo?c0000003;0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0
lo?c08<0lo?c00<000000??cl`000000c03clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`00
0000lo?c0??cl`230??cl`030000003clo<000000<`0lo?c00<000000??cl`3clo<0503clo<004@0
lo?c00<000000??cl`3clo<0P`3clo<00`000000lo?c0000003<0??cl`030000003clo<0lo?c01@0
lo?c00140??cl`030000003clo<0lo?c08<0lo?c00<000000??cl`000000c03clo<00`000000lo?c
0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`230??cl`030000003clo<000000<`0lo?c00<0
00000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0P`3clo<00`000000lo?c0000003<
0??cl`030000003clo<0lo?c01@0lo?c00140??cl`<00000P`3clo<00`000000lo?c0000003:0??c
l`<000005P3clo<004@0lo?c00<000000??cl`3clo<0P`3clo<00`000000lo?c0000003<0??cl`03
0000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c08<0lo?c00<000000??cl`000000
c03clo<00`000000lo?c0??cl`0D0??cl`00A03clo<00`000000lo?c0??cl`230??cl`030000003c
lo<000000<`0lo?c00<000000??cl`3clo<0503clo<004@0lo?c00<000000??cl`3clo<0P`3clo<0
0`000000lo?c0000003<0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c
08<0lo?c0P00003=0??cl`030000003clo<0lo?c01@0lo?c00140??cl`030000003clo<0lo?c08<0
lo?c0P00003=0??cl`030000003clo<0lo?c01@0lo?c000`0??cl`<000000`3clo<00`000000lo?c
0??cl`03000000P0lo?c00<000000??cl`3clo<0Q03clo<00`000000lo?c0??cl`3;0??cl`030000
003clo<0lo?c01@0lo?c000_0??cl`050000003clo<0lo?c0??cl`0000001P3clo<00`000000lo?c
0??cl`070??cl`030000003clo<0lo?c08@0lo?c00<000000??cl`3clo<0b`3clo<00`000000lo?c
0??cl`0D0??cl`00;`3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c00<000000??cl`3clo<0
1`3clo<00`000000lo?c0??cl`3o0??cle<0lo?c00<000000??cl`3clo<0503clo<002l0lo?c00D0
00000??cl`3clo<0lo?c000000060??cl`030000003clo<0lo?c00L0lo?c0`0000140??cl`030000
003clo<0lo?c04`0lo?c00<000000??cl`3clo<0B`3clo<00`000000lo?c0??cl`1<0??cl`030000
003clo<0lo?c01d0lo?c0`00000F0??cl`00;`3clo<01@000000lo?c0??cl`3clo<0000000H0lo?c
00<000000??cl`3clo<01`3clo<00`000000lo?c0??cl`050??cl`030000003clo<0lo?c00d0lo?c
00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c
00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c
00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c
00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c
00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00`0lo?c
00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0=0??cl`030000003clo<0lo?c00d0lo?c
00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`040000003clo<0lo?c000001H0
lo?c000_0??cl`050000003clo<0lo?c0??cl`0000001P3clo<00`000000lo?c0??cl`070??cl`03
0000003clo<0lo?c00D0lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c00d0lo?c00<000000??cl`3clo<0303clo<00`000000lo?c0??cl`0=0??cl`03
0000003clo<0lo?c00d0lo?c00<000000??cl`3clo<03@3clo<00`000000lo?c0??cl`0<0??cl`03
0000003clo<0lo?c00d0lo?c00@000000??cl`3clo<000005P3clo<002l0lo?c00D000000??cl`3c
lo<0lo?c000000060??cl`030000003clo<0lo?c00L0lo?co`00001G000001H0lo?c000_0??cl`05
0000003clo<0lo?c0??cl`0000001@3clo<200000?l0lo?cMP3clo<00300lo?c0`0000070??cl`03
0000003clo<0lo?c0?l0lo?cM03clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<0
0>H0lo?c0`0000370??cl`00i@3clo<01@000000lo?c0??cl`3clo<000000<H0lo?c003U0??cl`05
0000003clo<0lo?c0??cl`000000aP3clo<00=/0lo?c0`0000040??cl`<0000000@0lo?c00000000
000000000`3clo<700000080lo?c0P0000050??cl`@00000/03clo<00=`0lo?c00<000000??cl`3c
lo<0103clo<00`000000lo?c000000050??cl`040000003clo<0lo?c00000080lo?c00@000000??c
l`3clo<000000P3clo<00`000000lo?c0??cl`060??cl`030000003clo<0lo?c0:d0lo?c003L0??c
l`030000003clo<0lo?c00@0lo?c00<000000??cl`3clo<00P0000030??cl`040000003clo<0lo?c
00000080lo?c00@000000??cl`3clo<000002`3clo<00`000000lo?c0??cl`2]0??cl`00g03clo<0
10000000lo?c0??cl`0000030??cl`030000003clo<000000080lo?c00@000000??cl`3clo<00000
0P3clo<010000000lo?c0??cl`0000020??cl`@00000203clo<00`000000lo?c0??cl`2]0??cl`00
g03clo<4000000<0lo?c00<000000??cl`0000000P3clo<010000000lo?c0??cl`0000020??cl`03
0000003clo<0lo?c0080000000D0lo?c0000003clo<0lo?c000000060??cl`800000/03clo<00=`0
lo?c00@000000??cl`3clo<000000P3clo<200000080lo?c1P000000103clo<000000000003clo<2
000000030??cl`000000lo?c00800000203clo<00`000000lo?c0??cl`2^0??cl`00g03clo<00`00
0000lo?c0??cl`0Q0??cl`030000003clo<0lo?c0:d0lo?c003L0??cl`050000003clo<0lo?c0??c
l`0000006`3clo<01@000000lo?c0??cl`3clo<000000:l0lo?c003K0??cl`H000000P3clo<00`00
0000lo?c0??cl`0G0??cl`<00000/03clo<00?l0lo?c/@3clo<00?l0lo?c/@3clo<00001\
\>"],
  ImageRangeCache->{{{0, 431}, {265.938, 0}} -> {0.521897, -0.0200598, \
0.00127489, 0.000493602}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.2 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 922}},
ScreenStyleEnvironment->"Working",
WindowToolbars->"RulerBar",
WindowSize->{1272, 882},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{
  Inherited, Inherited, Cell[ "Kocabas", "Header"]}, {
    Cell[ "Drude-Lorentz Fit To Experimental Permittivity Data", "Header"], 
      Inherited, Inherited}},
StyleDefinitions -> "TMJv4Style.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 102, 1, 55, "Subtitle"],
Cell[1881, 56, 86, 4, 40, "Author"],
Cell[1970, 62, 393, 7, 46, "Text"],
Cell[2366, 71, 251, 3, 60, "DisplayFormula"],
Cell[2620, 76, 783, 24, 46, "Text"],
Cell[3406, 102, 390, 12, 23, "Reference"],
Cell[3799, 116, 172, 4, 37, "Text"],
Cell[3974, 122, 2075, 36, 500, "InputOnly"],

Cell[CellGroupData[{
Cell[6074, 162, 953, 20, 225, "Input"],
Cell[7030, 184, 53, 1, 43, "Output"],
Cell[7086, 187, 53, 1, 43, "Output"]
}, Open  ]],
Cell[7154, 191, 389, 8, 46, "Text"],

Cell[CellGroupData[{
Cell[7568, 203, 1206, 23, 241, "Input"],
Cell[8777, 228, 185, 3, 43, "Output"],
Cell[8965, 233, 364, 6, 43, "Output"],
Cell[9332, 241, 383, 6, 60, "Output"]
}, Open  ]],
Cell[9730, 250, 131, 3, 27, "Text"],
Cell[9864, 255, 593, 9, 187, "Input"],
Cell[10460, 266, 190, 4, 27, "Text"],

Cell[CellGroupData[{
Cell[10675, 274, 283, 5, 63, "Input"],
Cell[10961, 281, 503, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11501, 295, 225, 5, 43, "Input"],
Cell[11729, 302, 518, 9, 43, "Output"]
}, Open  ]],
Cell[12262, 314, 141, 3, 27, "Text"],
Cell[12406, 319, 1302, 26, 163, "Input"],
Cell[13711, 347, 109, 2, 27, "Text"],
Cell[13823, 351, 217, 5, 27, "Text"],
Cell[14043, 358, 168, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[14236, 365, 3969, 77, 483, "Input"],

Cell[CellGroupData[{
Cell[18230, 446, 44643, 1201, 283, 17898, 867, "GraphicsData", "PostScript", \
"Graphics"],
Cell[62876, 1649, 90773, 2603, 283, 42681, 2006, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],
Cell[153676, 4256, 917, 18, 65, "Text"],
Cell[154596, 4276, 3503, 58, 607, "Input"],

Cell[CellGroupData[{
Cell[158124, 4338, 29, 0, 61, "Section"],

Cell[CellGroupData[{
Cell[158178, 4342, 65, 0, 54, "Subsection"],
Cell[158246, 4344, 484, 10, 46, "Text"],

Cell[CellGroupData[{
Cell[158755, 4358, 382543, 4725, 92, 382443, 4722, "GraphicsData", \
"Metafile", "PlacedGraphics"],
Cell[541301, 9085, 352869, 4359, 95, 352769, 4356, "GraphicsData", \
"Metafile", "PlacedGraphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[894207, 13449, 3645, 62, 667, "Input"],
Cell[897855, 13513, 917, 20, 45, "Print"],
Cell[898775, 13535, 197453, 5660, 550, 90629, 4338, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1096277, 19201, 66, 0, 54, "Subsection"],
Cell[1096346, 19203, 170, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[1096541, 19210, 2719, 47, 526, "Input"],
Cell[1099263, 19259, 1357, 32, 44, "Print"],

Cell[CellGroupData[{
Cell[1100645, 19295, 90866, 2706, 228, 46624, 2157, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1191514, 22003, 55008, 1371, 283, 19360, 928, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

